#!bin/bash

declare G_LOG_FILE="/tmp/fi-multiClusterTools.log"
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    log.sh
## Description: 日志函数，由各个模块调用
## Version:     FusionInsight Hadoop V100R002C02
## Created:     Thursday, May 16, 2013
################################################################################

# ---------------------------------------------------------------------------- #
#                                   函数定义                                   #
# ---------------------------------------------------------------------------- #
################################################################################
# Function: log
# Description: 记录到日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: 该函数是最低层日志函数，不会被外部函数直接调用
################################################################################
log()
{
    # 调用日志打印函数的函数名称
    local func_name="$1"
    shift

    # 打印的日志级别
    local log_level="$1"
    shift

    # 外部调用日志打印函数时所在的行号
    local lineNO="$1"
    shift

    # 打印时间、日志级别、日志内容、脚本名称、调用日志打印函数的函数、打印时的行号及脚本的进程号
    local log_time="$(date -d today +'%Y-%m-%d %H:%M:%S')"
    printf "[${log_time}] ${log_level} $* [${g_scriptName}(${func_name}):${lineNO}]($$)\n" \
        >> "${G_LOG_FILE}" 2>&1

    return 0
}

################################################################################
# Function: log_error
# Description: 对外部提供的日志打印函数：记录EEROR级别日志到日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
log_error()
{
    # FUNCNAME是shell的内置环境变量，是一个数组变量，其中包含了整个调用链上所有
    # 的函数名字，通过该变量取出调用该函数的函数名
    log "${FUNCNAME[1]}" "ERROR" "$@"
}

################################################################################
# Function: log_info
# Description: 对外部提供的日志打印函数：记录INFO级别日志到日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
log_info()
{
    # FUNCNAME是shell的内置环境变量，是一个数组变量，其中包含了整个调用链上所有
    # 的函数名字，通过该变量取出调用该函数的函数名
    log "${FUNCNAME[1]}" "INFO" "$@"
}

################################################################################
# Function: log_debug
# Description: 对外部提供的日志打印函数：记录DEBUG级别日志到日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
log_debug()
{
    # FUNCNAME是shell的内置环境变量，是一个数组变量，其中包含了整个调用链上所有
    # 的函数名字，通过该变量取出调用该函数的函数名
    log "${FUNCNAME[1]}" "DEBUG" "$@"
}

################################################################################
# Function: log_warn
# Description: 对外部提供的日志打印函数：记录WARN级别日志到日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
log_warn()
{
    # FUNCNAME是shell的内置环境变量，是一个数组变量，其中包含了整个调用链上所有
    # 的函数名字，通过该变量取出调用该函数的函数名
    log "${FUNCNAME[1]}" "WARN" "$@"
}

################################################################################
# Function: showLog
# Description: 记录日志到文件并显示到屏幕
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: 该函数是低层日志函数，不会被外部函数直接调用
################################################################################
showLog()
{
    # 把日志打印到日志文件。FUNCNAME是shell的内置环境变量，是一个数组变量，其中
    # 包含了整个调用链上所有的函数名字，通过该变量取出调用该函数的函数名
    log "${FUNCNAME[2]}" "$@"

    # 如果是EEROR日志级别，则显示在屏幕上要带前缀：ERROR
    if [ "$1" = "ERROR" ]; then
        echo -e "[`date +%y-%m-%d\" \"%T`]: ERROR:$3"
    elif [ "$1" = "WARN" ];then
        echo -e "[`date +%y-%m-%d\" \"%T`]: WARN: $3"
    else
        echo -e "[`date +%y-%m-%d\" \"%T`]: $3"
    fi
}

showEcho()
{
    echo -e "`date +%y-%m-%d\" \"%T`: $@"
}

################################################################################
# Function: showLog_error
# Description: 对外部提供的日志打印函数：记录ERROR级别日志到文件并显示到屏幕
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
showLog_error()
{
    showLog ERROR "$@"
}

################################################################################
# Function: showLog_info
# Description: 对外部提供的日志打印函数：记录INFO级别日志到文件并显示到屏幕
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
showLog_info()
{
    showLog INFO "$@"
}

################################################################################
# Function: showLog_warn
# Description: 对外部提供的日志打印函数：记录WARN级别日志到文件并显示到屏幕
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
showLog_warn()
{
    showLog WARN "$@"
}

################################################################################
# Function: showLog_debug
# Description: 对外部提供的日志打印函数：记录DEBUG级别日志到文件并显示到屏幕
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
showLog_debug()
{
    showLog DEBUG "$@"
}