#!/bin/bash

declare g_curPath=""                                            # 当前脚本所在的目录

####################################################################
## @Usage getCurPath 获取脚本所在的当前路径，并取脚本名
####################################################################
getCurPath()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    cd - > /dev/null
}

####################################################################
#  FUNCTION     : hue原生页面配置
####################################################################
recoverHueConfig()
{
    if [ -f ${CONTROLLER_HOME}/etc/om/hueport.properties ]; then
        local CLUSTER_ID="${1}"
        ## hue的httpd监听规则是组件自己独有的,规则为212+集群ID
        local webFloatIP="$(cat ${CONTROLLER_HOME}/inst/conf/oms-config.ini | grep -w "ws_float_ip" | awk -F '=' '{print $2}')"
        local clusterIdLength=$(expr length ${CLUSTER_ID})
        if [ ${clusterIdLength}X == "1"X ]; then
            local hueport="2120"${CLUSTER_ID}
        else
            local hueport="212"${CLUSTER_ID}
        fi
        local tmp="Listen ${webFloatIP}:${hueport} https"
        sed -i "s/#${tmp}/${tmp}/g" ${OM_HTTPD_HOME}/conf/httpd.conf
        return $?
    fi
}

main()
{
    if [ $# != 1 ]; then
        showLog_error "${LINENO}" "Invalid parameter :$@"
        exit 1
    fi
    recoverHueConfig $@
    if [ $? -ne 0 ];then
        showLog_error "${LINENO}" "Modify hue httpd conf failed"
    fi
    showLog_info "${LINENO}" "Modify hue httpd conf successfully"
    
    restartOmsAndSyncallfile || exit 1
}

# 判断是否为 omm 用户
if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can execute."
    exit 1
fi

getCurPath

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/../../func/functions.sh" || { echo "Failed to load functions.sh."; exit 1; }
. "${g_curPath}/../../func/log.sh" || { echo "Failed to load log.sh."; exit 1; }
. "${g_curPath}/../../func/constants.sh" || { echo "Failed to load constants.sh."; exit 1; }

main $@
