#!/bin/bash

declare g_curPath=""                                            # 当前脚本所在的目录

####################################################################
## @Usage getCurPath 获取脚本所在的当前路径，并取脚本名
####################################################################
getCurPath()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    cd - > /dev/null
}

####################################################################
#  FUNCTION     : 修改oozie客户端配置，主备都需要改，需重新下载客户端
####################################################################
recoverOozieConfig()
{
    for file in `find   $BIGDATA_HOME/components/FusionInsight_Porter_6.5.1.*/Oozie/client -name job.properties`; do
        if [ -f ${file} ] ; then
            local CLUSTER_ID="${1}"
            local prefix="c""${CLUSTER_ID}"
            sed -i "s/hdfs:\/\/hacluster/hdfs:\/\/${prefix}hacluster/g" $file
        fi
    done
}

main()
{
    if [ $# != 1 ]; then
        showLog_error "${LINENO}" "Invalid parameter :$@"
        exit 1
    fi
    recoverOozieConfig $@
    if [ $? -ne 0 ];then
        showLog_error "${LINENO}" "Modify oozie job properties failed"
    fi
    showLog_info "${LINENO}" "Modify oozie job properties successfully"
}

# 判断是否为 omm 用户
if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can execute."
    exit 1
fi

getCurPath

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/../../func/functions.sh" || { echo "Failed to load functions.sh."; exit 1; }
. "${g_curPath}/../../func/log.sh" || { echo "Failed to load log.sh."; exit 1; }
. "${g_curPath}/../../func/constants.sh" || { echo "Failed to load constants.sh."; exit 1; }

main $@