#!/bin/bash

# ---------------------------------------------------------------------------- #
#                                   变量定义                                    #
# ---------------------------------------------------------------------------- #
## 所有要替换的表,用于导入前保留原始数据
declare -a g_all_public_schema_tables=("OM_PACKS" "OM_PACK_PACKAGES" "OM_NODES" "TENANT" "TENANT_SERVICES" "TENANT_RESOURCES" "RESOURCEPOOL" "RESOURCEPOOL_SERVICE" "RESOURCEPOOL_HOSTS" "RESOURCES_PROPERTIES" "OM_CLIENT_INFO" "OM_AZS" "AOS_ROLE_INFO" "AOS_ROLE_USER_GROUP" "OM_USER_PRIVILEGES" "OM_PRIVILAGES_TOPOLY" "TBL_FM_HEALTHCHECK_CONFINFO" "TBL_BASELINE" "TBL_BASELINE_TEMPLET" "TBL_MONITOR_CUSTOMIZE" "OM_CLUSTERS" "OM_SERVICES" "OM_ROLES" "OM_INSTANCE_GROUP" "OM_TOPOLOGY" "OM_CONFIGURATIONS" "OM_NODES_ISOLATION" "OM_FSMTRANSITIONS" "OM_CLUSTER_PATCH_INFO"  "TBL_SYSTEM_LICENSE" "TBL_UPGRADE_MARK" "OM_PACK_PACKAGES" "OM_PACKS" "OM_INSTANCE_GROUP_ID_SEQ" "USERPRIVILAGE_ID_GEN_SEQ" "TBL_BACKUP_CONFIGS" "TBL_BACKUP_TASKINFO" "TBL_RECOVERY_CONFIGS" "TBL_RECOVERY_TASKINFO" "MPP_LOGIC_CLUSTER" "MPP_LOGIC_CLUSTER_LOOP_HOSTS" "MPP_LOGIC_CLUSTER_REDISTRIBUTION_TASK" "OM_MPPDB_CLIENT_INFO" "TBL_FM_ALARM_LOCATIONINFO" "TBL_FM_EVENT_ADDITIONALINFO" "TBL_FM_EVENT_LOCATIONINFO" "TBL_FM_EVENT_RES" "OM_CLIENT_INFO_CLIENT_ID_SEQ" "OM_PACKS_PACK_ID_SEQ")
declare -a g_all_omm_schema_tables=("TBL_PM_ALARMTHRESHOLD")
declare -a g_all_redis_schema_tables=("CLUSTER" "INSTANCE" "OMINSTANCES" "SLOT")

## public schema的表
declare -a g_complete_tables=("OM_PACKS" "OM_PACK_PACKAGES" "OM_NODES" "TENANT" "TENANT_SERVICES" "TENANT_RESOURCES" "RESOURCEPOOL" "RESOURCEPOOL_SERVICE" "RESOURCEPOOL_HOSTS" "RESOURCES_PROPERTIES" "OM_CLIENT_INFO" "OM_AZS" "AOS_ROLE_INFO" "AOS_ROLE_USER_GROUP" "OM_USER_PRIVILEGES" "OM_PRIVILAGES_TOPOLY" "TBL_FM_HEALTHCHECK_CONFINFO" "TBL_BASELINE" "TBL_BASELINE_TEMPLET" "TBL_MONITOR_CUSTOMIZE" "OM_CLUSTERS" "OM_SERVICES" "OM_ROLES" "OM_INSTANCE_GROUP" "OM_TOPOLOGY" "OM_CONFIGURATIONS" "OM_NODES_ISOLATION" "OM_FSMTRANSITIONS" "OM_CLUSTER_PATCH_INFO"  "TBL_SYSTEM_LICENSE" "TBL_UPGRADE_MARK" "TBL_BACKUP_CONFIGS" "TBL_BACKUP_TASKINFO" "TBL_RECOVERY_CONFIGS" "TBL_RECOVERY_TASKINFO" "MPP_LOGIC_CLUSTER" "MPP_LOGIC_CLUSTER_LOOP_HOSTS" "MPP_LOGIC_CLUSTER_REDISTRIBUTION_TASK" "OM_MPPDB_CLIENT_INFO" "TBL_FM_ALARM_LOCATIONINFO" "TBL_FM_EVENT_ADDITIONALINFO" "TBL_FM_EVENT_LOCATIONINFO" "TBL_FM_EVENT_RES")
## OMM_1 schema的表
declare -a g_ommschema_tables=("TBL_PM_ALARMTHRESHOLD")
## 需要提前处理的表
declare -a g_predelete_tables=("OM_PACK_PACKAGES" "OM_PACKS")
## 序列需要特殊处理
declare -a g_sequence_list=("OM_INSTANCE_GROUP_ID_SEQ" "USERPRIVILAGE_ID_GEN_SEQ" "OM_CLIENT_INFO_CLIENT_ID_SEQ" "OM_PACKS_PACK_ID_SEQ");
## Redis逻辑集群迁移
declare -a g_redis_logical=("CLUSTER" "INSTANCE" "OMINSTANCES" "SLOT")

declare CLUSTER_ID=""
declare CURRENT_PATH=""
declare DBCOMMAND=""                                            # ommdba用户数据库sql执行语句
declare DBPASSWORD=""                                           # ommdba用户的gsdb数据库密码
declare OMMDBCOMMAND=""                                         # omm用户的数据库sql执行语句
declare OMMDBPASSWORD=""                                        # omm用户的gsdb数据库密码
declare BACKTABLEDIR=""                                         # 当前oms备份被覆盖表目录

# ---------------------------------------------------------------------------- #
#                                   函数定义                                    #
# ---------------------------------------------------------------------------- #

####################################################################
## @Usage getCurPath 获取脚本所在的当前路径，并取脚本名
####################################################################
getCurPath()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    CURRENT_PATH="$(pwd)"
    cd - > /dev/null
}

####################################################################
#  FUNCTION     : 备份被替换文件
####################################################################
backupFiles()
{
    log_info "${LINENO}" "Replace the placeholder value of the $CURRENT_PATH/recover.lst"
    BACKUP_LIST_FILE="$CURRENT_PATH/recover.lst"
    DST_FILE="$CURRENT_PATH/backupOverData/backupFile.tar"
    local common_home=`dirname ${JAVA_HOME}`

    mkdir -vp $CURRENT_PATH/backupOverData >/dev/null 2>&1
    chmod 775 $CURRENT_PATH/backupOverData

    sed -i "s:#INSTALLCONTROLLERPATH#:${CONTROLLER_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#INSTALLTOMCATPATH#:${OM_TOMCAT_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#INSTALLOMSPATH#:${OMS_RUN_PATH}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#BIGDATADATAHOME#:${BIGDATA_DATA_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#BIGDATAHOME#:${BIGDATA_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#JAVAHOME#:${JAVA_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#OMHTTPDHOME#:${OM_HTTPD_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#BIGDATA_COMMON#:${BIGDATA_COMMON}:g" "$BACKUP_LIST_FILE"
    sed -i "s/#CLUSTER_ID#/${CLUSTER_ID}/g" "$BACKUP_LIST_FILE"
    sed -i "s:#COMMON_HOME#:${common_home}:g" "$BACKUP_LIST_FILE"

    tar -rPp -T "$BACKUP_LIST_FILE" -f "$DST_FILE" >/dev/null 2>&1
    chmod 775 "$DST_FILE"
}

####################################################################
#  FUNCTION     : 判断表是否存在
####################################################################
isTableExist(){
    local tableSchema=$1
    local tableName=$2
    local isTableExistCommand="SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='${tableSchema}' AND TABLE_NAME='${tableName}';"
    local result=$(${DBCOMMAND} -W ${DBPASSWORD} -c "${isTableExistCommand}")
    local tableNum=$(echo ${result} | awk  '{printf $3}')
    if [ ${tableNum}X == "1"X ];then
        return 0
    else
        return 1
    fi
}

####################################################################
#  FUNCTION     : 判断序列是否存在
####################################################################
isSequenceExist()
{
    local sequenceName=$1
    COMMAND="SELECT COUNT(*) FROM USER_SEQUENCES WHERE SEQUENCE_NAME='${sequenceName}';"
    local result=$(${OMMDBCOMMAND} -W ${OMMDBPASSWORD} -c "$COMMAND")
    local sequenceNum=$(echo $result | awk  '{printf $3}')
    if [ ${sequenceNum}X == "1"X ];then
        return 0
    else
        return 1
    fi
}

####################################################################
#  FUNCTION     : 执行sql
####################################################################
executeSql(){
    local command="${1}"
    local tableName="${2}"
    local tableSchema="${3}"
    local executeType="${4}"
    ${DBCOMMAND} -W ${DBPASSWORD} -c "${command}" >/dev/null 2>&1
    if [ $? -ne 0 ];then
        isTableExist ${tableSchema} ${tableName}
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "table ${tableSchema}.${tableName} does not exist."
        else
            showLog_error "${LINENO}" "Failed to execute command :'${command}'."
            exit 1
        fi
    else
        log_info "${LINENO}" "Success to execute command :'${command}'."
        if [ "${executeType}"X == ${IMPORT_EXECUTE_TYPE}X ]; then
            showLog_info "${LINENO}" "Import Table : ${tableSchema}.${tableName} Successfully"
        fi
    fi
}

####################################################################
#  FUNCTION     : 备份被覆盖的public schema 表
####################################################################
backupPublicSchemaTables()
{
    for table in "${g_all_public_schema_tables[@]}"
    do
        touch ${BACKTABLEDIR}/${PUBLIC_SCHEMA}-$table
        chmod 775 ${BACKTABLEDIR}/${PUBLIC_SCHEMA}-$table
        COMMAND="COPY (select * from $table) TO '${BACKTABLEDIR}/${PUBLIC_SCHEMA}-$table';"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}" "${EXPORT_EXECUTE_TYPE}"
    done
}

####################################################################
#  FUNCTION     : 备份被覆盖的omm schema 表
####################################################################
backupOmmSchemaTables(){
    for table in "${g_all_omm_schema_tables[@]}"
    do
        touch ${BACKTABLEDIR}/${OMM_SCHEMA}-$table
        chmod 775 ${BACKTABLEDIR}/${OMM_SCHEMA}-$table
        COMMAND="COPY (select * from ${OMM_SCHEMA}.$table) TO '${BACKTABLEDIR}/${OMM_SCHEMA}-$table';"
        executeSql "${COMMAND}" "${table}" "${OMM_SCHEMA}" "${EXPORT_EXECUTE_TYPE}"
    done
}

backupRedisSchemaTables(){
   for table in "${g_all_redis_schema_tables[@]}"
   do
       touch ${BACKTABLEDIR}/${REDIS_SCHEMA}-$table
       chmod 775 ${BACKTABLEDIR}/${REDIS_SCHEMA}-$table
       COMMAND="COPY (select * from ${REDIS_SCHEMA}.$table) TO '${BACKTABLEDIR}/${REDIS_SCHEMA}-$table';"
       executeSql "${COMMAND}" "${table}" "${REDIS_SCHEMA}" "${EXPORT_EXECUTE_TYPE}"
   done
}

####################################################################
#  FUNCTION     : 备份被覆盖的表数据
####################################################################
backupDbTables(){
    chmod 775 $(dirname $(dirname ${CURRENT_PATH})) -R

    backupPublicSchemaTables

    backupOmmSchemaTables

    backupRedisSchemaTables
}

####################################################################
#  FUNCTION     : 部件包不区分集群,待导入的包信息可能存在多余包信息,需要删除
####################################################################
deleteRedundantPack(){
    log_info "${LINENO}" "Begin deleting Redundant Packages..."
    rm -rf $CURRENT_PATH/tmp && mkdir $CURRENT_PATH/tmp
    touch $CURRENT_PATH/tmp/OM_PACKS
    touch $CURRENT_PATH/tmp/OM_PACK_PACKAGES
    chmod 775 $(dirname $CURRENT_PATH) -R
    COMMAND="COPY (select * from OM_PACKS) TO '$CURRENT_PATH/tmp/OM_PACKS';"
    executeSql "${COMMAND}" "OM_PACKS" "${PUBLIC_SCHEMA}"

    COMMAND="COPY (select * from OM_PACK_PACKAGES) TO '$CURRENT_PATH/tmp/OM_PACK_PACKAGES';"
    executeSql "${COMMAND}" "OM_PACK_PACKAGES" "${PUBLIC_SCHEMA}"

    while read LINE
    do
        packName=$(echo ${LINE} |  awk '{print $5}')
        grep -w ${packName} $CURRENT_PATH/tmp/OM_PACKS >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            log_info "${LINENO}" "clean OM_PACKS info : ${LINE}"
            sed -i "/${packName}/d"  /tmp/backupDir/tables/OM_PACKS
        fi
    done  < /tmp/backupDir/tables/OM_PACKS

    while read LINE
    do
        packName=$(echo ${LINE} |  awk '{print $4}')
        grep -w ${packName} $CURRENT_PATH/tmp/OM_PACK_PACKAGES >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            log_info "${LINENO}" "clean OM_PACK_PACKAGES info : ${LINE}"
            sed -i "/${packName}/d"  /tmp/backupDir/tables/OM_PACK_PACKAGES
        fi
    done  < /tmp/backupDir/tables/OM_PACK_PACKAGES
    rm -rf $CURRENT_PATH/tmp
    log_info "${LINENO}" "End deleted Redundant Packages."
}

####################################################################
#  FUNCTION     : 修改node表
####################################################################
modifyOmNodes(){
    log_info "${LINENO}" "begin modify om_nodes info..."
    touch /tmp/backupDir/tables/OMS_NODE_INFO
    chmod 775 /tmp/backupDir/tables/OMS_NODE_INFO
    COMMAND="COPY (select * from OM_NODES) TO '/tmp/backupDir/tables/OMS_NODE_INFO';"
    executeSql "${COMMAND}" "OM_NODES" "${PUBLIC_SCHEMA}"

    while read LINE
    do
        local nodeName=$(echo ${LINE} |  awk '{print $1}')
        grep -w ${nodeName} /tmp/backupDir/tables/OM_NODES >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "${LINE}" >> /tmp/backupDir/tables/OM_NODES
        fi
    done  < /tmp/backupDir/tables/OMS_NODE_INFO
    rm -rf /tmp/backupDir/tables/OMS_NODE_INFO
    log_info "${LINENO}" "end modify om_nodes info..."
}

####################################################################
#  FUNCTION     : 修改node表
####################################################################
preDeleteTables()
{
    for table in "${g_predelete_tables[@]}"
    do
        log_debug "${LINENO}" "predelete DbTables ${table}."
        COMMAND="delete from ${table};"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}" "${DELETE_EXECUTE_TYPE}"
    done
}

####################################################################
#  FUNCTION     : 修改序列
####################################################################
modifySequence()
{
    initOmmDBInfo
    for sequence in "${g_sequence_list[@]}"
    do
        log_debug "${LINENO}" "importing sequence ${sequence}."
        isSequenceExist ${sequence}
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "sequence ${sequence} does not exist. skip it "
            continue
        fi
        local backData="/tmp/backupDir/tables/${sequence}"
        local last_value=$(cat ${backData} |  awk '{print $2}')
        COMMAND="select setval('${sequence}',${last_value});"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "$COMMAND" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            showLog_error "${LINENO}" "Failed to modify Sequence :${COMMAND}."
            exit 1
        else
            log_info "${LINENO}" "Success to execute command :${COMMAND}."
        fi
    done
}

####################################################################
#  FUNCTION     : 导入ommschema表数据
####################################################################
recoverOMMSchemaTables()
{
    for table in "${g_ommschema_tables[@]}"
    do
        log_debug "${LINENO}" "importing DbTables ${table}."
        isTableExist ${OMM_SCHEMA} ${table}
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "table ${OMM_SCHEMA}.${table} does not exist. skip it "
            continue
        fi
        COMMAND="delete from ${OMM_SCHEMA}.${table};"
        executeSql "${COMMAND}" "${table}" "${OMM_SCHEMA}" "${DELETE_EXECUTE_TYPE}"

        COMMAND="COPY ${OMM_SCHEMA}.$table from '/tmp/backupDir/tables/${table}';"
        executeSql "${COMMAND}" "${table}" "${OMM_SCHEMA}" "${IMPORT_EXECUTE_TYPE}"
    done
}

####################################################################
#  FUNCTION     : 导入redis逻辑集群数据
####################################################################
recoverRedisLogical()
{
    for table in "${g_redis_logical[@]}"
    do
        log_debug "${LINENO}" "importing DbTables ${REDIS_SCHEMA}.${table}."
        isTableExist ${REDIS_SCHEMA} ${table}
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "table ${REDIS_SCHEMA}.${table} does not exist. skip it "
            continue
        fi
        COMMAND="delete from ${REDIS_SCHEMA}.${table};"
        executeSql "${COMMAND}" "${table}" "${REDIS_SCHEMA}" "${DELETE_EXECUTE_TYPE}"

        COMMAND="COPY ${REDIS_SCHEMA}.$table from '/tmp/backupDir/tables/${REDIS_SCHEMA}/${table}';"
        executeSql "${COMMAND}" "${table}" "${REDIS_SCHEMA}" "${IMPORT_EXECUTE_TYPE}"
    done
}

####################################################################
#  FUNCTION     : 导入表数据
####################################################################
recoverDbTables()
{
    for table in "${g_complete_tables[@]}"
    do
        log_debug "${LINENO}" "importing DbTables ${table}."
        isTableExist ${PUBLIC_SCHEMA} ${table}
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "table ${PUBLIC_SCHEMA}.${table} does not exist. skip it "
            continue
        fi
        COMMAND="delete from ${table};"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}" "${DELETE_EXECUTE_TYPE}"

        if [ -s "/tmp/backupDir/tables/${table}" ];then
            COMMAND="COPY $table from '/tmp/backupDir/tables/${table}';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}" "${IMPORT_EXECUTE_TYPE}"
        else
            showLog_warn "${LINENO}" "/tmp/backupDir/tables/${table} is empty,skip it"
        fi
    done
}

####################################################################
#  FUNCTION     : 恢复Okerberos/Oldap进程数据
####################################################################
recoverOkerberos_Oldap()
{
    showLog_info "${LINENO}" "recover oldap begin..."
    \cp -rf /tmp/backupDir/${BIGDATA_DATA_HOME}/ldapData/oldap ${BIGDATA_DATA_HOME}/ldapData/
    \cp -rf /tmp/backupDir/${CONTROLLER_HOME}/kerberos_user_specific_binay/kerberos/var/krb5kdc ${CONTROLLER_HOME}/kerberos_user_specific_binay/kerberos/var/
    cp -r /tmp/backupDir/${CONTROLLER_HOME}/etc/om/omsconfig/ldapserver/KEYENV_VARS ${CONTROLLER_HOME}/etc/om/omsconfig/ldapserver/KEYENV_VARS
    cp -r /tmp/backupDir/${CONTROLLER_HOME}/etc/om/omsconfigmodle/ldapserver/KEYENV_VARS ${CONTROLLER_HOME}/etc/om/omsconfigmodle/ldapserver/KEYENV_VARS
    cp -r /tmp/backupDir/${CONTROLLER_HOME}/etc/om/omsconfigmodle/initconfig/ldap.properties ${CONTROLLER_HOME}/etc/om/omsconfigmodle/initconfig/ldap.properties
    chmod -R 700 ${BIGDATA_DATA_HOME}/ldapData/
    showLog_info "${LINENO}" "recover oldap end..."
}

####################################################################
#  FUNCTION     : 恢复Kerberos进程数据
####################################################################
recoverKerberos()
{
    showLog_info "${LINENO}" "recover Kerberos begin..."
    cp -r /tmp/backupDir/${CONTROLLER_HOME}/packaged-distributables/home/keytabs/*.COM ${CONTROLLER_HOME}/packaged-distributables/home/keytabs/
    cp -r /tmp/backupDir/${CONTROLLER_HOME}/packaged-distributables/home/keytabs/KERBEROS_SERVER_PWD ${CONTROLLER_HOME}/packaged-distributables/home/keytabs/KERBEROS_SERVER_PWD
    cp -r /tmp/backupDir/${CONTROLLER_HOME}/packaged-distributables/home/keytabs/service_passwd ${CONTROLLER_HOME}/packaged-distributables/home/keytabs/service_passwd
    chmod -R 700 ${BIGDATA_DATA_HOME}/ldapData/
    showLog_info "${LINENO}" "recover Kerberos end..."
}

####################################################################
#  FUNCTION     : 恢复CAS进程数据
####################################################################
recoverCAS(){
    cp -r /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/cas/WEB-INF/classes/config/4A.properties ${OM_TOMCAT_HOME}/webapps/cas/WEB-INF/classes/config/4A.properties
}

####################################################################
#  FUNCTION     : 恢复ACS_AOS进程数据
####################################################################
recoverACS_AOS(){
    \cp -r /tmp/backupDir/${CONTROLLER_HOME}/share/om/aos/plugin/${CLUSTER_ID} ${CONTROLLER_HOME}/share/om/aos/plugin/
}

####################################################################
#  FUNCTION     : 恢复WEB进程数据
####################################################################
recoverWeb(){
    cp -r /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/dock4A_white_ip.properties ${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/dock4A_white_ip.properties
}

####################################################################
#  FUNCTION     : 恢复License进程数据
####################################################################
recoverLicense()
{
    showLog_info "${LINENO}" "recover license begin..."
    mkdir -p ${CONTROLLER_HOME}/etc/om/license/used
    mkdir -p ${CONTROLLER_HOME}/etc/om/license/temp
    \cp -r /tmp/backupDir/${CONTROLLER_HOME}/etc/om/license/used/${CLUSTER_ID} ${CONTROLLER_HOME}/etc/om/license/used
    \cp -r /tmp/backupDir/${CONTROLLER_HOME}/etc/om/license/temp/${CLUSTER_ID} ${CONTROLLER_HOME}/etc/om/license/temp
    showLog_info "${LINENO}" "recover license end..."
}

####################################################################
#  FUNCTION     : 恢复互信数据
####################################################################
recoverOmmPassphrase()
{
    showLog_info "${LINENO}" "recover omm passphrase begin..."
    cp -f /tmp/backupDir/home/omm/passphrase.file  /home/omm
    cp -f /tmp/backupDir/home/omm/passphrase.file.sha256 /home/omm
    \cp -r /tmp/backupDir/home/omm/.ssh /home/omm
    showLog_info "${LINENO}" "recover omm passphrase end..."
}

####################################################################
#  FUNCTION     : 恢复北向对接数据
####################################################################
recoverSnmp()
{
    showLog_info "${LINENO}" "recover snmp config begin..."
    [ -e /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/conf/northbound.property ] && cp -f /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/conf/northbound.property  ${OM_TOMCAT_HOME}/webapps/web/WEB-INF/conf/northbound.property
    [ -e /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/snmpConfig.xml ] && cp -f /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/snmpConfig.xml ${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/snmpConfig.xml
    [ -e /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/monitor_dump_config.properties ] && cp -f /tmp/backupDir/${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/monitor_dump_config.properties ${OM_TOMCAT_HOME}/webapps/web/WEB-INF/classes/config/monitor_dump_config.properties
    showLog_info "${LINENO}" "recover snmp config end..."
}

####################################################################
#  FUNCTION     : 恢复证书数据
####################################################################
recoverCrt()
{
    showLog_info "${LINENO}" "recover crt config begin..."
    \cp -rf /tmp/backupDir/${CONTROLLER_HOME}/security/cert/root  ${CONTROLLER_HOME}/security/cert/
    \cp -rf /tmp/backupDir/${CONTROLLER_HOME}/security/cert/subcert ${CONTROLLER_HOME}/security/cert/
    cp -rf /tmp/backupDir/${CONTROLLER_HOME}/packaged-distributables/ntpKeyFile ${CONTROLLER_HOME}/packaged-distributables/ntpKeyFile
    # 删除新oms使用的ntpkeys,保证使用备份数据生成ntpkeys
    rm -rf $OMS_RUN_PATH/workspace/conf/ntp*keys
    showLog_info "${LINENO}" "recover crt config end..."
}

####################################################################
#  FUNCTION     : 恢复ACS进程数据
####################################################################
recoverAcs()
{
    showLog_info "${LINENO}" "recover acs config begin..."
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/KRB_CONFIG  ${CONTROLLER_HOME}/share/om/acs/config/KRB_CONFIG
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/KRB_CONFIG_SRC ${CONTROLLER_HOME}/share/om/acs/config/KRB_CONFIG_SRC
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/KRB_REALM_CONFIG ${CONTROLLER_HOME}/share/om/acs/config/KRB_REALM_CONFIG
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/REALM_RESET_CONFIG ${CONTROLLER_HOME}/share/om/acs/config/REALM_RESET_CONFIG
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/REALM_RESET_CONFIG_KRB ${CONTROLLER_HOME}/share/om/acs/config/REALM_RESET_CONFIG_KRB
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/kerberos_realm.conf ${CONTROLLER_HOME}/share/om/acs/config/kerberos_realm.conf
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/AD_CONFIG ${CONTROLLER_HOME}/share/om/acs/config/AD_CONFIG
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/share/om/acs/config/AD_REALM_CONFIG   ${CONTROLLER_HOME}/share/om/acs/config/AD_REALM_CONFIG
    showLog_info "${LINENO}" "recover acs config end..."
}

####################################################################
#  FUNCTION     : 恢复其他配置数据
####################################################################
recoverOthers(){
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/etc/om/static-service-pool-config.xml  ${CONTROLLER_HOME}/etc/om/static-service-pool-config.xml
    cp -f /tmp/backupDir/${CONTROLLER_HOME}/etc/om/security.properties ${CONTROLLER_HOME}/etc/om/security.properties
}

####################################################################
#  FUNCTION     : hue原生页面配置
####################################################################
recoverHueConfig()
{
    if [ -f /tmp/backupDir/${CONTROLLER_HOME}/etc/om/hueport.properties ]; then
        cp -f /tmp/backupDir/${CONTROLLER_HOME}/etc/om/hueport.properties  ${CONTROLLER_HOME}/etc/om/hueport.properties
        ## hue的httpd监听规则是组件自己独有的,规则为212+集群ID
        local webFloatIP="$(cat ${CONTROLLER_HOME}/inst/conf/oms-config.ini | grep -w "ws_float_ip" | awk -F '=' '{print $2}')"
        local clusterIdLength=$(expr length ${CLUSTER_ID})
        if [ ${clusterIdLength}X == "1"X ]; then
            local hueport="2120"${CLUSTER_ID}
        else
            local hueport="212"${CLUSTER_ID}
        fi
        local tmp="Listen ${webFloatIP}:${hueport} https"
        sed -i "s/#${tmp}/${tmp}/g" ${OM_HTTPD_HOME}/conf/httpd.conf
    fi
}

####################################################################
#  FUNCTION     : 组件配置恢复
####################################################################
recoverComponentConfig()
{
    recoverHueConfig
}

####################################################################
#  FUNCTION     : 重启互信进程
####################################################################
restartSshAgent()
{
    showLog_info "${LINENO}" "restart ssh-agent process..."
    local ssh_agent_pid=$(ps -ef | grep "ssh-agent" | grep -v grep | awk '{print $2}')
    if [ -n "${ssh_agent_pid}" ]; then
        kill -9  ${ssh_agent_pid} >/dev/null 2>&1
    fi
}

initDBInfo()
{
    # 获取 DB 端口号及密码
    [ -f "$OMS_RUN_PATH/product_adapter.properties" ] && source $OMS_RUN_PATH/product_adapter.properties
    DBCOMMAND="gsql -U ommdba -d omm -p $OMS_GAUSS_PORT"
    DBPASSWORD=`getDBPasswd`
    if [ -z "${DBPASSWORD}" ]; then
        showLog_error "${LINENO}" "get db password failed."
        exit 1
    fi
}

initOmmDBInfo()
{
    # 获取 omm用户DB 端口号及密码
    [ -f "$OMS_RUN_PATH/product_adapter.properties" ] && source $OMS_RUN_PATH/product_adapter.properties
    OMMDBCOMMAND="gsql -U omm -d omm -p $OMS_GAUSS_PORT"
    OMMDBPASSWORD=`getOmmDBPassword`
    if [ -z "${OMMDBPASSWORD}" ]; then
        showLog_error "${LINENO}" "get omm user db password failed."
        exit 1
    fi
}

initLogAndTmpDir()
{
    if [ ! -f ${G_LOG_FILE} ]; then
        touch ${G_LOG_FILE}
        if [ $? -ne 0 ]; then
            echo "Failed to create log file: ${G_LOG_FILE}";
            exit 1
        fi
    fi
    local rootDirectory=$(dirname $CURRENT_PATH)
    local rootDirectoryOwner=$(ls -ld ${rootDirectory} | awk '{print $3}')
    if [ ${rootDirectoryOwner}X != "omm"X ];then
        showLog_error "${LINENO}" "${rootDirectory} owner is not omm:wheel,Change the file owner first."
        exit 1
    fi
    mkdir -p -m 775 "$CURRENT_PATH/backupOverData/tables"
    BACKTABLEDIR="$CURRENT_PATH/backupOverData/tables"
}

unzipBackupFile()
{
    if [ -f /tmp/backupDir.tar.gz ];then
        local backupTarOwner=$(ls -ld /tmp/backupDir.tar.gz | awk '{print $3}')
        if [ ${backupTarOwner}X != "omm"X ];then
            showLog_error "${LINENO}" "/tmp/backupDir.tar.gz owner is not omm:wheel,Change the file owner first."
            exit 1
        fi
        if [ -d /tmp/backupDir ];then
            chown omm:wheel -R /tmp/backupDir >/dev/null 2>&1
            if [ $? -ne 0 ];then
                showLog_error "${LINENO}" "/tmp/backupDir/ owner is not omm:wheel,Change the file owner first."
                exit 1
            fi
            rm -rf /tmp/backupDir
        fi
        tar xzf /tmp/backupDir.tar.gz -C /tmp >/dev/null 2>&1
        tar xf /tmp/backupDir/backupFile.tar -C /tmp/backupDir >/dev/null 2>&1
        if [ ! -d /tmp/backupDir/${CONTROLLER_HOME} ];then
            showLog_error "${LINENO}" "Data to be imported is missing."
            exit 1
        fi
        chmod 775 -R /tmp/backupDir
        chown omm:wheel -R /tmp/backupDir
    else
        showLog_error "${LINENO}" "/tmp/backupDir.tar.gz does not exist."
        exit 1
    fi
}

initParams()
{
    if [ $# != 1 ]; then
        showLog_error "${LINENO}" "Invalid parameter :$@"
        exit 1
    fi
    rm -rf ${CURRENT_PATH}/backupDir*
    # 集群 ID, 取值为 1 2 3 4...
    CLUSTER_ID="${1}"
    validate_clusterId "${CLUSTER_ID}"
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Invalid cluster id :${CLUSTER_ID}"
        exit 1
    fi
}

####################################################################
#  FUNCTION     : 恢复表数据
####################################################################
recoverTables()
{
    backupDbTables

    deleteRedundantPack

    modifyOmNodes

    preDeleteTables

    modifySequence

    recoverDbTables

    recoverOMMSchemaTables

    recoverRedisLogical
}

####################################################################
#  FUNCTION     : 恢复管理数据
####################################################################
recoverConfigFile()
{
    backupFiles

    recoverSnmp

    recoverCrt

    recoverAcs

    recoverOkerberos_Oldap

    recoverKerberos

    recoverCAS

    recoverACS_AOS

    recoverWeb

    recoverLicense

    recoverOthers

    recoverOmmPassphrase

    recoverComponentConfig
}

main()
{
    initLogAndTmpDir

    # 初始化入参: $1 集群ID
    initParams $@

    unzipBackupFile

    initDBInfo

    forbidOmsSwitch || exit 1

    recoverTables

    # 导入oldap需要先停掉oms,防止导入过程中数据紊乱
    stopActiveOms  || exit 1

    recoverConfigFile

    restartSshAgent

    restartOmsAndSyncallfile || exit 1
}

# 判断是否为 omm 用户
if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can export data."
    exit 1
fi

getCurPath

find ${CURRENT_PATH} -type f | xargs  dos2unix {}  > /dev/null 2>&1

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${CURRENT_PATH}/../func/functions.sh" || { echo "Failed to load functions.sh."; exit 1; }
. "${CURRENT_PATH}/../func/log.sh" || { echo "Failed to load log.sh."; exit 1; }
. "${CURRENT_PATH}/../func/constants.sh" || { echo "Failed to load constants.sh."; exit 1; }

# $1 : cluster ID
main $@