#!/bin/bash
## no  config  and start agent
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    install.sh
## Description: 安装FusionInsight Hadoop om的入口脚本
## Version:     FusionInsight Hadoop V100R002C02
## Created:     Thursday, May 23, 2013
################################################################################
# ---------------------------------------------------------------------------- #
#                                   变量定义                                   #
# ---------------------------------------------------------------------------- #

declare g_softwareHome=""                       # 软件包根目录
declare g_productHome=""                        # 产品要安装到的根目录
declare g_productDataHome=""                    # 产品数据的存放根目录
declare g_logHome=""                            # 产品日志根目录
declare g_omHome=""                             # 产品安装完成后，om的安装目录
declare g_omInstallFile=""                      # om安装包(om-0.0.1.tar.gz)
declare g_omFloatIP=""                          # 各个节点访问主controller的浮动ip
declare g_logFile=""                            # 日志文件名
declare g_curPath=""                            # 当前脚本所在的目录
declare g_scriptName=""                         # 当前脚本的脚本名称
declare g_operatorType=""                       # 当前操作的类别
declare -i retVal=0                             # 返回值
declare sudoScript="sudoExecute.sh"             # nodeagent执行sudo命令的脚本sudoExecute.sh
declare g_sudo_profile="/etc/profile.d/sudoProfile.sh" # sudo脚本中用到的环境变量文件
declare g_omm_profile="/home/omm/.profile"      # omm用户的环境变量文件

declare g_omServerWorkspace=""                  # om-server安装的工作区目录
declare g_omServerSoftLink=""                   # om-server安装的工作区目录soft link
declare g_omAgentWorkspace=""                   # om-agent安装的工作区目录
declare g_omAgentSoftLink=""                    # om-agent安装的工作区目录soft link
declare g_bigdataCommon=""                      # common工作区目录 /opt/huawei/Bigdata/common

declare g_ommid=""                              # omm用户id
declare g_ommdbaid=""                           # omm DBA用户id
declare g_installFile=""                        # 安装配置文件路径
declare g_super_user="root"                     # 超级用户
# ---------------------------------------------------------------------------- #
#                                   常量定义                                   #
# ---------------------------------------------------------------------------- #
declare -r TEMP_LOG_PATH="/tmp/scriptlog"       # 临时存放日志的目录
declare -r GLOBAL_PARAM_FILE="paramValue.ini"   # 保存变量值，供父shell读取
declare GLOBAL_INSTALL_USER=""                  # 安装用户

declare g_packageDistributeSubdir=""            # 包分发子目录 ${OS}-${PLATFORM}
declare g_os_type=""
declare g_cpu_type=""

declare g_patch_install="false"
declare g_oms_patch_install="false"
declare g_pack_patch_install="false"
declare g_oms_patch_pkgname=""
declare -a g_pack_patch_pkgnames
declare g_patch_tempDir=""

declare g_patch_list=""
declare g_prePackName=""
declare g_step_index=0

declare g_ha_status=""
declare HA_ACTIVE_STATUS="active"
declare HA_STANDBY_STATUS="standby"

declare -a package_name_array

#clusterId在该接口中并没有使用，这里先传入0
declare -r RGISTER_PKG_CONTROLLER_URL="http://127.0.0.1:20008/update/v1/clusters/0/installation/action/registerPackage/silent"
declare -r REQUEST_HEAD="Content-Type:application/json"
declare -r REQUEST_METHOD="POST"

declare -r RGISTER_PACK_CONTROLLER_URL="http://127.0.0.1:20008/update/v1/clusters/0/installation/action/registerPatchPack/silent"

declare -r PACKAGE_RESPONSE_NODE="status"
declare -r PACK_RESPONSE_NODE="isRegisterSuccess"
declare -r RESPONSE_ERROR_MSG="errorMsg"

declare -i CONTROLLER_STARTUP_TMOUT=1200
declare -r CAS_CERKEY_NAME="tomcat"

# ---------------------------------------------------------------------------- #
#                                   函数定义                                   #
# ---------------------------------------------------------------------------- #
################################################################################
# Function: get_cur_path
# Description: 获取脚本所在的目录及脚本名
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="${PWD}"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    cd - >/dev/null
}

################################################################################
# Function: init_log
# Description: 初始化日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_log()
{
    # controller安装日志存放路径。在用户安装时传入的日志目录参数生效之前，把日志打印到/tmp
    # 目录里
    local logPath="${TEMP_LOG_PATH}"
    g_logFile="${logPath}/install.log"
    
    # 创建日志文件和修改文件权限
    if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
    mkdir -p "${logPath}"
    if [ ! -f "${g_logFile}" ] ; then
        touch "${g_logFile}"
    fi

    chmod 700 "${logPath}" -R
    else
        sudo ${INSTALL_SUDO_PATH} tmp_log_mkdir "${logPath}"
        if [ $? -ne 0 ]; then
           echo "ERROR: Failed mkdir ${logPath}."
           return 1
        fi
        if [ ! -f "${g_logFile}" ] ; then
           sudo ${INSTALL_SUDO_PATH} tmp_log_touch "${g_logFile}"
           if [ $? -ne 0 ]; then
              echo "ERROR: Failed touch ${g_logFile}."
              return 1
           fi
        fi
        sudo ${INSTALL_SUDO_PATH} tmp_log_chmod "${logPath}"
        if [ $? -ne 0 ]; then
            echo "ERROR: Failed chmod ${logPath}."
            return 1
        fi
        sudo ${INSTALL_SUDO_PATH} tmp_log_chown "${logPath}"
        if [ $? -ne 0 ]; then
            echo "ERROR: Failed chown ${logPath}."
            return 1
        fi
    fi
    return 0
}

################################################################################
# Function: reinit_log
# Description: 重新设置日志的根目录为用户传递的目录，并保存此函数调用前打印的日志
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
reinit_log()
{
    # 把日志路径重设为安装时用户传入的日志路径
    local logPath="${g_logHome}/controller/scriptlog"
    g_logFile="${logPath}/install.log"
    g_controller_logFile="${g_logHome}/controller/controller.log"
    # 创建日志目录
    if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
    mkdir -p "${logPath}"
    else
       sudo ${INSTALL_SUDO_PATH} log_mkdir "${logPath}"
    fi
    if [ $? -ne 0 ]; then
        echo "ERROR: Failed mkdir ${logPath}."
        return 1
    fi
    # 把以前在tmp目录下打印的日志文件拷贝到新日志目录下
    if [ -f "${TEMP_LOG_PATH}/install.log" ] ; then
        if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
           cat "${TEMP_LOG_PATH}/install.log" >> "${g_logFile}"
        else
           sudo ${INSTALL_SUDO_PATH} tmp_log_cp 
        fi
    else
        if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
        touch "${g_logFile}"
        else
           sudo ${INSTALL_SUDO_PATH} log_File_touch "${g_logFile}"
        fi
    fi
    if [ $? -ne 0 ]; then
        echo "ERROR: Failed touch ${g_logFile}."
        return 1
    fi

    if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
    chmod 700 "${logPath}" -R
    else
       sudo ${INSTALL_SUDO_PATH} log_dir_chmod "${logPath}"
    fi
    if [ $? -ne 0 ]; then
        echo "ERROR: Failed chmod ${logPath}."
        return 1
    fi
    if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
    chmod 600 "${g_logFile}"
    else
       sudo ${INSTALL_SUDO_PATH} log_chmod "${g_logFile}"
    fi
    if [ $? -ne 0 ]; then
        echo "ERROR: Failed chmod ${g_logFile}."
        return 1
    fi
    
    #使用非root升级或安装时，创建日志目录后修改日志目录权限
    if [ "${GLOBAL_INSTALL_USER}" != "${g_super_user}" ]; then
        if [ "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
            sudo ${INSTALL_SUDO_PATH} log_dir_chown "${g_logHome}" "${GLOBAL_INSTALL_USER}"
        else
            sudo ${INSTALL_SUDO_PATH} restrict_chown -R $OMM_USER:$OMM_USER_GROUP "${g_logHome}/controller"
        fi
    fi
    
    # 删除掉临时日志目录
    rm -rf "${TEMP_LOG_PATH}"

    return ${INS_SUCCESS}
}

################################################################################
# Function: init_global_param
# Description: 初始化全局变量
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_global_param()
{
    # 初始化g_softwareHome
    g_softwareHome="${g_curPath}"
    log_debug "${LINENO}" "g_softwareHome=${g_softwareHome}"

    # 自适应计算om安装包文件名和om的安装目录，避免写死
    g_omInstallFile="$(ls ${g_softwareHome}/om/package | grep '^om.*\.tar\.gz$')"
    local omInstallFileNum="$(echo "${g_omInstallFile}" | wc -w)"
    if [ "${omInstallFileNum}" != "1" ]; then
        showLog_error "${LINENO}" "The om install file is not unique."
        return 1
    fi
    g_omHome="${g_omInstallFile%.tar.gz}"

    #初始化工作区
    init_workspace
    
    #g_omHome point to the om under workspace
    g_omHome="${g_omServerWorkspace}/${g_omHome}"
    
    g_omInstallFile="${g_softwareHome}/om/package/${g_omInstallFile}"
    log_debug "${LINENO}" "g_omHome=${g_omHome}"
    log_debug "${LINENO}" "g_omInstallFile=${g_omInstallFile}"
    
    g_controllerConfHome="${g_omHome}/etc/om"
    log_debug "${LINENO}" "g_controllerConfHome=${g_controllerConfHome}"
    
    if [ -z ${BIGDATA_SUDO_HOME} ]; then
        BIGDATA_SUDO_HOME="/var/lib/sudo/Bigdata"
    fi
    
    return ${INS_SUCCESS}
}

################################################################################
# Function: init_workspace
# Description: 初始化工作区
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_workspace()
{
    #初始化om-server工作区
    g_omServerSoftLink="${g_productHome}/om-server"
    g_omServerWorkspace="${g_productHome}/om-server_${version}"
    log_debug "${LINENO}" "g_omServerWorkspace=${g_omServerWorkspace}"
    
    mkdir -p ${g_omServerWorkspace}
    
    #初始化om-server节点的agent工作区
    g_omAgentSoftLink="${g_productHome}/om-agent"
    g_omAgentWorkspace="${g_productHome}/om-agent_${version}"
    log_debug "${LINENO}" "g_omAgentWorkspace=${g_omAgentWorkspace}"
    
    mkdir -p ${g_omAgentWorkspace}
    
    #初始化common工作区
    g_bigdataCommon="${g_productHome}/${COMMON_HOME_NAME}"
    
    #升级时只创建工作区，不切换软连接
    if [ "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
        [ -L "${g_omServerSoftLink}" ] &&  rm -rf "${g_omServerSoftLink}"
        ln -sf "${g_omServerWorkspace}" "${g_omServerSoftLink}"
           
        [ -L "${g_omAgentSoftLink}" ] &&  rm -rf "${g_omAgentSoftLink}"
        ln -sf "${g_omAgentWorkspace}" "${g_omAgentSoftLink}"
    fi
}

################################################################################
# Function: get_free_space
# Description: 获取目录所在磁盘分区的可用空间
# Parameter:
#   input:
#   $1 -- diskPath 要检测的目录
#   output:
#   1  -- 目录所在磁盘分区的可用空间
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_free_space()
{
    local diskPath="$1"

    # 检查输入参数是否为空
    if [ -z "${diskPath}" ] ; then
        showLog_error "${LINENO}" "The input parameter is empty."
        return 1
    fi

    # 检查待检测的目录是否存在
    if [ ! -d "${diskPath}" ] ; then
        showLog_error "${LINENO}" "The ${diskPath} path does not exist."
        return 1
    fi

    echo $(($(df "${diskPath}" -Pk | tail -1 | awk '{print $4}' | sed 's/e/*10^/g;s/+//g')/1024))

    return ${INS_SUCCESS}
}

################################################################################
# Function: check_space
# Description: 判断目录所在磁盘分区的可用空间是否大于指定值
# Parameter:
#   input:
#   $1 -- diskPath 要检测的目录
#   $2 -- minSpace 需要满足的最小可用空间值
#   output:
#   1 --  freeSpace 当前可用的空间
# Return:
#   0 -- 目录所在磁盘分区的可用空间大于或者等于指定值
#   1 -- 目录所在磁盘分区的可用空间小于指定值
# Others: N/A
################################################################################
check_space()
{
    local diskPath="$1"
    local minSpace="$2"
    local freeSpace=""
    local isEnough=""

    # 检查输入参数个数是否正确
    if [ $# -ne 2 ] ; then
        showLog_error "${LINENO}" "The input parameter is incorrect."
        return 1
    fi

    # 检查待检测的目录是否存在
    if [ ! -d "${diskPath}" ] ; then
        showLog_error "${LINENO}" "The ${diskPath} path does not exist."
        return 1
    fi

    # 校验diskPath所在磁盘空间的可用空间是否满足要求
    # 获取可用空间大小
    freeSpace="$(get_free_space ${diskPath})"
    # 打印输出可用空间大小
    echo "${freeSpace}"
    
    isEnough=1
    [ ${freeSpace} -ge ${minSpace} ] || isEnough=0
    
    # 如果表达式(${freeSpace} >= ${minSpace})成立,isEnough等于非0
    [ "${isEnough}" != "0" ]
    return $?
}

################################################################################
# Function: check_disk_space
# Description: 检查磁盘空间是否足够
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
check_disk_space()
{
    log_info "${LINENO}" "Start to check disk_space."
    local curAvailableSpace=""

    # 检查安装目录的可用磁盘空间
    curAvailableSpace=$(check_space "${g_productHome}" "${INSTALL_DIR_MIN_SPACE}")
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "The ${g_productHome} directory on the active OMS must have at least ${INSTALL_DIR_MIN_SPACE} MB free space. The current free space is ${curAvailableSpace} Mb."
        return 1
    fi

    # 检查日志目录的可用磁盘空间
    curAvailableSpace=$(check_space "${g_logHome}" "${LOG_DIR_MIN_SPACE}")
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "The ${g_logHome} directory on the active OMS must have at least ${LOG_DIR_MIN_SPACE} MB free space. The current free space is ${curAvailableSpace} Mb."
        return 1
    fi
    log_info "${LINENO}" "Succeed to check disk_space."
    return ${INS_SUCCESS}
}


################################################################################
# Function: create_env_variable_file
# Description: 生成环境变量文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
create_env_variable_file()
{
    log_info "${LINENO}" "Start to create env_variable_file."

    # controller profile should be include in the workspace
    local profileFile="${g_omServerWorkspace}/.controller_profile"
    echo '## Huawei OM Controller profile file ## DO NOT MODIFY MANUALLY' > "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BASE_DIR=${g_productHome}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_HOME=${g_productHome}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_OM_SERVER_HOME=${g_omServerSoftLink}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_COMMON=${g_bigdataCommon}/${COMMON_WORKSPACE_NAME}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_DATA_HOME=${g_productDataHome}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_MANAGER_DATA_HOME=${g_productDataHome}/${OM_DATA_DIR}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_LOG_HOME=${g_logHome}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export CONTROLLER_HOME=${g_omHome}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export CONTROLLER_CONF_HOME=${g_controllerConfHome}/" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    echo "export BIGDATA_SUDO_HOME=${BIGDATA_SUDO_HOME}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}

    chmod 750 "${profileFile}"

    log_info "${LINENO}" "Succeed to create env_variable_file."
    return ${INS_SUCCESS}
}

################################################################################
# Function: extract_jdk
# Description：将Agent安装包中的JDK拷贝到指定om-server目录
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function extract_jdk()
{
    log_info "${LINENO}" "extract jdk from agent package.."

    #获取agent安装路径
    local agentPackageFile="${g_softwareHome}/om/package/${AGENT_PACKAGE_NAME}"    
    log_debug "$LINENO" "agentPackageFile=${agentPackageFile}"   
    [ ! -f ${agentPackageFile} ] && die "get agent install package failed."
    
    #将Agent包拷贝到om-server目录
    if [ ! -d "${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}/" ];then
        mkdir -p "${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}/"
        chmod 700 "${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}/"
    fi
    /bin/cp -f "${agentPackageFile}" "${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}/"
 
    #创建临时目录
    local tmp_dir="${g_omHome}/packaged-distributables/${AGENT_PACKAGE_NAME}_tmp"
    mkdir -p "${tmp_dir}"

    #解压agent包抽取jdk
    tar -xzf  "${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}/${AGENT_PACKAGE_NAME}" -C "${tmp_dir}" >> "$g_logFile" 2>&1
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "tar -xzf ${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}/${AGENT_PACKAGE_NAME} -C ${tmp_dir} failed."
        return 1
    fi
    
    if [ ! -d "${g_omHome}/inst/jre/${g_packageDistributeSubdir}" ]; then
        mkdir -p "${g_omHome}/inst/jre/${g_packageDistributeSubdir}"
    fi

    mv "${tmp_dir}/setup/jre"/* "${g_omHome}/inst/jre/${g_packageDistributeSubdir}/" 
    /bin/rm -rf "${tmp_dir}"
    
    log_info "${LINENO}" "Succeed to extract jdk from ${AGENT_PACKAGE_NAME}."

    return 0
}

################################################################################
# Function: post_install
# Description: 安装后处理
# Parameter:
#   input:
#   retCode -- main主函数的返回码
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
post_install()
{
    retCode="$1"
    # 如果安装成功，则提示安装成功
    if [ ${retCode} = "0" ];then
        showLog_info "${LINENO}" "Installation is successful. "

    # 如果安装失败，则提示调用卸载脚本
    else
        if [ "${g_patch_install}" == "true" ];then
            showLog_error "${LINENO}" "Installation failed. For details about the error, see the log file ${g_logFile} and ${g_controller_logFile}."
        else
            showLog_error "${LINENO}" "Installation failed. For details about the error, see the log file ${g_logFile}."
        fi 
        source "${g_sudo_profile}" > /dev/null 2>&1 
        source "${g_omm_profile}"  > /dev/null 2>&1 
        local uninstallCmd="${BIGDATA_HOME:-${HADOOP_HOME}}/om-server/om/inst/uninstall.sh"
        # 若卸载脚本存在，再提示调用卸载脚本进行卸载
        if [ -f "${uninstallCmd}" ] ; then
            chmod +x "${uninstallCmd}"
            echo "      Please run the following script to delete useless files:"
            echo "      ${uninstallCmd}"
        fi
    fi

    return ${retCode}
}

################################################################################
# Function: prepare_components
# Description: 安装前准备
# Parameter:
#   input:
#   retCode --
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
prepare_components()
{
    #还原瘦身包中安装包
    chmod +x ${g_curPath}/restore_package.sh >> "${g_logFile}"
    ${g_curPath}/restore_package.sh
    if [ $? -ne 0 ];then
        showLog_error "${LINENO}" "Recovery slimmed package failed."
        return 1
    fi

    # 解压om安装包
    if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" ]; then
        rm -rf ${g_omHome}
        mkdir ${g_omHome}
    else
       sudo ${INSTALL_SUDO_PATH} controller_install_mkdir_chown "${g_omHome}" "${GLOBAL_INSTALL_USER}"
    fi
    
    tar -xzf "${g_omInstallFile}" -C "${g_omServerWorkspace}" >> "${g_logFile}" 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to decompress the ${g_omInstallFile} file."
        return 1
    fi

    #拷贝Controller的JRE安装包至Agent
    extract_jdk
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to copy jre package to agent install package."
        return 1
    fi

    # 生成环境变量文件，保证卸载时，始终可以导入BIGDATA_DATA_HOME和BIGDATA_LOG_HOME变量
    create_env_variable_file
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to create environment variable file."
        return ${retVal}
    fi

    # 把根据输入参数生成的配置文件拷贝到controller指定目录下
    local configFile="${g_softwareHome}/om/script/${CONFIG_FILE}"
    local configDir="${g_omHome}/inst/conf"
    cp -af ${configFile} ${configDir}
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to copy the ${configFile} file to the ${configDir} directory."
        return 1
    fi
    # 拷贝完成，删除掉生成的临时配置文件
    rm -f "${configFile}"
    rm -f "${g_softwareHome}/om/script/${GLOBAL_PARAM_FILE}"

    # 将Manager OMS相关组件包拷贝到controller的指定目录下
    rsync -q --exclude="${g_omInstallFile}" "${g_softwareHome}/om/package/"* "${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}"
    ret=$?
    if [ $ret -ne 0 ]; then
        showLog_error $LINENO "Failed to copy software packages to ${g_omHome}/packaged-distributables/${g_packageDistributeSubdir}."
        return $ret
    fi
    
    if [ "${GLOBAL_INSTALL_USER}" = "${g_super_user}" -a "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
        if [ ! -d ${BIGDATA_SUDO_HOME} ]; then
            rm -rf ${BIGDATA_SUDO_HOME}
            mkdir -p ${BIGDATA_SUDO_HOME}
        fi
        cp "-rf" "${g_omServerWorkspace}/om/sbin/sudo" "$BIGDATA_SUDO_HOME"
        mkdir -p "$BIGDATA_SUDO_HOME/sudo/install/components"
    else
        sudo ${INSTALL_SUDO_PATH} restrict_cp "-rf" "${g_omServerWorkspace}/om/sbin/sudo" "$BIGDATA_SUDO_HOME"  >> ${g_logFile} 2>&1 || return $?
        sudo ${INSTALL_SUDO_PATH} restrict_mkdir "-p" "$BIGDATA_SUDO_HOME/sudo/install/components"  >> ${g_logFile} 2>&1 || return $?
    fi

    #release文件拷贝om-server配置目录
    /bin/cp "${g_softwareHome}/${RELEASE_CONF_FILENAME}" "${g_controllerConfHome}"; ret=$?
    if [ $ret -ne 0 ]; then
        log_error $LINENO "exec_cmd cp ${g_softwareHome}/${RELEASE_CONF_FILENAME} ${g_controllerConfHome} failed."
        return 1
    fi
    
    # 改变controller安装目录及其以下所有文件的权限
    chmod 700 ${g_omHome} -R
    
    # 改变controller目录权限为750，omm的ommdba用户需要访问security文件夹key文件
    chmod 750 ${g_omHome}
}

################################################################################
# Function: waitforNtp
# Description: 等待ntp启动
# Parameter:
#   input:
#   output:
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
waitforNtp()
{
    if [ "$g_operatorType" = "$UPGRADE_OPERATOR" ]; then
        log_info "${LINENO}" "UPGRADE, no need to start ntp."
        return 0
    fi
    local -i MAX_TIMES=60
    local -i i=0

    for ((i=0; i<MAX_TIMES; i++)) ; do
        sudo ${INSTALL_SUDO_PATH}  ntpq_np > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            sleep 10
        else
            break
        fi
    done

    if  [ $i -ge $MAX_TIMES ] ; then
        log_error "${LINENO}"  "Wait for ntp to startup timeout."
        return 1
    fi

    return ${INS_SUCCESS}
}

function validate_cpu_type()
{
    os_type_ini="${g_curPath}/osType.ini"
    
    cpu_type=$(get_key_value "cpu" "cpu_type" "${os_type_ini}")
    if [ $? -ne 0 ];then
       showLog_error "$LINENO" "Get cpu type is error from ${os_type_ini}."
       return 1
    fi
    
    if [ -z "${cpu_type}" ]; then
       showLog_error "$LINENO" "cpu type is empty in ${os_type_ini}."
       return 1
    fi
    
    cpu_type_support=$(get_key_value "cpu" "cpu_type_support" "${os_type_ini}")
    if [ $? -ne 0 ];then
       showLog_error "$LINENO" "Get list of Support cpu type is error from ${os_type_ini}."
       return 1
    fi
    
    check_cpu_type "${cpu_type}" "${cpu_type_support}"
    if [ $? -ne 0 ];then
       return 1
    fi
    
    log_info "$LINENO" "Succeed to validate cpu_type."
}

################################################################################
# Function: validate_os_type
# Description: 校验OS类型是否匹配
# Parameter:
#   input:
#   output:
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function validate_os_type()
{
    os_type_ini="${g_curPath}/osType.ini"
    
    os_type=$(get_key_value "os" "os_type" "${os_type_ini}")
    if [ $? -ne 0 ];then
       showLog_error "$LINENO" "Get os type is error from ${os_type_ini}."
       return 1
    fi
    
    if [ -z "${os_type}" ]; then
       showLog_error "$LINENO" "os type is empty in ${os_type_ini}."
       return 1
    fi
    
    os_type_support_redhat=$(get_key_value "os" "os_type_support_redhat" "${os_type_ini}")
    if [ $? -ne 0 ];then
       showLog_error "$LINENO" "Get list of Support redhat os type is error from ${os_type_ini}."
       return 1
    fi
    
    os_type_support_suse=$(get_key_value "os" "os_type_support_suse" "${os_type_ini}")
    if [ $? -ne 0 ];then
       showLog_error "$LINENO" "Get list of Support redhat os type is error from ${os_type_ini}."
       return 1
    fi
    
    checkOS "${os_type}" "${os_type_support_suse}" "${os_type_support_redhat}"
    if [ $? -ne 0 ];then
       return 1
    fi
    
    log_info "$LINENO" "Succeed to validate os_type."
}
################################################################################
# Function: validate_ha_ssl_cert
# Description: 校验ha ssl证书
# Parameter:
#   input:
#   output:
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function validate_ha_ssl_cert()
{
    log_info "$LINENO" "Start to validate ha_ssl_cert."
    hasslCertDir="${g_curPath}/hasslCert"
    fileCount=$(ls -l "${hasslCertDir}" | grep "^-" | wc -l )
    if [ ${fileCount} -gt 1 ];then
       if [ ! -f ${hasslCertDir}/root-ca.crt ]; then
          log_error "$LINENO" "${hasslCertDir}/root-ca.crt is not exist."
          return 1
       fi
       if [ ! -f ${hasslCertDir}/root-ca.pem ]; then
          log_error "$LINENO" "${hasslCertDir}/root-ca.pem is not exist."
          return 1
       fi
       if [ ! -f ${hasslCertDir}/hasslCert.ini ]; then
          log_error "$LINENO" "${hasslCertDir}/hasslCert.ini is not exist."
          return 1
       fi
       ha_ssl_cert_pwd=$(cat ${hasslCertDir}/hasslCert.ini | cut -d '=' -f2- | sed 's/^[[:space:]]*//g;s/[[:space:]]*$//g')
       export ha_ssl_cert_pwd
       openssl rsa -in "${hasslCertDir}/root-ca.pem" -passin env:ha_ssl_cert_pwd > /dev/null 2>&1
       if [ $? -ne 0 ]; then
          log_error "$LINENO" "HA SSL Cert passwod is mismatch with ${hasslCertDir}/hasslCert.ini."
          unset ha_ssl_cert_pwd
          return 1
       fi
       unset ha_ssl_cert_pwd
    fi
    log_info "$LINENO" "Succeed to validate ha_ssl_cert."
}

################################################################################
# Function: installPack
# Description: 安装packs目录下的部件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
installPack()
{
    local packs_path="$g_softwareHome/packs"
    if [ ! -d "$packs_path" ]; then
        return 0
    fi
    
    packs=$(ls "${packs_path}" | grep .tar.gz$)
    if [ -z "${packs}" ]; then
        return 0
    fi
    
    /bin/bash "${g_omServerWorkspace}"/om/sbin/autoInstallPacks.sh "${packs_path}" >> "${g_logFile}" 2>&1 &
    spin_wait "=== STEP ${g_step_index} Installing the packs." "$!"
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
       showLog_error "${LINENO}"  "Failed to install the packs.For details about the error,see the log file ${BIGDATA_LOG_HOME}/controller/scriptlog/installPack.log   and  ${BIGDATA_LOG_HOME}/controller/scriptlog/install.log."
       return ${retVal}
    fi
    g_step_index=$(($g_step_index+1))

    return 0
}

################################################################################
# Function: start_oms
# Description: 启动OMS所有进程
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
start_oms()
{
    if [ "$g_operatorType" = "$UPGRADE_OPERATOR" ]; then
        log_info $LINENO "The current operation is upgrade, no need to startup om-server."
        return 0
    fi

    sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "${g_omServerWorkspace}/om/sbin/start-oms.sh" >> ${g_logFile} 2>&1 &
    spin_wait "=== STEP ${g_step_index} Starting the OMS." "$!"
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to start OMS."
        return ${retVal}
    fi
    g_step_index=$(($g_step_index+1))
}


get_patch_request()
{
    declare -r REQUEST_BODY_TEMPLATE='{"entities":{"packages":[<PATCH_PACKAGES_INFO>]}}'
    declare -r PACKAGE_INFO_TEMPLATE='{"pkgPath":<PKG_PATCH>, "checkSum":<CHECK_SUM>}'


    local patch_package_info=""
    for pack in "${package_name_array[@]}"
    do
        local package_path="${g_patch_tempDir}/$pack"
        local package_check_sum=$("sha256sum" "$package_path" | head -1 | awk '{print $1}')

        package_path="\\\"$package_path\\\""
        package_check_sum="\\\"$package_check_sum\\\""

        local package_info=$(echo ${PACKAGE_INFO_TEMPLATE} | sed -e "s@<PKG_PATCH>@${package_path}@")
        local package_info=$(echo ${package_info} | sed -e "s@<CHECK_SUM>@${package_check_sum}@")

        #拼接请求体
        if [ -z "${patch_package_info}" ]; then
            patch_package_info="$package_info"
        else
            patch_package_info="${patch_package_info}, $package_info"
        fi
    done

    local request_body=$(echo ${REQUEST_BODY_TEMPLATE} | sed -e "s@<PATCH_PACKAGES_INFO>@${patch_package_info}@")

    echo "${request_body}"
}

get_patch_pack_request()
{
    declare -r PACK_REQUEST_TEMPLATE='{"entities":{"patchPacks":[<PACKS_INFO>]}}'
    declare -r PACK_INFO_TEMPLATE='{"packName":<PACK_NAME>, "packDisplayName":<PACK_DISPLAY_NAME>}'

    #安装包新增一个patch目录，放置静默补丁包
    local patch_path="$g_softwareHome/patch"

    local patch_packs_info=""
    index=0
    for pack in "${g_pack_patch_pkgnames[@]}"
    do
        local patch_dir=${pack%.tar.gz}
        local package_dir="$patch_path/$patch_dir"
        local patch_sdf="${package_dir}/package.sdf"

        if [ ! -f "${patch_sdf}" ]; then
            log_error "$LINENO" "patch sdf file does not exist."
            return 1
        fi

        local pack_name=$(grep 'packName' ${patch_sdf} | awk -F '=' '{print $2}')
        local pack_display_name=$(grep 'packDisplayName' ${patch_sdf} | awk -F '=' '{print $2}')

        pack_name="\\\"$pack_name\\\""
        pack_display_name="\\\"$pack_display_name\\\""

        local pack_info=$(echo ${PACK_INFO_TEMPLATE} | sed -e "s@<PACK_NAME>@${pack_name}@")
        local pack_info=$(echo ${pack_info} | sed -e "s@<PACK_DISPLAY_NAME>@${pack_display_name}@")

        #拼接请求体
        if [ -z "${patch_packs_info}" ]; then
            patch_packs_info="$pack_info"
        else
            patch_packs_info="${patch_packs_info}, $pack_info"
        fi
    done

    local request_body=$(echo ${PACK_REQUEST_TEMPLATE} | sed -e "s@<PACKS_INFO>@${patch_packs_info}@")

    echo "${request_body}"
}

##############################################################
## @Usage getXmlNode
## @Description 读取XML指定标签的内容
##############################################################
getXmlNode()
{
    local xmlContext="$1"
    local nodeName="$2"
    
    [ -n "${nodeName}" ] || return 1
    [ -n "$nodeEnd" ] || nodeEnd="${nodeName}"
    
    echo $(echo "${xmlContext}" | awk -F"<${nodeName}>" '{print $2}' | awk -F"</${nodeName}>" '{print $1}')
    return 0
}

copyPatchPackagesToTempDir()
{
    local patch_path="$g_softwareHome/patch"

    if [ -d "${g_patch_tempDir}" ];then
        rm -rf "${g_patch_tempDir}"
    fi
    mkdir -p "${g_patch_tempDir}"
    chown omm:wheel "${g_patch_tempDir}"

    for pack in "${package_name_array[@]}"
    do

        cp -pf "${patch_path}/${pack}" "${g_patch_tempDir}" >> "${g_logFile}" 2>&1
        if [ $? -ne 0 ] ; then
            log_error "cp ${patch_path}/${pack} to ${g_patch_tempDir} failed."
            return 1
        fi

        if ! tar -pxzf "${patch_path}/${pack}" -C "${g_patch_tempDir}"; then
            log_error "Failed to decompress the pack ${patch_path}/${pack} to ${g_patch_tempDir}."
            return 1
        fi

        chmod 660 "${g_patch_tempDir}/${pack}"

    done

    chown -R omm:wheel "${g_patch_tempDir}"

    return 0
}

distributeExtractOmsPatchOnStandy()
{
    local patch_path="$g_patch_tempDir"

    local localIp=""
    local omsConfigFile="${g_omHome}/inst/conf/oms-config.ini"
    localIp="$(cat ${omsConfigFile} | grep 'server1_ip*' | cut -f 2 -d '=' | sed 's/[[:space:]]//g')"

    local distrPatchScript="${g_omServerWorkspace}/om/sbin/distrExtractPatchOnOMS.sh"    

    local patch_package_info=""
    for pack in "${package_name_array[@]}"
    do
        local patch_dir=${pack%.tar.gz}
        local package_dir="$patch_path/$patch_dir"
        #识别patchlist，无需排序
        #这里需要解压一下先
        local patchList=""
        local index=0
        for cp_pakcage in $(ls "${package_dir}" | grep .tar.gz$)
        do
            local cp_pakcage_path="${package_dir}/${cp_pakcage}"
            local cp_dir=${cp_pakcage%.tar.gz}
            if [ -d "${package_dir}/${cp_dir}" ];then
                if ! rm -rf "${package_dir}/${cp_dir}"; then 
                    log_error "rm -rf ${package_dir}/${cp_dir} failed."
                    return 1
                fi
                log_info "delete ${package_dir}/${cp_dir} successfully."
            fi

            if ! tar -pxzf "${cp_pakcage_path}" -C "${package_dir}"; then
                log_error "Failed to decompress the cp_pakcage ${cp_pakcage_path} to ${package_dir}."
                return 1
            fi

            patchList[${index}]="${cp_dir}"
            index=$(($index+1))
        done

        chown -R omm:wheel "${g_patch_tempDir}"

        if [ $(id -u) -eq 0 ]; then
            su - omm -c "/bin/bash \"${distrPatchScript}\" \"${localIp}\" \"single\" \"${package_dir}\" \"${patchList}\" " >> ${g_logFile} 2>&1
            ret=$?
        else
            /bin/bash "${g_omServerWorkspace}/om/sbin/distrExtractPatchOnOMS.sh" "${localIp}" "single" "${package_dir}" "${patchList}" >> "${g_logFile}" 2>&1
            ret=$?
        fi

        return $ret

    done
}

check_controller_startup()
{
    local controller_properties="${g_omServerWorkspace}/om/etc/om/controller.properties"

    if [ ! -f "${controller_properties}" ]; then
        log_info "${LINENO}" "controller_properties does not exist."
        return 1
    fi
    
    local RestUrl=$(grep -w 'controller.rest.server.url' $controller_properties | grep -v '^[ ]*#' | awk -F'=' '{print $2}' 2>/dev/null)
    if [ -z "$RestUrl" ]; then
        return 2
    fi

    local subRet=0
    curl -X GET $RestUrl/clusters/getCurrentClusterInfo >/dev/null 2>&1; subRet=$?
    return $subRet
}

wait_controller_startup()
{
    if [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
        return 0
    fi

    local -i maxNum=${CONTROLLER_STARTUP_TMOUT}
    for (( i=1; i<=${maxNum}; i=i+5 ))
    do
        check_controller_startup
        if [ $? -eq 0 ]; then
            return 0
        fi

        sleep 5
        log_info "${LINENO}" "Wait ${i}s for controller startup."
    done

    return 1
}


register_patch()
{
    if [ "${g_patch_install}" == "false" ];then
        log_info "${LINENO}" "No need to register patch."
        return 0
    fi

    log_info "${LINENO}" "Begin to register patch."  

    g_ha_status=$("${g_omServerWorkspace}/OMS/workspace/ha/module/hacom/script/get_harole.sh")
    if [[ "$g_ha_status" != "${HA_ACTIVE_STATUS}" ]] && [[ "$g_ha_status" != "${HA_STANDBY_STATUS}" ]];then
        log_error "$LINENO" "HA status is $g_ha_status , maybe abnormal."
        return 1
    fi

    #备节点后续查询补丁数据库时 会出现连接数据库失败或者查不出结果的问题，需要重启一下oms
    if [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
        sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "${g_omServerWorkspace}/om/sbin/restart-oms.sh" >> ${g_logFile} 2>&1
        retVal=$?
        if [ ${retVal} -ne 0 ]; then
            log_error "${LINENO}" "Failed to restart oms on standby."
            return ${retVal}
        fi
    fi

    g_patch_tempDir="${g_productHome}/tmp/patch"
    copyPatchPackagesToTempDir
    if [ $? -ne 0 ];then
        log_error "$LINENO" "Copy patch package to ${g_patch_tempDir} failed."
        return 1
    fi

    #备节点只做补丁包拷贝与解析
    if [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
        distributeExtractOmsPatchOnStandy
        if [ $? -ne 0 ];then
            log_error "$LINENO" "Register patch packages on standby oms failed."
            return 1
        fi
        log_info "${LINENO}" "Register patch on standy successfully."
        return 0
    fi

    if ! wait_controller_startup "${g_ha_status}"; then
        log_error "${LINENO}" "wait controller startup failed."
        return 1
    fi

    local request_body=$(get_patch_request)
    log_info "$LINENO" "register patch requestbody: ${request_body} "

    #curl命令调用注册补丁流程
    local response=$(curl -s -H "${REQUEST_HEAD}" -X "${REQUEST_METHOD}" -d "${request_body}" "${RGISTER_PKG_CONTROLLER_URL}")

    #分析rest返回结果
    local result="$(getXmlNode "$response" "${PACKAGE_RESPONSE_NODE}")"   
    if [ -z "$result" ]; then
        log_error "${LINENO}" "Failed to install the packs."
        return 1
    fi 

    if [ "${result}" -eq 0 ]; then
        log_info "${LINENO}" "The packs are installed suceessfully."
        return 0
    else
        local errorMessage="$(getXmlNode "$response" "${RESPONSE_ERROR_MSG}")"
        log_error "${LINENO}" "Failed to register patch packages. Status: $result , Error message: $errorMessage"
        return 1
    fi

    log_info "${LINENO}" "Register patch successfully."  

    return 0
}

sortPatchList()
{
    len=${#g_patch_list[@]}

    for((i=0; i<$len; i++))
    {
        for((j=i+1; j<$len; j++))
        {

            if [[ ${g_patch_list[i]} > ${g_patch_list[j]} ]];then
              temp=${g_patch_list[i]}
              g_patch_list[i]=${g_patch_list[j]}
              g_patch_list[j]=$temp
            fi

        }
    }

    return 0
}

generatePrePackName()
{
    local packName="$1"
    
    #备节点直接生成基础版本的prepackName 不查询数据库，因为主备同时执行时，备节点执行较快，可能查出来的是空
    local prePackFlag=$(echo "$packName" | awk -F "${version}" '{print $1}')
    g_prePackName="${prePackFlag}${version}"
    
    return 0
}

queryPrePackName()
{
    local packName="$1"
    
    queryStatment="SELECT PRE_PACK_NAME FROM OM_PACKAGE WHERE PACK_NAME='${packName}';"
    log_info "${LINENO}" "Query prePackName with sql statment: $queryStatment."

    local execSqlTool="${g_omServerWorkspace}/om/tools/execSql.sh"

    if [ $(id -u) -eq 0 ]; then
        query_result=$(su - omm -c "\"${execSqlTool}\" -s \"${queryStatment}\" ") ; ret=$?
    else
        query_result="$(${execSqlTool}  -s "${queryStatment}")" ; ret=$?
    fi

    log_info $LINENO "query_result: ${query_result}"

    if [ $ret -ne 0 ];then
        log_error "${LINENO}" "Failed to query prePackName from database, errorCode=$ret."
        return $ret
    fi

    g_prePackName=$(echo "${query_result}" | awk 'NR==3{print}' | sed -e 's/\(^ *\)//' -e 's/\( *$\)//')

    echo "$g_prePackName" | grep "rows" >> "${g_logFile}" 2>&1
    if [ $? -eq 0 ]; then
        log_error "${LINENO}" "Query prePackName from database failed."
        return 1
    fi

    return 0
}

registerPatchPackOnStandby()
{
    local patch_path="$g_softwareHome/patch"

    for pack in "${g_pack_patch_pkgnames[@]}"
    do
        local patch_dir=${pack%.tar.gz}
        local package_dir="$patch_path/$patch_dir"
        local patch_sdf="${package_dir}/package.sdf"

        if [ ! -f "${patch_sdf}" ]; then
            log_error "$LINENO" "patch sdf file does not exist."
            return 1
        fi

        local pack_name=$(grep 'packName' ${patch_sdf} | awk -F '=' '{print $2}')
        local pack_display_name=$(grep 'packDisplayName' ${patch_sdf} | awk -F '=' '{print $2}')

        local package_name="${pack}"

        #扫描cplist并排序
        g_patch_list=""
        index=0
        for cp_pakcage in $(ls "${package_dir}" | grep .tar.gz$)
        do
            local cp_dir=${cp_pakcage%.tar.gz}

            g_patch_list[${index}]="${cp_dir}"
            index=$(($index+1))
        done

        #排序
        sortPatchList

        #拼接cplist 入参格式为："Manager_6.5.RC2.1#Manager_6.5.RC2.2#Manager_6.5.RC2.1"
        local cp_names=""
        for cp in "${g_patch_list[@]}"
        do
            if [ -z "$cp_names" ];then
                cp_names="$cp"
            else
                cp_names="${cp_names}#${cp}"
            fi
        done

        #从数据库表中查询prePackName
        g_prePackName=""
        generatePrePackName "${pack_name}"
        if [ $? -ne 0 ];then
            log_error "$LINENO" "query prePackName from datebase failed."
            return 1
        fi

        #执行注册补丁Pack
        local register_patch_pack_script="${g_omServerWorkspace}/om/sbin/register_patch_pack.sh"
        if [ $(id -u) -eq 0 ]; then
            su - omm -c "/bin/bash \"${register_patch_pack_script}\" \"single\" \"${package_name}\" \"${pack_display_name}\" \"${g_prePackName}\" \"${pack_name}\" \"${cp_names}\" " >> "${g_logFile}" 2>&1
            ret=$?
        else
            /bin/bash "${g_omServerWorkspace}/om/sbin/register_patch_pack.sh" "single" "${package_name}" "${pack_display_name}" "${g_prePackName}" "${pack_name}" "${cp_names}" >> "${g_logFile}" 2>&1
            ret=$?
        fi

        if [ "$ret" -ne 0 ];then
            log_error "$LINENO" "execute register_patch_pack.sh failed."
            return 1
        fi

    done

    return 0

}

de_active_old_pack()
{
    local pre_pack_name="$1"
    local execSqlTool="${g_omServerWorkspace}/om/tools/execSql.sh"

    updateStatment="UPDATE OM_PACKS SET STATUS='detached' WHERE PACK_NAME='${pre_pack_name}';"
    log_info "${LINENO}" "Update om_packs with sql statment: $updateStatment."

    if [ $(id -u) -eq 0 ]; then
        update_result=$(su - omm -c "\"${execSqlTool}\" -s \"${updateStatment}\" ") ; ret=$?
    else
        update_result="$(${execSqlTool}  -s "${updateStatment}")" ; ret=$?
    fi

    log_info $LINENO "update_result: ${update_result}"

    if [ $ret -ne 0 ];then
        log_error "${LINENO}" "Failed to update om_packs, errorCode=$ret."
        return $ret
    fi
}

active_new_pack()
{
    local new_pack_name="$1"
    local execSqlTool="${g_omServerWorkspace}/om/tools/execSql.sh"

    updateStatment="UPDATE OM_PACKS SET STATUS='activated' WHERE PACK_NAME='${new_pack_name}';"
    log_info "${LINENO}" "Update om_packs with sql statment: $updateStatment."

    if [ $(id -u) -eq 0 ]; then
        update_result=$(su - omm -c "\"${execSqlTool}\" -s \"${updateStatment}\" ") ; ret=$?
    else
        update_result="$(${execSqlTool}  -s "${updateStatment}")" ; ret=$?
    fi

    log_info $LINENO "update_result: ${update_result}"

    if [ $ret -ne 0 ];then
        log_error "${LINENO}" "Failed to update om_packs, errorCode=$ret."
        return $ret
    fi
}

active_patch_pack()
{
    if [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
        log_info "${LINENO}" "No need to active patch pack on standby."
        return 0
    fi

    if [ "${g_pack_patch_install}" == "false" ];then
        log_info "${LINENO}" "No need to active patch."
        return 0
    fi

    log_info "${LINENO}" "Begin to active patch pack."

    local patch_path="$g_softwareHome/patch"

    for pack in "${g_pack_patch_pkgnames[@]}"
    do
        local patch_dir=${pack%.tar.gz}
        local package_dir="$patch_path/$patch_dir"
        local patch_sdf="${package_dir}/package.sdf"

        if [ ! -f "${patch_sdf}" ]; then
            log_error "$LINENO" "patch sdf file does not exist."
            return 1
        fi

        local pack_name=$(grep 'packName' ${patch_sdf} | awk -F '=' '{print $2}')

        #从数据库表中查询prePackName
        g_prePackName=""
        queryPrePackName "${pack_name}"
        if [ $? -ne 0 ];then
            log_error "$LINENO" "query prePackName from datebase failed."
            return 1
        fi

        #更新OM_PACKS数据库表
        de_active_old_pack "${g_prePackName}" || return 1
        active_new_pack "${pack_name}" || return 1

    done

    log_info "${LINENO}" "Active patch packs successfully."

    return 0
}

#激活所有补丁: 将OM_PACKAGE所有补丁记录PACKAGE_STATE 设置为INSTALLED
active_patch()
{
    if [ "${g_patch_install}" == "false" ];then
        log_info "${LINENO}" "No need to active patch."
        return 0
    fi

    if [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
        log_info "${LINENO}" "No need to active patch on standby."
        return 0
    fi

    log_info "${LINENO}" "Begin to active patch."

    updateStatment="UPDATE OM_PACKAGE SET PACKAGE_STATE='INSTALLED';"
    log_info "${LINENO}" "Update om_package with sql statment: $updateStatment."

    local execSqlTool="${g_omServerWorkspace}/om/tools/execSql.sh"

    if [ $(id -u) -eq 0 ]; then
        update_result=$(su - omm -c "\"${execSqlTool}\" -s \"${updateStatment}\" ") ; ret=$?
    else
        update_result="$(${execSqlTool}  -s "${updateStatment}")" ; ret=$?
    fi

    log_info $LINENO "update_result: ${update_result}"

    if [ $ret -ne 0 ];then
        log_error "${LINENO}" "Failed to update om_package, errorCode=$ret."
        return $ret
    fi
    
    log_info "${LINENO}" "Active patch successfully."

    return 0    
}

#注册补丁部件包
register_patch_packs()
{
    if [ "${g_pack_patch_install}" == "false" ];then
        log_info "${LINENO}" "No need to register patch packs."
        return 0
    fi

    log_info "${LINENO}" "Begin to register patch packs."

    #备节点只做component补丁新版本目录创建与新配置目录切换
    if [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
        registerPatchPackOnStandby
        if [ $? -ne 0 ];then
            log_error "$LINENO" "Register patch pack on standby oms failed."
            return 1
        fi
        log_info "${LINENO}" "Register patch pack on standy successfully."
        return 0
    fi

    #读取每个补丁包的sdf文件，主备{packName,displayName}}列表
    local request_body=$(get_patch_pack_request)

    local response=$(curl -s -H "${REQUEST_HEAD}" -X "${REQUEST_METHOD}" -d "${request_body}" "${RGISTER_PACK_CONTROLLER_URL}")

    #分析rest返回结果
    local result="$(getXmlNode "$response" "${PACK_RESPONSE_NODE}")"   
    if [ -z "$result" ]; then
        log_error "${LINENO}" "Failed to install the packs."
        return 1
    fi 

    if [ "${result}" == "true" ]; then
        log_info "${LINENO}" "The patch packs are registered suceessfully."
        return 0
    else
        local errorMessage="$(getXmlNode "$response" "${RESPONSE_ERROR_MSG}")"
        log_error "${LINENO}" "Failed to register patch packs. Result: $result , Error message: $errorMessage"
        return 1
    fi

    return 0
}


install_oms_patch()
{
    if [ "${g_oms_patch_install}" == "false" ]; then
        log_info "${LINENO}" "No need to install oms patch."
        return 0
    else
        log_info "${LINENO}" "Begin to install oms patch."  
    fi

    local patch_path="$g_softwareHome/patch"

    #调用install_oms_patch.sh   可能的问题（脚本中调用最新的CP包打补丁，如果没有oms目录，则报错）
    local oms_patch_pakcage="${patch_path}/${g_oms_patch_pkgname}"

    local install_oms_patch_script="${g_omServerWorkspace}/om/sbin/install_oms_patch.sh"
    if [ $(id -u) -eq 0 ]; then
        su - omm -c "/bin/bash \"${install_oms_patch_script}\" \"${oms_patch_pakcage}\" \"silent\" " >> ${g_logFile} 2>&1
        ret=$?
    else
        /bin/bash "${g_omServerWorkspace}/om/sbin/install_oms_patch.sh" "${oms_patch_pakcage}" "silent" >> "${g_logFile}" 2>&1
        ret=$?
    fi

    if [ "${ret}" -ne 0 ];then
        showLog_error "$LINENO" "Failed to install oms patch."
        return 1
    fi

    log_info "${LINENO}" "Install oms patch successfully."  

    return 0
}


################################################################################
# Function: setOmsConfigByNodeNum
# Description: 根据节点数量规模设置OMS相关参数, 该值为空则默认按最低配走
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
setOmsConfigByNodeNum()
{
    #集群规模参数最低时按照最低配走，
    if [ -z "$g_clsNodeNum" ]; then
        g_clsNodeNum=1
    fi
    
    log_info "$LINENO" "Begin modify oms config, node num: $g_clsNodeNum."
    
    sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "${g_omServerWorkspace}/om/sbin/oms_config_info.sh -s $g_clsNodeNum -k -l -f" >> ${g_logFile} 2>&1
    if [ $? -ne 0 ]; then
        showLog_error "$LINENO" "Failed to adapter oms configurations by cluster nodes number."
        return 1;
    fi
}

################################################################################
# Function: setPackageDistributeSubDir
# Description: 获取平台类型&操作系统信息 用于创建包分发子目录
# Others: N/A
################################################################################
setPackageDistributeSubDir()
{
    local os_type_ini="${g_curPath}/osType.ini"
    
    local os_type=$(get_key_value "os" "os_type" "${os_type_ini}")
    ## /opt/FusionInsight_Manager/software/osType.ini 中增加一个字段
    local cpu_type=$(get_key_value "cpu" "cpu_type" "${os_type_ini}")

    #直接读取配置拼接
    g_packageDistributeSubdir="${os_type}-${cpu_type}"

    g_os_type="${os_type}"
    g_cpu_type="${cpu_type}"

    export g_os_type
    export g_cpu_type
}

clear_patch_temp_packages()
{
    local patch_path="$g_softwareHome/patch"
    if [ ! -d "${patch_path}" ]; then
        return 0
    fi

    for pack in "${package_name_array[@]}"
    do
        local dst_dir="$patch_path"
        local patch_dir=${pack%.tar.gz}

        if [ -d "${dst_dir}/${patch_dir}" ];then
            if ! rm -rf "${dst_dir}/${patch_dir}"; then 
                log_error "rm -rf ${dst_dir}/${patch_dir} failed."
                return 1
            fi
            log_info "delete ${dst_dir}/${patch_dir} successfully."
        fi
    done

    return 0
}

scan_patch_packages()
{
    local patch_path="$g_softwareHome/patch"
    if [ ! -d "${patch_path}" ]; then
        log_info "$LINENO" "$g_softwareHome/patch dir does not exist, no patch."
        return 0
    fi

    #扫描目录 排序得到每个部件的最大版本补丁包
    local pre_pack_flag=""
    local cur_pack_flag=""
    index=0
    for pack in $(ls "${patch_path}" | grep .tar.gz$ | sort)
    do
        cur_pack_flag=$(echo "$pack" | awk -F "${version}" '{print $1}')

        if [ -z "${cur_pack_flag}"  ]; then
            continue
        fi

        if [ "${cur_pack_flag}" != "${pre_pack_flag}" -a "${pre_pack_flag}" != ""  ]; then
            index=$(($index+1))
        fi

        package_name_array[${index}]="${pack}"
        pre_pack_flag="${cur_pack_flag}"
    done

    #解压补丁SPC包
    for pack in "${package_name_array[@]}"
    do
        local package_path="$patch_path/$pack"
        local dst_dir="$patch_path"

        local patch_dir=${pack%.tar.gz}
        if [ -d "${dst_dir}/${patch_dir}" ];then
            if ! rm -rf "${dst_dir}/${patch_dir}"; then 
                log_error "rm -rf ${dst_dir}/${patch_dir} failed."
                return 1
            fi
            log_info "delete ${dst_dir}/${patch_dir} successfully."
        fi

        if ! tar -pxzf "${package_path}" -C "${dst_dir}"; then
            log_error "Failed to decompress the pack ${package_path} to ${dst_dir}."
            return 1
        fi
    done

    #初始化静默补丁开关
    local length=${#package_name_array[@]}
    if [ ${length} -gt 0 ]; then
        g_patch_install="true"
    else
        return 0
    fi

    #读取sdf文件 初始化oms补丁开关
    local pack_index=0
    for pack in "${package_name_array[@]}"
    do
        local patch_dir=${pack%.tar.gz}
        local patch_package_dir="${patch_path}/${patch_dir}"

        #oms补丁识别方法：判断packDisplayName=Manager
        local patch_sdf="${patch_package_dir}/package.sdf"

        if [ ! -f "${patch_sdf}" ]; then
            log_error "$LINENO" "patch sdf file does not exist."
            return 1
        fi

        local pack_name=$(grep 'packName' ${patch_sdf} | awk -F '=' '{print $2}')
        local pack_display_name=$(grep 'packDisplayName' ${patch_sdf} | awk -F '=' '{print $2}')

        if [ "${pack_display_name}" == "${PRODUCT_MANAGER}" ];then
            g_oms_patch_install="true"
            g_oms_patch_pkgname="${pack}"
        else
            g_pack_patch_install="true"
            g_pack_patch_pkgnames[${pack_index}]="${pack}"
            pack_index=$(($pack_index+1))
        fi
    done
}

install_silent_patch()
{
    #0.安装oms补丁
    install_oms_patch || return $?

    #1.注册补丁: 解析oms节点补丁+更新补丁相关的3个数据库表
    register_patch || return $?

    #2.注册补丁部件: 更新om_packs表+创建component新工作区+切换component工作区
    register_patch_packs || return $?

    #3.激活补丁
    active_patch || return $?

    #4.激活补丁部件
    active_patch_pack || return $?

    #激活补丁与激活补丁部件涉及数据库刷新 需重启一下oms 否则安装集群时不会修正adapter目录以及组件安装目录
    if [ "${g_ha_status}" == "${HA_ACTIVE_STATUS}" ];then
        sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "${g_omServerWorkspace}/om/sbin/restart-oms.sh" >> ${g_logFile} 2>&1
        retVal=$?
        if [ ${retVal} -ne 0 ]; then
            log_error "${LINENO}" "Failed to restart oms on active."
            return ${retVal}
        fi
    fi

    if [ -d "${g_patch_tempDir}" ];then
        chmod 660 "${g_patch_tempDir}" -R
        rm -rf "${g_patch_tempDir}"
    fi

    return 0
}

################################################################################
# Function: writeRealmIp
# Description: 将kerberos域名解析ip写入controller.properties中
# Others: N/A
################################################################################
writeRealmIp()
{
    source ${g_omm_profile}
    local CONTROLLER_FILE="${CONTROLLER_HOME}/etc/om/controller.properties"
    if [ ! -f ${CONTROLLER_FILE} ];then
        log_warn "${LINENO}" "file not exists , write realmParseIp to controller.properties failed."
        return 1
    fi
    echo "om.realm.parse.ip=${g_realmParseIp}" >> ${CONTROLLER_FILE}
    if [ $? -ne 0 ];then
        log_warn "${LINENO}" "write realmParseIp to controller.properties failed."
       return 1
    fi
}

################################################################################
# Function: configAndStartAgent
# Description: 安装nodeagent所需需要的证书文件，并启动nodeagent
# Others: N/A
################################################################################
configAndStartAgent()
{
    g_omFloatIP="$(get_key_value "${HA}" "${OM_FLOAT_IP}" "${g_omsConfigFile}")"
    
    #安装web登陆安全证书，给组件机机登陆使用，需要从主oms节点拷贝，oms节点安装agent时，oms还没有启动，因此放到这里拷贝
    source ${g_omm_profile} >> "${g_logFile}" 2>&1
    
    #等待keytab生成
    waitForKeytab
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}"  "Waiting for controller to generate keytab failed."
        return ${retVal}
    fi

    install_security_cert
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}"  "Falied to install security cert for oms node."
        return ${retVal}
    fi
    
    #安装agent证书，给组件机机登陆使用，需要从主oms节点拷贝，oms节点安装agent时，oms还没有启动，因此放到这里拷贝
    install_agent_cert
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}"  "Falied to install agent cert for oms node."
        return ${retVal}
    fi
    
    #等待okerberos启动21701端口，否则agent会启动认证失败
    waitForKerberosPort
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}"  "Wait for process ${OKERBEROS_PROC_NAME} port ${KDC_PORT} to be available timeout."
        return ${retVal}
    fi
    
    #启动nodeagent
    sudo ${INSTALL_SUDO_PATH} exec_omm_cmd ${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/bin/start-agent.sh >> "${g_logFile}" 2>&1
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}"  "Falied to start agent."
        return ${retVal}
    fi
}

################################################################################
# Function: waitForKeytab
# Description: 等待controller生成keytab文件
# Parameter:
#   input:
#   output:
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
waitForKeytab()
{
    if [ "$g_operatorType" = "$UPGRADE_OPERATOR" ]; then
        log_info "${LINENO}" "UPGRADE, no need to generate keytab."
        return 0
    fi
    local -i MAX_TIMES=60
    local -i i=0

    for ((i=0; i<MAX_TIMES; i++)) ; do
        g_ha_status=$("${g_omServerWorkspace}/OMS/workspace/ha/module/hacom/script/get_harole.sh")
        if [ "${g_ha_status}" == "${HA_ACTIVE_STATUS}" ];then
            if [ -f ${CONTROLLER_HOME}/etc/om/omm.keytab ]; then
                break
            fi
        elif [ "${g_ha_status}" == "${HA_STANDBY_STATUS}" ];then
            #防止主节点.ssh publickey没有及时同步到备，导致备节点ssh的时候，提示输入密码导致安装流程卡主
            sleep 10
            exec_omm_user_cmd ssh $g_omFloatIP "[ -f ${CONTROLLER_HOME}/etc/om/omm.keytab ]"
            if [ $? -eq 0 ]; then
                break
            fi
        else
            log_warn "${LINENO}"  "Invalid ha status ${g_ha_status} while waiting for generating ${CONTROLLER_HOME}/etc/om/omm.keytab."
        fi
        
        sleep 2
    done

    if  [ $i -ge $MAX_TIMES ] ; then
        log_error "${LINENO}"  "File ${CONTROLLER_HOME}/etc/om/omm.keytab not created in 120s."
        return 1
    fi

    return ${INS_SUCCESS}
}

################################################################################
# Function: waitForKerberosPort
# Description: 等待okerboers端口启动
# Parameter:
#   input:
#   output:
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
waitForKerberosPort()
{
    if [ "$g_operatorType" = "$UPGRADE_OPERATOR" ]; then
        log_info "${LINENO}" "UPGRADE, no need to wait for kerberos port."
        return 0
    fi
    local -i MAX_TIMES=60
    local -i i=0

    for ((i=0; i<MAX_TIMES; i++)) ; do
        netstat -anp | grep ":${KDC_PORT}"
        if [ $? -eq 0 ];then
            break
        else
            log_warn "${LINENO}"  "Process ${OKERBEROS_PROC_NAME} port ${KDC_PORT} not found. Waiting ..."
        fi
        
        sleep 5
    done

    if  [ $i -ge $MAX_TIMES ] ; then
        log_error "${LINENO}"  "Enable port ${KDC_PORT} for process ${OKERBEROS_PROC_NAME} failed in 300s."
        return 1
    fi

    return ${INS_SUCCESS}
}

##############################################################
## @Usage getEncryptKey
## @Return 明 文
## @Description AGENT方法
##############################################################
getEncryptWord()
{

    local STR_SRC="${1}"
    local encType="${2}"
    local KEY=""    

    local wcc=$(find ${NODE_AGENT_HOME}/share/om/lib/ -maxdepth 1 -name wc2frm*.jar)
    local log4j=$(find ${NODE_AGENT_HOME}/share/om/lib -maxdepth 1 -name log4j*.jar)
    local commons=$(find ${NODE_AGENT_HOME}/share/om/lib -maxdepth 1 -name commons-codec*.jar)
    local javaArgs="-Dbeetle.application.home.path=${WCC_PROFILE_DIR}"
    local classpath="${wcc}:${log4j}:${commons}:${NODE_AGENT_HOME}/tools/main.jar"
    local className="com.omm.encrypt.DecryptCommandMain"
    local javaCommand="${JAVA_HOME}/bin/java -cp ${classpath} ${javaArgs} ${className}"

    CMD_STRING=${javaCommand}
    CMD_STRING=${CMD_STRING}" strSrc:"$STR_SRC
    CMD_STRING=${CMD_STRING}" encType:"$encType
    CMD_STRING=${CMD_STRING}" key:"$KEY
    ${CMD_STRING}
}

################################################################################
# Function: install_security_cert
# Description: 安装web登陆安全证书，给组件机机登陆使用
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
install_security_cert()
{
    log_info "${LINENO}" "Installing web security certificate for oms node."
    
    local ENCRYPT_PATH="${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/tools"
    
    exec_omm_user_cmd source "${g_bigdataCommon}/${COMMON_WORKSPACE_NAME}/.common_profile"
    exec_omm_user_cmd source "${g_omAgentWorkspace}/.om_profile"
    exec_omm_user_cmd scp $g_omFloatIP:"$CONTROLLER_HOME/etc/om/security.properties" "${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/etc/agent/"  >> ${g_logFile} 2>&1 || return ${INS_COPY_FILE_FAILURE}
    exec_omm_user_cmd scp $g_omFloatIP:$CONTROLLER_HOME/packaged-distributables/${CAS_CERKEY_NAME}.crt ${OMM_USER_HOME}/  >> ${g_logFile} 2>&1 || return ${INS_COPY_FILE_FAILURE}
    source "${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/etc/agent/security.properties"
    dejreStorePass=$(getEncryptWord ${jreStorePass} AES256_CBC_IGNORE_KEY)

    exec_omm_user_cmd keytool -delete -alias ${CAS_CERKEY_NAME} -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
$dejreStorePass
EOF
    exec_omm_user_cmd keytool -import -noprompt -trustcacerts -alias ${CAS_CERKEY_NAME} \
        -file ${OMM_USER_HOME}/${CAS_CERKEY_NAME}.crt -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
$dejreStorePass
EOF

    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install web security certificate."
        return 1
    fi

    exec_omm_user_cmd chmod 600 ${OMM_USER_HOME}/${CAS_CERKEY_NAME}.crt

    return ${INS_SUCCESS}
}

################################################################################
# Function: generate_cacertfiles
# Description: 依据ca.crt生成中间证书
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
generate_cacertfiles()
{
    local cert_file="$1"
    local cert_tmp_path="$2"
    local cert_num=0
    local cert_file_names=""
    
    if [ ! -d "${cert_tmp_path}" ];then
        exec_omm_user_cmd mkdir "${cert_tmp_path}"
    fi
    
    cert_num=$(cat "${cert_file}" | grep "\-\-\-\-\-END CERTIFICATE\-\-\-\-\-" | wc -l)
    if [ ${cert_num} -le 0 ];then
        showLog_error "${LINENO}" "invalid cert file num ${cert_num}." 
        return 1
    fi
    
    local i=1
    for((i=1;i<=${cert_num};i++));
    do
        if [ -f "${cert_tmp_path}/ca_level$i.crt" ];then
            cat /dev/null > "${cert_tmp_path}/ca_level$i.crt"
        fi
    done
    
    local j=1
    while read line
    do
        echo "${line}" >> "${cert_tmp_path}/ca_level$j.crt"
        if [ $? -ne 0 ];then
            showLog_error "${LINENO}" "create ${cert_tmp_path}/ca_level$j.crt failed." 
            return 1
        fi
        
        exec_cmd chown ${OMM_USER}:${OMM_USER_GROUP} ${cert_tmp_path}/ca_level$j.crt
        if [ $? -ne 0 ];then
            showLog_error "${LINENO}" "chown ${cert_tmp_path}/ca_level$j.crt failed." 
            return 1
        fi        
        
        echo "${line}" | grep "\-\-\-\-\-END CERTIFICATE\-\-\-\-\-" > /dev/null
        if [ $? -eq 0 ];then
            if [ $j -eq 1 ];then
                cert_file_names="ca_level$j.crt"
            else
                cert_file_names="${cert_file_names} ca_level$j.crt"
            fi
            j=$[ $j + 1 ]
        fi
    done < "${cert_file}"
    
    echo "${cert_file_names}"
    return 0
    
}

################################################################################
# Function: import_cacertfiles_to_ketstore
# Description: 将证书链导入JRE的keystore
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
import_cacertfiles_to_ketstore()
{
    local middlecert_path="${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/security/middlecert"
    local ca_cert="${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/security/cert/subcert/certFile/ca.crt"
    local middle_cert_names=""
    local j=0
    middle_cert_names=$(generate_cacertfiles "${ca_cert}" "${middlecert_path}")
    if [ 0 -ne $? ]; then
        showLog_error "${LINENO}" "Failed to generate cert files according to ${ca_cert}" 
        return 1
    fi
    #Get an array of names of certificate chains
    cert_files_list=($(echo "${middle_cert_names}" | awk -F ' ' '{for(i=1; i<=NF; i++){print $i;};}'))
    
    cert_num=${#cert_files_list[@]}
    if [ ${cert_num} -le 0 ];then
        showLog_error "${LINENO}" "invalid cert file num ${cert_num}"
        return 1
    fi
    
    #import ca_level${j} to JRE.keystore
    for((i=0;i<${cert_num};i++));
    do
        let j=i+1
        exec_omm_user_cmd keytool -delete -alias ca_level${j} -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
${dejreStorePass}
EOF
        exec_omm_user_cmd keytool -import -noprompt -trustcacerts -alias ca_level${j} -file "${middlecert_path}/${cert_files_list[${i}]}" -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
${dejreStorePass}
EOF
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to import ca_level${j} to JRE.keystore." 
            return 1
        fi
    done
    
    log_info "${LINENO}" "Successful to import ca_level${j} to JRE.keystore." 
    
    return 0
    
}

################################################################################
# Function: install_agent_cert
# Description: 从oms节点拷贝证书到agent节点
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
install_agent_cert()
{
    log_info "${LINENO}" "Installing agent certificate for oms node."

    local certpath="${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/security/cert/subcert/certFile"
    local ENCRYPT_PATH="${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/tools"

    exec_cmd source "${g_bigdataCommon}/${COMMON_WORKSPACE_NAME}/.common_profile"
    exec_omm_user_cmd scp -r  $g_omFloatIP:"$CONTROLLER_HOME/security/cert/subcert/certFile/"* "${certpath}/"  || return ${INS_COPY_FILE_FAILURE}
    source "${NODE_AGENT_HOME}/etc/agent/security.properties"
    
    dejreStorePass=$(getEncryptWord ${jreStorePass} AES256_CBC_IGNORE_KEY)
    
    #import httpd security certificate.
    exec_omm_user_cmd keytool -delete -alias httpdcert -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
${dejreStorePass}
EOF

    exec_omm_user_cmd keytool -import -noprompt -trustcacerts -alias httpdcert \
        -file  ${certpath}/proxy_ssl.cert  -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
${dejreStorePass}
EOF

    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to import httpd security certificate."
        return 1
    fi

    #import ca_level$i security certificate.
    import_cacertfiles_to_ketstore
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to import ca_level$i security certificate."
        return 1
    fi

    #import SSL security certificate.
    exec_omm_user_cmd keytool -delete -alias fusionInsightsubroot -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
${dejreStorePass}
EOF

    exec_omm_user_cmd keytool -import -noprompt -trustcacerts -alias fusionInsightsubroot \
        -file  ${certpath}/sslservercert.crt  -keystore ${JAVA_HOME}/jre/lib/security/cacerts  >> ${g_logFile} 2>&1 <<EOF
${dejreStorePass}
EOF

    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to import SSL security certificate."
        return 1
    fi

    source ${certpath}/password.property
    if [ -z ${password} ] ; then
        showLog_error "${LINENO}" "Failed to get cert password"
        return 1
    fi

    if [ -f  ${certpath}/password.changed ] ;then
        exec_omm_user_cmd cp  ${certpath}/password.property  ${certpath}/password.file
        log_info "${LINENO}" "  agent  password file has changed ."
    else
        local password=$(getEncryptWord ${password} AES256)
        local randomKeyPassWord=$(${ENCRYPT_PATH}/encryptWithoutParam.sh <<EOF
--strSrc ${password}
--encType AES256_CBC_IGNORE_KEY
EOF
)
        echo "password=${randomKeyPassWord}"  >  ${certpath}/password.file
        log_info "${LINENO}" "generate  agent  password file."
    fi

    local opensslCnf="${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}/security/cert/subcert/demoCA/openssl.cnf"
    if [ ! -f "${opensslCnf}" ] ; then
        showLog_error "${LINENO}" "The ${opensslCnf} file does not exist."
        return 1
    fi
    sed -i "s:#NODE_AGENT_HOME#:${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}:g" "${opensslCnf}"

    chmod 600  ${certpath}/password.file

    return ${INS_SUCCESS}
}

backupLdapserver(){
    local ldapserverData=${g_productDataHome}/ldapData/ldapserver
    if [ -d ${ldapserverData} ];then
        rm -rf /tmp/backupLdapserver && mkdir /tmp/backupLdapserver
        cp -arf ${ldapserverData} /tmp/backupLdapserver/
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to backup ${g_productDataHome}/ldapData/ldapserver."
            return 1
        fi
        if [ $(id -u) -eq 0 ] ; then
            chown omm:wheel /tmp/backupLdapserver
        fi
    fi
}

recoverLdapserver(){
    if [  -d /tmp/backupLdapserver/ldapserver ] && [  ! -d ${BIGDATA_DATA_HOME}/ldapData/ldapserver ];then
        cp -arf /tmp/backupLdapserver/ldapserver ${BIGDATA_DATA_HOME}/ldapData/
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to recover ${BIGDATA_DATA_HOME}/ldapData/ldapserver."
            return 1
        fi
    fi
}
################################################################################
# Function: main
# Description: 主函数
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    umask 0027
    
    showLog_info "${LINENO}" "=================================== Welcome ==================================="
    showLog_info "${LINENO}" "=== STEP 1 Checking the parameters."

    # 判断OS类型
    validate_os_type
    if [ $? -ne 0 ];then
       return 1
    fi

    # 校验平台类型
    validate_cpu_type
    if [ $? -ne 0 ];then
       return 1
    fi

    #获取平台子目录
    setPackageDistributeSubDir
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to setPackageDistributeSubDir."
        return 1
    fi

    # 判断ha SSL证书的合法性
    validate_ha_ssl_cert
    if [ $? -ne 0 ];then
       return 1
    fi
    # 获取安装输入参数并校验合法性，然后保存到配置文件
    export GLOBAL_INSTALL_USER
    export TEMP_LOG_PATH
    export GLOBAL_PARAM_FILE
    export INSTALLSH_PATH=${PWD}
    
    #解析输入参数
    ${g_curPath}/om/script/initInputParam.sh "$@"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        log_error "$LINENO" "Failed to init input parameters."
        return ${retVal}
    fi
    
    # 获取initInputParam.sh计算的变量值：子shel把计算的变量值保存到GLOBAL_PARAM_FILE
    # 文件中，供父shell读取
    source ${g_curPath}/om/script/${GLOBAL_PARAM_FILE}
    
    # 获取用户传递的日志根目录后，把此处以前打印的日志移到用户设置的日志根目录里。不会
    # 出错，不用判断返回值
    reinit_log

    # 初始化全局变量值
    init_global_param
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    # 检查磁盘空间,如检查失败，则删除安装根目录和数据根目录
    check_disk_space
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to check disk space."
        return 1
    fi

    # 检查主机名是否合法,合法的主机名：以字母或者数字开头并且由字母、数字和中划线(-)组成
    hostname | grep '^[[:alnum:]][[:alnum:]\-]*$' > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Invalid host name $(hostname). The host name must start with a letter or digit and consist of letters, digits, or hyphens (-)."
        return 1
    fi

    #安装包瘦身还原
    g_step_index=2
    prepare_components >> "${g_logFile}" 2>&1 &
    spin_wait "=== STEP ${g_step_index} Preparing for installation components." "$!"
    if [ $? -ne 0 ]; then
        showLog_info "$LINENO" "Prepare failed."
        return 1
    fi
    g_step_index=$(($g_step_index+1))
    
    # 导出日志根目录环境变量(其值为用户传入的目录),供其子shell install.sh使用
    declare -x BIGDATA_LOG_HOME="${g_logHome}"

    # 将用户omm id和DB ommdba用户id设置为全局变量
    if [ $(id -u) -ne 0 ] ; then
        g_ommid=$(id -u)
    fi
    export g_ommid
    export g_ommdbaid
    export g_softwareHome

    # 备份当前节点的$BIGDATA_DATA_HOME/ldapData/ldapserver目录
    backupLdapserver || return 1

    # 执行om的安装脚本，安装controller
    "${g_omHome}/inst/install.sh" "${g_productDataHome}" "${g_operatorType}" "${g_kerberos_realm}">> "${g_logFile}" 2>&1 &
    spin_wait "=== STEP ${g_step_index} Installing the manager." "$!"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi
    g_step_index=$(($g_step_index+1))

    source ${HOME}/.bashrc > /dev/null 2>&1

    #将解析域名写入到controller.properties中，升级场景由适配代码完成realm.parse.ip配置
    if [ "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
        writeRealmIp
        if [ $? -ne 0 ] ; then
            log_warn "${LINENO}" "write realmParseIp to controller.properties failed."
        fi
    fi

    # 安装部件包
    if [ "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
        installPack || return 1
    fi
    
    # 根据节点规模调整OMS相关参数
    setOmsConfigByNodeNum || return 1


    #判断补丁包存放目录是否有补丁，初始化打补丁开关
    scan_patch_packages || return 1

    # 启动OMS的所有进程
    start_oms || return $?
    
    if [ "${g_patch_install}" == "true" ];then
        install_silent_patch >> "${g_logFile}" 2>&1 &
        spin_wait "=== STEP ${g_step_index} Installing the patch." "$!"
        if [ $? -ne 0 ]; then
            showLog_info "$LINENO" "Install oms patch failed."
            return 1
        fi

        clear_patch_temp_packages || return 1

        g_step_index=$(($g_step_index+1))
    fi
    
    # 等待NTP进入同步状态
    if [ -z "${USE_EXTERNAL_NTP}" -o "${USE_EXTERNAL_NTP}" != "YES" ]; then
        waitforNtp >> "${g_logFile}" 2>&1 &
        spin_wait "=== STEP ${g_step_index} Waiting for ntp to startup." "$!"
        retVal=$?
        if [ ${retVal} -ne 0 ]; then
            showLog_error "${LINENO}"  "Wait for ntp to startup timeout."
            return ${retVal}
        fi
        g_step_index=$(($g_step_index+1))
    fi

    g_omsConfigFile="${g_omHome}/inst/conf/oms-config.ini"
    
    # 改变sudoExecute.sh及其父的权限：root: 500
    sudo ${INSTALL_SUDO_PATH} modify_oms_sudo_script
    
    local docFile=""
    local tmpdocfile=""
    tmpdocfile=$( ls ${g_curPath} | grep "FusionInsight" )
    if [ ! -z "${tmpdocfile}" ]; then
        docFile="FusionInsight*.doc*"
    else
        tmpdocfile=$( ls ${g_curPath} | grep "GaussDB" )
        if [ ! -z "${tmpdocfile}" ]; then
            docFile="GaussDB*.doc*"
        else
            showLog_error "${LINENO}" "Failed to find open source software notices."
            return 1
        fi
    fi
    
    if [ "x${docFile}" == "x" ]; then
        showLog_error "${LINENO}" "Failed to find open source software notices."
        return 1
    fi
    
    cp ${g_curPath}/${docFile} ${g_productHome} > /dev/null 2>&1
    if [ $? -ne 0 ]; then
       showLog_error "${LINENO}" "Failed to copy fusionInsiht open source software notices."
       return 1
    fi
    chmod 600 ${g_productHome}/${docFile} > /dev/null 2>&1
    chown omm:wheel ${g_productHome}/${docFile} > /dev/null 2>&1

    
    #删除HA ssl证书的明文密码
    if [ -f "${g_softwareHome}/hasslCert/hasslCert.ini" ]; then
       sed -i s/password=.*/password=/g "${g_softwareHome}/hasslCert/hasslCert.ini"
    fi
    
    g_wsFloatIP="$(get_key_value "${HA}" "${WS_FLOAT_IP}" "${g_omsConfigFile}")"
    #禁止ommdba远程登录
    sudo ${INSTALL_SUDO_PATH} forbid_ommdba_login
    if [ $? -ne 0 ]; then
        log_debug "${LINENO}" "failed to execute forbid_ommdba_login."
    fi

    recoverLdapserver
    if [ $? -ne 0 ]; then
        log_error "${LINENO}" "recover Ldapserver failed,Manually restore LDAP data."
    fi
    showLog_info "${LINENO}" "============================= Install Successfully ============================"
    showLog_info "${LINENO}" "Please visit http://${g_wsFloatIP}:${CONTROLLER_PORT}/web/ to continue cluster installation."
}

# ---------------------------------------------------------------------------- #
#                                 获取当前路径                                 #
# ---------------------------------------------------------------------------- #
get_cur_path
cd "${g_curPath}"
# ---------------------------------------------------------------------------- #
#                                 判断安装用户                                #
# ---------------------------------------------------------------------------- #
if [ $(id -u) -eq 0 ] ; then
    GLOBAL_INSTALL_USER=${g_super_user}
else
    source ${HOME}/.bashrc > /dev/null 2>&1
    if [ $(id -un) != "${INSTALL_USER}" ]; then
       echo "ERROR: The user $(id -un) does not have the permission to execute the script."
       exit 1
    else
       GLOBAL_INSTALL_USER=$(id -un)
    fi
fi
# ---------------------------------------------------------------------------- #
#                                 初始化日志文件                               #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/om/script/func/log.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${g_curPath}/om/script/func/log.sh."; exit 1;}
init_log || exit 1

# ---------------------------------------------------------------------------- #
#                                   导入头文件                                 #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/om/script/func/constants.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/om/script/func/constants.sh."; exit 1; }
. "${g_curPath}/om/script/func/functions.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/om/script/func/functions.sh."; exit 1; }
. "${g_curPath}/${RELEASE_CONF_FILENAME}" || { log_error "${LINENO}" "Failed to load ${g_curPath}/${RELEASE_CONF_FILENAME}."; exit 1; }

# ---------------------------------------------------------------------------- #
#                                 脚本开始运行                                 #
# ---------------------------------------------------------------------------- #
main "$@"
post_install $?
ret=$?

#在操作系统日志中记录审计日志
syslog "manager" "install.sh" "$ret" "Install oms."
exit $ret


