#!/bin/bash
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    uninstall.sh
## Description: uninstall controller
## Version:     FusionInsight Hadoop V100R002C02
## Created:     Tuesday, June 04, 2013
################################################################################
# ---------------------------------------------------------------------------- #
#                                   变量定义                                   #
# ---------------------------------------------------------------------------- #
declare g_operatorType="$1"                         # 当前操作的类别
declare g_logFile=""                                # 日志文件名
declare g_osOptimizatLogFile=""                     # 系统优化日志文件名
declare g_controllerHome=""                         # controller安装目录
declare g_bigdataHome=""                            # bigdata安装目录
declare g_omServerWorkspace=""                        # om-server workspace
declare g_tomcatHome=""                             # bigdata安装目录
declare g_omsRunPath=""                             # OMS运行目录
declare g_omaRunPath=""                             # OMA运行目录
declare g_bigdataDataHome=""                        # 产品数据存放目录
declare g_logHome=""                                # 日志根目录
declare g_curPath=""                                # 当前脚本所在的目录
declare g_scriptName=""                             # 当前脚本的脚本名称
declare -i retVal=0                                 # 返回值
declare  SUDO_PATH_OMS=""                            # OMS侧sudoExecute.sh路径


# ---------------------------------------------------------------------------- #
#                                   常量定义                                   #
# ---------------------------------------------------------------------------- #
declare -r UNINSTALL_LOG_HOME="/var/log"            # 卸载日志根目录

# ---------------------------------------------------------------------------- #
#                                   函数定义                                   #
# ---------------------------------------------------------------------------- #
################################################################################
# Function: get_cur_path
# Description: 获取脚本所在的目录及脚本名
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="${PWD}"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    cd - > /dev/null 2>&1
}
################################################################################
# Function: set_install_sudo_env
# Description: root用户卸载时设置环境变量
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function set_install_sudo_env()
{
    user_home="$HOME/.bashrc"
    if [ ! -f "${user_home}" ] ; then
        touch ${user_home}
    fi
    user_env="$HOME/env_file"
    if [ ! -f "${user_env}" ] ; then
        touch ${user_env}
    fi

    grep INSTALL_SUDO_PATH "${user_env}" >> "${g_logFile}" 2>&1
    if [ $? -eq 0 ]; then
        sed -i '/export INSTALL_SUDO_PATH.*/d' "${user_env}"
    fi
    
    echo "export INSTALL_SUDO_PATH=${BIGDATA_SUDO_HOME}/sudo/install/installSudoExecute.sh" >> "${user_env}"
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    grep INSTALL_USER "${user_env}" >> "${g_logFile}" 2>&1
    
    if [ $? -eq 0 ]; then
        sed -i '/export INSTALL_USER.*/d' "${user_env}"
    fi
    echo "export INSTALL_USER=$(id -un)" >> "${user_env}"
    grep "^source ${user_env}" ${user_home}  > /dev/null 2>&1 
    if [ $? -ne 0 ] ; then
        echo "source ${user_env}" >> ${user_home} 
    fi
    source ${user_home} > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        return 1
    fi
} 

################################################################################
# Function: init_log
# Description: 初始化日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_log()
{
    if [ "$(id -u)" = "0" ] ; then 
        # controller卸载日志存放路径
        local logPath="${UNINSTALL_LOG_HOME}"
        g_logFile="${logPath}/bigdataControllerUninstall.log"
        g_osOptimizatLogFile="${logPath}/bigdataControllerUninstallOsOptimization.log"

        # 创建日志文件和修改文件权限
        mkdir -p "${logPath}"
        chmod u+rwx "${logPath}"

        rm -rf "${g_logFile}"
        touch "${g_logFile}"
        chmod 600 "${g_logFile}"

        rm -rf "${g_osOptimizatLogFile}"
        touch "${g_osOptimizatLogFile}"
        chmod 600 "${g_osOptimizatLogFile}"

        return 0
    else    
        # 创建controller卸载日志
        local logPath="${UNINSTALL_LOG_HOME}"
        g_logFile="${logPath}/bigdataControllerUninstall.log"
        g_osOptimizatLogFile="${logPath}/bigdataControllerUninstallOsOptimization.log"
        
        sudo ${INSTALL_SUDO_PATH} creat_uninstall_oms_log "${logPath}"
        if [ "$?" != "0" ] ; then
            showLog_error "${LINENO}" "Failed to creat the uninstall log."
            return 1
        fi
        return 0
    fi
}
################################################################################
# Function: init_global_param
# Description: 初始化全局变量
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_global_param()
{
    log_debug "${LINENO}" "Initialize the global parameters."

    # 初始化g_controllerHome
    g_controllerHome=${CONTROLLER_HOME}
    log_debug "${LINENO}" "g_controllerHome=${g_controllerHome}."

    # 初始化g_serverWorkspace
    g_omServerWorkspace="$(dirname "${g_controllerHome}")"
    log_debug "${LINENO}" "g_serverWorkspace=${g_omServerWorkspace}."

    # 初始化g_bigdataHome
    g_bigdataHome="$(dirname "${g_omServerWorkspace}")"
    log_debug "${LINENO}" "g_bigdataHome=${g_bigdataHome}."
    
    source ${OMM_USER_HOME}/.profile >> "${g_logFile}" 2>&1
    
    # 初始化g_omsRunPath，若计算的该变量为空，说明OMS没有安装，则不卸载OMS
    g_omsRunPath="$(echo ${OMS_RUN_PATH})"
    log_debug "${LINENO}" "g_omsRunPath=${g_omsRunPath}."

    # 初始化g_omaRunPath，若计算的该变量为空，说明OMA没有安装，则不卸载OMA
    g_omaRunPath="$(echo ${OMA_RUN_PATH})"
    log_debug "${LINENO}" "g_omaRunPath=${g_omaRunPath}."

    # 初始化g_bigdataDataHome
    g_bigdataDataHome="$(echo ${BIGDATA_DATA_HOME})"
    log_debug "${LINENO}" "g_bigdataDataHome=${g_bigdataDataHome}."
    
    # 初始化g_logHome
    g_logHome="${BIGDATA_LOG_HOME}"
    log_debug "${LINENO}" "g_logHome=${g_logHome}."
    
    # 初始化g_tomcatHome
    g_tomcatHome="$(echo ${OM_TOMCAT_HOME})"
    log_debug "${LINENO}" "g_tomcatHome=${g_tomcatHome}."

    if [ -z ${BIGDATA_SUDO_HOME} ]; then
        BIGDATA_SUDO_HOME="/var/lib/sudo/Bigdata"
    fi

    return ${INS_SUCCESS}
}

################################################################################
# Function: print_usage
# Description: 打印卸载提示信息
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
print_usage()
{
    showLog_info "${LINENO}" "------------------------------------------------------------------------------------------"
    showLog_info "${LINENO}" "Prerequisites:"
    showLog_info "${LINENO}" "1. The system has no cluster. If clusters have been installed, uninstall them."
    showLog_info "${LINENO}" "2. No operation is being performed."
    showLog_info "${LINENO}" "Check that all conditions are met before the uninstallation."
    showLog_info "${LINENO}" "------------------------------------------------------------------------------------------"

}

################################################################################
# Function: uninstallOlK
# Description: 卸载Okerberos和oldap
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
uninstallOlK()
{
     # 卸载Okerberos
     showLog_info "${LINENO}" "Uninstalling okerberos."

     local okerberosUninstallCMD="${g_controllerHome}/meta-0.0.1-SNAPSHOT/kerberos/scripts/krb_uninstall.sh"
     if [ -f "${okerberosUninstallCMD}" ] ; then
        sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "$okerberosUninstallCMD" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            log_warn "${LINENO}" "Failed to uninstall okerberos."
        fi
     else
        log_warn "${LINENO}" "The okerberos is not installed."
     fi

     # 卸载oldap
     showLog_info "${LINENO}" "Uninstalling oldap."
     local oldapUninstallfile="${g_controllerHome}/ldapserver/ldapserver/ldapserver_uninstall.sh"
     local oldapUninstallCMD="${g_controllerHome}/ldapserver/ldapserver/ldapserver_uninstall.sh uninstall"
     if [ -f "${oldapUninstallfile}" ] ; then
        sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "$oldapUninstallCMD" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            log_warn "${LINENO}" "Failed to uninstall oldap."
        fi
     else
        log_warn "${LINENO}" "The oldap is not installed."
     fi
}

################################################################################
# Function: uninstall
# Description: 卸载controller
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
uninstall()
{
    showLog_info "${LINENO}" "Uninstallation in progress."
   
    # 停止OMS
    showLog_info "${LINENO}" "Stopping the OMS..."
    local omsStopCMD="${g_controllerHome}/sbin/stop-oms.sh"
    if [ -f "${omsStopCMD}" ] ; then
        sudo ${INSTALL_SUDO_PATH} exec_omm_cmd "$omsStopCMD" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            log_warn "${LINENO}" "Failed to stop OMS."
        fi
    else
        log_warn "${LINENO}" "The OMS is not installed."
    fi

    # 卸载OMS
    showLog_info "${LINENO}" "Uninstalling the OMS..."
    if [ -n "${g_omsRunPath}" -a -d "${g_omsRunPath}" ] ; then
        ${g_omsRunPath}/setup/uninstall.sh > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            showLog_warn "${LINENO}" "Failed to uninstall the OMS."
        fi
    fi

    # root下USE_EXTERNAL_NTP变量为YES时，禁止FI的NTP功能
    if [ -z "${USE_EXTERNAL_NTP}" -o "${USE_EXTERNAL_NTP}" != "YES" ]; then
        
        local backup_dir="${BIGDATA_HOME}/om-agent/configbackup"
        local ntp_sys_conf="/etc/ntp.conf"
        local ntp_backup_conf="$backup_dir/ntp.conf"
        IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
        if [ -n "$IS_SUSE_OS" ]; then
            ntp_sys_keys="/etc/ntp.keys"
            ntp_backup_keys="$backup_dir/ntp.keys"
        else
            ntp_sys_keys="/etc/ntp/ntpkeys"
            ntp_backup_keys="$backup_dir/ntpkeys"
        fi
        
        if [ -f $ntp_backup_conf ] ; then
            sudo ${INSTALL_SUDO_PATH} uninstall_ntp_sys_conf $ntp_backup_conf $ntp_sys_conf  >> "${g_logFile}" 2>&1
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}" "Failed to rollback $ntp_backup_conf to $ntp_sys_conf."
                return 1
            fi
        fi
        
        if [ -f $ntp_backup_keys ] ; then
            sudo ${INSTALL_SUDO_PATH} uninstall_ntp_sys_conf $ntp_backup_keys $ntp_sys_keys  >> "${g_logFile}" 2>&1
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}" "Failed to rollback $ntp_backup_conf to $ntp_sys_keys."
                return 1
            fi
        fi   
    fi

    # 清除crontab
    if [ -f "${CRONTAB_FILE}" ] ; then
        sed -i '/om-server/d' "${CRONTAB_FILE}"
    fi

    # 重启crontab服务
    sudo ${INSTALL_SUDO_PATH} restart_crontab_service "${CRON_SERVICE_NAME}" > /dev/null 2>&1
    
    # 清除DNS配置
    sudo ${INSTALL_SUDO_PATH} remove_DNS_config "${g_logFile}" > /dev/null 2>&1
    
    # 删除数据目录
    sudo ${INSTALL_SUDO_PATH} delete_dbdata_om "${g_bigdataDataHome}/dbdata_om" > /dev/null 2>&1
    if [ "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
        sudo ${INSTALL_SUDO_PATH} delete_Manager "${g_bigdataDataHome}/Manager" > /dev/null 2>&1
        sudo ${INSTALL_SUDO_PATH} delete_manager_backup_link "${g_bigdataDataHome}/LocalBackup" > /dev/null 2>&1
    fi

    log_info "${LINENO}" "Uninstallation succeeded."
    return ${INS_SUCCESS}
}

################################################################################
# Function: uninstall_os_optimization
# Description: 执行卸载OS优化脚本
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function uninstall_os_optimization
{
    log_debug "${LINENO}" "Start to uninstall os_optimization."
    find ${g_controllerHome}/inst/plugin/ -maxdepth 2 -mindepth 2 -name "uninstall.sh" 2>/dev/null | while read line
    do
        chmod 700 $line
        $line ${g_osOptimizatLogFile} "${INSTALL_SUDO_PATH}"
        retVal=$?
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to optimize the operating system."
            return ${INS_CREATE_FILE_FAILURE}
        fi
    done

    log_debug "${LINENO}" "Succeed to uninstall os_optimization."
    return ${INS_SUCCESS}
}

################################################################################
# Function: get_user_command
# Description:get user input command
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function get_user_command()
{
    echo -n "Are all the conditions met? (y/n):"
    maxRetry=0
    read response   
    ((maxRetry=maxRetry+1))
    while [ "$maxRetry" -lt 5 ]
    do
       if [ "${response}" = "y" ] || [ "$response" = "Y" ] ; then
           return 0
       elif [ "${response}" = "n" ] || [ "$response" = "N" ] ; then
           log_info "${LINENO}" "Canceled the uninstallation."
           return  1
       else
           ((maxRetry=maxRetry+1))
            echo -n "Input error: ${response}, please re-enter(y/n):"
            read response
       fi
    done

    return 1
}

################################################################################
# Function: in
# Description: 主函数
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    # 提示卸载信息,不会出错，不用判断返回值
    print_usage
    
    #get user input command
    get_user_command
    if [ $? -ne 0 ]; then
        exit 1
    fi 
    
    # 初始化全局变量
    init_global_param
  
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to initialize the global parameters."
        return 1
    fi
    
    #如果是root用户，则设置root用户的环境变量
    if [ "$(id -u)" = "0" ] ; then
        set_install_sudo_env
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to set install_sudo_env."
            return 1
        fi       
    fi

    # 调用系统优化脚本
    uninstall_os_optimization > /dev/null 2>&1
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to uninstall OS optimization of script."
        showLog_error "${LINENO}" "For details about the error, see the log file ${UNINSTALL_LOG_HOME}/bigdataControllerUninstallOsOptimization.log"
        return ${retVal}
    fi

    # 卸载Okerberos和oldap
    uninstallOlK
    if [ $? -ne 0 ] ; then
        return 1
    fi

    # 开始卸载controller
    uninstall
    if [ $? -ne 0 ] ; then
        #sync disk
        sync
        return 1
    fi

    if [ -n "${BIGDATA_HOME}" ];then
        rm -rf ${BIGDATA_HOME}/components
        rm -f ${BIGDATA_HOME}/FusionInsight*.docx
        rm -rf ${BIGDATA_HOME}/LocalBackup
        rm -rf ${BIGDATA_HOME}/Manager
        rm -rf ${BIGDATA_HOME}/om-server
        rm -rf ${BIGDATA_HOME}/om-server_6.5.1
        rm -rf ${BIGDATA_HOME}/packaged-distributables
    fi

    [ -e /home/omm/.profile ] && sed -i "/om-server/d" /home/omm/.profile

    #sync disk
    sync

    return ${INS_SUCCESS}
}

# ---------------------------------------------------------------------------- #
#                                 获取当前路径                                 #
# ---------------------------------------------------------------------------- #
get_cur_path
cd "${g_curPath}"

# 校验卸载用户
if [ $(id -u) -ne 0 ] ; then
    if [ $(id -un) != "${INSTALL_USER}" ]; then
        echo "ERROR: The user does not have the permission to execute the script."
        exit 1
    fi
fi
init_log || exit 1

source "/home/omm/.profile" >> "${g_logFile}" 2>&1
# ---------------------------------------------------------------------------- #
#                                 初始化日志文件                               #
# ---------------------------------------------------------------------------- #
. "${CONTROLLER_HOME}/inst/func/log.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${g_curPath}/func/log.sh."; exit 1;}

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                   #
# ---------------------------------------------------------------------------- #
. "${CONTROLLER_HOME}/inst/func/constants.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/func/constants.sh."; exit 1; }
. "${CONTROLLER_HOME}/inst/func/functions.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/func/functions.sh."; exit 1; }
. "${CONTROLLER_HOME}/inst/oss/interface/interface.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/oss/interface/interface.sh."; exit 1; }

# ---------------------------------------------------------------------------- #
#                                 脚本开始运行                                 #
# ---------------------------------------------------------------------------- #


main
retVal=$?
if [ ${retVal} -eq 0 ];then
    showLog_info "${LINENO}" "Uninstallation is successful."
else
    showLog_error "${LINENO}" "Uninstallation failed."
fi

#在操作系统日志中记录审计日志
syslog "manager" "uninstall.sh" "$retVal" "Uninstall oms."

exit ${retVal}
