#!/bin/bash
################################################################################
## Filename:    permissionManager.sh
## Description: 拆分后处理用户,用户组,角色等数据
################################################################################

##############################################################
## @Usage getCurPath
## @Description 获取当前路径
##############################################################
function getCurPath() {
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    cd - >/dev/null
}

##############################################################
## @Usage initLogAndTmpDir
## @Description 日志
##############################################################
function initLogAndTmpDir() {
    if [ ! -f ${G_LOG_FILE} ]; then
        touch ${G_LOG_FILE}
        if [ $? -ne 0 ]; then
            echo "Failed to create log file: ${G_LOG_FILE}"
            exit 1
        fi
    fi
}

##############################################################
## @Usage deleteUserGroup
## @Description 删除用户组
##############################################################
function deleteUserGroup() {
    if [ $# != 1 ]; then
        showLog_error "${LINENO}" "number of user groups is not 1,parameters: [$*]."
        exit 1
    fi
    local userGroup="${1}"
    local response=$(curl -k "http://127.0.0.1:20006/usergroup/forcedeleteusergroup/${userGroup}" -X DELETE 2>/dev/null)
    local resultCode="$(echo "${response}" | sed 's/,/\n/g' | grep "resultCode" | sed 's/:/\n/g' | sed '1d' | sed '1d')"
    if [ "$resultCode" == 0 ]; then
        showLog_info "${LINENO}" "Succeeded in deleting the user group ${userGroup}."
        return 0
    else
        local errorInfo="$(echo "${response}" | sed 's/,/\n/g' | grep -w "resultDesc" | sed 's/:/\n/g' | sed '1d')"
        showLog_error "${LINENO}" "Failed to delete the user group ${userGroup}. Error message : ${errorInfo}"
        return 1
    fi
}

##############################################################
## @Usage deleteUser
## @Description 删除用户  TODO 暂无方案,待完善
##############################################################
function deleteUser() {
    return 1
}

##############################################################
## @Usage deleteRole
## @Description 删除角色  TODO 暂无方案,待完善
##############################################################
function deleteRole() {
    return 1
}

function show_help() {
    echo
    echo "Usage: $(basename $0) [option]"
    echo "options:"
    echo "cleanUserGroup          Notes: Deleting a User Group."
    echo "cleanUser               Notes: Deleting a User."
    echo "cleanRole               Notes: Deleting a Role."
    echo "cleanMutualTrust        Notes: Deleting Mutual Trust."
    echo "help                    Notes: show help."
    echo
    echo "Example:"
    echo "$(basename $0)"
    echo "$(basename $0) cleanUserGroup userGroupName"
    echo "$(basename $0) cleanUser userName"
    echo "$(basename $0) cleanRole roleName"
    echo "$(basename $0) help"
    echo
    return 0
}

function main() {
    case $1 in
    cleanUserGroup)
        deleteUserGroup "$2"
        exit $?
        ;;
    cleanUser)
        deleteUser "$2"
        exit $?
        ;;
    cleanRole)
        deleteRole "$2"
        exit $?
        ;;
    help)
        show_help
        exit $?
        ;;
    *)
        showLog_error "${LINENO}" "Unknown cmd[$*]."
        show_help
        exit 1
        ;;
    esac
}

if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can export data."
    exit 1
fi
getCurPath
# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/../func/functions.sh" || {
    echo "Failed to load functions.sh."
    exit 1
}
. "${g_curPath}/../func/log.sh" || {
    echo "Failed to load log.sh."
    exit 1
}
. "${g_curPath}/../func/constants.sh" || {
    echo "Failed to load constants.sh."
    exit 1
}

main "$@"
exit $?
