#!/bin/bash
# --------------------------------------------------------------------------- #
#                           函数定义
# --------------------------------------------------------------------------- #


init_log()
{
    # 创建日志路径
    local logDir="${BIGDATA_LOG_HOME}/patch/"
    local logFile="decompress.log"
    mkdir -p "${logDir}"
    chmod 700 "${logDir}"

    # 创建日志文件
    g_logFile="${logDir}/${logFile}"
    touch "${g_logFile}"
    chmod 700 "${g_logFile}"
    
}

log()
{
    # 打印时间、日志级别、日志内容、脚本名称、调用日志打印函数的函数、打印时的行号及脚本的进程号
    local logTime=
    logTime="$(date -d today +'%Y-%m-%d %H:%M:%S')"
    printf "[%s] $*($$)\n" "${logTime}">> "${g_logFile}" 2>&1

    return 0
}

log_info()
{
    log "INFO:" "$@"
}

log_error()
{
    log "ERROR:" "$@"
}
##############################################################
## @Usage main
## @Input
##            $1        # 补丁包下载路径
##            $2        # 大数据安装目录
## @Return
## @Description 解压补丁包
##############################################################
main()
{
    local files="$@"

    local srcDir="${BIGDATA_HOME}/packages/patch/"
    local dstDir="${BIGDATA_HOME}/patch/install/"
    
    init_log
    
    log "-------------------start-----------------------------"
    
    if [ -z "$files" ]; then
        log_error "File list is empty."
        return 1
    fi
    
    if ! [ -d "${BIGDATA_HOME}/packages" ]; then
        log_error "${BIGDATA_HOME}/packages is not dir."
        return 1
    fi
    
    log_info "files : ${files}"
    log_info "srcDir : ${srcDir}"
    
    #创建补丁包最终的解压目录
    if [ ! -d "${dstDir}" ]; then
       if ! mkdir -p "${dstDir}"; then
          log_error "mkdir ${dstDir} failed."
          return 1
       fi
    fi
    
    for file in ${files}
    do
        local patchDir=${file%.tar.gz}
        if [ ! -f "${srcDir}/${file}" ]; then
            log_error "${srcDir}/${file} does not exit."
            return 1
        fi
        
        if [ -f "${dstDir}/${patchDir}/decompression.succeed" ]; then
            if [ -f "${srcDir}/${file}.sha256" ]; then
                newFileSha256=$(awk '{print $1}' "${srcDir}/${file}.sha256")
                oldFileSha256=$(awk '{print $1}' "${dstDir}/${patchDir}/decompression.succeed")
                if [ "$newFileSha256" = "$oldFileSha256" ]; then
                    continue;
                fi 
            fi
        fi
        
        if [ -d "${dstDir}/${patchDir}" ];then
            if ! rm -rf "${dstDir:?}/${patchDir}"; then 
                log_error "rm -rf ${dstDir}/${patchDir} failed."
                return 1
            fi
            log_info "delete ${dstDir}/${patchDir} successfully."
        fi
        
        if ! tar -pxzf "${srcDir}/${file}" -C "${dstDir}"; then
            log_error "Failed to decompress the file ${srcDir}/${file} to ${dstDir}."
            return 1
        fi
        if [ -f "${dstDir}/${patchDir}/slim_restore_patch.sh" ]; then
            if ! /bin/bash "${dstDir}/${patchDir}/slim_restore_patch.sh"; then
                log_error "slim restore patch ${dstDir} failed."
                return 1
            fi
        fi
        #解压后同一个修改权限为700
        chmod -R 700 "${dstDir}/${patchDir}"
        
        #生成补丁解压成功标志
        rsync -ar --delete "${srcDir}/${file}.sha256" "${dstDir}/${patchDir}/decompression.succeed"
        
        log_info "decompress the file ${srcDir}/${file} to ${dstDir} successfully."
    done
    return 0
}

source ~/.profile
# --------------------------------------------------------------------------- #
#                          脚本开始运行
# --------------------------------------------------------------------------- #

main "$@"
exit $?
