#!/bin/bash

# $1: 是否当前为升级场景
# $2：nodeagent安装目录
# $3: 日志输出目录
# $4：om浮动IP
# $5：NTP OPTIONS
# $6: NTP OPTIONS File
# $7: INSTALL_SUDO_PATH sudo脚本所在路径
# $8: controller安装目录
# $9: timestamp目录
# $10: STOP_NTP_CMD
#
# Note: root用户下安装执行，omm用户安装，需要依赖preset时的提权操作
init_params()
{
    g_if_upgrade_operation="${1}"
    g_nodeagent_home="${2}"
    g_logFile_path="${3}"
    g_omFloatIP="${4}"
    g_ntp_options="${5}"
    g_ntp_options_file="${6}"
    g_sudo_path="${7}"
    g_controller_home="${8}"
    g_timestamp_path="${9}"
    g_stop_ntp_cmd="${10}"
    
    g_omAgentWorkspace="$(dirname ${g_nodeagent_home})"
    
    if [ -z "${g_if_upgrade_operation}" ]; then
        g_if_upgrade_operation=0
    fi

    if [ -z "${g_logFile_path}" ]; then
        g_logFile_path="${BIGDATA_LOG_HOME}/nodeagent/scriptlog/"
    fi
    #检查chrony包是否存在
    rpm -qa | grep chrony >> /dev/null 2>&1
    if [ $? -eq 0 ]; then
        g_chrony_exist="yes"
    fi

    g_logFile="${g_logFile_path}/installntp.log"
}

check_params()
{
    if [ -z "${g_nodeagent_home}" ]; then
        Log "${LINENO}" "ERROR" "g_nodeagent_home is null."
        return 1
    fi
    
    if [ -z "${g_omFloatIP}" ]; then
        Log "${LINENO}" "ERROR" "g_omFloatIP is null."
        return 1
    fi
    
    if [ -z "${g_ntp_options}" ]; then
        Log "${LINENO}" "ERROR" "g_ntp_options is null."
        return 1
    fi
    
    if [ -z "${g_ntp_options_file}" ]; then
        Log "${LINENO}" "ERROR" "g_ntp_options_file is null."
        return 1
    fi
    
    if [ -z "${g_sudo_path}" ]; then
        Log "${LINENO}" "ERROR" "g_sudo_path is null."
        return 1
    fi
    
    if [ -z "${g_controller_home}" ]; then
        Log "${LINENO}" "ERROR" "g_controller_home is null."
        return 1
    fi
    
    if [ -z "${g_timestamp_path}" ]; then
        Log "${LINENO}" "ERROR" "g_timestamp_path is null."
        return 1
    fi
    
    if [ -z "${g_stop_ntp_cmd}" ]; then
        Log "${LINENO}" "ERROR" "g_stop_ntp_cmd is null."
        return 1
    fi
    
    return 0
}

make_log_directory()
{  
    if [ ! -d "${g_logFile_path}" ]; then
        mkdir -p "${g_logFile_path}"
    fi
}

Log()
{  
    printf "`date -d today +\"%Y-%m-%d %H:%M:%S\"`,000 $*\n" >> "${g_logFile}" 2>/dev/null
    
    logFileSize=$(du -sm "${g_logFile}" | awk '{print $1}')
    if [ "${logFileSize}" -ge ${MAX_LOG_SIZE} ]; then
        mv ${g_logFile} ${g_logFile}_bak
        chown omm:wheel "${g_logFile}_bak"
        chmod 600 "${g_logFile}_bak"
        
        return 0
    fi
    
    chown omm:wheel "${g_logFile}"
    chmod 600 "${g_logFile}"
    
    return 0
}

cron_ntp_checker()
{
    #需要先强制同步时间，然后再配置crontab文件，否则ntp服务会被拉起来
    #如果是升级，刷新重启时间戳，防止强制同步时间失败（升级会启停HA/ntp等服务）
    if [ "${g_if_upgrade_operation}" -eq 1 ]; then
        if [ $(id -u) -eq 0 ] ; then 
            su - omm -c "mkdir -p ${g_timestamp_path}/timestamp" > /dev/null 2>&1
        else
            mkdir -p ${g_timestamp_path}/timestamp > /dev/null 2>&1 
        fi

        if [ $(id -u) -eq 0 ] ; then 
            su - omm -c "date -d \"\$(LANG=en_US.UTF-8 && who -b | sed 's/system boot//g')\" +%s" > ${g_timestamp_path}/timestamp/restart_stamp
        else
            date -d "$(LANG=en_US.UTF-8 && who -b | sed 's/system boot//g')" +%s > ${g_timestamp_path}/timestamp/restart_stamp
        fi

        sudo ${g_sudo_path} chowntimestamp
    fi
    
    # 如果不是OMS节点，停止NTP服务，补偿时间。升级场景也会执行initnode.sh安装agent，此时升级不能强行同步时间
    if [ "${g_if_upgrade_operation}" -eq 0 ]; then            
        sudo ${g_sudo_path} initnode_ntp_conf "${g_stop_ntp_cmd}" "${g_omFloatIP}"
    fi
	
	sudo ${g_sudo_path} m_clear_ntp_config > /dev/null 2>&1
	sudo ${g_sudo_path} kill_ntp_process > /dev/null 2>&1
	
    local ntpChecker="${g_nodeagent_home}/bin/ntpChecker.sh"
    local mail="MAILTO"
    # 重置crontab文件及权限
    sudo ${g_sudo_path} m_add_crontab "${ntpChecker}" "${mail}" "${g_logFile}"
    if [ $? -ne 0 ] ; then
        Log "${LINENO}" "ERROR" "sudo ${g_sudo_path} m_add_crontab failed。"
        return 1
    fi
 
    #重启crontab服务
    sudo ${g_sudo_path} m_restart_cron >> "${g_logFile}" 2>&1
    
    #开启NTP监控
    if [ $(id -u) -eq 0 ] ; then
        su - omm -c "sh ${ntpChecker}"
    else
        sh ${ntpChecker}
    fi

    return 0
}

function backup_ntp_conf(){
    Log "${LINENO}" "INFO" "Begin to backup ntp config."
    
    local backup_dir="${g_omAgentWorkspace}/configbackup"
    local ntp_sys_conf="/etc/ntp.conf"
    local ntp_backup_conf="${backup_dir}/ntp.conf"
    
    if [ ! -d $backup_dir ] ; then 
        mkdir -p $backup_dir;
        chmod 750 "$backup_dir"
        chown omm:wheel "$backup_dir"
    fi
    
    local ntp_sys_keys=""
    local ntp_backup_keys=""

    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then        
        ntp_sys_keys="/etc/ntp.keys"
        ntp_backup_keys="$backup_dir/ntp.keys"
    else
        ntp_sys_keys="/etc/ntp/ntpkeys"
        ntp_backup_keys="$backup_dir/ntpkeys"
    fi
    
    if [ -f $ntp_sys_conf ] ; then 
        sudo ${g_sudo_path} backup_ntp_sys_conf "${ntp_sys_conf}" "${ntp_backup_conf}" >>/dev/null 2>&1   
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" "ERROR" "Failed to sudo backup_ntp_sys_conf ${ntp_sys_conf}  ${ntp_backup_conf}."
            return 1
        fi 
    fi
     
    if [ -f $ntp_sys_keys ] ; then 
        sudo ${g_sudo_path} backup_ntp_sys_conf "${ntp_sys_keys}" "${ntp_backup_keys}" >>/dev/null 2>&1
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" 'ERROR' "Failed to sudo backup_ntp_sys_conf ${ntp_sys_keys}  ${ntp_backup_keys}."
            return 1
        fi
    fi    
    Log "${LINENO}" "INFO" "End to backup ntp config." 
    return 0
}

function backup_chrony_conf(){
    Log "${LINENO}" "INFO" "Begin to backup chrony config."
    
    local backup_dir="${g_omAgentWorkspace}/configbackup"
    local ntp_sys_conf="/etc/chrony.conf"
    local ntp_backup_conf="${backup_dir}/chrony.conf"
    
    if [ ! -d $backup_dir ] ; then 
        mkdir -p $backup_dir;
        chmod 750 "$backup_dir"
        chown omm:wheel "$backup_dir"
    fi
    
    local ntp_sys_keys=""
    local ntp_backup_keys=""

    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then        
        ntp_sys_keys="/etc/chrony.keys"
        ntp_backup_keys="$backup_dir/chrony.keys"
    else
        ntp_sys_keys="/etc/chrony.keys"
        ntp_backup_keys="$backup_dir/chrony.keys"
    fi
    
    if [ -f $ntp_sys_conf ] ; then 
        sudo ${g_sudo_path} backup_chrony_sys_conf "${ntp_sys_conf}" "${ntp_backup_conf}" >>/dev/null 2>&1   
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" "ERROR" "Failed to sudo backup_chrony_sys_conf ${ntp_sys_conf}  ${ntp_backup_conf}."
            return 1
        fi 
    fi
     
    if [ -f $ntp_sys_keys ] ; then 
        sudo ${g_sudo_path} backup_chrony_sys_conf "${ntp_sys_keys}" "${ntp_backup_keys}" >>/dev/null 2>&1
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" 'ERROR' "Failed to sudo backup_chrony_sys_conf ${ntp_sys_keys}  ${ntp_backup_keys}."
            return 1
        fi
    fi    
    Log "${LINENO}" "INFO" "End to backup chrony config." 
    return 0
}

is_suse12_sp0()
{
    local version=0
    local patch_level=0
    
    version=$(cat /etc/*release|grep "VERSION[[:space:]]\{1,\}" | awk -F '=' '{print $2}' | tr -d "[[:space:]]")
    patch_level=$(cat /etc/*release|grep "PATCHLEVEL" | awk -F '=' '{print $2}'| tr -d "[[:space:]]")
    
    if [[ ${version} -eq 12 ]];then
        if [ ${patch_level} -eq 0 ];then
            return 0
        fi
    fi    
    
    return 1
}

is_rh7xVersion()
{
    local rh_os_type=$(cat /etc/*release | tr [A-Z] [a-z]  | grep "redhat")
    local euler_os=$(cat /etc/*release | tr [A-Z] [a-z]  | grep "euler")

    if [ ! -z "${rh_os_type}" -o ! -z "${euler_os}" ]; then
        Log "${LINENO}" "INFO" "OS is redhat7.X or centOS7.x or Euler2.X version,need to adapte."
        return 0
    fi

    return 1
}

main()
{
    init_params "$@"
    
    make_log_directory
    
    Log "${LINENO}" "INFO" "Begin to install ntp..."
    
    check_params
    if [ $? -ne 0 ]; then
        Log "${LINENO}" "ERROR" "check params failed."
        return 1
    fi
    
    is_suse12_sp0
    if [ $? -eq 0 ] ; then
        Log "${LINENO}" "INFO" "Suse 12.0, need to adjust ntp.pid config in apparmor."
        sudo ${g_sudo_path} adjust_ntp_apparmor_config  >> "${g_logFile}" 2>&1
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" "ERROR" "Failed to adjust ntp apparmor config."
            return 1
        fi
    fi
    
    local ntpFile="${g_nodeagent_home}/ntp.conf.agent"
    
    sudo ${g_sudo_path} agent_config_ntp "${ntpFile}" "${g_omFloatIP}"
    if [ $? -ne 0 ]; then
        Log "${LINENO}" "ERROR" "generate ntp.conf.agent failed."
        return 1
    fi
    
    if [ $g_chrony_exist = 'yes' ]; then
        local chronyFile="${g_nodeagent_home}/chrony.conf.agent"
    sudo ${g_sudo_path} agent_config_chrony "${chronyFile}" "${g_omFloatIP}"
    if [ $? -ne 0 ]; then
        Log "${LINENO}" "ERROR" "generate ntp.conf.agent failed."
        return 1
    fi
    fi
    
    sudo ${g_sudo_path} m_update_drift
    
    #如果已经安装OMS，则NTP已经完成配置，不用再配置
    if [ ! -f "${g_controller_home}/omsetup.completed" ]; then
      
        backup_ntp_conf
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" "ERROR" "Failed to backup ntp config."
            return 1
        fi
        
        if [ $g_chrony_exist = 'yes' ]; then
            backup_chrony_conf
            if [ $? -ne 0 ] ; then
                Log "${LINENO}" "ERROR" "Failed to backup ntp config."
                return 1
            fi
        fi

        sudo ${g_sudo_path} m_replace_ntp_conf "${ntpFile}"
        if [ $? -ne 0 ]; then
            Log "${LINENO}" "ERROR" "cp ${ntpFile} /etc/ntp.conf failed."
            return 1
        fi
        
        if [ $g_chrony_exist = 'yes' ]; then
            sudo ${g_sudo_path} m_replace_chrony_conf "${chronyFile}"
            if [ $? -ne 0 ]; then
                Log "${LINENO}" "ERROR" "cp ${chronyFile} /etc/chrony.conf failed."
                return 1
            fi
        fi
        
        sudo ${g_sudo_path} m_modify_ntp_file ${g_ntp_options} ${g_ntp_options_file} ${g_logFile}
        if [ $? -ne 0 ] ; then
            Log "${LINENO}" "ERROR" "Failed to modify ntp file."
            return 1
        fi

        is_rh7xVersion
        if [ $? -eq 0 ]; then
            Log "${LINENO}" "INFO" "OS is redhat7.X or centOS7.X,need to preset ntpd.service system file in case that ntp updated."
            sudo ${g_sudo_path} ntpdService_file_preset >> "${g_logFile}" 2>&1
            ret_ntpdService_file_preset=$?
            case ${ret_ntpdService_file_preset} in 
                0)
                    Log "${LINENO}" "INFO" "Succeed to set ntpd.service."
                    ;;
                1)
                    Log "${LINENO}" "ERROR" "/usr/lib/systemd/system/ntpd.service is softlink,or method ntpdService_file_preset does not in sudo."
                    return 1
                    ;;
                2)
                    Log "${LINENO}" "ERROR" "Failed to echo Service realm to ntpd.service,Service does not exist in ntpd.service."
                    return 1
                    ;;
                3)
                    Log "${LINENO}" "ERROR" "Failed to add ExecStart config to ntpd.service."
                    return 1
                    ;;
                4)
                    Log "${LINENO}" "ERROR" "Failed to set correct ExecStart to ntpd.service."
                    return 1
                    ;;
                10)
                    Log "${LINENO}" "ERROR" "ntpd is not installed."
                    return 1
                    ;;
                *)
                    Log "${LINENO}" "ERROR" "Failed to set ntpd.service system file."
                    return 1
                    ;;
            esac
        fi
    fi

    # 把ntpChecker.sh加到定时任务，让其定时执行
    cron_ntp_checker
    if [ $? -ne 0 ] ; then
        Log "${LINENO}" "ERROR" "Failed to add ntp checker to crontab."
        return 1
    fi
    
    Log "${LINENO}" "INFO" "install ntp successfully."
    return 0
}


declare MAX_LOG_SIZE=10

main "$@"
exit $?