#!/bin/bash

#每次补丁需修改版BACKUP_HOME本号
declare PATCH_VERSION="6.5.1.6"
declare BACKUP_HOME="/opt/os_optimization_tool_backup_${PATCH_VERSION}"

###################################################
#
# 获取绝对路径
#
###################################################
get_path()
{
    local path=""

    pushd $(dirname "$0") > /dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi

    CURRENT_DIR="${path}"
    PARENT_PATH=$(dirname "${path}")
    echo "PARENT_PATH=$PARENT_PATH."
    if [ -z "$PARENT_PATH" ]; then
        echo "Init parameter PARENT_PATH failed."
        return 1
    fi

    return 0
}

init_env()
{
    #引入日志处理函数
    get_path
    source "${PARENT_PATH}/common/log.sh"
    
    #引入omm用户环境变量
    if [ -f "/home/omm/.profile" ] ;then
        source "/home/omm/.profile"
    else
        log_and_show "ERROR" "${LINENO}" "no /home/omm/.profile, source omm env failed"
        return 1
    fi
    
    return 0
}

main()
{
    init_env
    if [ $? -ne 0 ];then
        log_and_show "ERROR" "${LINENO}" "init env failed"
        return 1
    fi 
    
    #replace sudo/install scripts
    /bin/bash "${PARENT_PATH}"/common/undo_replace_sudoExecute.sh
    if [ $? -ne 0 ];then
        return 1
    fi
    
    #replace diskmgt/ini-plugin/notify/datanode_notify.sh
    /bin/bash "${PARENT_PATH}"/common/undo_replace_datanode_notify.sh
    if [ $? -ne 0 ];then
        return 1
    fi
    
    #undo add path of updateElectronicTag.sh to osFile 
    /bin/bash "${PARENT_PATH}"/common/undo_add_updateElectronicTag_to_os.sh
    if [ $? -ne 0 ];then
        log_and_show "WARNING" "${LINENO}" "undo add updateElectronicTag to os failed."
    fi
    
    #卸载成功，则删除备份文件
    rm -fr "${BACKUP_HOME}"
    if [ $? -ne 0 ];then
       log_and_show "WARNING" "${LINENO}" "remove $BACKUP_HOME failed."
    fi
    
    return 0
}

#脚本解释器 强制设置为 /bin/bash
if [ "$BASH" != "/bin/bash" ]; then
   /bin/bash $0 "$@"
   exit $?
fi


main
exit $?