#!/bin/bash
TARGET_PATH="${HOME}"
LOCAL_SCRIPT=$(dirname "$0")/scriptFI/
PARSEPATH="${LOCAL_SCRIPT}"/function/JSON.sh
LOCAL_HOST_OPERATION_PATH="${LOCAL_SCRIPT}"/function/checkEnvironmentOnEachHost.sh
TARGET_HOST_OPERATION_PATH="${TARGET_PATH}"/scriptFI/function/checkEnvironmentOnEachHost.sh
ANALYSIS="${LOCAL_SCRIPT}"/function/analyse.py
local_ip=127.0.0.1

#创建precheck日志目录
LOG_PATH=$(cd "$(dirname "$0")";pwd)/log
LOG_FILE="${LOG_PATH}/precheck.log"
VER_FILE="${LOG_PATH}/ver.log"
FAILED_FILE="${LOG_PATH}/precheck_failed.log"
MISSING_RPM_FILE="${LOG_PATH}/missing_rpms.lst"

mkdir -p "${LOG_PATH}"
touch "${LOG_FILE}"
echo "" > "${LOG_FILE}"
chmod 600 "${LOG_FILE}"

touch "${FAILED_FILE}"
echo "" > "${FAILED_FILE}"
chmod 600 "${FAILED_FILE}"

touch "${VER_FILE}"
echo "" > "${VER_FILE}"
chmod 600 "${VER_FILE}"

touch "${MISSING_RPM_FILE}"
echo "" > "${MISSING_RPM_FILE}"
chmod 600 "${MISSING_RPM_FILE}"

SCRIPT_NAME="$(basename "${BASH_SOURCE-$0}")"

SCRIPT_DIR="$(cd "$(dirname "$0")";pwd)"
PARSE_CFG_TMP_RESULT="${SCRIPT_DIR}/parse_json_config.result.temp"
MANAGER_IP_LIST="${SCRIPT_DIR}/scriptFI/modules/OMSServer/manager_IPAddr.lst"
BUSINESS_IP_LIST="${SCRIPT_DIR}/scriptFI/modules/OMSServer/business_IPAddr.lst"
PRECHECK_CONSTANTS="${SCRIPT_DIR}/scriptFI/precheck_constants"

function LOG()
{
    # 打印的日志级别
    local logLevel="$1"
    shift

    # 外部调用日志打印函数时所在的行号
    local lineNO="$1"
    shift

    # 打印时间、日志级别、日志内容、脚本名称、调用日志打印函数的函数、打印时的行号及脚本的进程号
    local logTime="$(date -d today +'%Y-%m-%d %H:%M:%S')"
    
    # 调用日志打印函数的函数名称
    local funcName="${FUNCNAME[1]}"
    
    echo "[${logTime}] ${logLevel} $* [${SCRIPT_NAME}(${funcName}):${lineNO}]($$)" | tee -a "$LOG_FILE"

    return 0
}

#-----------------------------------------------------------------------------------------------------------------
#fun : 记录审计日志
#-----------------------------------------------------------------------------------------------------------------
function syslog()
{
    local component=$1
    local filename=$2
    local status=$3
    local message=$4

    which logger >/dev/null 2>&1
    [ "$?" -ne "0" ] && return 0;

    login_user_ip="$(who -m | sed 's/.*(//g;s/)*$//g')"
    execute_user_name="$(whoami)"
    logger -p local0.notice -i "FusionInsight;${component};[${filename}];${status};${login_user_ip};${execute_user_name};${message}"
    return 0
}


#-----------------------------------------------------------------------------------------------------------------
#fun : 获取配置文件的解析结果
#param : 配置文件名
#return : 配置文件的解析结果
#-----------------------------------------------------------------------------------------------------------------
function parseConfigFile() {
    LOG "[INFO]" "${LINENO}" "localhost: start parsing the configuration file."

    rm -rf "${PARSE_CFG_TMP_RESULT}"
    "${PARSEPATH}" -l < "${CONFIG_FILE_PATH}" > "${PARSE_CFG_TMP_RESULT}" 
    #判断是否解析成功
    if [ $? != 0 ]
    then
        LOG "[ERROR]" "${LINENO}" "localhost: fail to parse configuaration file, check the file name or the content of config file."
        LOG "[ERROR]" "${LINENO}" "localhost: quit the programme."
        exit 1
    else
        LOG "[INFO]" "${LINENO}" "localhost: parse the configuration file success."
    fi
}


#-----------------------------------------------------------------------------------------------------------------
#func : 针对每一个主机的配置信息进行分割
#param : 无
#return : 每个主机的一段配置信息，数组形式返回；并生成管理平面和业务平面的IP列表
#-----------------------------------------------------------------------------------------------------------------
function getConfigForEachHost() {
    LOG "[INFO]" "${LINENO}" "localhost: start divide the global config file to get configurations for each host."

    #记录ip信息所在行，并记录行号和对应的ip，以方便进行分割
    unset ipLineNumArray
    local ipLineNum

    local count=0
    for ipLineNum in $(cat "${PARSE_CFG_TMP_RESULT}" | awk '/"nodes",[[:digit:]]+,"node","ip"/{print NR}')
    do
        ipLineNumArray["${count}"]=$ipLineNum
        count=$(expr $count + 1)
    done

    #根据每个ip对应的行号进行分割，分别获取每个主机的配置信息
    for i in "${!ipLineNumArray[@]}"
    do
        #判断是否是最后一个行号
        if [ "${i}" != $(( ${#ipLineNumArray[@]} - 1 )) ]
        then
            hostConfigArray["${i}"]=$(cat "${PARSE_CFG_TMP_RESULT}" | sed -n ${ipLineNumArray[${i}]},$(( ${ipLineNumArray[ ((${i} + 1)) ]} - 1 ))p)
        else
            hostConfigArray["${i}"]=$(cat "${PARSE_CFG_TMP_RESULT}" | sed -n "${ipLineNumArray[${i}]}"',$p')
        fi
    done

    LOG "[INFO]" "${LINENO}" "localhost: divide the global config file for each host success."

    #获取配置文件中的管理IP列表和业务IP列表，用于检查网络平面是否互通
    local manager_ip=""
    local service_ip=""
    rm -rf "$MANAGER_IP_LIST"
    rm -rf "$BUSINESS_IP_LIST"
    for i in "${!hostConfigArray[@]}"
    do
        manager_ip=""
        service_ip=""
        manager_ip=$(echo "${hostConfigArray[${i}]}" | egrep '"nodes",[[:digit:]]+,"node","ip"' | cut -d '"' -f 8)
        service_ip=$(echo "${hostConfigArray[${i}]}" | egrep '"nodes",[[:digit:]]+,"node","serviceip"' | cut -d '"' -f 8)
        
        #判断ip是否为空
        if [ -z "${manager_ip}" ]
        then
            LOG "[ERROR] the ip:${ip} of host ${i} is empty."
            LOG "[ERROR] stop checking on this host."
            continue
        fi

        echo "$manager_ip" >> "$MANAGER_IP_LIST"
        echo "$service_ip" >> "$BUSINESS_IP_LIST"
    done
}

function create_precheck_constants_file()
{
    LOG "[INFO]" "${LINENO}" "localhost: start create precheck constants file."
    rm -rf "${PRECHECK_CONSTANTS}"
    #从配置文件中获取到安装路径的值。
    g_install_home=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_install_home" | cut -d '"' -f 4)

    if [ -z "${g_install_home}" ]
    then
        LOG "[ERROR] install home is null"
        return 0
    fi
    
    #将安装目录的变量写入文件
    echo "g_install_home=${g_install_home}" >> "${PRECHECK_CONSTANTS}"
    
    #从配置文件中获取到处理器类型的值。
    g_platform=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_platform" | cut -d '"' -f 4)
    if [ -z "${g_platform}" ]
    then
        LOG "[ERROR] g_platform is null"
        return 0
    fi
    #将处理器类型的变量写入文件
    echo "g_platform=${g_platform}" >> "${PRECHECK_CONSTANTS}"
}

function auto_scp()
{
    ip=$1
    username=$2
    password=$3
    srcpath=$4
    dstpath=$5
    expect -c "set timeout 180;
                spawn -noecho scp -r ${srcpath} ${username}@${ip}:${dstpath} ;
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                         timeout { exit 1 };
                         FATAL {exit 2 };
                         \"*Connection refused\" { exit 3 };
                         \"*Permission denied*\" { exit 4 };
                         \"*Host key verification failed*\" { exit 5 };
                         \"*Illegal host key*\" { exit 6 };  
                         \"*No route to host\" { exit 7 };  
                         \"Received disconnect*\" { exit 9 };  
                         \"lost connection\" { exit 10 };
                       }
                expect timeout { exit 8 };
                set my_ret [wait];
                exit [lindex \$my_ret 3];"
}

function auto_reverse_scp()
{
    ip=$1
    username=$2
    password=$3
    srcpath=$4
    dstpath=$5
    expect -c "set timeout 180;
                spawn -noecho scp -r ${username}@${ip}:${srcpath} ${dstpath};
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                         timeout { exit 1 };
                         FATAL {exit 2 };
                         \"*Connection refused\" { exit 3 };
                         \"*Permission denied*\" { exit 4 };
                         \"*Host key verification failed*\" { exit 5 };
                         \"*Illegal host key*\" { exit 6 };  
                         \"*No route to host\" { exit 7 };  
                         \"Received disconnect*\" { exit 9 };  
                         \"lost connection\" { exit 10 };
                       }
                expect timeout { exit 8 };
                set my_ret [wait];
                exit [lindex \$my_ret 3];"
}

function auto_scp_IPv6()
{
    ip=$1
    username=$2
    password=$3
    srcpath=$4
    dstpath=$5
    expect -c "set timeout 180;
                spawn -noecho scp -r ${srcpath} ${username}@\[${ip}\]:${dstpath} ;
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                         timeout { exit 1 };
                         FATAL {exit 2 };
                         \"*Connection refused\" { exit 3 };
                         \"*Permission denied*\" { exit 4 };
                         \"*Host key verification failed*\" { exit 5 };
                         \"*Illegal host key*\" { exit 6 };
                         \"*No route to host\" { exit 7 };
                         \"Received disconnect*\" { exit 9 };
                         \"lost connection\" { exit 10 };
                       }
                expect timeout { exit 8 };
                set my_ret [wait];
                exit [lindex \$my_ret 3];"
}

function auto_reverse_scp_IPv6()
{
    ip=$1
    username=$2
    password=$3
    srcpath=$4
    dstpath=$5
    expect -c "set timeout 180;
                spawn -noecho scp -r ${username}@\[${ip}\]:${srcpath} ${dstpath};
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                         timeout { exit 1 };
                         FATAL {exit 2 };
                         \"*Connection refused\" { exit 3 };
                         \"*Permission denied*\" { exit 4 };
                         \"*Host key verification failed*\" { exit 5 };
                         \"*Illegal host key*\" { exit 6 };
                         \"*No route to host\" { exit 7 };
                         \"Received disconnect*\" { exit 9 };
                         \"lost connection\" { exit 10 };
                       }
                expect timeout { exit 8 };
                set my_ret [wait];
                exit [lindex \$my_ret 3];"
}

function execute_shell()
{
    ip=$1
    username=$2
    password=$3
    command=$4
    params=$5
    ommid=$6


    expect -c "set timeout 900;
                spawn -noecho ssh ${username}@${ip} $command {'`sed "s/'/'\\\\\\\\''/g" <<< \"$params\"`' ${ommid}};
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                       }
                expect timeout { exit 8 }
                set my_ret [wait];
                exit [lindex \$my_ret 3];" >> "$LOG_PATH/${ip}.result"
}

function getuserhomeprofile()
{
    ip=$1
    username=$2
    password=$3
    
    expect -c "set timeout 180;
                spawn -noecho ssh ${username}@${ip} cat /etc/passwd|grep ^"${username}:"|cut -d ':' -f 6;
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                         timeout { exit 1 };
                         FATAL {exit 2 };
                         \"*Connection refused\" { exit 3 };
                         \"*Permission denied*\" { exit 4 };
                         \"*Host key verification failed*\" { exit 5 };
                         \"*Illegal host key*\" { exit 6 };  
                         \"*No route to host\" { exit 7 };  
                         \"Received disconnect*\" { exit 9 };  
                         \"lost connection\" { exit 10 };
                       }
                expect timeout { exit 8 };
                set my_ret [wait];
                exit [lindex \$my_ret 3];"   
}


# 不传入配置文件参数，只对本机环境进行检查
localHostCheck()
{
    LOG "[INFO]" "${LINENO}" "${local_ip}: start checking local host, please wait several minutes..." 
    . ${LOCAL_SCRIPT}"/function/sharefuncs.sh"
    for checkItem in $(find "${LOCAL_SCRIPT}"/modules/ -name "check.sh" | sort )
    do
        exec_cmd "${checkItem}" "${local_ip}" "${g_ommUserid}"
    done
    
    mv -f "${LOCAL_SCRIPT}"/rpm_lists/*_missing-rpms.lst ${MISSING_RPM_FILE}
    if [ $? -ne 0 ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: failed to mv missing rpms flie to ${MISSING_RPMS_DIR}."
    fi
}


exec_cmd()
{
    local -i retVal=0

    # 第一个参数为系统命令名，后面的参数为该命令参数
    local    cmd="$1"
    
    # 参数移位，第一个参数丢失
    shift
    
    # $@加上双引号，可以让每个参数都加上双引号，防止参数字符串有空格
    "${cmd}" "$@" >> "$LOG_FILE"
    retVal=$?
    
    # 返回码255代表执行/scripts/om/check.sh中的检查os类型或kernel类型失败，该两项如果失败，整个机器上的检查都停止
    if [ "${retVal}" -eq 255 ] ; then
        echo "[ERROR] Stop other checks on host ${ip}and restore os/kernel first."
        # 梳理precheck结果
        checkResult
        exit 1
    fi

    return 0
}

function CheckOneHost()
{
    ip=$1
    g_user_name=$2
    password=$3
    params=$4
   
    result=$(getuserhomeprofile "${ip}" "${g_user_name}" "${password}" 2> /dev/null)
    if [ $? -eq 0 ] && [ -n "${result}" ];then
        USERHOME=$(echo "${result}"|grep ^"/"|tr -d '\r') 
        if [ -n "${USERHOME}" ];then
            TARGET_PATH="${USERHOME}"
            TARGET_HOST_OPERATION_PATH="${TARGET_PATH}"/scriptFI/function/checkEnvironmentOnEachHost.sh
        fi
    fi

    LOG "[INFO]" "${LINENO}" "localhost: start transferring scripts to host ${ip}${TARGET_PATH}."
    if [ ${g_ip_model} == "IPv4" ]; then
        auto_scp "${ip}" "${g_user_name}" "${password}" "${LOCAL_SCRIPT}" "${TARGET_PATH}" > /dev/null 2>&1
    else
        auto_scp_IPv6 "${ip}" "${g_user_name}" "${password}" "${LOCAL_SCRIPT}" "${TARGET_PATH}" > /dev/null 2>&1
    fi
    result=$?
    if [ $result -eq 0 ]
    then
        LOG "[INFO]" "${LINENO}" "localhost: transfer success, start checking the host ${ip}"
    
        execute_shell "${ip}" "${g_user_name}" "${password}" "${TARGET_HOST_OPERATION_PATH}" "$params" "${g_ommUserid}" > /dev/null 2>&1
        retVal=$?
        
        if [ "${retVal}" -eq 0 ]
        then
            LOG "[INFO]" "${LINENO}" "localhost: finish checking the host ${ip}."
        elif [ "${retVal}" -eq 255 ]
        then
            LOG "[ERROR]" "${LINENO}" "localhost: Stop other checks on host ${ip} and restore os/kernel first."
        else
            LOG "[ERROR]" "${LINENO}" "localhost: Failed to check host ${ip}. Execute scripts error."
        fi
        
        #把各节点缺少的rpm包列表文件拷贝到当前节点
        local missing_rpms_file="${LOCAL_SCRIPT}"/rpm_lists/*_missing-rpms.lst
        if [ ${g_ip_model} == "IPv4" ]; then
            auto_reverse_scp "${ip}" "${g_user_name}" "${password}" "${missing_rpms_file}" "${LOG_PATH}" > /dev/null 2>&1
        else
            auto_reverse_scp_IPv6 "${ip}" "${g_user_name}" "${password}" "${missing_rpms_file}" "${LOG_PATH}" > /dev/null 2>&1
        fi
        if [ $? -ne 0 ];then
            LOG "[ERROR]" "${LINENO}" "localhost: failed to scp missing rpms flie from ${ip}."
        else
            LOG "[INFO]" "${LINENO}" "localhost: finish copying missing rpms flie from ${ip}."
        fi
        
        execute_shell "${ip}" "${g_user_name}" "${password}" "rm -rf" "${TARGET_PATH}/scriptFI" "" > /dev/null 2>&1
        retVal=$?
        if [ "${retVal}" -ne 0 ]
        then
            LOG "[ERROR_DEBUG]" "${LINENO}" "Failed to clear the path ${TARGET_PATH}/scriptFI of ${ip}."
        fi
    else
        LOG "[ERROR]" "${LINENO}" "localhost: cannot transfer the operation scripts to host ${ip}."
        LOG "[ERROR]" "${LINENO}" "localhost: fail to check host ${ip} ${result}, possible reasons: (1) password error/ip address error. (2) failed to connect remote host."
    fi
}

function checkNtpServer()
{
    if [ -z "${OMS_INSTALL_FILE}" -o ! -f "${OMS_INSTALL_FILE}" ]; then
        LOG "[WARNING]" "${LINENO}" "localhost: OMS install file does not exist, so ntp server will not be verified."
        return 0
    fi

    ${LOCAL_SCRIPT}/function/checkNtpServer.sh ${OMS_INSTALL_FILE} ${MANAGER_IP_LIST} ${BUSINESS_IP_LIST} ${LOG_FILE} >> ${LOG_FILE} 2>&1
}

# 传入配置文件，对配置文件中主机环境进行检查
remoteHostsCheck()
{
    #主机配置信息数组
    declare -a hostConfigArray

    #存放配置文件的解析结果
    parseConfigFile $1

    #存放每个主机的配置信息,数组形式
    getConfigForEachHost
    
    #生成变量文件
    create_precheck_constants_file
    
    #判断配置文件中是否存在ip行，不存在就直接退出程序
    if [ "${#ipLineNumArray[@]}" == 0 ]
    then
        LOG "[INFO]" "${LINENO}" "localhost: no host config exist in config file, make sure the format of config file is correct."
        LOG "[INFO]" "${LINENO}" "localhost: quit the program."
        exit 1
    fi

    if [ -n "$g_passwd_aes_file" ]
    then
        password=`openssl enc -aes-256-cbc -salt -a -d -k "$g_aes_rand_key" -in "$g_passwd_aes_file" 2> /dev/null`
        if [ $? != 0 ]
        then
            LOG "[INFO]" "${LINENO}" "Can't get passwd from file"
            password=
        fi
    fi

    g_user_name=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_user_name" | cut -d '"' -f 4)
    if [ -z "${g_user_name}" ];then
        g_user_name='root'
    fi
    
    g_ntp_server=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_ntp_server" | cut -d '"' -f 4)
    g_float_ip=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_float_ip" | cut -d '"' -f 4)

    g_dbserver_ip=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_dbserver_ip" | cut -d '"' -f 4)
    g_omsserver_ip=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_omsserver_ip" | cut -d '"' -f 4)
    g_ip_model=$(cat "${PARSE_CFG_TMP_RESULT}" | grep "g_ip_model" | cut -d '"' -f 4)

    # g_ip_model 若为空，则默认为 IPv4
    if [ -z "$g_ip_model" ]; then
         g_ip_model="IPv4"
    fi
    
    #删除临时文件
    rm -rf "${PARSE_CFG_TMP_RESULT}"
    
    tmpIFS="$IFS"
    IFS=$'\n'
    #如果输入密码为空,无法区分,再输一边    
    if [ -z "${password}" ]
    then
        echo "Please enter the SSH password for ${g_user_name}: "
        stty -echo
        read -r password
        stty echo
    fi

    password=$(echo ${password} | sed 's/\\/\\&/g')
    password=$(echo ${password} | sed 's/'{'/\\&/g')
    password=$(echo ${password} | sed 's/\}/\\&/g')
    password=$(echo ${password} | sed 's/\[/\\&/g')
    password=$(echo ${password} | sed 's/\]/\\&/g')
    password=$(echo ${password} | sed 's/\$/\\&/g')
    password=$(echo ${password} | sed 's/\"/\\&/g')

    Nproc=200    # 可同时运行的最大作业数
    PID=() # 记录PID到数组, 检查PID是否存在以确定是否运行完毕    

    for i in "${!hostConfigArray[@]}"
    do
        ip=$(echo "${hostConfigArray[${i}]}" | egrep '"nodes",[[:digit:]]+,"node","ip"' | cut -d '"' -f 8)

        #判断ip是否为空
        if [ -z "${ip}" ]
        then
            LOG "[ERROR] the ip:${ip} of host ${i} is empty."
            LOG "[ERROR] stop checking on this host."
            continue
        fi
        
        #控制并发,检查PID是否存在以确定是否运行完毕,找到一个就退出 
        while true
        do
            for((Ijob=0; Ijob<Nproc; Ijob++))
            do
                if [[ ! "${PID[Ijob]}" ]] || ! kill -0 ${PID[Ijob]} 2> /dev/null
                then
                    break
                fi
            done
            
            if [[ "$Ijob" -lt "$Nproc" ]]
            then 
                break
            fi
            
            sleep 2
        done

        CheckOneHost "${ip}" "${g_user_name}" "${password}" "${hostConfigArray[${i}]}"&
        PID[Ijob]=$!
       
    done
    wait
    IFS="${tmpIFS}"

    # 收集脚本日志
    for log_file in `find ${LOG_PATH} -name *.result`; do
        cat ${log_file} >> ${LOG_FILE}
        rm -f ${log_file}
    done

    #收集rpm缺包列表
    for log_file in `find ${LOG_PATH} -name *missing-rpms.lst`; do
        cat ${log_file} >> ${MISSING_RPM_FILE}
        rm -f ${log_file}
    done

    #检查ntpserver的可用性
    ${LOCAL_SCRIPT}/function/checkNtpServer.sh "${g_ntp_server}" "${g_float_ip}" "${MANAGER_IP_LIST}" "${BUSINESS_IP_LIST}" "${LOG_FILE}" >> ${LOG_FILE} 2>&1
    #检查DBserviceIP是否在同一网段
    if [ -n "$g_dbserver_ip" ]; then
        DBserviceIP1=$(echo $g_dbserver_ip | awk -F ',' '{print $1}')
        DBserviceIP2=$(echo $g_dbserver_ip | awk -F ',' '{print $2}')
        ${LOCAL_SCRIPT}/function/checkIP.sh "${DBserviceIP1}" "${DBserviceIP2}" "${g_user_name}" "${password}" "${g_ip_model}" "${LOG_FILE}" >> ${LOG_FILE} 2>&1
        if [ $? -ne 0 ];then
            LOG "[ERROR]" "${LINENO}" "localhost: check DBserviceIP failed."
        else
            LOG "[INFO]" "${LINENO}" "localhost: finish check DBserviceIP."
        fi
    fi
    #检查omsserverIP是否在同一网段
    if [ -n "$g_omsserver_ip" ]; then
        omsserverIP1=$(echo $g_omsserver_ip | awk -F ',' '{print $1}')
        omsserverIP2=$(echo $g_omsserver_ip | awk -F ',' '{print $2}')
        ${LOCAL_SCRIPT}/function/checkIP.sh "${omsserverIP1}" "${omsserverIP2}" "${g_user_name}" "${password}" "${g_ip_model}" "${LOG_FILE}" >> ${LOG_FILE} 2>&1
        if [ $? -ne 0 ];then
            LOG "[ERROR]" "${LINENO}" "localhost: check omsserverIP failed."
        else
            LOG "[INFO]" "${LINENO}" "localhost: finish check omsserverIP."
        fi
    fi
}

#-----------------------------------------------------------------------------------------------------------------
#shell主体开始
#-----------------------------------------------------------------------------------------------------------------
main()
{
    syslog "precheck" "precheck.sh" "begin" "precheck.sh $* begin."
    
    CONFIG_FILE_PATH=$1
    
    local ret=0
    # 获取omm用户名id（工具独立发布，FusionInsight_SetupTool目录下不存在userid.ini文件，不再从该文件中读取）
    g_ommUserid=2000

    if [ -z $CONFIG_FILE_PATH ]; then
        LOG "[INFO] no precheck config file input, so just check the local host."
        localHostCheck
    else
        LOG "[INFO]" "${LINENO}" "start checking each hosts."
        remoteHostsCheck
    fi
    
    rm -rf ${MANAGER_IP_LIST}
    rm -rf ${BUSINESS_IP_LIST}
    rm -rf ${PRECHECK_CONSTANTS}

    # 梳理precheck结果
    checkResult
    ret=$?
    syslog "precheck" "precheck.sh" "exit=${ret}" "precheck.sh $* end."
    
    return $ret
}

checkResult()
{
    # 过滤错误结果
    echo ""
    echo ""
    echo "==========Summary Output=============================="
   
    grep "\[WARNING\]" "${LOG_FILE}" >> ${FAILED_FILE}
    grep "\[ERROR\]" "${LOG_FILE}" >> ${FAILED_FILE}
    grep "RPM_VER" "${LOG_FILE}" >> ${VER_FILE}
    python3 ${ANALYSIS} "${VER_FILE}" "${FAILED_FILE}"
    
    fail=$(cat ${FAILED_FILE}) 
    if [ -z "${fail}" ];then 
        echo "Environment check success."
    else
        echo "Environment check failed,you can get more information from ${FAILED_FILE}"
    fi
    
    
}

main "$@"
