#!/bin/bash

#导入公共日志函数
SCRIPT_NAME="$(basename "${BASH_SOURCE-$0}")"
LOG_FUNC_FILE="$(dirname "$0")"/../../function/log.sh
. "${LOG_FUNC_FILE}" || { echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [ERROR]: Failed to load ${LOG_FUNC_FILE}. [${SCRIPT_NAME}:${LINENO}]($$)"; exit 1;}

# ---------------------------------------------------------------------------- #
#                                   常量定义                                   #
# ---------------------------------------------------------------------------- #
declare -r MIN_IP_LOCAL_PORT_ID=32768          #OS随机端口范围的最小值，小于此值可能会与FI所使用端口冲突
declare -r MANAGER_IP_LIST="$(dirname "$0")"/manager_IPAddr.lst
declare -r BUSINESS_IP_LIST="$(dirname "$0")"/business_IPAddr.lst

main()
{
    ip=$1
    g_ommUserid=$2
    g_ommUsername="omm"
    hostName=$3
    serviceip=$4
    
    # 加载各种检查脚本
    sourceFile
    
    # 检查OS版本是否支持
    checkOS "${ip}" 
    if [ $? -ne 0 ]; then
       exit 255
    fi

    # 检查OS内核版本是否支持
    checkKernel "${ip}" 
    if [ $? -ne 0 ]; then
       exit 255
    fi
    
    # 检查SSH是否安装
    checkSSH "${ip}" 
    
    # 检查python版是否正确
    checkPython "${ip}" 
    
    # 检查缺少的rpm包
    checkRpms "${ip}" 
    
    # 检查是否存在expect命令
    checkExpect "${ip}" 
    
    # 检查/etc/hosts是否配置正确
    checkLocalHostConfigured "${ip}" "${hostName}" "${serviceip}"
    
    #检测hostname是否满足
    checkHostName "${ip}" 
    #检测IPv6是否开启
    checkipv6status "${ip}"
    # 检查OS最大共享内存值
    checkKernelShmmax "${ip}" 

    # 检查omm用户和id=2000用户是否已经存在
    currentUser=$(whoami)
    if [ "${currentUser}" != "${g_ommUsername}" ];then
        checkUserAndId "${ip}" "${g_ommUsername}" "${g_ommUserid}" 
    fi    
    # 检查swap分区是否存在
    checkSwap "${ip}" 
    
    # 检查是否安装了NTP服务
    checkNtp "${ip}"
    
    # 检查/etc/ntp.conf是否为空
    checkNtpConf "${ip}"
    
    # 检查lib64
    checkLib64 "${ip}"    
    
    # 检查NSS包的版本和配置
    checkNSS "${ip}"

    # 检查是否配置了DNS
    checkDns "${ip}"    
    
    #检查/etc/sudoers中是否配置Defaults requiretty
    checksudoers "${ip}"
     
    checkCrontabRight "${ip}"
    
    checkHadoopConfigured "${ip}"

    checkZoneReclaim "${ip}"
    
    #ipmitool 工具检查
    checkipmitool "${ip}"
    
    #iostat工具检查
    checkiostat "${ip}"

    #检查OS随机端口范围
    checkLocalPortRange "${ip}"
    
    #检查文件权限
    check_permission "${ip}"
    
    #检查挂载点
    check_mounted_attribute "${ip}"

    checkNetwork "${ip}"
    
    #检查处理器类型
    check_platform "${ip}"

    checkWheelGroupID "${ip}"

    checkCPUEnergyEfficiencyMode "${ip}"
}

sourceFile()
{
    for checkScripts in $(ls $(dirname "$0")/checkItems/ | grep '.sh')
    do
        source $(dirname "$0")/checkItems/"${checkScripts}"    > /dev/null 2>&1
    done
}

main "$@"
