#!/bin/bash
#-----------------------------------------------------------------------------#
#    Copyright @ Huawei Technologies Co., Ltd. 2015-2016. All rights reserved.
#                    版权所有 (C), 2015-2016, 华为技术有限公司
#-----------------------------------------------------------------------------#

########################################################################################################################
#对其他节点进行ping操作，读取ping结果的丢包率和平均时延(对应检查序号4,5)
########################################################################################################################

function pingcheck()
{
    local targetIP=$1             #目标IP
    local -i lossRate="0"         #网络丢包率阈值
    local -i delay="200"          #时延阈值20ms
    local -i pingCount="3"        #发包数量
    local errorOut=""
    local resultOutput=""
    local iLossRate=""
    local iDelay=""
    
    #交换机或者当前主机上没有对端的arp信息，第一个ping包高概率出现loss的现象，因此先ping一个包，不统计结果
    ping -c 1 ${targetIP} >> /dev/null 2>&1
    resultOutput=$(ping -c ${pingCount} ${targetIP})
    iLossRate=$(echo ${resultOutput}|grep -Eo '[0-9]+% packet loss'|grep -Eo '[0-9]+')
    iDelay=$(echo ${resultOutput}|awk -F '/' '{print $5}')

    if [ "${iLossRate}" -gt "${lossRate}" ]; then
        errorOut="${errorOut}${targetIP}: ${iLossRate}% packet loss;"
    fi
    if [ -z "${iDelay}" ] || [ $(echo "$iDelay > ${delay}"|bc) -eq 1 ]; then
        errorOut="${errorOut}${targetIP}: rtt avg=${iDelay};"
    fi

    if [ -z "${errorOut}" ];then
        log "[INFO]" "${LINENO}" "${ip}: ping ${targetIP} ${iLossRate}% packet loss, rtt avg=${iDelay}."
        echo -e "ping -c ${pingCount} ${targetIP}\n${resultOutput}"
    else
        log "[WARNING]" "${LINENO}" "${ip}: Network exception to ${targetIP}: ${iLossRate}% packet loss, rtt avg=${iDelay}."
        echo -e "ping -c ${pingCount} ${targetIP}\nerrorOut|${errorOut}\n${resultOutput}"
    fi
}

function parallelCheck()
{
    local IP_LIST="$1"
    local MaxThread=10
    local PID=() # 记录PID到数组, 检查PID是否存在以确定是否运行完毕 

    while read ipAddr
    do
        #控制并发,检查PID是否存在以确定是否运行完毕,找到一个就退出 
        while true
        do
            for((Ijob=0; Ijob<MaxThread; Ijob++))
            do
                if [ ! "${PID[Ijob]}" ] || ! kill -0 ${PID[Ijob]} 2> /dev/null
                then
                    break
                fi
            done
                
            if [ "$Ijob" -lt "$MaxThread" ]
            then 
                break
            fi
            
            sleep 1
        done
        
        pingcheck "${ipAddr}" &
        PID[Ijob]=$!
    done  <"${IP_LIST}"
    
    wait
}

function checkNetwork()
{
    if [ ! -f ${MANAGER_IP_LIST} ]; then
        log "[DEBUG]" "${LINENO}" "${ip}: ${MANAGER_IP_LIST} does not exist. Skip checking network."
        return 0
    fi
    
    log "[INFO]" "${LINENO}" "${ip}: Start checking network."
    log "[INFO]" "${LINENO}" "${ip}: Check the ping package loss rate and delay with all the other nodes."
    log "[INFO]" "${LINENO}" "${ip}: The ping package loss rate threshold is 0% and delay threshold is 5ms."    

    log "[INFO]" "${LINENO}" "${ip}: Check the ping package loss rate and delay within management plane."
    parallelCheck "${MANAGER_IP_LIST}"

    diff "${MANAGER_IP_LIST}" "${BUSINESS_IP_LIST}" 1> /dev/null
    if [ $? -eq 0 ]; then
        log "[INFO]" "${LINENO}" "${ip}: ${BUSINESS_IP_LIST} is same as ${MANAGER_IP_LIST}. Skip checking network within service plane."
    else
        log "[INFO]" "${LINENO}" "${ip}: Check the ping package loss rate and delay within service plane."
        parallelCheck "${BUSINESS_IP_LIST}"
    fi
    
    log "[INFO]" "${LINENO}" "${ip}: End checking network."
}