#!/bin/bash

########################################################################################
########################################################################################
#一共四个通知接口
#--------------------------------------------------
#"0"   "2"   "${dir}"  "${module}"  "mount ok"
#"1"   "2"   "${dir}"  "${module}"  "partiton lost"
#"2"   "0"   "${dir}"  "${module}"  "readonly"
#"2"   "1"   "${dir}"  "${module}"  "readonly resume"
#--------------------------------------------------
#参数1为告警ID
#0：目录变为可用：比如之前目录不可用（拔盘），现在找到新的盘恢复了
#1：目录变为不可用：比如所挂载的盘符消失（拔盘）
#2：目录变为只读: 执行mount命令查看，挂载有ro属性
#/srv/BigData/clickhouse/data1/metadata/
#参数2为告警类型
#0：发生告警
#1：恢复告警
#2：事件（无需恢复）
#
#参数3为目录，如/srv/BigData/clickhouse/data1
#
#参数4为模块名
#
#参数5为附加信息
########################################################################################
########################################################################################

##参数定义
ParaNum=5  #参数合法个数
RESULT=0   #返回值

##5个入参定义
ALARM_ID=0
ALARM_TYPE=0
ALARM_DIR=""
ALARM_MODULE=""
ALARM_INFO=""

##参数打印
echo `date` "`basename $0` paras:" "[$*]"
echo `date` "`basename $0` para[1]:" "$1"
echo `date` "`basename $0` para[2]:" "$2"
echo `date` "`basename $0` para[3]:" "$3"
echo `date` "`basename $0` para[4]:" "$4"
echo `date` "`basename $0` para[5]:" "$5"

if [ $# -ne ${ParaNum} ]; then
    echo `date` "`basename $0` failed: paras num($#) is not ${ParaNum}"
    exit 1
fi


##确定参数合法后开始赋值
ALARM_ID=$1
ALARM_TYPE=$2
ALARM_DIR="$3"   #目录名，请用这个
ALARM_MODULE="$4"   #模块名，clickhouse
ALARM_INFO="$5"  #附加信息，可以不关注

if [[ $UID -ne 0 ]]; then
    DISKMGT_BIGDATA_HOME="${BIGDATA_HOME}"
else
    DISKMGT_BIGDATA_HOME="$(su - omm -c 'echo $BIGDATA_HOME')"
fi

if [ -z "${DISKMGT_BIGDATA_HOME}" ]; then
    DISKMGT_BIGDATA_HOME="/opt/huawei/Bigdata"
fi

DISK_CONFIG_DIR="${DISKMGT_BIGDATA_HOME}/tmp/disk_manager"
DISABLE_DIR_LIST="${DISK_CONFIG_DIR}/disable_dir_list"
RESULT_FILE="${DISK_CONFIG_DIR}/ck_result"
declare IS_METADATA_DISK=false
declare CK_SERVER_CONFIG=""
BIN=$(cd "$(dirname -- "$0")";pwd)
#状态标注文件，文件由本脚本产生，server端清理，每次覆盖写入，格式如下：
#dir=xxx
#type=0
#type:
#0 mount ok
#1 partiton lost
#2 readonly
#3 readonly resume
STATUS_TAG="${DISK_CONFIG_DIR}/CK_DiskManagerStatus"

#副本同步百分比，当前认为所有表都同步才算副本同步完成
SYNCED_RATE=100
CK_ORDER=""
AVALIABLE_SERVER=""

CLOUD_FLAG_FILE="/usr/local/diskmgt/ini/cloud"
DISKMGT_NOTIFY_LOG_HOME="/var/log/diskmgt/notify"
LOGFILE="${DISKMGT_NOTIFY_LOG_HOME}/clickhouse_notify.log"
CK_LOGFILE="/var/log/Bigdata/clickhouse/clickhouseServer/clickhouse_notify.log"

function clickhouse_client_CMD()
{
    server="$1"
    sql="$2"
    log_file="$3"
    ENV_SH=$(su - omm -c "find ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseServer/etc/clickhouse-env.sh")
    ENV_VARS=$(su - omm -c "find ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseServer/etc/ENV_VARS")
    source "${ENV_SH}"
    source "${ENV_VARS}"
    INSTALL_HOME=$(su - omm -c "source ${ENV_SH}; echo $CLICKHOUSE_INSTALL_HOME")
    ret=$(su - omm -c "source ${ENV_SH}; source ${ENV_VARS}; source ${INSTALL_HOME}/clickhouse/sbin/clickhouse-common.sh; clickhouse_client_cmd \"${server}\" \"${sql}\" \"${log_file}\"")

    if [ $? -ne 0 ]; then
        return 1
    else
        echo "${ret}"
        return 0
    fi
}

function LOG()
{
    local log_size=0

    if [ ! -d ${DISKMGT_NOTIFY_LOG_HOME} ]; then
        mkdir -p ${DISKMGT_NOTIFY_LOG_HOME}
        chown root:root ${DISKMGT_NOTIFY_LOG_HOME}
        chmod 700 ${DISKMGT_NOTIFY_LOG_HOME}
    fi

    if [ ! -e "$LOGFILE" ]; then
        touch "$LOGFILE"
        chown root:root "$LOGFILE"
        chmod 600 ${LOGFILE}
        LOG "INFO" "Set ${LOGFILE} file's permission is $(ls -ld ${LOGFILE})"
    fi

    printf "`date -d today +\"%Y-%m-%d %H:%M:%S\"`,000 $1 $2\n" >> "$LOGFILE" 2>/dev/null

    log_size=`du -ms $LOGFILE |cut -f1`
    if [ $log_size -gt 50 ]; then
        #如果日志文件大于50M，则备份日志文件
        mv ${LOGFILE} ${LOGFILE}_bak
        LOG "INFO" "Change the ${LOGFILE}_bak file's permission to $(ls -ld ${LOGFILE}_bak)"
    fi
}

################################
#功能：进入运维模式
#入参：config.xml
#返回：处理成功返回0；否则返回非0
################################
function enter_maintanence_mode()
{
    LOG "INFO" "Enter maintanence mode, new requests will be rejected."
    su - omm -c "sed -i '/<\/yandex\>/i\  <working_mode>maintenance<\/working_mode>' "${CK_SERVER_CONFIG}""
    result=$?
    return ${result}
}

################################
#功能：退出运维模式
#入参：config.xml
#返回：处理成功返回0；否则返回非0
################################
function restore_working_mode()
{
    LOG "INFO" "Exit maintanence mode, node recovery."
    su - omm -c "sed -i '/working_mode/d' ${CK_SERVER_CONFIG}"
}

function export_env()
{
    #nodeagent env
    export NODE_AGENT_HOME=$(su - omm -c 'echo ${NODE_AGENT_HOME}')
    export WCC_PROFILE_DIR=$(su - omm -c 'echo ${WCC_PROFILE_DIR}')
    export JAVA_HOME=$(su - omm -c 'echo ${JAVA_HOME}')
    export JAVA=${JAVA_HOME}/bin/java

    #clickhouse env
    local ENV_SH=$(su - omm -c 'find '${DISKMGT_BIGDATA_HOME}'/FusionInsight_Current/*_*_ClickHouseServer/etc/clickhouse-env.sh')
    local ENV_VARS=$(su - omm -c 'find '${DISKMGT_BIGDATA_HOME}'/FusionInsight_Current/*_*_ClickHouseServer/etc/ENV_VARS')
    if [ -z "${ENV_SH}" ] || [ -z "${ENV_VARS}" ]; then
       LOG "ERROR" "Env vars does not exist, unable to check sync status"
       return 1
    fi

    source "${ENV_SH}"
    source "${ENV_VARS}"

    #balancer env
    export NGINX_PATH=$(su - omm -c 'source '${ENV_SH}'; echo $CLICKHOUSE_INSTALL_HOME/clickhouse/clickHouse_balancer/nginx/')
    export DISABLE_SERVER_LIST="${DISKMGT_BIGDATA_HOME}"/tmp/LB_DISABLE_SERVER_LIST
    export DIRMETA=$(su - omm -c 'source '${ENV_VARS}';basename $CLICKHOUSE_DATADIR')

    #判断是否是元数据的盘坏掉了
    if [ $(su - omm -c 'source '${ENV_VARS}'; echo ${CLICKHOUSE_DATADIR}') == "${ALARM_DIR}/" ] || [ $(su - omm -c 'source '${ENV_VARS}'; echo ${CLICKHOUSE_DATADIR}') == "${ALARM_DIR}/${DIRMETA}/" ]; then
        IS_METADATA_DISK=true
    fi

    CK_SERVER_CONFIG=$(su - omm -c "find "${DISKMGT_BIGDATA_HOME}"/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/config.xml")
    if [ -z "${CK_SERVER_CONFIG}" ]; then
       LOG "ERROR" "Config file does not exist, unable to enter or exit maintenance mode"
       return 1
    fi
}

################################
#功能：检查副本同步状态，所有表同步完成则表明副本同步完成
#入参：无
#返回：同步完成则返回0，未同步完成则返回1
################################
function check_sync_status()
{
    local CLICKHOUSE_BINARY=$(su - omm -c 'source ${BIGDATA_HOME}/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/clickhouse-env.sh; find "${CLICKHOUSE_INSTALL_HOME}"/clickhouse/bin/clickhouse')
    export CLICKHOUSE_SECURITY_ENABLED="${CLICKHOUSE_SECURITY_ENABLED}"
    export CLICKHOUSE_CONF_DIR="${CLICKHOUSE_CONF_DIR}"

    query="select table, log_max_index, log_pointer from system.replicas where total_replicas > 1 and engine like '%Replicated%'"
    clickhouse_client_CMD "${AVALIABLE_SERVER}" "${query}" "${CK_LOGFILE}" > "${RESULT_FILE}"
    result=$(cat ${RESULT_FILE})
    if [[ $? -ne 0 ]]; then
        LOG "ERROR"  "Failed to get sync status"
        return 1
    fi

    #处理每一个表的返回结果
    local totalTableNum=0
    local syncedTableNum=0
    if [ -z "${result}" ]; then
        LOG "INFO" "There is no table in clickhouse, no need to wait"
        return 0
    fi
    while read line; do
        totalTableNum=$((${totalTableNum}+1))
        array=(${line})
        if [[ ${array[1]} -eq 0 ]] && [[ ${array[2]} -eq 0 ]]; then
            syncedTableNum=$((${syncedTableNum}+1))
        elif [ $((${array[1]}+1)) -eq ${array[2]} ]; then
            syncedTableNum=$((${syncedTableNum}+1))
        fi
    done < ${RESULT_FILE}
    rm -rf "${RESULT_FILE}"

    rate=$(echo "scale=2; ${syncedTableNum}/${totalTableNum}*100"|bc)
    LOG "INFO" "There are ${totalTableNum} tables, ${syncedTableNum} of them are synced, rate is ${rate}"
    rm -rf "${CK_ENV_FILE}"
    if [[ $(echo "${rate} >= ${SYNCED_RATE}" | bc) -eq 1 ]]; then
        return 0
    else
        return 1
    fi
}


################################
#功能：获取本节点逻辑集群名称
#入参：无
#返回：返回逻辑集群名存在结果文件里面
################################
function get_clustername()
{
    local CLICKHOUSE_BINARY=$(su - omm -c 'source ${BIGDATA_HOME}/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/clickhouse-env.sh; find "${CLICKHOUSE_INSTALL_HOME}"/clickhouse/bin/clickhouse')
    export CLICKHOUSE_SECURITY_ENABLED="${CLICKHOUSE_SECURITY_ENABLED}"
    export CLICKHOUSE_CONF_DIR="${CLICKHOUSE_CONF_DIR}"

    query="select cluster from system.clusters where host_name='$(hostname)'"
    clickhouse_client_CMD "${AVALIABLE_SERVER}" "${query}" "${CK_LOGFILE}" > "${RESULT_FILE}"
}

################################
#功能：在LB中删除该节点
#返回：
################################
function remove_node_from_LB()
{
    balancer_server_list=${CK_BALANCER_LIST//,/ }
    for balancer_server in ${balancer_server_list}
    do
        LOG "INFO" "Begin to update ClickHouseBalancer configuration on ${balancer_server}"
        #把需要屏蔽的目录和节点信息写入lb节点的屏蔽目录文件中
        su - omm -c "ssh -n ${balancer_server} \"echo ${ALARM_DIR}:$(hostname) >> ${DISABLE_SERVER_LIST}\""
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to set the node as a shielded node"
            return 1
        fi
        LOG "INFO" "Succeed to set the node as a shielded node"

        #获取lb的环境变量
        BALANCER_ENV_VARS=$(su - omm -c "ssh -n ${balancer_server} \"find ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseBalancer/etc/ENV_VARS\"")
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to get the environment variables of ClickHouseBalancer"
            return 1
        fi

        # 1.修改nginx.conf
        # 2.reload配置文件
        deleteserver="server $(hostname):[0-9]\{5\};"
        su - omm -c "ssh -n ${balancer_server} \"sed -i \\\"s/${deleteserver}//g\\\" ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseBalancer/etc/nginx.conf\""
        su - omm -c "ssh -n ${balancer_server} 'source ${BALANCER_ENV_VARS}; source ${CLICKHOUSE_INSTALL_HOME}/clickhouse/sbin/updateLb.sh; synConf ${LOGFILE}'"
        LOG "INFO" "Succeed to update ClickHouseBalancer configuration on ${balancer_server}"
    done
    return 0
}

################################
#功能：在LB中添加该节点
#返回：
################################
function restore_node_from_LB()
{
    balancer_server_list=${CK_BALANCER_LIST//,/ }
    for balancer_server in ${balancer_server_list}
    do
        LOG "INFO" "Begin to update ClickHouseBalancer configuration on ${balancer_server}"
        #把需要屏蔽的目录和节点信息写入lb节点的屏蔽目录文件中
        su - omm -c "ssh -n ${balancer_server} \"sed -i \\\"/$(echo "$(hostname)" | sed -e 's/\//\\\//g')/d\\\" ${DISABLE_SERVER_LIST}\""
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to unblock node"
            return 1
        fi
        LOG "INFO" "Succeed to unblock node"

        #获取lb的环境变量
        BALANCER_ENV_VARS=$(su - omm -c "ssh -n ${balancer_server} \"find ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseBalancer/etc/ENV_VARS\"")
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to get the environment variables of ClickHouseBalancer"
            return 1
        fi

        # 1.修改nginx.conf
        # 2.reload配置文件
        get_clustername
        clustername=$(cat ${RESULT_FILE})
        LOG "INFO"  "This node is in cluster ${clustername}"
        upstream_list=$(su - omm -c "ssh -n ${balancer_server} \"grep -n \\\"upstream \\\" ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseBalancer/etc/nginx.conf | grep -E '(ckhttp|ckhttps|clickhouse|clickhouseSSL)${clustername} {' | cut -d ':' -f 1 \"")
        for i in $upstream_list;
        do
            line=$(($i+1))
            upstream=$(su - omm -c "ssh -n ${balancer_server} \"sed -n \\\"${i}p\\\" ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseBalancer/etc/nginx.conf \"")
            local port=0
            if [[ $(echo ${upstream} | grep "ckhttp${clustername} {") != "" ]]; then
                port=$(xmllint --xpath '/yandex/http_port' ${CK_SERVER_CONFIG} | sed -En 's/<.?http_port>/\n/gp' | sed  '/^[[:space:]]*$/d')
            elif [[ $(echo ${upstream} | grep "ckhttps${clustername} {") != "" ]]; then
                port=$(xmllint --xpath '/yandex/https_port' ${CK_SERVER_CONFIG} | sed -En 's/<.?https_port>/\n/gp' | sed  '/^[[:space:]]*$/d')
            elif [[ $(echo ${upstream} | grep "clickhouse${clustername} {") != "" ]]; then
                port=$(xmllint --xpath '/yandex/tcp_port' ${CK_SERVER_CONFIG} | sed -En 's/<.?tcp_port>/\n/gp' | sed  '/^[[:space:]]*$/d')
            elif [[ $(echo ${upstream} | grep "clickhouseSSL${clustername} {") != "" ]]; then
                port=$(xmllint --xpath '/yandex/tcp_port_secure' ${CK_SERVER_CONFIG} | sed -En 's/<.?tcp_port_secure>/\n/gp' | sed  '/^[[:space:]]*$/d')
            fi
            newserver="server $(hostname):${port};"
            su - omm -c "ssh -n ${balancer_server} \"sed -i \\\"${line} s/$/&${newserver}/\\\" ${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_ClickHouseBalancer/etc/nginx.conf \""
        done
        su - omm -c "ssh -n ${balancer_server} 'source ${BALANCER_ENV_VARS}; source ${CLICKHOUSE_INSTALL_HOME}/clickhouse/sbin/updateLb.sh; synConf ${LOGFILE}'"
        LOG "INFO" "Succeed to update ClickHouseBalancer configuration on ${balancer_server}"
    done
    return 0
}

wait_until_start ()
{
    sleep 10
    time_i=0
    retval=1
    while [ $time_i -lt 300 ]
    do
        curpid=$(pgrep -f "clickhouse --server --config-file")
        LOG "INFO" "Check no : $time_i, process id: $curpid ."
        if [ -n "$curpid" ]; then
            LOG "INFO" "Clickhouse server running normally."
            su - omm -c 'echo "${curpid}" > "${CLICKHOUSE_PID_FILE}"'
            retval=0
            break;
        fi

        ((time_i=time_i+1))
        sleep 1
    done
    if [ $retval -ne 0 ];then
        su - omm -c 'source ${BIGDATA_HOME}/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/ENV_VARS; rm -rf "${CLICKHOUSE_PID_FILE}"'
        log "${START_LOG}" "ERROR" "Start clickhouse failed."
        return 1
    fi
}

start_clickhouse_service() {
    ret=$(pgrep -f 'clickhouse --server --config-file' | grep -v grep | grep -c -v start)

    if [ "${ret}" -eq 0 ]; then
        LOG "INFO" "Wait for clickhouse server start."
    else
        LOG "INFO" "Clickhouse server already running now."
    fi
    wait_until_start
}

################################
#从副本节点上把元数据信息拷贝过来，重启服务
#处理成功返回0；否则返回非0
################################
function copy_metadata_from_replica() {
    local local_hostname=$(hostname)
    su - omm -c "${NODE_AGENT_HOME}/bin/config-trust.sh ${AVALIABLE_SERVER}"

    su - omm -c "ssh -n ${AVALIABLE_SERVER} \"${NODE_AGENT_HOME}/bin/config-trust.sh ${local_hostname}\""
    #在副本节点打包metadata目录
    su - omm -c "ssh -n ${AVALIABLE_SERVER} \"cd ${ALARM_DIR}; tar -cf metadata.tar.gz ${DIRMETA}\""
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to pack the metadata directory on the replica node"
        return 1
    fi
    LOG "INFO"  "Succeed to pack the metadata directory on the replica node"

    #scp到当前节点，解压
    su - omm -c "scp ${replica}:${ALARM_DIR}/metadata.tar.gz $(hostname):/${ALARM_DIR}/; cd ${ALARM_DIR}; tar -xf metadata.tar.gz; rm -rf ${ALARM_DIR}/metadata.tar.gz"
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to copy ${replica}:${ALARM_DIR}/metadata.tar.gz to current node and decompress"
        return 1
    fi
    LOG "INFO"  "Succeed to copy ${replica}:${ALARM_DIR}/metadata.tar.gz to current node and decompress"

    #删除副本节点的tar包
    su - omm -c "ssh -n ${replica} \"rm -rf ${ALARM_DIR}/metadata.tar.gz\""
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to clear pack on the replica node"
        return 1
    fi

    #启动进程
    start_clickhouse_service
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to start clickhouse server"
        return 1
    fi
    LOG "INFO" "Succeed to sync metadata from replica"
    return 0
}

################################
#修改zk上的server路径模拟进入运维模式
#普通模式路径：/clickhouse/instances/${listen_host}:${tcp_port}
#安全模式路径：/clickhouse/instances/${listen_host}:${tcp_port_secure}
#运维模式下写1，普通模式写0
#处理成功返回0；否则返回非0
################################
function modify_status_on_zk()
{
    for jar in "${CLICKHOUSE_INSTALL_HOME}"/adapter/uploadZk/lib/*.jar; do
        CK_CLASSPATH=${CK_CLASSPATH}:${jar}
    done
    ZOOKEEPER_ADDRESS="${ZK_SERVER}"
    LOG "INFO" "zookeeper ssl is ${ZK_SSL_ENABLE}"
    CK_SSL_OPTS=""
    if [[ ${ZK_SSL_ENABLE} = "true" ]];then
        ZOOKEEPER_ADDRESS="${ZK_SERVER_SSL}"
        CK_SSL_OPTS="-Dzookeeper.clientCnxnSocket=org.apache.zookeeper.ClientCnxnSocketNetty -Dzookeeper.client.secure=true"
    fi
    STATUS=${1}
    TCP_PORT=$(su - omm -c 'source ${BIGDATA_HOME}/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/ENV_VARS; cat ${CLICKHOUSE_CONF_DIR}/config.xml | grep \<tcp_port_secure\>' | awk -F '[<>]' '{print $3}')
    ZK_NODE=${ZK_ROOT_PATH}/instances/$(hostname):${TCP_PORT}
    ids=$(su - omm -c "${JAVA} -Djaas.path="${CLICKHOUSE_INSTALL_HOME}"/clickhouse/keytab -Duser.krb5.file="${KRB5_CONFIG}" -Dclickhouse.safemode="${SECURITY_MODE}" -Duser.keytab="${KRB5_KTNAME}" -Duser.principal="${KEYTAB_SRC_PRINCIPAL}" -Dzookeeper.znode.path="${ZK_NODE}" -Dzookeeper.address="${ZOOKEEPER_ADDRESS}" ${CK_SSL_OPTS} -Dstatus="${STATUS}" -Dinstance.id="${INSTANCE_ID}"  -cp "${CK_CLASSPATH}" com.huawei.clickhouse.uploadzk.UpdateStatusOnZK")
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Change status on znode ${ZK_NODE} failed."
        return 1
    fi
    echo "${ids[@]}" >> "${LOGFILE}"
    LOG "INFO" "Succeed to change status on znode ${ZK_NODE}."

    TCP_PORT_SECURE=$(su - omm -c 'source ${BIGDATA_HOME}/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/ENV_VARS; cat ${CLICKHOUSE_CONF_DIR}/config.xml | grep \<tcp_port_secure\>' | awk -F '[<>]' '{print $3}')
    ZK_NODE=${ZK_ROOT_PATH}/instances/$(hostname):${TCP_PORT_SECURE}
    ids=$(su - omm -c "${JAVA} -Djaas.path="${CLICKHOUSE_INSTALL_HOME}"/clickhouse/keytab -Duser.krb5.file="${KRB5_CONFIG}" -Dclickhouse.safemode="${SECURITY_MODE}" -Duser.keytab="${KRB5_KTNAME}" -Duser.principal="${KEYTAB_SRC_PRINCIPAL}" -Dzookeeper.znode.path="${ZK_NODE}" -Dzookeeper.address="${ZOOKEEPER_ADDRESS}" ${CK_SSL_OPTS} -Dstatus="${STATUS}" -Dinstance.id="${INSTANCE_ID}"  -cp "${CK_CLASSPATH}" com.huawei.clickhouse.uploadzk.UpdateStatusOnZK")
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Change status on znode ${ZK_NODE} failed."
        return 1
    fi
    echo "${ids[@]}" >> "${LOGFILE}"
    LOG "INFO" "Succeed to change status on znode ${ZK_NODE}."
    return 0
}

clear_old_dir()
{
    su - omm -c "rm -rf ${ALARM_DIR}/clickhouse/*"
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to clear "${ALARM_DIR}"/clickhouse/*"
        restore_working_mode
        return 1
    fi
    LOG "INFO" "Succeed to clear "${ALARM_DIR}"/clickhouse/*"

    if "${IS_METADATA_DISK}"; then
        LOG "INFO" "Try to restore the metadata disk, this process will take a long time"
        #遍历所有数据磁盘避免冗余数据产生，最优方法是找到uuid和表之间的关系，修改软连接
        su - omm -c "rm -rf ${ALARM_DIR}/${DIRMETA}/*"
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to clear "${ALARM_DIR}"/${DIRMETA}/*"
            restore_working_mode
            return 1
        fi
        LOG "INFO" "Succeed to clear "${ALARM_DIR}"/${DIRMETA}/*"
        disks=${CH_DATA_PATH//,/ }
        for disk in ${disks}; do
            su - omm -c "rm -rf ${disk}/clickhouse/*"
            if [ $? -ne 0 ]; then
                LOG "ERROR" "Failed to clear "${disk}"/clickhouse/*, the directory may generate redundant data"
            else
                LOG "INFO" "Succeed to clear "${disk}"/clickhouse/*"
            fi
        done
    fi
    LOG "INFO" "Succeed to clear old directory"
}

function get_avaliable_server()
{
    HOSTNAME=$(hostname)
    replicas=$("${BIN}"/clickhouse_get_replica.py --metrika_file "${CLICKHOUSE_CONF_DIR}/metrika.xml" \
                                  --current_hostname ${HOSTNAME}  \
                                  --log_file "${LOGFILE}")
    python_result=$?
    if [ "${python_result}" -ne 0 ]; then
         LOG "ERROR" "Failed to get avaliable server"
         return 1
    fi
    LOG "INFO" "Avaliable server list is ${replicas}"

    for replica in ${replicas}; do
        if [ ${replica} = $(hostname) ]; then
            continue
        fi
        # just check result
        clickhouse_client_CMD "${replica}" "select 1" "${CK_LOGFILE}"
        # $?=0 represent healthy of ck instance, $?>0 represent unhealthy of ck instance
        status=$?
        if [[ "${status}" -ne 0 ]]; then
            LOG "ERROR" "${replica} is not avaliable, curl result is ${status}"
            continue
        fi
        AVALIABLE_SERVER=${replica}
        LOG "INFO" "Avaliable server is ${AVALIABLE_SERVER}"
        return 0
    done
}

function mkdir_data_dirs()
{
    local DATA_DIRS="$1"

    for DIR in $DATA_DIRS
    do
        LOG "INFO" "checking ${DIR} exists"
        if [ ! -d "${DIR}" ]; then
            LOG "INFO" "mkdir ${DIR}"
            su - omm -c "mkdir -p "$DIR""
            ret_code=$?
            LOG "INFO" "mkdir ${DIR} exit with ${ret_code}"
            if [ "${ret_code}" -ne 0 ]; then
                return 1
            fi
            LOG "INFO" "chown and chmod ${DIR}"
            chmod 700 "${DIR}"
        fi
    done
}

################################
#获取副本个数
#处理成功返回0；否则返回非0
################################
function check_replica_num() {
    serverlist=${CK_SERVER_LIST//,/ }
    # shellcheck disable=SC2068
    for server in ${serverlist[@]}
    do
        sql="select countDistinct(replica_num) from system.clusters where cluster=(select cluster from system.clusters where host_name='$(hostname)')"
        REPLICA_NUM=$(clickhouse_client_CMD "${server}" "${sql}" "${CK_LOGFILE}")
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Execute sql (get replica number) on server ${server} failed."
            continue
        fi
        echo "${REPLICA_NUM}"
        return 0
    done
    LOG "ERROR" "Execute sql (get replica number) on each clickhouse server failed."
    return 1
}

################################
#目录恢复挂载事件处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_mount_ok()
{
    LOG "INFO" "module(${ALARM_MODULE}): dir(${ALARM_DIR}) mount ok."
    #清理目录
    clear_old_dir
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to clear old dir"
        return 1
    fi
    local tmp_dir=$(su - omm -c "echo ${ALARM_DIR} | sed 's#\/#\\\/#g'")
    su - omm -c "sed -i '/${tmp_dir}/d' "${DISABLE_DIR_LIST}""

    #写标志文件，内核会用，将恢复后的磁盘状态改为正常
    su - omm -c "echo "dir=${ALARM_DIR}" >> "${STATUS_TAG}"; echo "type=0" >> "${STATUS_TAG}""

    local DATA_DIRS=`xmllint --xpath '/yandex/storage_configuration//path' ${CK_SERVER_CONFIG} | sed -En 's/<.?path>/\n/gp' | sed  '/^[[:space:]]*$/d'`

    #创建数据目录
    mkdir_data_dirs "${DATA_DIRS}"
    if [ $? -ne 0 ]; then
        LOG "ERROR" "There is no avaliable server"
        restore_working_mode
        return 1
    fi

    #获取合理的获取副本同步状态的节点，当元数据盘坏的时候取可用的副本节点，副本节点都不可用时脚本执行失败，仅当数据盘坏的时候取当前节点即可
    if "${IS_METADATA_DISK}"; then
        #获取clickhouse客户端, 安全模式一定有ssl端口，非安全模式一定有不加密端口
        if [[ -z "$(su - omm -c 'source ${BIGDATA_HOME}/FusionInsight_ClickHouse_*/*_*_ClickHouseServer/etc/clickhouse-env.sh; find "${CLICKHOUSE_INSTALL_HOME}"/clickhouse/bin/clickhouse')" ]]; then
           LOG "ERROR" "Failed to get clickhouse binary"
           restore_working_mode
           return 1
        fi

        REPLICA_NUM=$(check_replica_num)
        if [ $? -ne 0 ]; then
           LOG "ERROR" "Get replica num failed."
           AVALIABLE_SERVER=$(hostname)
           restore_working_mode
           return 1
        fi
        if [ $? -eq 0 ] && [ ${REPLICA_NUM} -eq 1 ]; then
           LOG "WARN" "Single replica cluster, there is no avaliable server"
           AVALIABLE_SERVER=$(hostname)
           restore_working_mode
           return 1
        fi

        if [ ${REPLICA_NUM} != 1 ]; then
            get_avaliable_server
            if [ $? -eq 0 ] && [ -z ${AVALIABLE_SERVER} ]; then
               LOG "ERROR" "There is no avaliable server"
               restore_working_mode
               return 1
            fi

            #从副本节点把metadata目录打包传回来，重启
            copy_metadata_from_replica
            if [ $? -ne 0 ]; then
                LOG "ERROR" "Failed to copy metadata from replica"
                restore_working_mode
                return 1
            fi
        fi
    else
        AVALIABLE_SERVER=$(hostname)
    fi

    #检查数据是否同步，超时时间为60min，若超时，则认为该节点数据异常
    local time=0

    while $(check_sync_status); [ $? -ne 0 ]; do
        LOG "INFO" "Waiting for the replica data synchronization to complete..."
        sleep 30
        time=$((${time}+30))

        if $(check_sync_status); [ $? -eq 0 ]; then
            LOG "INFO" "More than 70 percent of the tables have been synchronized from the replica node, thinking that the data has been synchronized"
            break
        fi

        if [ ${time} -gt 3600 ]; then
            LOG "ERROR" "The data cannot be synchronized or the synchronization is too slow, please check the node status"
            restore_working_mode
            return 1
        fi
    done

    #判断标志文件中是否还有未恢复的磁盘，如果还有就不走下面的退出运维模式的流程.因为运维模式是节点粒度，得等这个节点所有磁盘都恢复后再退出运维模式
    grep "dir=" "${DISABLE_DIR_LIST}" >/dev/null 2>&1
    if [ $? -eq 0 ];then
        LOG "WARN" "Do not exit the O&M mode because some disks $(grep "dir=" "${DISABLE_DIR_LIST}") are not restored."
        return 0
    fi

    #退出运维模式
    restore_working_mode
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to exit maintenance mode"
        return 1
    fi

    if ${IS_METADATA_DISK}; then
        #将zk上的状态位置为正常状态
        modify_status_on_zk 0
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to update status on zookeeper"
            return 1
        fi
    fi

    #在LB中添加该节点
    restore_node_from_LB
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to exit maintenance mode"
        return 1
    fi

    return 0
}

################################
#目录挂载设备丢失事件处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_partition_lost()
{
    LOG "INFO" "module(${ALARM_MODULE}): dir(${ALARM_DIR}) lost."

    #记录哪个目录挂载的磁盘丢失了,后续恢复运维模式需要判断是否都所有的磁盘都恢复了
    su - omm -c "echo "dir=${ALARM_DIR}" >> ${DISABLE_DIR_LIST}"

    #元数据盘丢失server进程无法正常拉起，此时需要修改zk状态，模拟进入运维模式，使得分布式表的请求不会分发到该节点
    #数据盘丢失，server进程能通过读取config.xml中的配置进入运维模式，分布式表的请求不会分发到该节点
    if ${IS_METADATA_DISK}; then
        modify_status_on_zk 1
        if [ $? -ne 0 ]; then
            LOG "ERROR" "Failed to modify status on zookeeper"
            return 1
        else
            LOG "INFO" "Succeed to modify status on zookeeper"
        fi
    fi

    #从lb中剔除改节点
    remove_node_from_LB
    if [ $? -ne 0 ]; then
        LOG "WARN" "Failed to remove node from ClickHouseBalancer"
        return 1
    fi

    #进入运维模式
    enter_maintanence_mode
    if [ $? -ne 0 ]; then
        LOG "ERROR" "Failed to enter maintenance mode"
        return 1
    fi

    return 0
}


################################
#目录只读告警处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_readonly()
{
    LOG "INFO" "module(${ALARM_MODULE}): dir(${ALARM_DIR}) readonly."
    #内核可感知，此处无需单独处理
    return 0
}


################################
#目录只读恢复处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_resume_readonly()
{
    LOG "INFO" "module(${ALARM_MODULE}): dir(${ALARM_DIR}) readonly resume."

    #写标志文件
    su - omm -c "echo "dir=${ALARM_DIR}" >> "${STATUS_TAG}"; echo "type=3" >> "${STATUS_TAG}""
    return 0
}
########################################################################################
# Function: is_dir_in_use
# Description: 判断传入的目录是否被作为数据目录使用，如果在使用中则返回0，未使用返回1，其他内部错误返回255
# Parameter:
#   input: 路径
#   N/A
# Return: 0 -- in use; 1 -- not in use; 255 -- internal error
# Others: N/A
########################################################################################
is_dir_in_use()
{
    local mountDir=${1}
    local configFile=${CLICKHOUSE_CONF_DIR}/config.xml

    if [ ! -f ${configFile} ]; then
        LOG "INFO" "Can not found ClickHouse config: ${currentConfigFile}."
        return 1
    fi

    if "${IS_METADATA_DISK}"; then
        LOG "INFO" "Match ${mountDir} in config value ${dataDirs}. So ${mountDir} is used by ClickHouse."
        return 0
    fi

    dataDirs=$(xmllint --xpath '/yandex/storage_configuration//path' "${CLICKHOUSE_CONF_DIR}"/config.xml | sed -En 's/<.?path>/\n/gp' | sed  '/^[[:space:]]*$/d')

    for dir in $dataDirs
    do
        echo ${dir} | grep -qw ${mountDir}
        if [ $? -eq 0 ]; then
            LOG "INFO" "Match ${mountDir} in config value ${dataDirs}. So ${mountDir} is used by ClickHouse."
            return 0
        fi
    done

    LOG "INFO" "Cann't match ${mountDir} in config value ${dataDirs}. So ${mountDir} is not used by ClickHouse."
    return 1
}

################################
#进入逻辑处理
################################
function main()
{
    #设置当前的umask
    UMASK=0077
    umask "$UMASK"

    if [ ! -d "${DISK_CONFIG_DIR}" ]; then
        su - omm -c "mkdir -p "$DISK_CONFIG_DIR""
        LOG "INFO" "Set ${DISK_CONFIG_DIR} file's permission is $(ls -ld ${DISK_CONFIG_DIR})"
    fi

    export_env
    # 判断是否镜像环境，以及目录是否被实例使用，如果是云上环境且本组件没有使用当前目录，则脚本直接返回成功；如果is_dir_in_use函数返回255，则说明脚本执行失败，脚本需要以非0值退出；其他情况需要执行分区事项相关接口的业务处理
    if [ -f ${CLOUD_FLAG_FILE} ]; then
        is_dir_in_use ${ALARM_DIR}
        local retVal=$?
        if [ $retVal -eq 1 ]; then
            LOG "INFO" "${ALARM_DIR} is not used by ClickHouse in cloud cluster. Skip to execute partition envent."
            return 0
        elif [ $retVal -eq 255 ]; then
            LOG "INFO" "Failed to judge whether ${ALARM_DIR} is used by ClickHouse in cloud cluster."
            return 1
        fi
    fi

    if [ ${ALARM_ID} -eq 0 ]; then
        echo `date` "handle mount ok."
        handle_mount_ok  #目录恢复挂载事件处理
        return $?
    elif [ ${ALARM_ID} -eq 1 ]; then
        echo `date` "handle partition lost."
        handle_partition_lost #目录挂载设备丢失事件处理
        return $?
    elif [ ${ALARM_ID} -eq 2 ]; then
        if [ ${ALARM_TYPE} -eq 0 ]; then
            echo `date` "handle ro."
            handle_readonly  #目录只读告警处理
            return $?
        elif [ ${ALARM_TYPE} -eq 1 ]; then
            echo `date` "handle ro resume."
            handle_resume_readonly  #目录只读恢复处理
            return $?
        else
            echo "Invalid AlarmType(${ALARM_TYPE})!"
            return 1
        fi
    else
        echo "Invalid AlarmID(${ALARM_ID})!"
        return 1
    fi
}

#执行主函数
main
exit $?
