#!/bin/bash

########################################################################################
########################################################################################
#一共四个通知接口
#--------------------------------------------------
#"0"   "2"   "${dir}" "${module}"  "mount ok"
#"1"   "2"   "${dir}" "${module}"  "partiton lost"
#"2"   "0"   "${dir}" "${module}"  "readonly"
#"2"   "1"   "${dir}" "${module}"  "readonly resume"
#--------------------------------------------------
#参数1为告警ID
#0：目录变为可用：比如之前目录不可用（拔盘），现在找到新的盘恢复了
#1：目录变为不可用：比如所挂载的盘符消失（拔盘）
#2：目录变为只读: 执行mount命令查看，挂载有ro属性
#
#参数2为告警类型
#0：发生告警
#1：恢复告警
#2：事件（无需恢复）
#
#参数3为目录，如/srv/BigData/datanode1
#
#参数4为模块名
#
#参数5个为附加信息
########################################################################################
########################################################################################

##参数定义
ParaNum=5  #参数合法个数
RESULT=0   #返回值

##5个入参定义
ALARM_ID=0
ALARM_TYPE=0
ALARM_DIR=""
ALARM_MODULE=""
ALARM_INFO=""
ENV_VARS=""

##参数打印
echo "$(date)" "$(basename "$0") paras:" "[$*]"
echo "$(date)" "$(basename "$0") para[1]:" "$1"
echo "$(date)" "$(basename "$0") para[2]:" "$2"
echo "$(date)" "$(basename "$0") para[3]:" "$3"
echo "$(date)" "$(basename "$0") para[4]:" "$4"
echo "$(date)" "$(basename "$0") para[5]:" "$5"

if [ $# -ne ${ParaNum} ]; then
    echo "$(date)" "$(basename "$0") failed: paras num($#) is not ${ParaNum}"
    exit 1
fi


##确定参数合法后开始赋值
ALARM_ID=$1
ALARM_TYPE=$2
ALARM_DIR="$3"   #目录名，请用这个
ALARM_MODULE="$4"   #模块名，比如zookkeep,bookeeper等，可以不关注
ALARM_INFO="$5"  #附加信息，可以不关注
if [[ $UID -ne 0 ]]; then
    DISKMGT_BIGDATA_HOME="${BIGDATA_HOME}"
    DISKMGT_BIGDATA_LOG_HOME="${BIGDATA_LOG_HOME}"
else
    DISKMGT_BIGDATA_HOME="$(su - omm -c 'echo $BIGDATA_HOME')"
    DISKMGT_BIGDATA_LOG_HOME="$(su - omm -c 'echo $BIGDATA_LOG_HOME')"
fi

if [ -z "${DISKMGT_BIGDATA_HOME}" ]; then
    DISKMGT_BIGDATA_HOME="/opt/huawei/Bigdata"
fi

if [ -z "${DISKMGT_BIGDATA_LOG_HOME}" ]; then
    DISKMGT_BIGDATA_LOG_HOME="/var/log/Bigdata"
fi

DISKMGT_NOTIFY_LOG_HOME="/var/log/diskmgt/notify"
LOGFILE="${DISKMGT_NOTIFY_LOG_HOME}/motservice_notify.log"

declare CLOUD_FLAG_FILE="/usr/local/diskmgt/ini/cloud"

################################
#进入逻辑处理
################################
function main()
{
    #设置当前的umask
    UMASK=0077
    umask "$UMASK"
    if [ -f ${CLOUD_FLAG_FILE} ]; then
        is_dir_in_use ${ALARM_DIR}
        local retVal=$?
        if [ $retVal -eq 1 ]; then
            LOG "INFO" "${ALARM_DIR} is not used by motservice in cloud cluster. Skip to execute partition envent."
            return 0
        elif [ $retVal -eq 255 ]; then
            LOG "INFO" "Failed to judge whether ${ALARM_DIR} is used by motservice in cloud cluster."
            return 1
        fi
    fi

    if [ "${ALARM_ID}" -eq 0 ]; then
        echo "$(date)" "handle mount ok."
        handle_mount_ok  #目录恢复挂载事件处理
        return $?
    elif [ "${ALARM_ID}" -eq 1 ]; then
        echo "$(date)" "handle partition lost."
        handle_partition_lost #目录挂载设备丢失事件处理
        return $?
    elif [ "${ALARM_ID}" -eq 2 ]; then
        if [ "${ALARM_TYPE}" -eq 0 ]; then
            echo "$(date)" "handle ro."
            handle_readonly  #目录只读告警处理
            return $?
        elif [ "${ALARM_TYPE}" -eq 1 ]; then
            echo "$(date)" "handle ro resume."
            handle_resume_readonly  #目录只读恢复处理
            return $?
        else
            echo "Invalid AlarmType(${ALARM_TYPE})!"
            return 1
        fi
    else
        echo "Invalid AlarmID(${ALARM_ID})!"
        return 1
    fi
}


########################################################################################
# Function: is_dir_in_use
# Description: 判断传入的目录是否被作为数据目录使用，如果在使用中则返回0，未使用返回1，其他内部错误返回255
# （实例未安装按照未使用该目录处理，返回1）
# Parameter:
#   input: 路径
#   N/A
# Return: 0 -- in use; 1 -- not in use; 255 -- internal error
# Others: N/A
########################################################################################
is_dir_in_use()
{
    # 分区事件对应的挂载目录
    local mountDir="${1}"

    # 数据目录所在配置文件路径，如果etc_UPDATED存在，优先使用etc_UPDATED路径下的配置文件
    local currentConfigFile=${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_MOTServer/etc
    local updateConfigFile=${DISKMGT_BIGDATA_HOME}/FusionInsight_Current/*_*_MOTServer/etc_UPDATED

    # 如果配置文件不存在，按照此notify脚本对应的实例未使用这个数据目录进行处理
    local configFile=""
    if [ -d ${updateConfigFile} ]; then
        configFile=${updateConfigFile}
    elif [ -d ${currentConfigFile} ]; then
        configFile=${currentConfigFile}
    else
        LOG "INFO" "motservice is not install at this node."
        return 1
    fi

    # 从配置文件中获取数据目录配置项对应的值，如果未获取到，返回错误码255

    local dataDirs=$(cat ${configFile}/ENV_VARS | grep "MOTSERVICE_DATA_DIR" | awk -F '=' '{print $2}')
    if [ -z ${dataDirs} ]; then
        LOG "ERROR" "Can not get motservice config item motservice.data.dir in ${configFile}."
        return 255
    fi

    # 判断此次分区事件对应的挂载点目录是否在数据目录配置项中。如果在，说明被该实例使用了，返回0；否则未被使用，返回1。
    echo "${dataDirs}" | grep -qw "${mountDir}"
    if [ $? -eq 0 ]; then
        LOG "INFO" "Match ${mountDir} in config value ${dataDirs}. So ${mountDir} is used by motservice."
        return 0
    else
        LOG "INFO" "Cann't match ${mountDir} in config value ${dataDirs}. So ${mountDir} is not used by motservice."
        return 1
    fi

}

function LOG()
{
    local log_size=0

    if [ ! -d ${DISKMGT_NOTIFY_LOG_HOME} ]; then
        mkdir -p ${DISKMGT_NOTIFY_LOG_HOME}
        chown root:root ${DISKMGT_NOTIFY_LOG_HOME}
        chmod 700 ${DISKMGT_NOTIFY_LOG_HOME}
    fi

    if [ ! -e "$LOGFILE" ]; then
        touch "$LOGFILE"
        chown root:root "$LOGFILE"
        LOG "INFO" "Set ${LOGFILE} file’s permission is $(ls -ld "${LOGFILE}")"
    fi

    printf "$(date -d today +"%Y-%m-%d %H:%M:%S",000) %s %s\n" "$1" "$2" >> "$LOGFILE" 2>/dev/null

    log_size=$(du -ms "$LOGFILE" |cut -f1)
    if [ "$log_size" -gt 50 ]; then
        #如果日志文件大于50M，则备份日志文件
        mv "${LOGFILE}" "${LOGFILE}"_bak
        LOG "INFO" "Change the ${LOGFILE}_bak file’s permission to $(ls -ld "${LOGFILE}"_bak)"
    fi
}

function initEnv()
{
    LOG "INFO" "module(${ALARM_MODULE}): Start init env."
    NODE_AGENT_HOME=$(su - omm -c "echo ${NODE_AGENT_HOME}")
    export NODE_AGENT_HOME
    ENV_VARS=$(find "${DISKMGT_BIGDATA_HOME}"/FusionInsight_Current/*_*_MOTServer/etc/ENV_VARS)
    if  [ -z "${ENV_VARS}" ]; then
        LOG "ERROR" "Env vars does not exist, unable to check sync status"
        return 1
    fi
    source "${ENV_VARS}"
    MOTSERVICE_LIST_IP=( ${MOTSERVICE_LIST//,/ } )
    LOG "INFO" "Mot service list is " "${MOTSERVICE_LIST_IP[@]}"
    ARRAY_LENGTH=${#MOTSERVICE_LIST_IP[@]}
    if [ "${ARRAY_LENGTH}" = "2" ]; then
        LOG "INFO" "The number of mot services is 2."
    else
        LOG "ERROR" "The number of mot services is not 2, is ${ARRAY_LENGTH}."
        return 1
    fi
    for ip in "${MOTSERVICE_LIST_IP[@]}"
    do
        if [ "${ip}" = "${MOTSERVICE_IP}" ]; then
            continue
        fi
        MOT_PEER_IP=${ip}
        LOG "INFO" "Peer mot service ip is ${MOT_PEER_IP}"
        export MOT_PEER_IP
    done
    if [ -z "${MOT_PEER_IP}" ]; then
        LOG "ERROR" "Not found peer mot service."
        return 1
    fi
}
################################
#目录恢复挂载事件处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_mount_ok()
{
    echo "module(${ALARM_MODULE}): dir(${ALARM_DIR}) mount ok."
    if ! initEnv; then
        return 1
    fi
    if [ ! -d "${MOTSERVICE_DATA_DIR}" ]; then
        mkdir -p "${MOTSERVICE_DATA_DIR}"
    fi
    chown omm:wheel "${MOTSERVICE_DATA_DIR}"
    LOG "INFO" "Start copy postgresql.conf, dir is ${MOTSERVICE_DATA_DIR}/postgresql.conf."
    if su - omm -c "scp -r $MOT_PEER_IP:${MOTSERVICE_DATA_DIR}/postgresql.conf ${MOTSERVICE_DATA_DIR}"; then
        LOG "INFO" "Copy postgresql.conf complete."
    else
        LOG "ERROR" "Copy postgresql.conf failed."
        return 1
    fi
    chown omm:wheel "${MOTSERVICE_DATA_DIR}"/postgresql.conf
    sed -i "s/listen_addresses = '.*'/listen_addresses = 'localhost,${MOTSERVICE_IP}'/" "${MOTSERVICE_DATA_DIR}/postgresql.conf"
    sed -i "s/local_bind_address = '.*'/local_bind_address = '${MOTSERVICE_IP}'/" "${MOTSERVICE_DATA_DIR}/postgresql.conf"
    PLACE_STRING=$(grep "^replconninfo1 = *" < "${MOTSERVICE_DATA_DIR}/postgresql.conf")
    PLACE_STRING=${PLACE_STRING/localhost="${MOT_PEER_IP}"/localhost="${MOTSERVICE_IP}"}
    PLACE_STRING=${PLACE_STRING/remotehost="${MOTSERVICE_IP}"/remotehost="${MOT_PEER_IP}"}
    sed -i "s/replconninfo1 = .*/${PLACE_STRING}/" "${MOTSERVICE_DATA_DIR}/postgresql.conf"
    restart_service
    return 0
}

function restart_service()
{
    local pid=""

    #杀掉mot进程，让定时器重新拉起mot
    pid=$(su - omm -c "ps -efww | grep MOTService | grep ha.bin |grep -v grep | awk '{print $2}' |xargs echo")
    if [ -z "${pid}" ]; then
        LOG "INFO" "Can not find mot process."
        return 0
    fi

    LOG "INFO" "Kill mot process, pid=[${pid}]."
    sudo /bin/kill -9 "${pid}"
}

################################
#目录挂载设备丢失事件处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_partition_lost()
{
    echo "module(${ALARM_MODULE}): dir(${ALARM_DIR}) lost."

    return 0
}


################################
#目录只读告警处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_readonly()
{
    echo "module(${ALARM_MODULE}): dir(${ALARM_DIR}) readonly."

    return 0
}


################################
#目录只读恢复处理
#只要在这里添加处理即可
#处理成功返回0；否则返回非0
################################
function handle_resume_readonly()
{
    echo "module(${ALARM_MODULE}): dir(${ALARM_DIR}) readonly resume."

    return 0
}


#执行主函数
main
exit $?