#!/bin/bash

###################################################
#
# 功能: 解析 分区配置表 与 主机名的关系
#
###################################################

###################################################
#
# 扩展主机分区配置:
#  1:删除配置中的空格回车,合并重复的分号和冒号
#  2:扩展配置中的主机名
#
###################################################
function extend_parted_conf()
{
    local parted_conf="$1"   #配置信息
    local new_conf=""        #新的配置
    local line=""
    local hosts=""
    local conf=""

    debug "[${parted_conf}] Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$parted_conf" ]; then
        error "[${FUNCNAME}:${LINENO}]" "parted conf is Null"
        return 1
    fi

    #1.删除配置中的空格回车,合并重复分号,删除最后一个分号 [由于IPv6，此处不合并冒号，改为#]
    parted_conf="$(echo "$parted_conf" | tr -d '[:space:]' | tr -s ';' | tr -s '#' | sed 's/^;//g' | sed 's/;$//g')"

    #2.扩展主机名
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi
        hosts="$(echo $line | awk -F'#' '{print $1}')"
        hosts="$(parse_host "${hosts}")"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "hosts invalid in [${line}]"
            return 1
        fi
        conf="$(echo $line | awk -F'#' '{print $2}')"
        if [ -z "$conf" ]; then
            error "[${FUNCNAME}:${LINENO}]" "conf in null in [${line}]"
            return 1
        fi

        new_conf="${new_conf};${hosts}#${conf}"

    done < <(echo "$parted_conf" | tr ';' '\n')

    if [ -z "$new_conf" ]; then
        error "[${FUNCNAME}:${LINENO}]" "parted conf invalid [${parted_conf}]"
        return 1
    fi

    #删除行首的分号
    new_conf="$(echo "$new_conf" | sed 's/^;//g')"

    debug "[${new_conf}] Leave [${FUNCNAME}:${LINENO}]"

    echo "${new_conf}"
    return 0
}

###################################################
#
# 合并配置中的主机名称
#
###################################################
function union_hosts()
{
    local parted_conf="$1"   #配置信息
    local line=""
    local host=""
    local hosts=""

    debug "[${parted_conf}] Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$parted_conf" ]; then
        error "[${FUNCNAME}:${LINENO}]" "parted conf is Null"
        return 1
    fi

    #1.删除配置中的空格回车,合并重复的分号,删除最后一个分号[由于支持IPv6，将分隔符:改为#]
    parted_conf="$(echo "$parted_conf" | tr -d '[:space:]' | tr -s ';' | tr -s '#' | sed 's/^;//g' | sed 's/;$//g')"

    #2.扩展主机名
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi
        host="$(echo $line | awk -F'#' '{print $1}')"
        if [ -z "$host" ]; then
            error "[${FUNCNAME}:${LINENO}]" "host is null in [${line}]"
            return 1
        fi

        #合并主机名
        hosts="${hosts},${host}"

    done < <(echo "$parted_conf" | tr ';' '\n')

    if [ -z "$hosts" ]; then
        error "[${FUNCNAME}:${LINENO}]" "hosts invalid [${parted_conf}]"
        return 1
    fi

    #此处不能使用uniq排除重复项
    hosts="$(echo "$hosts" | tr ',' '\n' | sort | tr '\n' ',' | sed "s/^,//g" | sed "s/,$//g")"

    debug "[${hosts}] Leave [${FUNCNAME}:${LINENO}]"

    #输出hosts
    echo "$hosts"
    return 0
}

