#!/bin/bash

###################################################
#
# 预检查 preinstall.ini 配置文件
#
###################################################
function pre_check_ini()
{
    #输出所有配置项内容
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_hosts=${g_hosts}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_parted=${g_parted}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_parted_conf=${g_parted_conf}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_add_pkg=${g_add_pkg}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_pkgs_dir=${g_pkgs_dir}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_hostname_conf=${g_hostname_conf}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_swap_off=${g_swap_off}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_feed_random_setup=${g_feed_random_setup}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_user_name=${g_user_name}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_port=${g_port}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_log_file=${g_log_file}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_check=${g_check}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_debug=${g_debug}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_platform=${g_platform}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_optimize=${g_optimize}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_kms_conf=${g_kms_conf}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_kms_user_name=${g_kms_user_name}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_ip_model=${g_ip_model}"
    info "[${FUNCNAME}:${LINENO}]" "preinstall.ini ==>> g_data_home=${g_data_home}"

    #去除 g_support_os 变量值的回车符 空格 及前后分号 合并重复分号
    g_support_os="$(echo "$g_support_os" | tr -d '[:space:]' | \
                  tr -s ';' | sed 's/^;//g' | sed 's/;$//g')"

    debug "Enter [${FUNCNAME}:${LINENO}:${g_support_os}]"

    #去除 g_hostname_conf 的空格
    g_hostname_conf="$(echo "$g_hostname_conf" | tr -d '[:space:]')"

    if [ "$g_platform" != "upgrade" ]; then   
        # g_platform 不能为空并且值为x86_64或aarch64
        if [ X"$g_platform" != X"x86_64" -a X"$g_platform" != X"aarch64" ]; then
            error "[${FUNCNAME}:${LINENO}]" "The value of g_platform is not x86_64 or aarch64."
            error "[${FUNCNAME}:${LINENO}]" "please check g_platform in preinstall.ini"
            return 1
        fi
    fi
    
    # g_hosts 不能为空
    if [ -z "$g_hosts" ]; then
        error "[${FUNCNAME}:${LINENO}]" "g_hosts can not be empty."
        error "[${FUNCNAME}:${LINENO}]" "please check g_hosts in preinstall.ini"
        return 1
    fi

    # g_parted_conf 不能为空
    if [ -z "$g_parted" ] || [ "$g_parted" -ne 0 ] ; then
        if [ -z "$g_parted_conf" ]; then
            error "[${FUNCNAME}:${LINENO}]" "g_parted_conf can not be empty, unless g_parted is 0."
            error "[${FUNCNAME}:${LINENO}]" "please check g_parted_conf in preinstall.ini"
            return 1
        fi
    fi

    # g_pkgs_dir 不能为空
    if [ -z "$g_add_pkg" ] || [ "$g_add_pkg" -ne 0 ] ; then
        if [ -z "$g_pkgs_dir" ]; then
            error "[${FUNCNAME}:${LINENO}]" "g_pkgs_dir can not be empty, unless g_add_pkg is 0."
            error "[${FUNCNAME}:${LINENO}]" "please check g_pkgs_dir in preinstall.ini"
            return 1
        fi
    fi

    # g_ip_model 若为空，则默认为 IPv4
    if [ -z "$g_ip_model" ]; then
         g_ip_model="IPv4"
    fi
    debug "[g_ip_model=${g_ip_model}] [${FUNCNAME}:${LINENO}]"
    
    # g_data_home 不允许为根目录“/”；若为空，则默认为 “/srv/BigData/”
    if [ "$g_data_home" = "/" ]; then
        error "[${FUNCNAME}:${LINENO}]" "g_data_home can not be root dir '/'."
        error "[${FUNCNAME}:${LINENO}]" "please check g_data_home in preinstall.ini"
    fi
    
    if [ -z "$g_data_home" ]; then
         g_data_home="/srv/BigData/"
    fi
    debug "[g_data_home=${g_data_home}] [${FUNCNAME}:${LINENO}]"

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}


###################################################
#
# 检查用于保存rpm包的目录的有效性
# 1:目录不能为 空 或 / 或 . 或 ..
# 2:必须为目录
# 3:目录下至少包含一个rpm包
#
###################################################
function check_rpms_dir()
{
    local rpms_dir="$1"

    debug "[${rpms_dir}] Enter [${FUNCNAME}:${LINENO}]"

    #rpm包目录 不能为 系统根目录
    if [ -z "$rpms_dir" -o "$rpms_dir" == "/" \
         -o "$rpms_dir" == "." -o "$rpms_dir" == ".." ]; then
        error "[${FUNCNAME}:${LINENO}]" "rpms_dir is invalid"
        return 1
    fi

    #rpm包目录 必须是一个目录
    if [ ! -d "$rpms_dir" ]; then
        error "[${FUNCNAME}:${LINENO}]" "rpms_dir(${rpms_dir}) is a directory"
        return 1
    fi

    #rpm包目录 必须是一个目录
    if [ ! -d "$rpms_dir" ]; then
        error "[${FUNCNAME}:${LINENO}]" "rpms_dir(${rpms_dir}) is a directory"
        return 1
    fi

    #目录下至少包含一个rpm包
    if [ "$(find ${rpms_dir} -name "*.rpm" | wc -l)" -eq 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "no rpm package in rpms_dir(${rpms_dir})"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 检查 preinstall.ini 中的 g_parted_conf 配置项
#
###################################################
function check_ini_parted_conf()
{
    local hosts_conf="$1"
    local hosts=""

    debug "[${hosts_conf}] Enter [${FUNCNAME}:${LINENO}]"

    # g_parted 为空,则默认设置为 2
    if [ -z "$g_parted" ]; then
        g_parted=2
    fi

    #判断是否格式化磁盘
    if [ "$g_parted" -eq 0 ]; then
        debug "[g_parted=0] [${FUNCNAME}:${LINENO}]"
        return 0
    fi

    debug "[g_parted=${g_parted}] [${FUNCNAME}:${LINENO}]"

    #解析并检查 g_parted_conf 配置项
    if [ -z "$g_parted_conf" ]; then
        error "[${FUNCNAME}:${LINENO}]" "please setup g_parted_conf in preinstall.ini"
        return 1
    else
        g_parted_conf="$(echo "$g_parted_conf" | tr -d '[:space:]')"
    fi

    #扩展配置中的主机名
    g_parted_conf="$(extend_parted_conf "$g_parted_conf")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "1.g_parted_conf invalid in preinstall.ini"
        return 1
    fi

    #合并配置中的主机名
    hosts="$(union_hosts "$g_parted_conf")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "2.g_parted_conf invalid in preinstall.ini"
        return 1
    fi

    #检查g_parted_conf和g_hosts中的主机名是否一致
    if [ "$hosts_conf" != "$hosts" ]; then
        error "[${FUNCNAME}:${LINENO}]" "hosts in g_parted_conf don't match g_hosts in preinstall.ini"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 检查 preinstall.ini 中的 g_pkgs_dir 配置项
#
###################################################
function check_ini_pkgs_dir()
{
    local line=""
    local type=""
    local dir=""

    debug "Enter [${FUNCNAME}:${LINENO}]"

    # g_add_pkg 为空,则默认设置为 1
    if [ -z "$g_add_pkg" ]; then
        g_add_pkg=1
    else
        g_add_pkg="$(echo "$g_add_pkg" | tr -d '[:space:]')"
    fi

    #判断是否添加缺失的rpm包
    if [ "$g_add_pkg" -eq 0 ]; then
        debug "[g_add_pkg=0] [${FUNCNAME}:${LINENO}]"
        return 0
    fi
    debug "[g_add_pkg=${g_add_pkg}] [${FUNCNAME}:${LINENO}]"

    #解析并检查 g_pkgs_dir 配置项
    if [ -z "$g_pkgs_dir" ]; then
        error "[${FUNCNAME}:${LINENO}]" "please setup g_pkgs_dir in preinstall.ini"
        return 1
    fi

    #解析并检查 g_pkgs_dir 配置项
    if [ -z "$g_pkgs_dir" ]; then
        error "[${FUNCNAME}:${LINENO}]" "please setup g_pkgs_dir in preinstall.ini"
        return 1
    fi

    #删除配置中的空格回车,合并重复的分号和冒号,删除最后一个分号
    g_pkgs_dir="$(echo "$g_pkgs_dir" | tr -d '[:space:]' | \
                  tr -s ';' | tr -s ':' | sed 's/^;//g' | sed 's/;$//g')"

    debug "[g_pkgs_dir=${g_pkgs_dir}] [${FUNCNAME}:${LINENO}]"

    #逐项校验
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi

        #校验os是否是支持的版本
        type="$(echo $line | awk -F':' '{print $1}')"
        echo "$g_support_os" | grep -E '^'${type}';|;'${type}';|;'${type}'$' > /dev/null
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "os type[${type}] don't support, at ${line}"
            return 1
        fi

        #校验目录是否符合要求
        dir="$(echo $line | awk -F':' '{print $2}')"
        check_rpms_dir "$dir"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "check ${dir} failed, at ${line}"
            return 1
        fi
    done < <(echo "$g_pkgs_dir" | tr ';' '\n')

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 检查 preinstall.ini 配置文件
#
###################################################
function check_ini()
{
    local hosts=""

    debug "Enter [${FUNCNAME}:${LINENO}]"

    # g_debug 为空,则默认设置为 0
    if [ -z "$g_debug" ]; then
        g_debug=0
    else
        g_debug="$(echo "$g_debug" | tr -d '[:space:]')"
    fi
    debug "[g_debug=${g_debug}] [${FUNCNAME}:${LINENO}]"
    
    # g_kms_conf 为空 则默认设置为 0
    if [ -z "$g_kms_conf" ]; then
        g_kms_conf=0
    else
        g_kms_conf="$(echo "$g_kms_conf" | tr -d '[:space:]')"
    fi
    debug "[g_kms_conf=${g_kms_conf}] [${FUNCNAME}:${LINENO}]"
    
    # g_kms_conf为1时,g_kms_user_name不能为空
    if [ $g_kms_conf -eq 1 ]; then
        if [ -z $g_kms_user_name ]; then
            error "[${FUNCNAME}:${LINENO}]" "please setup g_kms_user_name in preinstall.ini"
            return 1
        fi
    fi

    # g_optimize 为空,则默认设置为 HD
    if [ -z "$g_optimize" ]; then
        g_optimize="default"
    else
        g_optimize="$(echo "$g_optimize" | tr -d '[:space:]')"
    fi
    debug "[g_optimize=${g_optimize}] [${FUNCNAME}:${LINENO}]"

    # g_user_name 为空,则默认设置为 root
    if [ -z "$g_user_name" ]; then
        g_user_name="root"
    fi
    debug "[g_user_name=${g_user_name}] [${FUNCNAME}:${LINENO}]"

    # g_port 为空,则默认设置为 22
    if [ -z "$g_port" ]; then
        g_port=22
    else
        g_port="$(echo "$g_port" | tr -d '[:space:]')"
    fi
    debug "[g_port=${g_port}] [${FUNCNAME}:${LINENO}]"

    # g_swap_off 为空,则默认设置为 1
    if [ -z "$g_swap_off" ]; then
        g_swap_off=1
    elif [ "${g_swap_off}" != "0" -a "${g_swap_off}" != "1" ]; then
        error "[${FUNCNAME}:${LINENO}]" "please check g_swap_off in preinstall.ini"
        return 1
    else
       g_swap_off="$(echo "$g_swap_off" | tr -d '[:space:]')" 
    fi
    debug "[g_swap_off=${g_swap_off}] [${FUNCNAME}:${LINENO}]"

    # g_feed_random_setup 为空,则默认设置为 1
    if [ -z "$g_feed_random_setup" ]; then
        g_feed_random_setup=1
    elif [ "${g_feed_random_setup}" != "0" -a "${g_feed_random_setup}" != "1" -a "${g_feed_random_setup}" != "2" ]; then
        error "[${FUNCNAME}:${LINENO}]" "please check g_feed_random_setup in preinstall.ini"
        return 1
     else
       g_feed_random_setup="$(echo "$g_feed_random_setup" | tr -d '[:space:]')"     
    fi
    debug "[g_feed_random_setup=${g_feed_random_setup}] [${FUNCNAME}:${LINENO}]"

    # g_log_file 为空,则默认设置为 ${HOME}/fi-preinstall.log
    if [ -z "$g_log_file" ]; then
        g_log_file="${HOME}/fi-preinstall.log"
    fi
    debug "[g_log_file=${g_log_file}] [${FUNCNAME}:${LINENO}]"

    # g_wce_conf 为空,则默认设置为 0
    if [ -z "$g_wce_conf" ]; then
        g_wce_conf=0
    fi
    debug "[g_wce_conf=${g_wce_conf}] [${FUNCNAME}:${LINENO}]"

    #校验 g_pkgs_dir 配置项
    check_ini_pkgs_dir
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "please check g_pkgs_dir in preinstall.ini"
        return 1
    fi

    debug "[g_hosts=${g_hosts}] [${FUNCNAME}:${LINENO}]"

    #去除g_hosts参数的空格
    g_hosts="$(echo "$g_hosts" | tr -d '[:space:]')"
    #深度检查并解析hosts字符串
    hosts="$(parse_host "$g_hosts")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "parse ${g_hosts} failed, please check g_hosts in preinstall.ini"
        return 1
    fi
    debug "[g_hosts=${hosts}] [${FUNCNAME}:${LINENO}]"

    #重新赋值g_hosts
    g_hosts="${hosts}"

    check_ini_parted_conf "${g_hosts}"
    if [ $? -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "check ini parted_conf failed"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
