#!/bin/bash

###################################################
# 功能: 设置密码
###################################################

###################################################
#
# 设置用户输入的集群服务器密码
#
###################################################
function set_passwd()
{
    local aes="$1"   #aes密码文件
    local noninteractive="$2"
    local passwd="$3"
    local empty=""   #密码是否为空
    local i=0        #循环控制变量
    local max=3      #最大的循环次数
    local show="Please enter cluster SSH password for $g_user_name: "

    debug "Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$aes" ]; then
        error "[${FUNCNAME}:${LINENO}]" "set passwd failed, aes is null"
        return 1
    fi
    if [ -n "${noninteractive}" -a "${noninteractive}" = "1" ]; then
        echo "${passwd}" | openssl enc -aes-256-cbc -salt -a -e -k "${g_aes_rand_key}" > $aes 2> /dev/null
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "password encrypt failed"
            return 1
        fi
        debug "Leave [${FUNCNAME}:${LINENO}]"
        return 0
    fi

    #如果输入了无效字符,可以使用Ctrl+Backspace键删除这些字符
    #说明:执行 stty erase '^H' 后,可以使用Backspace键删除字符
    #     但这改变了默认的行为,有可能会对其他程序产生影响
    #echo "Tips:Press the Ctrl+Backspace key to delete wrong characters."
    while true
    do
        ((i++))
        #输入超时时间设置为60秒
        #说明:使用系统内置REPLY来保存输入信息,
        #     可以完全保留输入信息中的空格
        #如果使用自定义变量,输入信息的最前端和最后端的空格将会被丢弃
        #每次read都会重新初始化REPLY内置变量
        read -t 60 -r -s -p "$show"
        echo ""
        if [ -n "$REPLY" ]; then
            echo "$REPLY" | openssl enc -aes-256-cbc -salt -a -e -k "${g_aes_rand_key}" > $aes 2> /dev/null
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "password encrypt failed"
                return 1
            fi
            break
        else
            #如果密码为空,需要进行确认
            read -t 60 -p "The password is empty(yes/no)? : " empty
            if [ "$empty" == "yes" -o "$empty" == "y" -o\
                 "$empty" == "YES" -o "$empty" == "Y" ]; then
                info "[${FUNCNAME}:${LINENO}]" "The password is empty."
                echo "" | openssl enc -aes-256-cbc -salt -a -e -k "${g_aes_rand_key}" > $aes 2> /dev/null
                if [ $? -ne 0 ]; then
                    error "[${FUNCNAME}:${LINENO}]" "password encrypt failed"
                    return 1
                fi
                break
            else
                #超过最大次数后,退出
                if [ ${i} -ge ${max} ]; then
                    error "[${FUNCNAME}:${LINENO}]" "Entered password is wrong"
                    return 1
                fi
                show="Please enter cluster SSH password for $g_user_name again[${i}/$((${max}-1))]: "
                debug "Again[${i}/$((${max}-1))] [${FUNCNAME}:${LINENO}]"
                continue
            fi
        fi
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}


###################################################
#
# 设置用户输入的集群服务器密码
#
###################################################
function set_kms_passwd()
{
    local kmc_tool_dir="${g_preinstall_path}/modules/070.autopart/autopart/kmsLuksMgt/kmc/"
    local kms_conf_file="${g_preinstall_path}/modules/070.autopart/autopart/ini-plugin/kms/kms_info"
    local kms_server=""
    local empty=""   #密码是否为空
    local i=0        #循环控制变量
    local max=3      #最大的循环次数
    local show="Please enter kms password for $g_kms_user_name: "

    debug "Enter [${FUNCNAME}:${LINENO}]"

    #如果输入了无效字符,可以使用Ctrl+Backspace键删除这些字符
    #说明:执行 stty erase '^H' 后,可以使用Backspace键删除字符
    #     但这改变了默认的行为,有可能会对其他程序产生影响
    #echo "Tips:Press the Ctrl+Backspace key to delete wrong characters."
    while true
    do
        ((i++))
        #输入超时时间设置为60秒
        #说明:使用系统内置REPLY来保存输入信息,
        #     可以完全保留输入信息中的空格
        #如果使用自定义变量,输入信息的最前端和最后端的空格将会被丢弃
        #每次read都会重新初始化REPLY内置变量
        read -t 60 -r -s -p "$show"
        echo ""
        if [ -n "$REPLY" ]; then
            #校验输入密码的有效性
            kms_server=$(cat ${kms_conf_file} | grep "^[[:space:]]*kms_server=" | awk '{print $1}' | sed 's/kms_server=//g' | head -n 1 2>/dev/null)
            kms_key_name=$(cat ${kms_conf_file} | grep "^[[:space:]]*kms_key_version_name=" | awk '{print $1}' | sed 's/kms_key_version_name=//g' | head -n 1 | awk -F '@' '{print $1}' 2>/dev/null)
            user_info=$(echo -n "$g_kms_user_name:$REPLY" | base64)
            curl -s  -i -H "Authorization: Basic ${user_info}" -k -X GET "https://${kms_server}/kms/v1/keys/names" | grep -w ${kms_key_name} >/dev/null 2>&1
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "Entered password is invalid"
                return 1
            fi
            cd $kmc_tool_dir > /dev/null 2>&1
            ./kmc_tool -e -i "$g_kms_user_name:$REPLY"
            if [ $? -ne 0 ]; then
                cd - > /dev/null 2>&1
                error "[${FUNCNAME}:${LINENO}]" "password encrypt failed"
                return 1
            fi
            cd - > /dev/null 2>&1
            break
        else
            #超过最大次数后,退出
            if [ ${i} -ge ${max} ]; then
                error "[${FUNCNAME}:${LINENO}]" "Entered password is wrong"
                return 1
            fi
            show="Please enter kms password for $g_kms_user_name again[${i}/$((${max}-1))]: "
            debug "Again[${i}/$((${max}-1))] [${FUNCNAME}:${LINENO}]"
            continue
        fi
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}