#!/bin/bash

if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
else
    CMD_CHMOD="chmod"
fi

###################################################
# 功能: 修改ping命令权限
###################################################
function chmod_ping()
{
    grep -i 'uos' /etc/product-info 2>/dev/null| grep -q 1060 
    if [ $? -eq 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "Start to chmod 755 /usr/bin/ping for uos-1060."
        ${CMD_CHMOD} 755 /usr/bin/ping
    fi    
}

###################################################
# 功能: 安装rpm包
###################################################

###################################################
#
# 根据rpm包名,在指定目录下获取rpm文件名
#
###################################################
function get_rpm_file()
{
    local rpms_dir="$1"   #rpm包目录
    local name="$2"       #需要安装的rpm包(可能包含类型)
    local check="$3"      #是否需要校验rpms_dir,1表示校验,0表示无需校验
    local new_name=""     #转换后的rpm包名称
    local file=""         #rpm包文件名
    local rpms=""         #查询到的所有的rpm包文件名
    local rpm_name=""     #rpm包名(不含类型)
    local rpm_name_r=""   #从rpm包文件中读出的rpm包名
    local rpm_output_file=""   #rpm -qpi命令输出的中间文件，因为rpm -qpi命令可能会挂住，所以增加一个检测
    local child_pid=""         #记录临时rpm -qpi后台命令的pid号

    debug "[${rpms_dir}:${name}:${check}] Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$name" ]; then
        error "[${FUNCNAME}:${LINENO}]" "name is null"
        return 1
    fi

    #入参检查
    if [ "$check" == "1" ]; then
        check_rpms_dir "${rpms_dir}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "check_rpms_dir failed"
            return 1
        fi
    fi

    #删除行中空格
    new_name="$(echo "$name" | tr -d '[:space:]')"

    #获取rpm包名
    rpm_name=$(echo "$new_name" | awk -F'.' '{print $1}')

    #把(.)分隔符,转换为(-*)
    new_name="$(echo "$new_name" | sed 's/\./\-\*/g')"

    #suse与redhat的文件名格式不同
    result=$(echo ${name} | grep "*")
    if [ "Y$result" != "Y" ]; then
        new_name=${name} 
    elif [ "$name" == "$new_name" ]; then
        #suse满足此条件
        new_name="${new_name}-*rpm"
    else
        #redhat满足此条件
        new_name="${new_name}*rpm"
    fi

    #有可能会同时匹配多个文件
    while read rpms
    do
        file=""
        if [ -z "$rpms" ]; then
            continue
        fi
        
        rpm_output_file="${HOME}/$name-$$-$name-qpi-output-$(date "+%N")"
        nohup rpm -qpi "$rpms" > "${rpm_output_file}" 2>&1 &
        child_pid=$!
        for ((i=0; i < 10; i++))
        do
            kill -0 "${child_pid}" > /dev/null 2>&1 || break
            sleep 1
        done

        #最后一次检查进程是否存在
        kill -0 "${child_pid}" > /dev/null 2>&1
        if [ $? -eq 0 ]; then
            #经过了10秒，rpm -qpi命令仍然没有结束，那么任务该进程挂住了，返回失败。
            error "[${FUNCNAME}:${LINENO}]" "rpm -qpi $rpms command execution timeout failure."
            return 1
        else
            #命令成功，回收僵尸进程
            wait "${child_pid}"
        fi
        
        rpm_name_r=$(cat "${rpm_output_file}" |  grep "^Name[[:space:]]*:" | awk '{print $3}')
        if [ "$rpm_name" == "$rpm_name_r" ]; then
            file="$rpms"
            rm -rf "${rpm_output_file}"
            break
        fi

        if [ "Y$result" != "Y" ]
        then
            result=$(echo "$rpm_name_r" | grep -E "${rpm_name}")
            if [ "Y$result" != "Y" ]
            then
                file="$rpms"
                rm -rf "${rpm_output_file}"
                break
            fi
        fi
        
        rm -rf "${rpm_output_file}"
    done < <(find "$rpms_dir" -name "${new_name}")
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "rpm command failed, please check."
        return 1
    fi

    #未找到对应的rpm包
    if [ -z "$file" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Don't find [${name}] in [${rpms_dir}]"
        return 1
    fi

    debug "[${file}] Leave [${FUNCNAME}:${LINENO}]"

    echo "${file}"

    return 0
}

###################################################
#
# 安装rpm包
#
###################################################
function install_rpm()
{
    local rpms_dir="$1"    #rpm包目录
    local name="$2"        #rpm包名称
    local file=""          #rpm包文件名

    debug "[${rpms_dir}:${name}] Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$name" ]; then
        error "[${FUNCNAME}:${LINENO}]" "name is null"
        return 1
    fi

    #检查并安装rpm包
    rpm -q "${name}" > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        file="$(get_rpm_file "${rpms_dir}" "${name}" "0")"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "get_rpm_file ${rpms_dir} ${name} failed"
            return 1
        fi
        
        ${CMD_RPM_INSTALL} "$file" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "rpm -ivh $file failed(${name})"
            return 1
        fi

        debug "Install [${file}] successfully"
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 安装expect相关的rpm包
#
###################################################
function install_expect()
{
    local expect="expect"  #expect包名
    local tcl="tcl"        #tcl包名(expect依赖此包)
    local rpms_dir=""      #rpm包目录

    debug "Enter [${FUNCNAME}:${LINENO}]"

    #检查expect命令
    hash expect > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        debug "Don't Need Install expect [${FUNCNAME}:${LINENO}]"
        return 0
    fi

    #非suse版本的需要增加类型
    if [ -z "$(echo "${g_local_os}" | grep "^suse")" ]; then
        if [ `uname -p` == "aarch64" ]; then
            expect="expect.aarch64"
            tcl="tcl.aarch64"
        else
            expect="expect.x86_64"
            tcl="tcl.x86_64"
        fi
    fi

    #获取本机OS对应的rpm包目录
    rpms_dir=$(get_pkg_dir_by_name ${g_local_os})
    if [ $? -ne 0 ]; then
        debug "get local rpms failed"
        return 1
    fi

    #检查rpms_dir
    check_rpms_dir "${rpms_dir}"
    if [ $? -ne 0 ]; then
        debug "check_rpms_dir failed"
        return 1
    fi

    #安装tcl包
    install_rpm "${rpms_dir}" "${tcl}"
    if [ $? -ne 0 ]; then
        debug "install_rpm ${rpms_dir} ${tcl} failed"
        return 1
    fi

    #安装expect包
    install_rpm "${rpms_dir}" "${expect}"
    if [ $? -ne 0 ]; then
        debug "install_rpm ${rpms_dir} ${expect} failed"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 安装dos2unix相关的rpm包
#
###################################################
function install_dos2unix()
{
    local dos2unix="dos2unix"  #dos2unix包名
    local rpms_dir=""      #rpm包目录

    debug "Enter [${FUNCNAME}:${LINENO}]"

    #检查dos2unix命令
    hash dos2unix > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        debug "Don't Need Install dos2unix [${FUNCNAME}:${LINENO}]"
        return 0
    fi
    
    if [ `uname -p` == "aarch64" ]; then
        dos2unix="dos2unix.aarch64"
    else
        dos2unix="dos2unix.x86_64"
    fi

    #获取本机OS对应的rpm包目录
    rpms_dir=$(get_pkg_dir_by_name ${g_local_os})
    if [ $? -ne 0 ]; then
        debug "get local rpms failed"
        return 1
    fi

    #检查rpms_dir
    check_rpms_dir "${rpms_dir}"
    if [ $? -ne 0 ]; then
        debug "check_rpms_dir failed"
        return 1
    fi

    #安装dos2unix包
    install_rpm "${rpms_dir}" "${dos2unix}"
    if [ $? -ne 0 ]; then
        debug "install_rpm ${rpms_dir} ${dos2unix} failed"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

function process_ssh_for_local()
{
    if [ ! -f "/etc/euleros-release" ]; then
        return 0
    fi

    cat /etc/euleros-release | grep "^Euler" > /dev/null 2>&1
    if [ $? -eq 0 ]
    then
        sed -i 's/^HostKeyAlgorithms/#HostKeyAlgorithms/' /etc/ssh/sshd_config
        sed -i 's/^PubkeyAcceptedKeyTypes/#PubkeyAcceptedKeyTypes/' /etc/ssh/sshd_config
	    service sshd restart > /dev/null 2>&1
    fi
}



###################################################
#
# 安装ssh相关的rpm包
#
###################################################
function install_ssh()
{
    if [ `uname -p` == "aarch64" ]; then
       local libedit="libedit.aarch64"            #libedit包名(openssh-clients依赖此包)
       local openssh_c="openssh-clients.aarch64"  #openssh-clients包名
    else
       local libedit="libedit.x86_64"            #libedit包名(openssh-clients依赖此包)
       local openssh_c="openssh-clients.x86_64"  #openssh-clients包名
    fi
   
    local rpms_dir=""                         #rpm包目录
    debug "Enter [${FUNCNAME}:${LINENO}]"

    #检查ssh命令
    hash ssh > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        debug "Don't Need Install ssh [${FUNCNAME}:${LINENO}]"
	
	if [ $(id -u) -eq 0 ]; then
            process_ssh_for_local
        else
            sudo ${INSTALL_SUDO_PATH} process_ssh_for_local
        fi

        return 0
    fi

    if [ -n "$(echo "${g_local_os}" | grep "^suse")" ]; then
        #脚本依赖于sshd服务,在suse中sshd与ssh命令在同一个包里
        #在redhat中,sshd与ssh分属于不同的rpm包
        #因此 只有redhat环境可能会出现缺失ssh命令的情况
        error "[${FUNCNAME}:${LINENO}]" "please install ssh command manually"
        return 1
    fi

    #获取本机OS对应的rpm包目录
    rpms_dir=$(get_pkg_dir_by_name ${g_local_os})
    if [ $? -ne 0 ]; then
        debug "get local rpms failed"
        return 1
    fi

    #检查rpms_dir
    check_rpms_dir "${rpms_dir}"
    if [ $? -ne 0 ]; then
        debug "check_rpms_dir failed"
        return 1
    fi

    #安装libedit包
    install_rpm "${rpms_dir}" "${libedit}"
    if [ $? -ne 0 ]; then
        debug "install_rpm ${rpms_dir} ${libedit} failed"
        return 1
    fi

    #安装openssh_c包
    install_rpm "${rpms_dir}" "${openssh_c}"
    if [ $? -ne 0 ]; then
        debug "install_rpm ${rpms_dir} ${openssh_c} failed"
        return 1
    fi

    if [ $(id -u) -eq 0 ]; then
        process_ssh_for_local
    else
        sudo ${INSTALL_SUDO_PATH} process_ssh_for_local
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

########################################################################
# 函数描述： 本地配置 打开/关闭 硬盘缓存
# 参数1说明：0 disable WCE
#            1 enable WCE
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_wce()
{
    local wce_val=$1
    local wcefile="${g_preinstall_path}/modules/070.autopart/autopart/ini/wce"
    if [ ${wce_val} -eq 0 ]; then
        echo 0 > ${wcefile}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "echo 0 > ${wcefile} failed."
            return 1
        fi
        
    elif [ ${wce_val} -eq 1 ]; then
        echo 1 > ${wcefile}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "echo 1 > ${wcefile} failed."
            return 1
        fi
    else
        error "[${FUNCNAME}:${LINENO}]" "wce_conf is not 0/1"
        return 1
    fi
    
    info "[${FUNCNAME}:${LINENO}]" "set wce value as ${wce_val} successfully."
    
    return 0
}

###################################################
#
# 本地功能增强(补充rpm包等)
#
###################################################
function enhance_local()
{
    debug "Enter [${FUNCNAME}:${LINENO}]"

    # 统信uos 1060e ping命令权限默认为744，omm用户没有权限执行ping命令
    chmod_ping
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to chmod /usr/bin/ping."
        return 1
    fi
    
    #安装expect相关的rpm包
    install_expect
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Preinstall depends on expect command."
        error "[${FUNCNAME}:${LINENO}]" "Please install expect tool or provide rpm packages."
        return 1
    fi

    #安装ssh相关的rpm包
    install_ssh
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Preinstall depends on ssh command."
        error "[${FUNCNAME}:${LINENO}]" "Please install ssh tool or provide rpm packages."
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

