#!/bin/bash

###################################################
# 功能: 在远端服务器运行命令 或 拷贝文件
###################################################

###################################################
#
# 在远端服务器运行命令
#
###################################################
function exec_cmd_r()
{
    local host="$1"       #远端主机名
    local timeout="$2"    #命令超时时间
    local command="$3"    #运行的命令
    local log="$4"        #是否记录日志(可以为空,默认记录日志)
    local ret=0           #返回值

    if [ -z "$log" ] || [ "$log" -ne 0 ]; then
        debug "[${host}:${timeout}:${command}]Enter [${FUNCNAME}:${LINENO}]"
    fi

    #入参判断
    if [ -z "$host" ]; then
        error "[${FUNCNAME}:${LINENO}]" "host is null"
        return 1
    fi

    if [ -z "$timeout" ]; then
        error "[${FUNCNAME}:${LINENO}]" "timeout is null"
        return 1
    fi

    if [ -z "$command" ]; then
        error "[${FUNCNAME}:${LINENO}]" "command is null"
        return 1
    fi
    
    #log为0时,不记录日志
    if [ -z "$log" ]; then
        log=1
    fi

    #当log入参为0时，不输出任何信息；
    #当log入参为1时，把输出信息定向到日志文件；
    #当log入参为其他时，回显输出信息。
    if [ $log -eq 0 ]; then
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "ssh-cmd" -c "$command" > /dev/null
        ret=$?
    elif [ $log -eq 1 ]; then
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "ssh-cmd" -c "$command" >> "$g_log_file"
        ret=$?
    else
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "ssh-cmd" -c "$command"
        ret=$?
    fi
    
    if [ $ret -ne 0 ]; then
        debug "Run $g_remote -i $host -P $g_port -u $g_user_name -a $g_passwd_aes_file -t $timeout -m ssh-cmd -c $command failed, ret code:${ret}"
    fi

    if [ -z "$log" ] || [ "$log" -ne 0 ]; then
        debug "Leave [${FUNCNAME}:${LINENO}]"
    fi

    return $ret
}

###################################################
#
# 把文件拷贝至远端服务器
#
###################################################
function cp_file_to_r()
{
    local host="$1"       #远端主机名
    local timeout="$2"    #命令超时时间
    local src="$3"        #源文件
    local dst="$4"        #目标文件
    local log="$5"        #是否记录日志(可以为空,默认记录日志)
    local ret=0           #返回值

    if [ -z "$log" ] || [ "$log" -ne 0 ]; then
        debug "[${host}:${timeout}:${src}:${dst}]Enter [${FUNCNAME}:${LINENO}]"
    fi

    #入参判断
    if [ -z "$host" ]; then
        error "[${FUNCNAME}:${LINENO}]" "host is null"
        return 1
    fi

    if [ -z "$timeout" ]; then
        error "[${FUNCNAME}:${LINENO}]" "timeout is null"
        return 1
    fi

    if [ -z "$src" ]; then
        error "[${FUNCNAME}:${LINENO}]" "src is null"
        return 1
    fi

    if [ -z "$dst" ]; then
        error "[${FUNCNAME}:${LINENO}]" "dst is null"
        return 1
    fi

    #log为0时,不记录日志
    if [ -z "$log" ]; then
        log=1
    fi

    if [ $log -ne 0 ]; then 
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "scp-out" -s "$src" -d "$dst" >> "$g_log_file"
        ret=$?
    else
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "scp-out" -s "$src" -d "$dst" > /dev/null
        ret=$?
    fi

    if [ $ret -ne 0 ]; then
        debug "Run $g_remote -i $host  -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file -t $timeout -m scp-out -s $src -d $dst failed, ret code:${ret}"
    fi

    if [ -z "$log" ] || [ "$log" -ne 0 ]; then
        debug "Leave [${FUNCNAME}:${LINENO}]"
    fi

    return $ret
}

###################################################
#
# 把远端服务器文件拷贝至本地
#
###################################################
function cp_file_from_r()
{
    local host="$1"       #远端主机名
    local timeout="$2"    #命令超时时间
    local src="$3"        #源文件
    local dst="$4"        #目标文件
    local log="$5"        #是否记录日志(可以为空,默认记录日志)
    local ret=0           #返回值

    if [ -z "$log" ] || [ "$log" -ne 0 ]; then
        debug "[${host}:${timeout}:${src}:${dst}]Enter [${FUNCNAME}:${LINENO}]"
    fi

    #入参判断
    if [ -z "$host" ]; then
        error "[${FUNCNAME}:${LINENO}]" "host is null"
        return 1
    fi

    if [ -z "$timeout" ]; then
        error "[${FUNCNAME}:${LINENO}]" "timeout is null"
        return 1
    fi

    if [ -z "$src" ]; then
        error "[${FUNCNAME}:${LINENO}]" "src is null"
        return 1
    fi

    if [ -z "$dst" ]; then
        error "[${FUNCNAME}:${LINENO}]" "dst is null"
        return 1
    fi

    #log为0时,不记录日志
    if [ -z "$log" ]; then
        log=1
    fi

    if [ $log -ne 0 ]; then 
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "scp-in" -s "$src" -d "$dst" >> "$g_log_file"
        ret=$?
    else
        bash $g_remote -i "$host" -F "$g_ip_model" -P "$g_port" -u "$g_user_name" -a "$g_passwd_aes_file" \
                  -t "$timeout" -m "scp-in" -s "$src" -d "$dst" > /dev/null
        ret=$?
    fi
    
    if [ $ret -ne 0 ]; then
        debug "Run $g_remote -i $host  -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file -t $timeout -m scp-in -s $src -d $dst failed, ret code:${ret}"
    fi

    if [ -z "$log" ] || [ "$log" -ne 0 ]; then
        debug "Leave [${FUNCNAME}:${LINENO}]"
    fi

    return $ret
}