#!/bin/bash

declare -a g_hosts_array=()    #用于保存 各节点的主机名（可能是IP地址）的数组
declare -a g_ipaddr_array=()   #用于保存 各节点的IP地址的数组
declare -a g_processor_array=()   #用于保存 各节点的IP地址的数组
declare -a g_parted_array=()   #用于保存 分区配置文件 数组
declare -a g_service_ip_array=()   #用于保存 各节点的service IP地址 数组
declare -a g_hostname_array=()     #用于保存 各节点的主机名(而不会是IP地址) 数组

##############################################################
## @Usage checkIPv6
## @Return
## @Description 检查是否是有效IP格式
##############################################################
checkIPv6()
{
    local inputIp="$1"

    regex="^([0-9a-fA-F]{0,4}:){1,7}[0-9a-fA-F]{0,4}$"

    if [ `echo $inputIp | grep -P $regex` ];then
        return 0;
    else
        return 1;
    fi
}


###################################################
#
# 检查是否可以成功登陆各节点
#
###################################################
function check_login()
{
    local hosts="$1"
    local host=""
    local ipaddr=""
    local i=0
    local num=0
    local line=""
    local conf=""
    local service_ip=""
    local hostname=""
    local hostinfo=
    local ret=0

    debug "[${hosts_conf}] Enter [${FUNCNAME}:${LINENO}]"
    
    #变量初始化
    g_hosts_array=() 
    g_ipaddr_array=() 
    g_parted_array=()
        
    #生成主机名数组
    g_hosts_array=($(echo "$hosts" | tr ',' ' '))

    #遍历各节点,逐个检查
    for ((i=0;i<${#g_hosts_array[@]};i++))
    do
        host=${g_hosts_array[${i}]}
        # host就是主机的IP地址，所以直接赋值
        ipaddr="$host"
        if [ ${g_ip_model} == "IPv4" ]; then
            echo "$ipaddr" | grep "^[0-9]*.[0-9]*.[0-9]*.[0-9]*$" > /dev/null
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "ipaddr IPv4 [${ipaddr}] invalid ${host}"
                return 1
            fi
        else
             checkIPv6 "$ipaddr" || { error "[${FUNCNAME}:${LINENO}]" "ipaddr IPv6 [${ipaddr}] invalid ${host}"; return 1; }
        fi

        #把获取的ip地址保存在ip地址数组中
        g_ipaddr_array[${i}]="$ipaddr"
        
        service_ip=""
        hostname=""
        hostinfo=$(echo "${g_hostname_conf}" | tr ';' '\n' | grep "^$ipaddr#")
        if [ $? -eq 0 ]; then
            service_ip="$(echo "$hostinfo" | awk -F'#' '{print $2}')"
            hostname="$(echo "$hostinfo" | awk -F'#' '{print $3}')"
            g_service_ip_array[${i}]="${service_ip}"
            g_hostname_array[${i}]="${hostname}"
            
            if [ -z "${service_ip}" ]; then
                error "[${FUNCNAME}:${LINENO}]" "host[${host}] no service_ip"
                return 1
            fi

            debug "[${g_hosts_array[${i}]}:${g_ipaddr_array[${i}]}:${g_service_ip_array[${i}]}:${g_hostname_array[${i}]}] [${FUNCNAME}:${LINENO}]"
        fi

        # 填充 g_parted_array 数组
        if [ "$g_parted" -eq 0 ]; then
            debug "[${g_hosts_array[${i}]}:${g_ipaddr_array[${i}]}] [${FUNCNAME}:${LINENO}]"
            continue
        fi

        conf=$(echo "$g_parted_conf" | tr ';' '\n' | grep -E '^'${host}',|,'${host}',|,'${host}'#|^'${host}'#' | awk -F'#' '{print $2}' 2>/dev/null)
        if [ -z "$conf" ]; then
            error "[${FUNCNAME}:${LINENO}]" "host[${host}] no parted_conf"
            return 1
        fi
        
        g_parted_array[${i}]="$conf"
        
        debug "[${g_hosts_array[${i}]}:${g_ipaddr_array[${i}]}:${g_parted_array[${i}]}] [${FUNCNAME}:${LINENO}]"
    done

    num="$(echo "${g_ipaddr_array[@]}" | tr ' ' '\n' | sort | uniq | wc -l)"

    #判断是否有不同主机名指向同一个IP地址的情况
    if [ "$num" -ne "${#g_ipaddr_array[@]}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr repeat [${g_ipaddr_array[*]}]"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}


###################################################
#
# 检查是否可以成功登陆各节点
#
###################################################
function only_check_login()
{
    local hosts="$1"
    local host=""
    local ipaddr=""
    local i=0
    local num=0
    local line=""
    local conf=""
    local ret=0

    debug "[${hosts_conf}] Enter [${FUNCNAME}:${LINENO}]"
    
    #变量初始化
    g_hosts_array=() 
    g_ipaddr_array=() 
    
    #生成主机名数组
    g_hosts_array=($(echo "$hosts" | tr ',' ' '))

    #遍历各节点,逐个检查
    for ((i=0;i<${#g_hosts_array[@]};i++))
    do
        host=${g_hosts_array[${i}]}

        #获取主机名对应的ip地址
        ipaddr=$(ping -c 1 "$host" -i 0.2 -W 1 | grep "^PING" | awk -F'(' '{print $2}' | awk -F')' '{print $1}')
        echo "$ipaddr" | grep "^[0-9]*.[0-9]*.[0-9]*.[0-9]*$" > /dev/null
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "ipaddr[${ipaddr}] invalid ${host}"
            return 1
        fi

        #把获取的ip地址保存在ip地址数组中
        g_ipaddr_array[${i}]="$ipaddr" 
    done

    num="$(echo "${g_ipaddr_array[@]}" | tr ' ' '\n' | sort | uniq | wc -l)"

    #判断是否有不同主机名指向同一个IP地址的情况
    if [ "$num" -ne "${#g_ipaddr_array[@]}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr repeat [${g_ipaddr_array[*]}]"
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
