#!/bin/bash

###################################################
# 功能: 检查远端服务器的OS是否匹配当前配置OS
#
# 接口:
#   函数: check_osname_r
#
#   备注:
###################################################


###################################################
#
# 功能:  获取远端OS版本
#
# cat /etc/centos-release 
# CentOS release 6.2 (Final)
#
# cat /etc/oracle-release 
# Oracle Linux Server release 6.2
#
# cat /etc/redhat-release 
# Red Hat Enterprise Linux Server release 6.4 (Santiago)
#
# cat /etc/SuSE-release 
# SUSE Linux Enterprise Server 11 (x86_64)
# VERSION = 11
# PATCHLEVEL = 1
#
###################################################
function get_os_version_r()
{
    local ipaddr="$1"   #主机名或ip地址
    local cent_release="/etc/centos-release"
    local ole_release="/etc/oracle-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local euler_release="/etc/euleros-release"
    local neokylin_relase="/etc/neokylin-release"
    local kylin_productinfo="/etc/.productinfo"
    local bclinux_release="/etc/bclinux-release"
    local openeuler_release="/etc/openEuler-release"
    local kylin_relase="/etc/kylin-release"
    local linxos_release="/etc/linxos-release"
    local name=""
    local version=""
    local local_os_version_file=${g_ctrl_space}/${ipaddr}_os_release

    debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$ipaddr" ]; then
        error "[${FUNCNAME}:${LINENO}]"  "ipaddr is null"
        return 1
    fi
    
    # 检查是否为北京凝思OS x86，OS命令规则和arm不同，需要适配
    exec_cmd_r "$ipaddr" "120" "ls ${linxos_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${linxos_release}" "${local_os_version_file}" "0" > /dev/null
        name=$(awk '{print $1"-"$2}' "${linxos_release}")
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    #检查是否为 euler
    exec_cmd_r "$ipaddr" "120" "ls ${euler_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${euler_release}" "${local_os_version_file}" "0" > /dev/null
        if grep -q "release" "${local_os_version_file}"; then
            name="$(cat $local_os_version_file | sed 's/x86_64//g' | awk -F " |SP|)" '{print "euleros-"$3"sp"$5}')"
        else
            name="euleros-$(cat "${local_os_version_file}" | awk '{print $2}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
        fi
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    exec_cmd_r "$ipaddr" "120" "ls ${neokylin_relase}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${neokylin_relase}" "${local_os_version_file}" "0" > /dev/null
        if grep -q "release" "${local_os_version_file}"; then
            name="$(cat "${local_os_version_file}" | awk '{print $6}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            if [ -z "${name}" ];then
                name="$(cat "${local_os_version_file}" | awk '{print $4}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            fi
            name="neokylin-$(echo ${name/update/'.'})"
        fi
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    #检查是否为 bclinux
    exec_cmd_r "$ipaddr" "120" "ls ${bclinux_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${bclinux_release}" "${local_os_version_file}" "0" > /dev/null
        if grep -q "Euler" "${local_os_version_file}";then
            version=$(cat ${local_os_version_file} | awk '{print $7}' | awk -F. '{print $1"."$2}')
        else
            version=$(cat ${local_os_version_file} | awk '{print $5}' | awk -F. '{print $1"."$2}')
        fi
        name="bclinux-${version}"
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    #检查是否为 CentOS
    exec_cmd_r "$ipaddr" "120" "ls ${cent_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${cent_release}" "${local_os_version_file}" "0" > /dev/null
        if grep -q "Linux" "${local_os_version_file}"; then
            name="centos-$(cat "${local_os_version_file}" | awk '{print $4}' | awk -F. '{print $1"."$2}')"
        else
            name="centos-$(cat "${local_os_version_file}" | awk '{print $3}')"
        fi
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    #检查是否为 Oracle
    exec_cmd_r "$ipaddr" "120" "ls ${ole_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${ole_release}" "${local_os_version_file}" "0" > /dev/null
        version=$(cat ${local_os_version_file} | grep "^Oracle Linux" | awk '{print $5}')
        name="oracle-${version}"
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    #检查是否为 RedHat
    exec_cmd_r "$ipaddr" "120" "ls ${rhel_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${rhel_release}" "${local_os_version_file}" "0" > /dev/null
        version=$(cat ${local_os_version_file} | grep "^Red Hat Enterprise" | awk '{print $7}')
        name="redhat-${version}"
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    
    #检查是否为 SuSE
    exec_cmd_r "$ipaddr" "120" "ls ${sles_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${sles_release}" "${local_os_version_file}" "0" > /dev/null
        version="$(cat ${local_os_version_file} | grep "^VERSION" | awk '{print $3}')"
        version="${version}"."$(cat ${local_os_version_file} | grep "^PATCHLEVEL" | awk '{print $3}')"
        name="suse-${version}"
        echo $name
        rm -rf ${local_os_version_file}
        return 0
    fi
    exec_cmd_r "$ipaddr" "120" "ls ${kylin_relase}" "0" > /dev/null
    if [ $? -eq 0 ] ; then
        cp_file_from_r "$ipaddr" "240" "${sles15_release}" "${local_os_version_file}" "0" > /dev/null
        if [ "KylinSecOS" = "$(grep "^ID=" ${local_os_version_file} | awk -F "\"" '{print $2}')" ];then
           name="kylinSecOS-$(grep "^VERSION_ID=" "${local_os_version_file}" | awk -F "\"" '{print $2}')"
           echo $name
           rm -rf ${local_os_version_file}
           return 0
        fi
    fi

    #检查是否为 SuSE15
    exec_cmd_r "$ipaddr" "120" "ls ${sles15_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        cp_file_from_r "$ipaddr" "240" "${sles15_release}" "${local_os_version_file}" "0" > /dev/null
        if [ "suse" = "$(cat "${local_os_version_file}" | grep "^ID_LIKE=" | awk -F "\"" '{print $2}')" ]; then
            name="suse-$(cat "${local_os_version_file}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            rm -rf ${local_os_version_file}
            return 0
        fi
        idLike=$(grep "^ID_LIKE=" "${local_os_version_file}" | awk -F "\"" '{print $2}')
        ID=$(grep "^ID=" "${local_os_version_file}" | awk -F '=' '{print $2}' | tr -d '"')
        VERSION_ID=$(grep "^VERSION_ID=" "${local_os_version_file}" | awk -F "\"" '{print $2}')
        if [ "${ID}" = "kylin" ] && [ "${idLike}" = "fedora" ]; then
            name="$ID-${VERSION_ID}-fedora"
            echo "$name"
            return 0
        fi
        if [ "${ID}" = "kylin" ]; then
            local local_kylin_productinfo_file=${g_ctrl_space}/${ipaddr}_kylin_productinfo
            cp_file_from_r "$ipaddr" "240" "${kylin_productinfo}" "${local_kylin_productinfo_file}" "0" > /dev/null
            local MINOR_VERSION=$(grep "${VERSION_ID}" "${local_kylin_productinfo_file}" | awk -F "(" '{print $2}' | awk -F ")" '{print $1}')
            name="$ID-${VERSION_ID}-${MINOR_VERSION}"
            echo "$name"
            rm -rf ${local_kylin_productinfo_file}
            return 0
        fi
        if [ "${ID}" = "uos" ]; then
            local local_uos_version_file=${g_ctrl_space}/${ipaddr}_uos_version
            cp_file_from_r "$ipaddr" "240" "/etc/os-version" "${local_uos_version_file}" "0" > /dev/null
            name="$ID-$(cat ${local_uos_version_file} | grep MinorVersion | awk -F "=" '{print $2}')"
            echo "$name"
            rm -rf ${local_uos_version_file}
            return 0
        fi
        #获取openEuler-SPx的版本
        if [ -f "${openeuler_release}" ] && [ "${ID}" = "openEuler" ];then
           cp_file_from_r "$ipaddr" "240" "${openeuler_release}" "${local_os_version_file}" "0" > /dev/null
           local MINOR_VERSION=$(cat "${local_os_version_file}" | awk -F "-" '{print $2}' | awk -F ")" '{print $1}')
           #openEuler-20.03-SP1保持之前的规则提取版本号
           local MINOR_VERSION_ID="${VERSION_ID}-${MINOR_VERSION}"
           if [ -n "${MINOR_VERSION}" -a "${MINOR_VERSION_ID}" != "20.03-SP1" ] ; then
               name="$ID-${VERSION_ID}-${MINOR_VERSION}"
               echo "$name"
               return 0
           fi
        fi
        name="$ID-$VERSION_ID"
        echo "$name"
        rm -rf ${local_os_version_file}
        return 0
        
    fi
    
    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 1
}

###################################################
#
# 检查远端服务器的OS是否匹配当前配置OS
#
###################################################
function check_osname_r()
{
    local ipaddr="$1"  #主机名或ip地址
    local osname=""    #OS名称
    local pkg_dir=""

    debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$ipaddr" ]; then
        error "[${FUNCNAME}:${LINENO}]"  "Parameter error. ipaddr is null."
        return 1
    fi
    
    if [ "$g_add_pkg" -eq 0 ]; then
        debug "[${ipaddr}][g_add_pkg=0] [${FUNCNAME}:${LINENO}]"
        return 0
    fi
    
    #获取远端的OS版本
    osname="$(get_os_version_r "${ipaddr}")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]"  "get osname(${osname}) from ${ipaddr} failed"
        return 1
    fi
    
    #逐项匹配
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi

        #获取对应的目录
        type="$(echo $line | awk -F':' '{print $1}')"
        if [ "${type}" == "${osname}" ]; then
            pkg_dir="$(echo $line | awk -F':' '{print $2}')"
        fi

        debug "g_pkgs_dir:${line}:${type}:${osname}:${pkg_dir}"
    done < <(echo "$g_pkgs_dir" | tr ';' '\n')
    
    if [ -z "${pkg_dir}" ]; then
        error "[${FUNCNAME}:${LINENO}]"  "current os version is ${osname} on ${ipaddr}."
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
