#!/bin/bash

###################################################
# 功能: 对各个节点的OS进行适配
###################################################

###################################################
# 失败时返回值范围: 1, 10-100
# 
###################################################

###################################################
# 警告: 101-120
# 
###################################################

###################################################
# 全局变量
###################################################
g_GENERAL_ERROR=1
g_UPGRADE_OS_FAILED=2
g_UPGRADE_SECURITY_FAILED=3
g_UPGRADE_RANDOM_DEVICE_FAILED=4

g_UPGRADE_GENERAL_ERROR=11
g_UPGRADE_STOP_DSKMGT_ERROR=12
g_UPGRADE_START_DSKMGT_ERROR=13

g_COMMIT_GENERAL_ERROR=21
g_COMMIT_NOT_UPGRADED_ERROR=22

g_ROLL_GENERAL_ERROR=31
g_ROLL_STOP_DSKMGT_ERROR=32
g_ROLL_START_DSKMGT_ERROR=33


###################################################
#
# 根据返回码 回显日志
#
###################################################
function show_upgrade_result()
{
    local ret=$1
    local ip=$2
    
    case "$ret" in    
        #通用错误
        "$g_GENERAL_ERROR")  
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] general error" 
            return 1
        ;;
        
        "$g_UPGRADE_OS_FAILED")  
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] Setup os failed" 
            return 1
        ;;
        
        "$g_UPGRADE_SECURITY_FAILED")  
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] Security reinforcement failed" 
            return 1
        ;;
        
        "$g_UPGRADE_RANDOM_DEVICE_FAILED")  
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] Set feed random data failed" 
            return 1
        ;;
         
        #upgrade 错误
        "$g_UPGRADE_GENERAL_ERROR")  
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] upgrade error, general error." 
            return 1
        ;;
     
        "$g_UPGRADE_STOP_DSKMGT_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] upgrade error, stop diskmgt failed."
            return 1
        ;;
           
        "$g_UPGRADE_START_DSKMGT_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] upgrade error, start diskmgt failed." 
            return 1
        ;;
         
        #commit 错误
        "$g_COMMIT_GENERAL_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] commit error, general error." 
            return 1
        ;;
        
        "$g_COMMIT_NOT_UPGRADED_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] commit error, general error." 
            return 1
        ;;
         
        #roll back 错误
        "$g_ROLL_GENERAL_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] roll back error, general error." 
            return 1
        ;;
        
        "$g_ROLL_STOP_DSKMGT_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] roll back error, stop diskmgt failed." 
            return 1
        ;;
         
        "$g_ROLL_START_DSKMGT_ERROR")
            error "[${FUNCNAME}:${LINENO}]" "[${ip}] roll back error, start diskmgt failed." 
            return 1
        ;;
    esac 
    
    error "[${FUNCNAME}:${LINENO}]" "[${ip}] unknown error" 
    return 1
}



g_UPGRADE_UPGRADED_WARN=101
g_UPGRADE_NOT_INSTALLED_WARN=102
g_UPGRADE_IS_LATEST_VERSION_WARN=107

g_COMMIT_COMMITTED_WARN=103
g_COMMIT_NOT_INSTALLED_WARN=104

g_ROLL_ROLLED_WARN=105
g_ROLL_NOT_INSTALLED_WARN=106


###################################################
#
# 根据返回码 警告日志
#
###################################################
function show_upgrade_warn_result()
{
    local ret=$1
    local ip=$2
    
    case "$ret" in    
        "$g_UPGRADE_UPGRADED_WARN")
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] upgrade warn, upgrade may be done, skipped." 
            return 1
        ;;
        
        "$g_UPGRADE_NOT_INSTALLED_WARN")  
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] upgrade warn, diskmgt may not be installed, skipped." 
            return 1
        ;;
        
        "$g_UPGRADE_IS_LATEST_VERSION_WARN")  
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] upgrade warn, diskmgt have the latest version, skiped." 
            return 1
        ;;
        
        "$g_COMMIT_COMMITTED_WARN")
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] commit warn, diskmgt may be committed, skipped." 
            return 1
        ;;
        
        "$g_COMMIT_NOT_INSTALLED_WARN")
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] commit warn, diskmgt may not be installed, skipped." 
            return 1
        ;;
        
        "$g_ROLL_ROLLED_WARN")
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] roll back warn, diskmgt may be rolled, skipped." 
            return 1
        ;;
        
         "$g_ROLL_NOT_INSTALLED_WARN")
            warn "[${FUNCNAME}:${LINENO}]" "[${ip}] roll back warn, diskmgt may not be installed, skipped." 
            return 1
        ;;
    esac 
    
    warn "[${FUNCNAME}:${LINENO}]" "[${ip}] unknown warn" 
    return 1
}


###################################################
#
# 适配各节点
#
###################################################
function upgrade_node()
{
    local ipaddr="$1"     #主机名或ip地址
    local index="$2"      #数组索引
    local parted_conf=""  #节点对应的格式化配置
    local services_ip=""  #节点对应的services IP地址
    local hostname=""     #节点对应的主机名
    local ret=0
    
    debug "[${ipaddr}:${index}]Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$ipaddr" ] || [ -z "$index" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Parameter error."
        return 1
    fi
    
    #ping目标端 检测是否可以ping通
    ping -c 1 "$ipaddr" -i 0.2 -W 1 > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "ping ${ipaddr} failed."
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #登陆到各节点并执行命令
    exec_cmd_r "$ipaddr" "120" "whoami" "0" > /dev/null
    ret=$?
    if [ $ret -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Can not login ${ipaddr}, permission denied. Return code:${ret}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #在各个节点创建工作空间
    gen_workspace "$g_work_space" "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Gen workspace failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "1"
    
    #检查远端服务器的OS是否匹配当前配置OS
    check_osname_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "os version is inconsistent with g_pkgs_dir=$g_pkgs_dir on ${ipaddr}."
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #在远端服务器安装ssh相关的rpm包
    install_ssh_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Install ssh failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "3"

    if [ "${ACTION}" == "UPGRADE" ]; then
        #在远端服务器安装缺失的rpm包
        add_packages_r "${ipaddr}"
        if [ $? -ne 0 ]; then
            info "[${FUNCNAME}:${LINENO}]" "Add packages failed on ${ipaddr}"
            set_node_run_failure "${ipaddr}"
            return 1
        fi
        
        #设置进度
        set_node_schedule "${ipaddr}" "8"
    fi

    #把适配脚本拷贝至远端服务器
    cp_file_to_r "$ipaddr" "240" "${g_preinstall_path}/modules" "${g_work_space}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Copy files to ${ipaddr} failed"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "12"

    #配置core dump
    config_coredump_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "config core dump failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "20"

    g_parted=0         #升级禁止格式化磁盘
    parted_conf=""     #升级禁止格式化磁盘
    hostname=""        #升级禁止设置用户名
    services_ip=""     #升级禁止设置用户名
    g_swap_off=0       #升级禁止设置swap

    #在各节点上执行适配
    exec_cmd_r "$ipaddr" "18000" "bash ${g_work_space}/modules/upgrade.sh -i \"$ipaddr\" -p \"$g_parted\" \
            -c \"$parted_conf\" -d \"$g_debug\" -k \"$g_check\" -o \"$g_support_os\" -w \"$g_swap_off\" -a \"$ACTION\" -r \"$g_feed_random_setup\" -v \"$g_ip_model\""
    ret=$?
    
    #返回失败
    if [ $ret -ge 1 -a $ret -le 100 ]; then
        show_upgrade_result "${ret}" "${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return $ret
    fi
    
    #返回值 101-120 为告警
    if [ $ret -ge 101 -a $ret -le 120 ]; then
        show_upgrade_warn_result "${ret}" "${ipaddr}"
    fi
    
    #删除节点上的工作空间
    del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1

    #设置进度为
    set_node_schedule "${ipaddr}" "100"

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 启动OS适配: 共分为6个步骤
#  1:在各节点创建工作目录        /tmp/fi-preinstall
#  1:把适配脚本拷贝至各节点     拷贝至 /tmp/fi-preinstall
#  2:在各节点调用启动适配脚本   /tmp/fi-preinstall/modules/install.sh
#
###################################################
function start_upgrade()
{
    local i=""       #循环变量
    local ipaddr=""

    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    #初始化
    init_schedule_array "${#g_ipaddr_array[@]}"

    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"

        #并行对各个节点进行适配
        upgrade_node "${ipaddr}" "${i}" &
        g_processor_array[${i}]=$!
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
