#!/bin/bash

###################################################
#
# 功能:  在远端配置操作系统coredump
#
###################################################
function config_coredump_r() {
    if [[ "$UID" -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
        local ipaddr="$1"
        local ret=0
        exec_cmd_r "$ipaddr" "120" "sudo ${INSTALL_SUDO_PATH} preinstall_config_coredump $g_core_dump $g_core_dump_dir"
        ret=$?
        if [ $? -ne 0 ]; then
            echo "Failed to execute sudo config core dump."
            return 1
        fi
        return 0
    else
        local ipaddr="$1"
        local ret=0
        local core_pattern="%e#%h#%t"
        local core_pattern_file="/proc/sys/kernel/core_pattern"
        local limitstr="omm soft core unlimited"
        local limits_file="/etc/security/limits.conf"
        local sysctl_conf_file="/etc/sysctl.conf"

        debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]"

        if [ $g_core_dump != 1 ]; then
            debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]: core dump is not opened."
            # clear limits config
            local grep_limits_command="grep \"^${limitstr}\" ${limits_file}"
            exec_cmd_r "$ipaddr" "120" "${grep_limits_command}"
            ret=$?
            if [ $? -eq 0 ]; then
                tmp=$(echo "^omm soft core unlimited" | sed -e 's/\//\\\//g')
                local clear_cmd="sed -i \"/$tmp/d\" ${limits_file}"
                exec_cmd_r "$ipaddr" "120" "${clear_cmd}"
                ret=$?
                if [ $? -ne 0 ]; then
                    echo "Failed to clear /etc/security/limits.conf."
                    return 1
                fi
            fi
            return 0
        fi

        if [ ! -z "$g_core_dump_dir" ]; then
            # 1.检查core_dump_dir合法性校验 
            if [ "${g_core_dump_dir:0:1}" != "/" ]; then
                error "[${FUNCNAME}:${LINENO}]" "g_core_dump_dir must be an absolute path."
                return 1 
            fi
            local n=${#g_core_dump_dir}-1 
            if [ "${g_core_dump_dir:n:1}" == "/" ]; then
                error "[${FUNCNAME}:${LINENO}]" "g_core_dump_dir cannot end with a slash."
                return 1 
            fi
            if [ "${g_core_dump_dir:1:1}" == "." ]; then
                error "[${FUNCNAME}:${LINENO}]" "g_core_dump_dir contains an invalid character."
                return 1
            fi

            #2. core_dump_dir必须为空
            exec_cmd_r "$ipaddr" "120" "ls ${g_core_dump_dir}/* > /dev/null 2>&1"
            ret=$?
            if [ $ret -eq 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "g_core_dump_dir must be empty , ${g_core_dump_dir} is not empty."
                return 1
            fi

            core_pattern="${g_core_dump_dir}/${core_pattern}"
        fi

        #检查入参
        if [ -z "$ipaddr" ]; then
            error "[${FUNCNAME}:${LINENO}]" "Parameter error. ipaddr is null."
            return 1
        fi

        #检查远端是否存在core_pattern和limits.conf文件
        exec_cmd_r "$ipaddr" "120" "ls ${core_pattern_file}"
        ret=$?
        if [ $ret -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "check core_pattern on [${ipaddr}] failed"
            return 1
        fi

        exec_cmd_r "$ipaddr" "120" "ls ${limits_file}"
        ret=$?
        if [ $ret -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "check limits.config on [${ipaddr}] failed"
            return 1
        fi

        exec_cmd_r "$ipaddr" "120" "ls ${sysctl_conf_file}"
        ret=$?
        if [ $ret -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "check sysctl.config on [${ipaddr}] failed"
            return 1
        fi

        #检查远端core_pattern和limits.conf文件中是否已配置,如果没有，怎添加配置
        local grep_core_pattern_command="grep \"^${core_pattern}\" ${core_pattern_file}"
        exec_cmd_r "$ipaddr" "120" "${grep_core_pattern_command}"
        ret=$?
        if [ $ret -eq 1 ]; then
            local core_pattern_config_command="echo \"${core_pattern}\" > ${core_pattern_file}"
            exec_cmd_r "$ipaddr" "120" "${core_pattern_config_command}"
            ret=$?
            if [ $ret -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "config core_pattern on [${ipaddr}] failed"
                return 1
            fi
        fi
        debug "[${FUNCNAME}:${LINENO}]" "config core_pattern on [${ipaddr}] success"

        local grep_limits_command="grep \"^${limitstr}\" ${limits_file}"
        exec_cmd_r "$ipaddr" "120" "${grep_limits_command}"
        ret=$?
        if [ $ret -eq 1 ]; then
            local limits_config_command="echo \"${limitstr}\" >> ${limits_file}"
            exec_cmd_r "$ipaddr" "120" "${limits_config_command}"
            ret=$?
            if [ $ret -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "config limits.config on [${ipaddr}] failed"
                return 1
            fi
        fi
        debug "[${FUNCNAME}:${LINENO}]" "config limits.config on [${ipaddr}] success"

        local grep_sysctl_command="grep \"^kernel.core_pattern = ${core_pattern}\" ${sysctl_conf_file}"
        exec_cmd_r "$ipaddr" "120" "${grep_sysctl_command}"
        ret=$?
        if [ $ret -eq 1 ]; then
            local grep_kernel_command="grep \"^kernel.core_pattern\" ${sysctl_conf_file}"
            exec_cmd_r "$ipaddr" "120" "${grep_kernel_command}"
            ret=$?
            if [ $ret -eq 0 ]; then
                local sysctl_config_command="sed -i \"/^kernel.core_pattern.*/d\" ${sysctl_conf_file}"
                exec_cmd_r "$ipaddr" "120" "${sysctl_config_command}"
                ret=$?
                if [ $ret -ne 0 ]; then
                    error "[${FUNCNAME}:${LINENO}]" "delete old pattern on [${ipaddr}] failed"
                    return 1
                fi
            fi

            local sysctl_config_command="echo \"kernel.core_pattern = ${core_pattern}\" >> ${sysctl_conf_file}"
            exec_cmd_r "$ipaddr" "120" "${sysctl_config_command}"
            ret=$?
            if [ $ret -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "config sysctl.confg on [${ipaddr}] failed"
                return 1
            fi
        fi
        debug "[${FUNCNAME}:${LINENO}]" "config sysctl.confg on [${ipaddr}] success"

        return 0
    fi
}
