#!/bin/bash

################################################
# Usage: get_missing_rpms_lst.sh
# 功能:  获取必须的rpm包列表
#
# 参数:  参数1  节点标识  -- 节点IP地址
#        参数2  日志信息 --  1表示输出日志
################################################

###################################################
# debug日志开关
###################################################
g_lst_debug=0

###################################################
# 输出debug日志
###################################################
function debug()
{
    #输出debug日志
    if [ "${g_lst_debug}" -eq 1 ]; then
        echo "Debug:" "$@"
    fi
}

###################################################
#
# 获取OS的名称
#
###################################################
function get_osname()
{
    local cent_release="/etc/centos-release"
    local ole_release="/etc/oracle-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local euler_release="/etc/euleros-release"
    local neokylin_relase="/etc/neokylin-release"
    local kylin_productinfo="/etc/.productinfo"
    local bclinux_release="/etc/bclinux-release"
    local openeuler_release="/etc/openEuler-release"
    local kylin_relase="/etc/kylin-release"
    local linxos_release="/etc/linxos-release"
    local name=""
    
    if [ -f "${linxos_release}" ]; then
        name=$(awk '{print $1"-"$2}' ${linxos_release})
        echo "$name"
        return 0
    fi
    
    #获取Euler的版本
    #必须在redhat之前检查
    if [ -f "${euler_release}" ]; then
        if grep -q "release" "${euler_release}"; then
            name="$(cat $euler_release | sed 's/x86_64//g' | awk -F " |SP|)" '{print "euleros-"$3"sp"$5}')"
        else
            name="euleros-$(cat "${euler_release}" | awk '{print $2}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
        fi
        echo "$name"
        return 0
    fi
    
    if [ -f "${neokylin_relase}" ];then
        if grep -q "release" "${neokylin_relase}"; then
            name="$(cat "${neokylin_relase}" | awk '{print $6}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            if [ -z "${name}" ];then
                name="$(cat "${neokylin_relase}" | awk '{print $4}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            fi
            name="neokylin-$(echo ${name/update/'.'})"
        fi
        echo "$name"
        return 0
    fi
    #获取KylinSecOS的版本
    if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}"|awk -F " " '{print $1}')" ]; then
        name="kylinSecOS-$(grep "^VERSION_ID=" "${sles15_release}" | awk -F "\"" '{print $2}')"
        echo "$name"
        return 0
    fi

    if [ -f "${bclinux_release}" ];then
        if grep -q "Euler" "${bclinux_release}";then
            name="bclinux-$(cat ${bclinux_release} | awk '{print $7}' | awk -F. '{print $1"."$2}')"
        else
            name="bclinux-$(cat "${bclinux_release}" | awk '{print $5}' | awk -F. '{print $1"."$2}')"
        fi
        echo "$name"
        return 0
    fi
    
    #备注:  CentOS 中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${cent_release}" ]; then
        if grep -q "Linux" "${cent_release}"; then
            name="centos-$(cat "${cent_release}" | awk '{print $4}' | awk -F. '{print $1"."$2}')"
        else
            name="centos-$(cat "${cent_release}" | awk '{print $3}')"
        fi
        echo "$name"
        return 0
    fi

    #备注: oracle linux中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${ole_release}" ]; then
        name="oracle-$(cat "${ole_release}" | awk '{print $5}')"
        echo "$name"
        return 0
    fi

    #获取RedHat的版本
    if [ -f "${rhel_release}" ]; then
        name="redhat-$(cat "${rhel_release}" | awk '{print $7}')"
        echo "$name"
        return 0
    fi

    #获取SuSE的版本
    if [ -f "${sles_release}" ]; then
        name="suse-$(cat "${sles_release}" | grep "^VERSION" | awk '{print $3}')"
        name="${name}.$(cat "${sles_release}" | grep "^PATCHLEVEL" | awk '{print $3}')"
        echo "$name"
        return 0
    fi
    
    #获取SuSE15的版本
    if [ -f "${sles15_release}" ]; then
        if [ "suse" = "$(cat "${sles15_release}" | grep "^ID_LIKE=" | awk -F "\"" '{print $2}')" ]; then
            name="suse-$(cat "${sles15_release}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            return 0
        fi
        idLike=$(grep "^ID_LIKE=" "${sles15_release}" | awk -F "\"" '{print $2}')
        ID=$(grep "^ID=" "${sles15_release}" | awk -F '=' '{print $2}' | tr -d '"')
        VERSION_ID=$(grep "^VERSION_ID=" "${sles15_release}" | awk -F "\"" '{print $2}')
        if [ "${ID}" = "kylin" ] && [ "${idLike}" = "fedora" ]; then
            name="$ID-${VERSION_ID}-fedora"
            echo "$name"
            return 0
        fi
        if [ -f "${kylin_productinfo}" ] && [ "${ID}" = "kylin" ]; then
            local MINOR_VERSION=$(grep "${VERSION_ID}" "${kylin_productinfo}" | awk -F "(" '{print $2}' | awk -F ")" '{print $1}')
            name="$ID-${VERSION_ID}-${MINOR_VERSION}"
            echo "$name"
            return 0
        fi
        if [ "${ID}" = "uos" ]; then
            name="$ID-$(cat /etc/os-version | grep MinorVersion | awk -F "=" '{print $2}')"
            echo "$name"
            return 0
        fi
        #获取openEuler-SPx的版本
        if [ -f "${openeuler_release}" ] && [ "${ID}" = "openEuler" ]; then
           local MINOR_VERSION=$(cat "${openeuler_release}" | awk -F "-" '{print $2}' | awk -F ")" '{print $1}')
           #openEuler-20.03-SP1保持之前的规则提取版本号
           local MINOR_VERSION_ID="${VERSION_ID}-${MINOR_VERSION}"
           if [ -n "${MINOR_VERSION}" -a "${MINOR_VERSION_ID}" != "20.03-SP1" ] ; then
               name="$ID-${VERSION_ID}-${MINOR_VERSION}"
               echo "$name"
               return 0
           fi
        fi
        name="$ID-$VERSION_ID"
        echo "$name"
        return 0
    fi
    
    return 1
}

################################################
#
# 获取系统中确实的rpm列表
#
################################################
function get_missing_rpms_lst()
{
    local symbol="$1"            #节点标志,IP地址或主机名
    local must_rpms=""           #系统中必须有的rpm包列表
    local missing_rpms=""        #用于保存缺失rpm列表的文件
    local osname=""              #os名称(suse/redhat)
    local line=""                #文件每行的内容
    local newline=""             #转换后的行的内容(去空格)

    debug "[${symbol}]Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$symbol" ]; then
        symbol="localhost"
    fi

    #获取OS名称
    osname="$(get_osname)"
    if [ $? -ne 0 ]; then
        echo "[$(date)][${symbol}] [ERROR] [${FUNCNAME}:${LINENO}] get_osname(${osname}) failed"
        return 1
    fi

    debug "[${osname}] [${FUNCNAME}:${LINENO}]"

    #列表文件 与 脚本在同一目录下
    
    

    if [ `uname -p` == "aarch64" ]; then
       must_rpms="$(dirname "$0")/conf/${osname}_must-rpms-aarch64.lst"
    else
       must_rpms="$(dirname "$0")/conf/${osname}_must-rpms.lst"
    fi
    
    missing_rpms="$(dirname "$0")/${osname}_${symbol}_missing-rpms.lst"

    #文件检查
    if [ ! -f "$must_rpms" ]; then
        echo "[$(date)][${symbol}] [ERROR] [${FUNCNAME}:${LINENO}] ${must_rpms} not a file in ${symbol}"
        return 1
    fi

    #直接写入文件,如果写入失败,直接返回
    echo "# " > $missing_rpms
    if [ $? -ne 0 ]; then
        echo "[$(date)][${symbol}] [ERROR] [${FUNCNAME}:${LINENO}] ${missing_rpms} is invalid, write failed"
        return 1
    fi

    echo "# This list was generated on $(date)" >> $missing_rpms
    echo "# " >> $missing_rpms
    echo "" >> $missing_rpms

    while read line
    do
        #排除非字母或数字开头的行
        if [ -z "$(echo "$line" | grep "^\**[[:alnum:]]")" ]; then
            continue
        fi

        #删除行中空格
        newline="$(echo "$line" | tr -d '[:space:]')"

        #判断系统中是否包含需要的rpm包
        rpm -q $newline > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "${newline}" | grep "*"
            if [ $? -eq 0 ]
            then
                rpm -qa | grep -E "${newline}"
                if [ $? -eq 0 ]
                then
                    continue
                fi
            fi

            echo "${newline}" >> $missing_rpms
            debug "Missing:${newline}"
        fi
    done < $must_rpms

    debug "Leave [${FUNCNAME}:${LINENO}]"
    if [ -f "/etc/openEuler-release" ]; then
        VERSION_ID=$(grep "^VERSION_ID=" "/etc/os-release" | awk -F "\"" '{print $2}')
        local MINOR_VERSION=$(cat "/etc/openEuler-release" | awk -F "-" '{print $2}' | awk -F ")" '{print $1}')
        local MINOR_VERSION_ID="${VERSION_ID}-${MINOR_VERSION}"
        if [ -n ${MINOR_VERSION} -a ${MINOR_VERSION_ID} == "22.03-SP1" ] ; then
           ln -s /usr/bin/python3 /usr/bin/python
        fi
    fi
    return 0
}

###################################################
# 入口函数
###################################################
function main()
{
    #判断是否为debug模式
    if [ -n "$2" ] && [ "$2" -eq 1 ]; then
        g_lst_debug=1
    fi

    get_missing_rpms_lst "$@"
    return $?
}

main "$@"
exit $?
