#!/bin/bash

g_logFile=""   #日志文件

######################################################################
#   DESCRIPTION: os性能调优日志管理接口                
######################################################################
function main()
{
    local fileSize=0                #当前日志文件大小
    local timeNow=0                 #系统当前时间
    local counter=2                 #压缩文件个数
    local backupFile=""             #备份文件名称
    local allBackupCounter=0        #备份文件数量
    local deleteFileName=0          #待删除压缩文件名称
    
    local MAX_LOG_FILE_SIZE=1024    #日志文件大小限制(K)
    local MAX_ALLbackupFile_SIZE=10 #压缩文件个数限制
    
    #设置当前的umask
    UMASK=0027
    umask "$UMASK"

    #入参检查
    if [ $# -ne 1 ] ; then
        echo "[$(date)] ERROR : scriptlog.sh parameter invalid."
        return 1
    fi   
    
    g_logFile="$1"
    
    if [ -z "${g_logFile}" ]; then
        echo "[$(date)] ERROR : ${g_logFile} is null."
        return 1
    fi
    
    if [ ! -f "${g_logFile}" ]; then
        echo "[$(date)] ERROR : ${g_logFile} does not exist."
        return 1
    fi
    
    #获取日志文件大小(K)
    fileSize=$(du -a "${g_logFile}" |cut -f1)
    if [ -z "${fileSize}" ]; then
        echo "[$(date)] ERROR : Exception occurs when getting the size of the log file."
        return 1
    fi
    
    #当前日志大小小于上限直接返回
    if [ "${fileSize}" -lt "${MAX_LOG_FILE_SIZE}" ]; then
        return 0
    fi
    
    #取得当前时间
    timeNow=$(date +'%Y%m%d%H%M%S')
    
    #获取当前压缩文件的名称
    backupFile="${g_logFile}.${timeNow}"
    while [ -f "${backupFile}" ]
    do
        backupFile="${g_logFile}.${timeNow}_${counter}"
        ((counter++))
    done
    
    #备份日志文件并压缩
    cp "${g_logFile}" "${backupFile}" && cat /dev/null > "${g_logFile}"    
    gzip "${backupFile}"
    chmod 440 "${backupFile}.gz"  
    #判断压缩文件数量 
    local DirPath=$(dirname "${g_logFile}") 
    local LogName=$(basename "${g_logFile}") 
    allBackupCounter=$(find "${DirPath}" -type f | grep "${LogName}"* |wc -l)    
    
    #当压缩文件数量超过上限时删除
    while [ "${allBackupCounter}" -gt "${MAX_ALLbackupFile_SIZE}" ]
    do
        deleteFileName=$(ls -lrt $g_logFile* | awk 'NR==1 {print $NF}')        
        rm -rf "${deleteFileName}"
        allBackupCounter=$(find "${DirPath}" -type f | grep "${LogName}"* |wc -l)
    done
    
    return 0
}

main "$@"
exit $?