#!/bin/bash

######################################################################
#   DESCRIPTION: 设置noatime,nodiratime，禁止访问文件系统时更新访问时间，减少系统IO 
######################################################################
function noatime_config()
{
    local fstabFile="/etc/fstab"
    
    #文件不存在退出
    if [ ! -f "${fstabFile}" ]; then
        echo "[$(date)] ERROR : ${fstabFile} does not exist, so set the noatime configure fails."
        return 1
    fi

    #备份
    if [ ! -f "${fstabFile}.bakk" ]; then
        cp ${fstabFile} ${fstabFile}.bakk
    fi

    #这里稍作改动:SuSE ext3,ext4都存在可能，只判断ext4存在问题
    #不能仅仅依据判断文档里是否包含noatime,nodiratime来判断是否已完成设置。
    #不一定所有的默认属性都是default，原来采用default字串匹配的方式存在缺陷
    #另外,只需要设置noatime,不需要设置nodiratime,因为后者是前者的子集    
    sed -n '/ext3/=;/ext4/=;/btrfs/=;/xfs/=' ${fstabFile} | while read row    #取EXT3,EXT4所在的列数 
    do
        VAL=$(awk "NR==${row}" ${fstabFile} | awk '{print $4}')
        echo "${VAL}" | grep -q "noatime" 
        if [ $? -ne 0 ];then #未找到noatime
            #指定行增加noatime项并写回
            sed -i "${row}s#[ext3,ext4,btrfs,xfs][ \t]*${VAL}[^ ]*#&,noatime#g" ${fstabFile}
            if [ $? -ne 0 ]; then    
                echo "[$(date)] ERROR : Set the noatime configure fails."
                return 1
            fi
        fi
    done 
    
    #重新挂载文件系统,保证上述设置马上生效
    df -l -P -t ext4 -t ext3 -t xfs -t btrfs | awk '{print $6}' | grep "^/" | while read line
    do
        if [ -z "$line" ]; then
            continue
        fi
        
        #重新挂载
        mount -o remount $line
        
    done
        
    return 0
}


######################################################################
#    DESCRIPTION: 替换fstab文件中的tab键             
######################################################################
function replace_tab()
{
    local etc_fstab="/etc/fstab"
    local tmp_etc_fstab=""

    tmp_etc_fstab="$(mktemp)" || return 1
    expand -t 4 "${etc_fstab}" > "${tmp_etc_fstab}" ||  return 1
    cat "${tmp_etc_fstab}" > "${etc_fstab}" || return 1

    rm -rf "${tmp_etc_fstab}"
    return 0
}


######################################################################
#   DESCRIPTION: 禁止访问文件系统时更新访问时间功能主执行函数
######################################################################
function main()
{   
    #替换/etc/fstab中的tab键，防止其干扰后续处理
    replace_tab
    if [ $? -ne 0  ]; then
        echo "[$(date)] ERROR : Exception occurs when replace tab in /etc/fstab."
        return 1
    fi

    #设置noatime为禁止访问文件系统时更新访问时间
    noatime_config
    if [ $? -ne 0 ]; then    
        echo "[$(date)] ERROR : Exception occurs when setting the noatime configure."
        return 1
    fi
    
    echo "[$(date)] INFO : Setting the noatime configure successfully."
    return 0
}

main
exit $?
