#!/bin/bash

declare -a g_devAarry    #声明存放磁盘设备名称的数组

######################################################################
#   DESCRIPTION: 获取DEV设备    
######################################################################
function get_dev()
{
    #获取DEV设备的设备名
    g_devAarry=( $(fdisk -l 2>/dev/null | grep "Disk /dev/" | grep -v "/dev/mapper/" | awk '{ print $2 }' | awk -F'/' '{ print $NF }' | sed s/:$//g) )
    
    if [ $? -ne 0 ]; then
        echo "[$(date)] DEBUG : Executing the script of lsscsi fails."
        return 1
    fi
    
    #如果没有获取到任何DEV设备，则认为是失败的
    if [ "${#g_devAarry[@]}" -le 0 ]; then
        echo "[$(date)] DEBUG : Could not found the DEV disk."
        return 1
    fi
          
    return 0
}

######################################################################
#   DESCRIPTION: 优化磁盘预读      
######################################################################
function set_readhead()
{
    local optimizeReadhead=32768     #预读大小，单位是扇区的个数
    local devIndex=0                 
    local sysDev=""
    
    #获取系统盘
    sysDev=$(mount | awk '{if( $3=="/" ) print $1}' | sed 's/\/dev\///' | sed 's/[0-9]//')
    if [ -z "$sysDev" ]; then
        echo "[$(date)] DEBUG : Could not found the system's disk."
        return 1
    fi
    
    #遍历所有的DEV设备
    for devIndex in $(seq 0 $((${#g_devAarry[@]} - 1)))
    do 
        #如果是sda盘，则认为是系统盘，系统盘不改变预读
        if [ "${g_devAarry[$devIndex]}" = "$sysDev" ]; then
            continue
        fi
        
        #如果不是系统盘，则认为是数据盘，加大预读
        blockdev --setra "${optimizeReadhead}" "/dev/${g_devAarry[$devIndex]}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : Setting the read head fails."
            return 1
        fi
        
    done
    
    return 0
}

######################################################################
#   DESCRIPTION: 设置设备的调度算法
######################################################################
function set_devScheduler()
{
    local devName=""    #磁盘名称
    local scheduler=""  #预读值
    
    #判断参数的个数
    if [ $# -ne 2 ]
    then
        echo "[$(date)] ERROR : set_devScheduler parameter invalid."
        return 1
    fi
    
    devName=$1
    scheduler=$2
    
    #设置设备的调度算法
    echo "$scheduler" > "/sys/block/$devName/queue/scheduler"
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Setting the scheduler of disk fails."
        return 1
    fi    
    
    return 0
}


######################################################################
#   DESCRIPTION: 调整deadline调度器的
######################################################################
function set_deadline_scheduler()
{  
    local devIndex=0                      #遍历索引号
    local curScheduler=""                 #调度算法
    
    #遍历所有的sd*设备
    for devIndex in $(seq 0 $((${#g_devAarry[@]} - 1)))
    do 
        #查看设备的调度算法，如果不是deadline，将其设置为deadline
        curScheduler=$(cat /sys/block/${g_devAarry[$devIndex]}/queue/scheduler | grep "\[deadline\]")
        if [ -z "${curScheduler}" ]; then
            set_devScheduler "${g_devAarry[$devIndex]}" "deadline"
        fi
    done
    
    return 0
}


######################################################################
#   DESCRIPTION: 优化deadline调度算法以及磁盘优化预读主执行函数   
######################################################################
function main()
{
    local base_dir=$(dirname $0)
    local adapter_script="${base_dir}/adapter.sh"
    local osperf_conf="${base_dir}/osperf.conf"
    
    . "${osperf_conf}"
    source "${adapter_script}"
    
    #获取机器上的设备名
    get_dev
    if [ $? -ne 0 ]
    then
        echo "[$(date)] ERROR : Exception occurs when getting the DEV disk."
        return 1
    fi
    
    #优化deadline调度算法
    set_deadline_scheduler
    if [ $? -ne 0 ]
    then
        echo "[$(date)] ERROR : Exception occurs when Setting deadline scheduler."
        return 1
    fi
    echo "[$(date)] INFO : Setting the deadline scheduler successfully."
    
    #优化预读
    set_readhead
    if [ $? -ne 0 ]
    then
        echo "[$(date)] ERROR : Exception occurs when Setting the read head."
        return 1
    fi    
    echo "[$(date)] INFO : Setting the read head successfully."   

    set_sysctl "vm.dirty_ratio" "40"
    if [ $? -ne 0 ]
    then
        echo "[$(date)] ERROR : Exception occurs when Setting the vm.dirty_ratio."
        return 1
    fi    
    echo "[$(date)] INFO : Setting the vm.dirty_ratio successfully."   
    
    return 0
}

main
exit $?