#!/bin/bash

######################################################################
#   DESCRIPTION: 设置vm.swappiness value to zero
######################################################################
function swappiness_config()
{
    #查找系统文件
    local sysCtlFile="/etc/sysctl.conf"

    #文件不存在退出
    if [ ! -f "${sysCtlFile}" ]; then
        echo "[$(date)] ERROR : ${sysCtlFile} does not exist."
        return 1
    fi

    #备份
    if [ ! -f "${sysCtlFile}.bakk" ]; then
        cp ${sysCtlFile} ${sysCtlFile}.bakk
    fi
    
    #获取原有配置
    vmSwap=$(sed -n '/vm.swappiness[ ]*='/p ${sysCtlFile})
    if [ -z "${vmSwap}" ]; then
        #新增
        sed -i '$ a\vm.swappiness = 0' ${sysCtlFile}
        if [ $? -ne 0 ]
        then
            echo "[$(date)] ERROR : Exception occurs when create a configure of the swappiness."
            return 1
        fi    
    else
        #修改
        sed -i 's/vm.swappiness[ ]*=[ ]*[0-9]*/vm.swappiness = 0/g' ${sysCtlFile}
        if [ $? -ne 0 ]
        then
            echo "[$(date)] ERROR : Exception occurs when modify the configure of the swappiness."
            return 1
        fi    
    fi
        
    return 0
}

######################################################################
#   DESCRIPTION: 解决长时间GC调优脚本主执行函数                           
######################################################################
function main()
{   
    #GC调优
    swappiness_config
    if [ $? -ne 0 ]; then    
        echo "[$(date)] ERROR : Exception occurs when Setting the swappiness."
        return 1
    fi

    echo "[$(date)] INFO : Setting the swappiness successfully."    
    return 0
}

main
exit $?