#!/bin/bash

######################################################################
#   DESCRIPTION: 设置CFS调度器
######################################################################
function set_cfs_scheduler()
{
    set_sysctl "kernel.sched_min_granularity_ns" "10000000"
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Exception occurs when Setting the kernel.sched_min_granularity_ns."
        return 1
    fi    
    
    set_sysctl "kernel.sched_wakeup_granularity_ns" "15000000"
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Exception occurs when Setting the kernel.sched_wakeup_granularity_ns."
        return 1
    fi    
    
    return 0
}

######################################################################
#   DESCRIPTION:设置CFS调度器主执行函数   
######################################################################
function main()
{
    local os_name=""
    local base_dir=$(dirname $0)
    local adapter_script="${base_dir}/adapter.sh"
    local osperf_conf="${base_dir}/osperf.conf"
    
    . "${osperf_conf}"
    source "${adapter_script}"
    os_name=$(get_osname)
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Don't support [$(cat /proc/version)]"
        return 1
    fi
    
    if [ "${osperf_cfs_scheduler}" -eq 1 ]; then
        #suse暂时不设置
        echo "${os_name}" | grep -E "suse" > /dev/null
        if [ $? -eq 0 ]; then
            return 0
        fi
    
        set_cfs_scheduler
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : Exception occurs when set cfs scheduler."
            return 1
        fi
    fi

    echo "[$(date)] INFO : Setting cfs scheduler successfully."    
    
    return 0
}

main
exit $?