#!/bin/bash

###################################################
# 功能: 获取OS的名称
#
# 接口:
#   函数: get_osname
###################################################

###################################################
#
# 获取OS的名称
#
###################################################
function get_osname()
{
    local cent_release="/etc/centos-release"
    local ole_release="/etc/oracle-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local euler_release="/etc/euleros-release"
    local name=""

    #备注: CentOS中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${cent_release}" ]; then
        if grep -q "Linux" "${cent_release}"; then
            name="centos-$(cat "${cent_release}" | awk '{print $4}' | awk -F. '{print $1"."$2}')"
        else
            name="centos-$(cat "${cent_release}" | awk '{print $3}')"
        fi
        echo "$name"
        return 0
    fi

    #备注: oracle linux中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${ole_release}" ]; then
        name="oracle-$(cat "${ole_release}" | awk '{print $5}')"
        echo "$name"
        return 0
    fi
    
    #获取Euler的版本
    if [ -f "${euler_release}" ]; then
        name="euleros-$(cat "${euler_release}" | awk '{print $2}' | cut -d "V" -f 2)"
        echo "$name"
        return 0
    fi

    #获取RedHat的版本
    if [ -f "${rhel_release}" ]; then
        name="redhat-$(cat "${rhel_release}" | awk '{print $7}')"
        echo "$name"
        return 0
    fi

    #获取SuSE的版本
    if [ -f "${sles_release}" ]; then
        name="suse-$(cat "${sles_release}" | grep "^VERSION" | awk '{print $3}')"
        name="${name}.$(cat "${sles_release}" | grep "^PATCHLEVEL" | awk '{print $3}')"
        echo "$name"
        return 0
    fi

    return 1
}

######################################################################
#   DESCRIPTION: 设置/etc/sysctl.conf文件
######################################################################
function set_sysctl()
{  
    #查找系统文件
    local sysCtlFile="/etc/sysctl.conf"
    local sysctlKey="$1"
    local sysctlValue="$2"
    local curSysctlValue=""
    
    #文件不存在退出
    if [ ! -f "${sysCtlFile}" ]; then
        echo "[$(date)] ERROR : ${sysCtlFile} does not exist."
        return 1
    fi

    #备份
    if [ ! -f "${sysCtlFile}.bakk" ]; then
        cp ${sysCtlFile} ${sysCtlFile}.bakk
    fi
    
    #获取原有配置
    curSysctlValue=$(sed -n '/^[ ]*'''${sysctlKey}'''[ ]*=/p' ${sysCtlFile})
    if [ -z "${curSysctlValue}" ]; then
        #新增
        sed -i '$a '''${sysctlKey}''' = '''${sysctlValue}'''' "${sysCtlFile}"
        if [ $? -ne 0 ]
        then
            echo "[$(date)] ERROR : Exception occurs when create a configure of the ${sysctlKey}."
            return 1
        fi    
    else
        #修改
        sed -i 's/^[ ]*'''${sysctlKey}'''[ ]*=.*/'''${sysctlKey}''' = '''${sysctlValue}'''/g' ${sysCtlFile}
        if [ $? -ne 0 ]
        then
            echo "[$(date)] ERROR : Exception occurs when modify the configure of the ${sysctlKey}."
            return 1
        fi    
    fi
    
    sysctl -q -w "${sysctlKey}"="${sysctlValue}"    
    rm -rf ${sysCtlFile}.bakk
        
    return 0
}
