#!/bin/bash

###############################################################################
#author:   none
#date:     2014/05/14
#function: 后台收集硬盘在格式化过程中的进程信息
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#日志目录
LOGPATH="${HOME}/diskmgt/"

#日志文件
LOGFILE="${LOGPATH}/atpt_collect.log"

#系统dmesg信息
LOG_DMESG_FILE="${LOGPATH}/atpt_dmesg.log"

#系统dmesg信息备份
LOG_DMESG_OLD_FILE="${LOGPATH}/atpt_dmesg.log.old"

#收集D状态进程信息，主要是为了记录格式化进程处于D状态的时间周期长短
LOG_DPROCESS_FILE="${LOGPATH}/atpt_Dprocess.log"


########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数1说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] [EMER] diskmgtmodule ap: mkdir -p ${LOGPATH} failed." >>/var/log/messages
            echo "[$(date)] ${log_info}" >>/var/log/messages
            return 1
        fi
    fi
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 收集系统的dmesg信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function collect_dmesg()
{
    if [ -f "${LOG_DMESG_FILE}" ]; then
        cp "${LOG_DMESG_FILE}" "${LOG_DMESG_OLD_FILE}" -arf
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${LOG_DMESG_FILE} ${LOG_DMESG_OLD_FILE} -arf failed." 
        fi
    fi
    
    dmesg >"${LOG_DMESG_FILE}" 2>&1
    
    return 0
}

########################################################################
# 函数描述： 收集系统的D状态进程信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function collect_Dprocess()
{
    echo `date` >>"${LOG_DPROCESS_FILE}"
    ps -wweo user,pid,ppid,tid,nlwp,nice,%cpu,%mem,vsz,rss,tname,stat,lstart,etime,time,cmd  | awk '{if ( $12 ~ "D" ) print $0}' >>"${LOG_DPROCESS_FILE}"
    echo >>"${LOG_DPROCESS_FILE}"
    echo >>"${LOG_DPROCESS_FILE}"
    echo >>"${LOG_DPROCESS_FILE}"
    echo >>"${LOG_DPROCESS_FILE}"
    echo >>"${LOG_DPROCESS_FILE}"
    
    return 0
}

########################################################################
# 函数描述： 信号处理函数
# 参数1说明：无
#
# 返回值  ： 2 返回信号处理特殊错误码
########################################################################
function sig_catch()
{
    logme 2 "[EMER] [${FUNCNAME}:${LINENO}] catch signal!!"
    exit 2
}

#捕获信号
trap "sig_catch" INT TERM QUIT

########################################################################
# 函数描述： 主入口函数，每5秒收集一次dmesg和D状态信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function main()
{
    local pid_ap=0
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] collect begin." 
    
    while :
    do
        #如果autopart.sh脚本在运行则收集，否则退出收集
        pid_ap=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "autopart.sh" | awk '{print $1}')
        if [ -z "${pid_ap}" ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] autopart.sh may exit, so collect ready to exit."
            
            sleep 3
            pid_ap=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "autopart.sh" | awk '{print $1}')
            if [ -z "${pid_ap}" ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] collect end."
                break
            fi
        fi
        
        collect_dmesg
        collect_Dprocess
        sync
        
        sleep 5
    done

    return 0
}

main
exit $?

