#!/bin/bash

###############################################################################
#author:   none
#date:     2013/12/14
#function: 日志压缩脚本
###############################################################################

#需要压缩的文件路径
LOG_LOGFILE=$1


########################################################################
# 函数描述： 主入口函数，压缩日志
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{
    local FILESIZE=0
    local TIME_NOW=0
    local COUNTER=2
    local BACKUPFILE=""
    local ALLBACKUPSIZE=0
    local DELETEFILE=0
    
    #最大单个文件大小，即超过此大小即被压缩，单位是K
    local MAX_LOG_FILE_SIZE=2048
    
    #最大所有备份日志大小，单位是M
    local MAX_ALLBACKUPFILE_SIZE=30

    UMASK=0027
    umask "$UMASK"
    
    if [ $# -ne 1 ]; then
        echo "[archivescriptlog] failed: para isnot 1!"
        return 1
    fi
    
    #判断输入压缩文件的有效性
    if [ -z "${LOG_LOGFILE}" ]; then
        echo "[archivescriptlog] failed: logname is empty!"
        return 1
    fi
    
    if [ ! -f "${LOG_LOGFILE}" ]; then
        echo "[archivescriptlog] failed: ${LOG_LOGFILE} isnot exist!"
        return 1
    fi
    
    #计算文件大小
    FILESIZE=$(du -a "${LOG_LOGFILE}" |cut -f1)
    if [ -z "${FILESIZE}" ]; then
        echo "[archivescriptlog] failed: ${LOG_LOGFILE} size is zero!"
        return 1
    fi
    
    #如果没有超过要压缩的上限，则直接退出
    if [ "${FILESIZE}" -lt "${MAX_LOG_FILE_SIZE}" ]; then
        return 0
    fi
    
    
    
    #否则就开始压缩
    TIME_NOW=$(date +'%Y%m%d%H%M%S')
    BACKUPFILE="${LOG_LOGFILE}.${TIME_NOW}"
    while [ -f "${BACKUPFILE}" ]
    do
        BACKUPFILE="${LOG_LOGFILE}.${TIME_NOW}_${COUNTER}"
        ((COUNTER++))
    done
    cp "${LOG_LOGFILE}" "${BACKUPFILE}" && cat /dev/null > "${LOG_LOGFILE}"
    
    gzip "${BACKUPFILE}"
    chmod 440 "${BACKUPFILE}.gz"
    
    #检查是否所有的备份文件加起来，超过上限
    #如果超过要进行删除最老的压缩包
    ALLBACKUPSIZE=$(du -mc "${LOG_LOGFILE}"* | grep total | cut -f1)
    while [ "${ALLBACKUPSIZE}" -gt "${MAX_ALLBACKUPFILE_SIZE}" ]
    do
        DELETEFILE=$(ls -lrt $LOG_LOGFILE* | awk 'NR==1 {print $NF}')
        rm -rf "${DELETEFILE}"
        ALLBACKUPSIZE=$(du -mc "${LOG_LOGFILE}"* | grep total | cut -f1)
    done
    
    return 0
}

main "$@"
exit $?

