#!/bin/bash

###############################################################################
#author:   none
#date:     2013/12/14
#function: 发送告警插件
#
#
#所有参数说明：
#
##uint64 i_internal_id          1.产品内部告警ID
##int32 i_alarm_type            2.告警类型（0:故障告警 1:恢复告警 2:事件）
##int32 i_alarm_level           3.告警级别（1:紧急 2:重要 3:次要 4:警告）
##int32 i_alarm_cause           4.告警原因（0）
##int32 i_dt_time               5.故障告警时间,不输入则工具会自动生成时间、但是必须保留””参数占位符
##char sv_moc[20]               6.填写"FusionInsight Hadoop"
##char sv_resourceID[20]        7.填写"diskmgtd"
##char sv_location_info[256]    8.故障所在位置信息
##char sv_additional_info[256]  9.附加信息
##必选项有:    id location moc resID
##恢复的前提： id location moc resID相同 (告警类型0-->1)
##判断重复告警的条件：id level moc resID location addinfo
##################################################################################################

#告警工具
AlarmTool=""

#参数合法个数
ParaNum=5

#打印参数，方便定位
echo `date` "sendalarmin paras:" "[$*]"
echo `date` "sendalarmin para[1]:" "$1"
echo `date` "sendalarmin para[2]:" "$2"
echo `date` "sendalarmin para[3]:" "$3"
echo `date` "sendalarmin para[4]:" "$4"
echo `date` "sendalarmin para[5]:" "$5"

#判断参数个数的有效性
if [ $# -ne ${ParaNum} ]; then
    echo `date` "sendalarmin failed: paras num($#) is not ${ParaNum}"
    exit 1
fi

#对输入的各个参数，转换为告警工具对应的接口
#也就是要从6个输入参数，转为9个告警接口的输入参数
G_alarmID=$1                                   #1
G_alarmCategory=$2                             #2
G_alarmLevel=2                                 #3
G_alarmCause=0                                 #4
G_occurTime=""                                 #5
G_mocName="${HOSTNAME}"                        #6
G_resourceID="diskmgtd"                        #7
G_locationInfo="Manager;NA;${HOSTNAME};$3"     #8
G_additionalInfo="$5"                          #9

USER_NAME="omm"

if [[ $UID -ne 0 ]]; then
    DISKMGT_BIGDATA_AGENT_HOME="${NODE_AGENT_HOME}"
    DISKMGT_BIGDATA_LOG_HOME="${BIGDATA_LOG_HOME}"
else
    DISKMGT_BIGDATA_AGENT_HOME="$(su - omm -c 'echo $NODE_AGENT_HOME')"
    DISKMGT_BIGDATA_LOG_HOME="$(su - omm -c 'echo $BIGDATA_LOG_HOME')"
fi

if [ -z "${DISKMGT_BIGDATA_AGENT_HOME}" ]; then
    DISKMGT_BIGDATA_AGENT_HOME="/opt/huawei/Bigdata/om-agent/nodeagent"
fi

if [ -z "${DISKMGT_BIGDATA_LOG_HOME}" ]; then
    DISKMGT_BIGDATA_LOG_HOME="/var/log/Bigdata"
fi

########################################################################
# 函数描述： 转换告警ID号，具体为产品申请的告警ID
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function translate_para()
{
    #设备消失
    if [ ${G_alarmID} -eq 1 ]; then
        G_alarmID=12014
        return 0
    fi
    
    #只读告警
    if [ ${G_alarmID} -eq 2 ]; then
        G_alarmID=12015
        return 0
    fi
    
    #非法告警ID
    echo `date` "sendalarmin failed: invalid id=${G_alarmID}"
    return 1
}


########################################################################
# 函数描述： 获取告警工具路径，这是和OM的接口
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_alarm_tool()
{
    local supression_alarm_agent="${DISKMGT_BIGDATA_AGENT_HOME}/bin/supressionSendAlarm.sh"
    local alarm_agent="${DISKMGT_BIGDATA_AGENT_HOME}/bin/sendAlarm"
    
    #分为数据节点和管理节点，优先考虑用数据节点的工具
    #C30版本已经不再区分
    if [ -f "${supression_alarm_agent}" ]; then
        AlarmTool="${supression_alarm_agent}"
        return 0
    fi
    
    if [ -f "${alarm_agent}" ]; then
        AlarmTool="${alarm_agent}"
        return 0
    fi
    
    #如果是未知节点，那么就找不到告警工具，无法发送告警了
    echo `date` "sendalarmin failed: alarm tool is not exist."
    return 1
}


########################################################################
# 函数描述： 获取磁盘序列号，获取成功则附加给告警附加信息G_additionalInfo
# 参数1说明：挂载目录
#
# 返回值 ： 无
########################################################################
function get_disk_serial_number()
{
    diskname=$(df -h | sed -e "1,1d" | grep "${1}$" | awk -F " " '{print $1}')
    # 如果设备被umount或拔盘umount，则使用df或mount命令将查不到挂载目录。此时尝试在/etc/fstab文件查找
    if [ -z "$diskname" ]; then
        echo `date` "The disk may have been unmounted."
        # /etc/fstab能查到挂载目录以及其对应的uuid，可通过uuid得到对应磁盘
        disk_uuid=$(cat /etc/fstab 2>/dev/null | grep "^UUID=" | awk '{print $1,$2}' | grep "${1}$" | awk '{print $1}')
        if [ -n "$disk_uuid" ]; then
            diskname="$(ls -l /dev/disk/by-uuid | grep -w "${disk_uuid:5}" | awk '{print $11}')"
        fi
    fi
    if [ -z "$diskname" ]; then
        # 如果都找不到该磁盘，那么磁盘很可能已经被拔出，则该磁盘对应的序列号也就消失了
        echo `date` "Failed to obtain the disk serial number. The disk does not exist, may be removed."
    else
        disk_serial_sumber="$(ls -l /dev/disk/by-id | grep -w "${diskname##*/}" | awk 'BEGIN{ORS=","} {print $9}')"
        G_additionalInfo="${G_additionalInfo};${disk_serial_sumber%,*}"
        echo `date` "The disk serial number is successfully obtained. G_additionalInfo=${G_additionalInfo}."
    fi
}


########################################################################
# 函数描述： 发送告警
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function sendout_alarm()
{
    local ret=0
    
    if [ -z "${AlarmTool}" ]; then
        echo `date` "sendalarmin failed: alarm tool is empty."
        return 1
    fi

    if [ ! -x "${AlarmTool}" ]; then
        echo `date` "sendalarmin failed: alarm tool is not executable."
        return 1
    fi

    su - "${USER_NAME}" -c "\"${AlarmTool}\" \"${G_alarmID}\" \"${G_alarmCategory}\" \"${G_alarmLevel}\" \"${G_alarmCause}\" \"${G_occurTime}\" \"${G_mocName}\" \"${G_resourceID}\" \"${G_locationInfo}\" \"${G_additionalInfo}\""
    ret=$?
    if [ ${ret} -eq 0 ]; then
        echo `date` "sendalarmin ok: ${AlarmTool} 1-ID[$G_alarmID] 2-Category[$G_alarmCategory] 3-Level-[${G_alarmLevel}] 4-Cause[${G_alarmCause}] 5-Time[${G_occurTime}] 6-Moc[${G_mocName}] 7-Res[${G_resourceID}] 8-LocalInfo[$G_locationInfo] 9-AddInfo[$G_additionalInfo]"
    else
        echo `date` "sendalarmin failed(${ret}): ${AlarmTool} 1-ID[$G_alarmID] 2-Category[$G_alarmCategory] 3-Level-[${G_alarmLevel}] 4-Cause[${G_alarmCause}] 5-Time[${G_occurTime}] 6-Moc[${G_mocName}] 7-Res[${G_resourceID}] 8-LocalInfo[$G_locationInfo] 9-AddInfo[$G_additionalInfo]"
    fi
    
    return ${ret}
}


########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function main()
{
    #转换告警ID
    translate_para
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #设定告警工具路径
    get_alarm_tool
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #开始发送告警，返回值返回给上层调用者
    sendout_alarm
    return $?
}

get_disk_serial_number "$3"
main
exit $?

