#!/bin/bash


###############################################################################
#author:   none
#date:     2014/02/12
#function: 安装硬盘管理服务
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#主机名或IP地址
IPADDR="$(echo ${SSH_CONNECTION} | awk '{print $3}')"

#设置当前的umask
UMASK=0077
umask "$UMASK"

#运行的用户/组
RUN_USER=2000
RUN_GROUP=wheel

#设置当前路径
CURRPATH=""

#设置临时路径，所有的临时文件都放在此目录下
TMPDIR=""

#输入的挂载文件参数
KMS_ENCRYPT_DEVS="kms_encrypt_devs"

KMSINFO="kms_info"

#检查系统之前是否安装过硬盘管理服务，判断依据是/usr/local/kmsLuksMgt 目录是否存在
IS_EMPTY=0


#服务目录和文件
SERVICEPATH="/usr/local/kmsLuksMgt"
SERVICERUN="/usr/local/kmsLuksMgt/script/mountKms.sh"

#日志目录
LOGPATH="${HOME}/kmsLuksMgt/"

#日志文件
LOGFILE="${LOGPATH}/kmsLuksMgt.log"
RESULT=0
###############################################################################


#############################################################
#下面为非root用户切换root用户需要执行的命令
if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo    
    CMD_WHICH="sudo ${INSTALL_SUDO_PATH} preinstall_which"
    CMD_MKDIR_P="sudo ${INSTALL_SUDO_PATH} preinstall_mkdir_p"
    CMD_READLINK_E="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_e"
    CMD_READLINK_M="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_m"
    CMD_CAT_FSTAB="sudo ${INSTALL_SUDO_PATH} cat_fstab"
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_FILE_S="sudo ${INSTALL_SUDO_PATH} preinstall_file_s"
    CMD_CHOWN="sudo ${INSTALL_SUDO_PATH} preinstall_chown"
    CMD_CHOWN_R="sudo ${INSTALL_SUDO_PATH} preinstall_chown_r"
    CMD_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
    CMD_CHMOD_R="sudo ${INSTALL_SUDO_PATH} preinstall_chmod_r"
    CMD_CP="sudo ${INSTALL_SUDO_PATH} preinstall_cp"
    CMD_INSTALL="sudo ${INSTALL_SUDO_PATH} preinstall_install"
    CMD_DOS2UNIX="sudo ${INSTALL_SUDO_PATH} preinstall_dos2unix"
    CMD_IS_DIR="sudo ${INSTALL_SUDO_PATH} preinstall_is_dir"
    CMD_IS_FILE="sudo ${INSTALL_SUDO_PATH} preinstall_is_file"
    CMD_SWAPON_S="sudo ${INSTALL_SUDO_PATH} preinstall_swapon_s"
else
    CMD_WHICH="which"
    CMD_MKDIR_P="mkdir -p"
    CMD_READLINK_E="readlink -e"
    CMD_READLINK_M="readlink -m"
    CMD_CAT_FSTAB=" cat /etc/fstab"
    CMD_FINDFS="findfs"
    CMD_FILE_S="file -s"
    CMD_CHOWN="chown"
    CMD_CHOWN_R="chown -R"
    CMD_CHMOD="chmod"
    CMD_CHMOD_R="chmod -R"
    CMD_CP="cp -arf"
    CMD_INSTALL="preinstall_install"
    CMD_DOS2UNIX="preinstall_dos2unix"
    CMD_IS_DIR="preinstall_is_dir"
    CMD_IS_FILE="preinstall_is_file"
    CMD_SWAPON_S="swapon -s"
fi
################################################################


########################################################################
#                           sudo封装开始
########################################################################
########################################################################
# 函数描述： 判断路径是否是一个目录
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_dir ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -d "$1" ]; then
        return 0
    fi

    return 1
}

########################################################################
# 函数描述： 判断路径是否是一个文件
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_file ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -f "$1" ]; then
                 return 0
        fi

    return 1
}


########################################################################
# 函数描述： 调用install命令
# 参数1说明：安装目录权限
# 参数2说明：安装路径
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_install()
{
    if [[ -z "$1" || -z "$2" ]]; then
        echo "arguments is null."
        return 1
    fi
    
        install -m "$1" -d "$2"
    return $?
}

########################################################################
# 函数描述： 将指定路径的文件全部执行dos2unix命令
# 参数1说明：待转换的路径
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_dos2unix()
{
        if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    find "$1" -type f | xargs dos2unix >/dev/null 2>&1
    return $?
}

########################################################################
#                           sudo封装结束
########################################################################

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi
    
    chmod 700 "${LOGPATH}"
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 记录脚本执行的上下文，主要用于定位用户的执行历史明细
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function log_begin()
{
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 "************************INSTALL-START************************"
    logme 1 "==$0 $*=="
    logme 1 "==who=="
    logme 1 "$(who)"
    logme 1 "************************INSTALL-END**************************"
    
    return 0
}


########################################################################
# 函数描述： 初始化路径变量
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_path()
{
    #初始化当前路径，为其他变量赋值做准备
    pushd $(dirname "$0") >/dev/null 2>&1
    CURRPATH="$(readlink -e "`pwd`")"
    popd >/dev/null 2>&1
    if [ -z "${CURRPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current path failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPATH is:${CURRPATH}"
    
    #设置全局保存设备变量
    set_temp_file
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] set temp file failed."
        return 1
    fi
    

    return 0
}

########################################################################
# 函数描述： 设置系统中的设备的保存文件绝对路径
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_temp_file()
{
    TMPDIR="${CURRPATH}/tmpdir"
    
    if [ ! -d "${TMPDIR}" ]; then
        mkdir -p "${TMPDIR}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkdir -p ${TMPDIR} failed."
            return 1
        fi
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查系统的有效信息，包括用户、操作系统、命令等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_system()
{
    #检查操作系统
    check_osversion
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check os version failed."
        return 1
    fi
    return 0
}


########################################################################
# 函数描述： 检查执行操作系统版本，目前支持redhat/suse/centos
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_osversion()
{
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local centos_release="/etc/centos-release"
    local neokylin_relase="/etc/neokylin-release"
    local bclinux_release="/etc/bclinux-release"
    
    #支持红帽/suse/centos
    #为了最大程度上的支持，都不会异常退出，但是会打印告警，表示可能不太支持，毕竟没有在其他操作系统上做过测试
    if [ -f "${centos_release}" ]; then
        cat "${centos_release}" | grep -i "CentOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is centos."
    #获取Euler的版本
    elif [ -f "${neokylin_relase}" ];then
        cat "${neokylin_relase}" | grep -i "NeoKylin" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not NeoKylin, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is NeoKylin."
    elif [ -f "${bclinux_release}" ];then
        cat "${bclinux_release}" | grep -i "BigCloud" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not bclinux, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is bclinux."        
    elif [ -f "${euler_release}" ]; then
        cat "${euler_release}" | grep -i "EulerOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not euleros, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is euleros."
    elif [ -f "${rhel_release}" ]; then
        cat "${rhel_release}" | grep -i "Red Hat" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not redhat, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is redhat."
    elif [ -f "${sles_release}" ]; then
        cat "${sles_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse."
    elif [ -f "${sles15_release}" ]; then
        cat "${sles15_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse-15, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse-15."
    else
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos/redhat/suse, maybe it will cause unknown result!"
        return 0
    fi
    
    
    return 0
}


########################################################################
# 函数描述： 安装硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function setup_service()
{
    local pid_d=0
    
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -eq 0 ]; then
        IS_EMPTY=1
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] diskmgt new."
    fi
    
    #既然确定要安装了，那么安装之前首先要清空之前可能存在的安装文件
    rollback
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] roll back failed."
        return 1
    fi
    
    #前面无法清空，那么失败
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -eq 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] diskmgt has been installed already."
        return 1
    fi
    
    #开始安装了，首先创建目录
    ${CMD_INSTALL} 750 "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install -m 750 -d ${SERVICEPATH} failed."
        return 1
    fi
    
    
    #挂载配置文件设定
    ${CMD_INSTALL} 750 "${SERVICEPATH}/ini"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install -m 750 -d ${SERVICEPATH}/ini failed."
        return 1
    fi
    if [ ! -d "${CURRPATH}/kmsLuksMgt/ini" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/ini is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/kmsLuksMgt/ini/${KMS_ENCRYPT_DEVS}" "${SERVICEPATH}/ini/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp {CURRPATH}/ini/${KMS_ENCRYPT_DEVS} ${SERVICEPATH}/ini/ -arf failed."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/ini-plugin/kms/${KMSINFO}" "${SERVICEPATH}/ini/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp {CURRPATH}/ini/${KMSINFO} ${SERVICEPATH}/ini/ -arf failed."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/kmsLuksMgt/ini/${KMS_ENCRYPT_DEVS}" "/usr/local/diskmgt/ini-plugin/kms/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/kmsLuksMgt/ini/${KMS_ENCRYPT_DEVS} /usr/local/diskmgt/ini-plugin/kms/ -arf failed."
        return 1
    fi
    
    #外部脚本拷贝
    if [ ! -d "${CURRPATH}/kmsLuksMgt/script" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/script is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/kmsLuksMgt/script" "${SERVICEPATH}/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/kmsLuksMgt/script ${SERVICEPATH} -arf failed."
        return 1
    fi
    
    #kmc工具拷贝
    if [ ! -d "${CURRPATH}/kmsLuksMgt/kmc" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/kmc is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/kmsLuksMgt/kmc" "${SERVICEPATH}/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/kmsLuksMgt/kmc ${SERVICEPATH} -arf failed."
        return 1
    fi
    
    #设置用户、群主、权限
    ${CMD_CHOWN_R} 0:0 "${SERVICEPATH}" 
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown 0:0 ${SERVICEPATH} -R failed."
        return 1
    fi
    
    ${CMD_CHMOD_R} 750 "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 755 ${SERVICEPATH} -R failed."
        return 1
    fi
    
    #安全起见，所有的配置文件和脚本都转为unix格式
    ${CMD_DOS2UNIX} "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] find ${SERVICEPATH} -type f | xargs dos2unix >/dev/null 2>&1 failed."
        return 1
    fi

    #把kmsLuksMgt做成系统服务，加入关机流程
    ${CMD_CONFIG_SERVICE}
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] config kmsLuksMgt service failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 回滚安装流程
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback()
{
    #如果没有主目录，杀死进程也是合理的
    if [ ${IS_EMPTY} -eq 1 ]; then
        #回退首先把服务的主目录删除干净
        ${CMD_IS_DIR} "${SERVICEPATH}"
        if [ $? -eq 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] uninstall old."
            rm -rf "${SERVICEPATH}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${SERVICEPATH} failed."
                return 1
            fi
        fi
    fi
    
    return 0
}


########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{
    #检查操作系统有效性
    check_system
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check system failed."
        return 1
    fi
    
    #安装硬盘管理服务
    setup_service
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] setup service failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 清除锁函数
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function clean_lock()
{
    rm -rf "${LOCKFILE}"
}


########################################################################
# 函数描述： 信号处理函数
# 参数1说明：无
#
# 返回值  ： 2 返回信号处理特殊错误码
########################################################################
function sig_catch()
{
    logme 2 "[EMER] [${FUNCNAME}:${LINENO}] catch signal!!"
    rollback
    clean_lock
    exit 2
}

#捕获信号
trap "sig_catch" INT TERM QUIT


########################################################################
# 函数描述： 以下为最外层函数，在锁保护的范围内
# 参数1说明：脚本的所有输入参数
#
# 返回值  ： 0 成功
#            1 普通失败错误码
#            2 返回信号处理特殊错误码
#            3 锁保护获取失败错误码
#            4 上次被强杀进程导致的锁遗留错误
#            5 后台正在运行硬盘管理服务错误码
########################################################################
log_begin "$@"
{
    #初始化路径全局变量
    init_path
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] init path failed."
        exit 1
    fi
    
    #配置diskmgt服务命令
    CMD_CONFIG_SERVICE="${CURRPATH}/kmsLuksMgt/config_kmsLuksMgt_service.sh"
    
    #执行主函数
    main
    RESULT=$?
    if [ ${RESULT} -ne 0 ]; then
        #只要失败就需要进行回退操作
        rollback
        logme 2 "[ERROR] [none:${LINENO}] execute `basename $0` failed."
    else
        logme 2 "[INFO] [none:${LINENO}] execute `basename $0` successfully."
    fi
}

