#!/bin/bash
#为保证关机过程中在blk-availability关闭加密分区前停止diskmgt服务
#把磁盘监控进程diskmgt做成系统服务，加入开关机流程

LOGPATH="${HOME}/diskmgt/"
LOGFILE="${LOGPATH}/insdmgt.log"

SERVICE_PATH="/etc/init.d/"
SERVICE_NAME="diskmgt"

SYSTEMD_SYSTEM_DIR="/usr/lib/systemd/system/"
SYSTEMD_UNIT_FILE="diskmgt.service"

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile
fi

if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_PREINSTALL_CP="sudo ${INSTALL_SUDO_PATH} preinstall_cp"
    CMD_PREINSTALL_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
    CMD_CREATE_SOFTLINK="sudo ${INSTALL_SUDO_PATH} preinstall_create_softlink"
    CMD_PREINSTALL_CHOWN="sudo ${INSTALL_SUDO_PATH} preinstall_chown"
    CMD_CHKCONFIG_CTL="sudo ${INSTALL_SUDO_PATH} chkconfig_ctl"
    CMD_SERVICES_CTL="sudo ${INSTALL_SUDO_PATH} services_ctl"
    CMD_SYSTEMCTL="sudo ${INSTALL_SUDO_PATH} system_ctl"
    CMD_ADD_DISKMGT_OPTIMIZED_TO_LOCAL="sudo ${INSTALL_SUDO_PATH} preinstall_add_diskmgt_optimized_to_local"
else
    CMD_PREINSTALL_CP="cp -arf"
    CMD_PREINSTALL_CHMOD="chmod"
    CMD_CREATE_SOFTLINK="ln -sf"
    CMD_PREINSTALL_CHOWN="chown"
    CMD_CHKCONFIG_CTL="chkconfig"
    CMD_SERVICES_CTL="service"
    CMD_SYSTEMCTL="systemctl"
    CMD_ADD_DISKMGT_OPTIMIZED_TO_LOCAL="preinstall_add_diskmgt_optimized_to_local"
fi

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme() {
    local log_mode=$1
    local log_info="$2"

    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi

    #没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi

    chmod 700 "${LOGPATH}"

    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}" #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi

    if [ -f "${LOGFILE}" ]; then
        chmod 640 "${LOGFILE}"
    fi

    return 0
}

function install_service() {
    if [ ! -d "$SERVICE_PATH" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] dir: $SERVICE_PATH is not exist."
        return 1
    fi

    if [ ! -f "$SERVICE_NAME" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] file: $SERVICE_NAME is not exist."
        return 1
    fi

    ${CMD_PREINSTALL_CP} $SERVICE_NAME $SERVICE_PATH/$SERVICE_NAME
    if [ $? -ne 0 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] cp $SERVICE_NAME $SERVICE_PATH/$SERVICE_NAME -f failed."
        return 1
    fi

    ${CMD_PREINSTALL_CHMOD} 755 $SERVICE_PATH/$SERVICE_NAME
    if [ $? -ne 0 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 755 $SERVICE_PATH/$SERVICE_NAME failed."
        return 1
    fi

    return 0
}

########################################################################
# 优化操作系统启动(操作系统启动时挂载加密分区)
########################################################################
function preinstall_add_diskmgt_optimized_to_local() {
    local osLocalFile="/etc/rc.local"
    local diskMgtServicePath="/usr/local/diskmgt"
    local crontabFile="/etc/crontab"
    local mountEncryptPartitionCmd="${diskMgtServicePath}/script/mount_encrypt_partition.sh"

    if [ -f "/etc/SuSE-release" ]; then
        osLocalFile="/etc/rc.d/after.local" #suse的local目录
    fi

    local realOsLocalFile=$(readlink -e "$osLocalFile")
    echo "os-local-file:[$osLocalFile], real-os-local-file:[${realOsLocalFile}]"
    if [ -z "${realOsLocalFile}" ] || [ ! -f "${realOsLocalFile}" ]; then
        echo "The rc-local.service file cannot be used to optimize the system startup. Therefore, the crontab scheduled task is used to optimize the system startup."
        cat "${crontabFile}" | grep "${mountEncryptPartitionCmd}" >/dev/null
        if [ $? -ne 0 ]; then
            echo "@reboot root sh ${mountEncryptPartitionCmd} 2>/dev/null" >> "${crontabFile}"
        else
            echo "The diskmgt-optimized script has been written down in the crontab-file[${crontabFile}]."
        fi
        return 0
    fi

    #往LOCAL文件登记REBOOT时执行的
    sed -n '1,$p' ${realOsLocalFile} | grep "${mountEncryptPartitionCmd}" >/dev/null
    if [ $? -ne 0 ]; then
        echo "sh ${mountEncryptPartitionCmd}" >>${realOsLocalFile}
    else
        echo "The diskmgt-optimized script has been written down in the local file."
    fi

    chmod +x "${realOsLocalFile}"
    echo "Written diskmgt-optimized script to ${realOsLocalFile} successfully."
    return 0
}

function main() {
    local cent_release="/etc/centos-release"
    local rhel_release="/etc/redhat-release"
    local euler_release="/etc/euleros-release"

    #对于redhat7.X版本和suse12.X版本，用systemctl方式把diskmgt加入开关机流程
    which "systemctl" >/dev/null 2>&1
    if [ $? -eq 0 -a -d "${SYSTEMD_SYSTEM_DIR}" ]; then
        ${CMD_PREINSTALL_CP} "${SYSTEMD_UNIT_FILE}" "${SYSTEMD_SYSTEM_DIR}/${SYSTEMD_UNIT_FILE}"

        ${CMD_PREINSTALL_CHMOD} 644 "${SYSTEMD_SYSTEM_DIR}/${SYSTEMD_UNIT_FILE}"
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 644 ${SYSTEMD_SYSTEM_DIR}/${SYSTEMD_UNIT_FILE} failed."
            return 1
        fi

        ${CMD_PREINSTALL_CHOWN} 0:0 "${SYSTEMD_SYSTEM_DIR}/${SYSTEMD_UNIT_FILE}"
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] chown 0:0 ${SYSTEMD_SYSTEM_DIR}/${SYSTEMD_UNIT_FILE} failed."
            return 1
        fi

        ${CMD_SYSTEMCTL} daemon-reload
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ${CMD_SYSTEMCTL} daemon-reload failed."
            return 1
        fi

        ${CMD_SYSTEMCTL} enable "$SERVICE_NAME"
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] systemctl enable $SERVICE_NAME on failed."
            return 1
        fi

    #如果是redhat系列低版本，用chkconfig方式把diskmgt加入开关机流程
    elif [ -f "${rhel_release}" -o -f "${cent_release}" -o -f "${euler_release}" ]; then
        install_service
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] call install_service failed."
            return 1
        fi

        ${CMD_CHKCONFIG_CTL} $SERVICE_NAME on
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] chkconfig $SERVICE_NAME on failed."
            return 1
        fi

    #分区加密特性不支持suse11.X
    else
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] skip to config $SERVICE_NAME service."
    fi

    ${CMD_ADD_DISKMGT_OPTIMIZED_TO_LOCAL} >>${LOGFILE} 2>&1

    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] config $SERVICE_NAME service success."
    return 0
}

cd $(dirname $0)

main
exit $?
