#!/bin/bash


###############################################################################
#author:   none
#date:     2014/02/16
#function: 硬盘管理服务对外的控制接口
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#设置当前的umask
UMASK=0077
umask "$UMASK"

#设置当前路径
CURRPATH=""

#服务目录和文件
SERVICEPATH="/usr/local/diskmgt"
SERVICERUN="/usr/local/diskmgt/inner/diskmgtd"

#控制debug日志的打印开关
DEBUGSWITCH="/usr/local/diskmgt/inner/debugswc"

#控制wce设置的文件
WCE_MAKR="/usr/local/diskmgt/tmpdir/wce_mark"

#记录WCE状态的文件
WCEFILE="/usr/local/diskmgt/ini/wce"

SYSTEMD_SYSTEM_DIR="/usr/lib/systemd/system/"

#启动变量
IS_START=0

#停止变量
IS_STOP=0

#重启变量
IS_RESTART=0

#系统中存在此脚本执行的总进程数
#主要用于此脚本如果被强杀那么就遗留锁文件，等待下一次进来此脚本被锁定判断其实是之前被强杀，就返回特定错误码告知此事
#并删除遗留锁
CMDCNT=0
TMPFILE=""

#锁文件
LOCKFILE="${HOME}/diskmgt.lock"

#日志目录和文件
LOGPATH="/var/log/diskmgt"
LOGFILE="/var/log/diskmgt/diskmgt.log"

RESULT=0
#############################################################################

S_CP="sudo /bin/cp"
S_RM="sudo /bin/rm"
S_CHMOD="sudo /bin/chmod"
S_CHOWN="sudo /bin/chown"
S_TEE="sudo /usr/bin/tee"
CMD_SYSTEMCTL="sudo ${INSTALL_SUDO_PATH} system_ctl"
if [ $(id -u) -eq 0 ]; then
    S_CP="cp"
    S_RM="rm"
    S_CHMOD="chmod"
    S_CHOWN="chown"
    S_TEE="tee"
    CMD_SYSTEMCTL="systemctl"
fi

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    if [ ! -f "${LOGFILE}" ]; then
        if [ $(id -u) -eq 0 ]; then
            if [ ! -e "${LOGPATH}" ]; then
                mkdir -p "${LOGPATH}"
            fi
            touch "${LOGFILE}"
            echo "[$(date)] [WARN] [${FUNCNAME}:${LINENO}] diskmgtmodule root need chown omm:wheel ${LOGFILE} where first create it." >>"${LOGFILE}"
            chown omm:wheel "${LOGFILE}"
        fi
    fi
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    
    return 0
}


########################################################################
# 函数描述： 获取文件的uid
# 参数1说明：文件名
#
# 返回值  ： 输出uid
########################################################################
function get_file_uid()
{
    stat -c %u "$1"
}

########################################################################
# 函数描述： 获取文件的gid
# 参数1说明：文件名
#
# 返回值  ： 输出gid
########################################################################
function get_file_gid()
{
    stat -c %g "$1"
}

########################################################################
# 函数描述： 获取文件的user
# 参数1说明：文件名
#
# 返回值  ： 输出user
########################################################################
function get_file_user()
{
    stat -c %U "$1"
}

########################################################################
# 函数描述： 获取文件的group
# 参数1说明：文件名
#
# 返回值  ： 输出group
########################################################################
function get_file_group()
{
    stat -c %G "$1"
}

########################################################################
# 函数描述： 检查执行此服务的用户角色
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_run_role()
{
    local curr_uid=0        #当前执行用户
    local curr_user=""
    
    local ser_run_user=""
    local ser_run_group=""
    
    local ser_path_uid=0    #文件1
    local log_path_uid=0    #文件2
    local ser_run_uid=0     #文件3
    
    
    #判断当前用户和群主
    curr_uid=$(id -u)
    if [ -z "${curr_uid}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] current uid is null."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] curr_uid is ${curr_uid}"
    
    if [ ${curr_uid} -eq 0 ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] root or sysadmin id 0 role."
        return 0
    fi
    
    curr_user="$(whoami)"
    if [ -z "${curr_user}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] current user is null."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] curr_user is ${curr_user}"
    
    
    #判断目录存在不
    if [ ! -d "${SERVICEPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${SERVICEPATH} is not exist."
        return 1
    fi
    if [ ! -d "${LOGPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${LOGPATH} is not exist."
        return 1
    fi
    
    
    #判断用户是否被删除掉
    ser_run_user="$(get_file_user ${SERVICERUN})"
    if [ -z "${ser_run_user}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ser_run_user is null."
        return 1
    fi
    getent passwd | grep "^${ser_run_user}:" >/dev/null
    if [ $? -ne 0 ]; then
        #用户不存在 
        ser_run_user=root
    fi
    if [ "${curr_user}" != "${ser_run_user}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] role deny for ser_run_user is ${ser_run_user}"
        return 1
    fi
    
    #判断群主是否被删除掉
    ser_run_group="$(get_file_group ${SERVICERUN})"
    if [ -z "${ser_run_group}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ser_run_group is null."
        return 1
    fi
    getent group | grep "^${ser_run_group}:" >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ser_run_group(${ser_run_group}) is not exist in system."
        return 1
    fi
    
    
    
    
    #判断各个文件和运行用户是否一致
    ser_path_uid=$(get_file_uid ${SERVICEPATH})
    log_path_uid=$(get_file_uid ${LOGPATH})
    ser_run_uid=$(get_file_uid ${SERVICERUN})
    
    if [ ${curr_uid} -ne ${ser_path_uid} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] role deny for ser_path_uid is ${ser_path_uid}"
        return 1
    fi
    
    if [ ${curr_uid} -ne ${log_path_uid} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] role deny for log_path_uid is ${log_path_uid}"
        return 1
    fi

    if [ ${curr_uid} -ne ${ser_run_uid} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] role deny for ser_run_uid is ${ser_run_uid}"
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 记录脚本执行的上下文，主要用于定位用户的执行历史明细
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function log_begin()
{
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 "************************DISKMGT-START************************"
    logme 1 "==$0 $*=="
    logme 1 "==who=="
    logme 1 "$(who)"
    get_call_stack
    logme 1 "************************DISKMGT-END**************************"
    
    return 0
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-s         Notes: start diskmgtd."
    echo "-k         Notes: stop diskmgtd."
    echo "-r         Notes: restart diskmgtd."
    echo "-l 0/1     Notes: close/open debug log."
    echo "-w 0/1     Notes: clear/set disk Writeback Cache Enable."
    echo "-h         Notes: show help."
    echo
    echo "Example:"
    echo "`basename $0` -s"
    echo "`basename $0` -k"
    echo "`basename $0` -r"
    echo "`basename $0` -l 0"
    echo "`basename $0` -l 1"
    echo "`basename $0` -h"
    echo
    
    return 0
}


########################################################################
# 函数描述： 初始化路径变量
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_path()
{
    #初始化当前路径
    pushd $(dirname "$0") >/dev/null 2>&1
    CURRPATH="$(readlink -e "`pwd`")"
    popd >/dev/null 2>&1
    if [ -z "${CURRPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current path failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPATH is:${CURRPATH}"
    
    #压缩日志
    compress_log
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] compress log failed."
        return 1
    fi

    return 0
}


########################################################################
# 函数描述： 压缩日志
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function compress_log()
{
    script_log="${CURRPATH}/../inner/scriptlog.sh"
    
    if [ ! -f "${script_log}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] compress script(${script_log}) is not exist."
        return 1
    fi
    
    if [ ! -x "${script_log}" ]; then
        chmod 755 "${script_log}"
    fi
    
    #压缩日志
    if [ -f "${LOGFILE}" ]; then
        "${script_log}" "${LOGFILE}"
    fi
    
    return 0
}

########################################################################
# 函数描述： 获取脚本执行的调用栈
# 参数1说明：进程pid号
#
# 返回值  ： 0 成功
########################################################################
function get_call_stack()
{
    local pid_val=$1
    local line=""

    if [ -z "${pid_val}" ]; then
        pid_val=$$
    fi
    
    logme 1 "==stack-begin=="
    
    #获取调用栈
    while [[ ${pid_val} != 1 && ${pid_val} != 0 ]]
    do
        line=$(ps -wweo pid,ppid,uname,tname,args | grep "^[[:space:]]*${pid_val}[[:space:]]")
        if [ -z "${line}" ]; then
           break
        fi
        pid_val=$(echo ${line} | awk '{print $2}')
        logme 1 "${line}"
    done
    
    logme 1 "==stack-end=="
    
    return 0
}

########################################################################
# 函数描述： 启动服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function diskmgtd_start()
{
    local run_pids=0
    
    #如果已经启动服务，则直接返回成功
    run_pids=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
    if [ ! -z "${run_pids}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] diskmgtd is running already."
        return 0
    fi
    
    which "systemctl" > /dev/null 2>&1
    if [ $? -eq 0 -a -d "${SYSTEMD_SYSTEM_DIR}" ]; then
        ${CMD_SYSTEMCTL} start "diskmgt.service"
        return $?
    else
        #cron配置文件不存在，说明安装目录被人破坏掉
        #具体请看安装流程会拷贝这个文件的
        if [ ! -f "${CURRPATH}/../inner/diskmgtd.cron" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${CURRPATH}/../inner/diskmgtd.cron is not exist."
            return 1
        fi
        
        #操作系统的cron机制被破坏了
        if [ ! -d "/etc/cron.d/" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir /etc/cron.d/ is not exist."
            return 1
        fi
        
        #开始拷贝cron配置，确保此服务能有监控机制
        ${S_CP} "${CURRPATH}/../inner/diskmgtd.cron" /etc/cron.d/ -arf
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_CP} ${CURRPATH}/../inner/diskmgtd.cron /etc/cron.d/ -arf failed."
            return 1
        fi
        
        #注意cron配置文件的权限和用户群主
        ${S_CHMOD} 644 "/etc/cron.d/diskmgtd.cron"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_CHMOD} 644 /etc/cron.d/diskmgtd.cron failed."
            return 1
        fi
        
        ${S_CHOWN} 0:0 "/etc/cron.d/diskmgtd.cron"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_CHOWN} root:root /etc/cron.d/diskmgtd.cron failed."
            return 1
        fi
        
        #后台运行服务，启动完成
        nohup ${SERVICERUN} >/dev/null 2>&1 &
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] running diskmgtd in the background."
    fi
    
    return 0
}

########################################################################
# 函数描述： 停止服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            其他返回值 失败
########################################################################
function diskmgtd_stop()
{
    which "systemctl" > /dev/null 2>&1
    if [ $? -eq 0 -a -d "${SYSTEMD_SYSTEM_DIR}" ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] diskmgtd_stop:${CMD_SYSTEMCTL} stop diskmgt.service"
        ${CMD_SYSTEMCTL} stop "diskmgt.service"
        return $?
    else
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] diskmgtd_stop by kill process"
        kill_diskmgtd
        return $?
    fi
}

########################################################################
# 函数描述： 重启服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            其他返回值 失败
########################################################################
function diskmgtd_restart()
{
    which "systemctl" > /dev/null 2>&1
    if [ $? -eq 0 -a -d "${SYSTEMD_SYSTEM_DIR}" ]; then
        ${CMD_SYSTEMCTL} restart "diskmgt.service"
        return $?
    else
        #先停止
        diskmgtd_stop
        if [ $? -ne 0 ]; then
            return 1
        fi
        
        #再启动
        diskmgtd_start
        return $?
    fi
}

########################################################################
# 函数描述： 停止服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function kill_diskmgtd()
{
    local run_pids=0
    local count=0
    local count_max=20
    
    #停止之前，切忌先把cron的配置清除，免得进程停止后又被拉起
    if [ -f "/etc/cron.d/diskmgtd.cron" ]; then
        ${S_RM} -rf "/etc/cron.d/diskmgtd.cron"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_RM} -rf /etc/cron.d/diskmgtd.cron failed."
            return 1
        fi
    fi
    
    #循环停止服务进程，如果120秒后还停止不了，则停止服务失败
    #一般要注意都是权限不对导致停止不了，比如omm用户要停止root的进程当然失败
    while [ ${count} -le ${count_max} ]
    do
        run_pids=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
        if [ -z "${run_pids}" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] kill deamon ok."
            return 0
        fi
        
        logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill deamon pid(${run_pids})."
        echo ${run_pids} | xargs kill >>"${LOGFILE}" 2>&1
        
        ((count++))
        if [ ${count} -eq ${count_max} ]; then
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill force deamon pid(${run_pids})."
            echo ${run_pids} | xargs kill -9 >>"${LOGFILE}" 2>&1
            ${S_RM} -f "/usr/local/diskmgtd.lock"
        fi
        
        sleep 1
    done
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] kill deamon failed after ${count_max} times."
    return 1
}

########################################################################
# 函数描述： 控制debug日志打印开关
# 参数1说明：0 开启debug打印
#            1 关闭debug打印
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_debuglog()
{
    local debug_val=$1
    
    if [ -z "${debug_val}" ]; then
        logme 2 "[ERROR] [none:${LINENO}] debug value para is null."
        return 1
    fi

    #DEBUGSWITCH文件的存在，对diskmgtd服务的logme的函数分支有影响
    if [ ${debug_val} -eq 0 ]; then
        ${S_RM} -f "${DEBUGSWITCH}"
    elif [ ${debug_val} -eq 1 ]; then
        touch "${DEBUGSWITCH}"
    else
        logme 2 "[ERROR] [none:${LINENO}] invalid debug value."
        return 1
    fi
    
    logme 2 "[ERROR] [none:${LINENO}] set debug value as ${debug_val} successfully."
    return 0
}

########################################################################
# 函数描述： 打开/关闭 硬盘缓存
# 参数1说明：0 Clear WCE
#            1 Set WCE
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_wce()
{
    local wce_val=$1
    
    if [ -z "${wce_val}" ]; then
        show_help
        return 1
    fi

    if [ ${wce_val} -eq 0 ]; then
        echo 0 > ${WCEFILE}
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] echo 0 > ${WCEFILE} failed."
            return 1
        fi
        
        #删除所有控制文件
        ${S_RM} -f "${WCE_MAKR}"*
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_RM} -f ${WCE_MAKR} failed."
            return 1
        fi
        
    elif [ ${wce_val} -eq 1 ]; then
        echo 1 > ${WCEFILE}
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] echo 1 > ${WCEFILE} failed."
            return 1
        fi
        
        #删除所有控制文件
        ${S_RM} -f "${WCE_MAKR}"*
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_RM} -f ${WCE_MAKR} failed."
            return 1
        fi
    else
        show_help
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] set wce value as ${wce_val} successfully."
    
    return 0
}

########################################################################
# 函数描述： 清除锁函数
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function clean_lock()
{
    ${S_RM} -rf "${LOCKFILE}"
}


########################################################################
# 函数描述： 信号处理函数
# 参数1说明：无
#
# 返回值  ： 2 返回信号处理特殊错误码
########################################################################
function sig_catch()
{
    logme 2 "[EMER] [${FUNCNAME}:${LINENO}] catch signal!!"
    clean_lock
    exit 2
}

#捕获信号
trap "sig_catch" INT TERM QUIT


########################################################################
# 函数描述： 以下为最外层函数，在锁保护的范围内
# 参数1说明：脚本的所有输入参数
#
# 返回值  ： 0 成功
#            1 普通失败错误码
#            2 返回信号处理特殊错误码
#            3 锁保护获取失败错误码
#            4 上次被强杀进程导致的锁遗留错误
#            5 后台正在运行硬盘管理服务错误码
########################################################################
log_begin "$@"

if [ $(id -u) -ne 0 ] && [ -f "${LOCKFILE}" ]; then
    ${S_CHOWN} omm:wheel "${LOCKFILE}"
fi

if [ $(id -u) -ne 0 ] && [ -d "${LOGPATH}" ]; then
    ${S_CHOWN} omm:wheel "${LOGPATH}" -R
fi

{    
    flock -n 3
    #如果获取锁失败，要判断之前是否是被强杀进程导致锁遗留
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] `basename $0` locked by another user."
        
        #这里判断到之前是否是被强杀进程导致锁遗留，就把遗留锁清理干净
        TMPFILE="${HOME}/diskmgttmpfile_`date +%s`.txt"
        ps -wweo pid,ppid,uname,tname,args | grep -v grep | grep "diskmgt.sh" | grep "/bin/" >"${TMPFILE}"
        CMDCNT=$(cat "${TMPFILE}" | wc -l)
        if [ ${CMDCNT} -eq 1 ]; then
            logme 2 "[ERROR] [none:${LINENO}] `basename $0` kill force last time, clean lock now."
            clean_lock
            exit 4
        fi
        
        exit 3
    fi
    
    #初始化路径全局变量
    init_path
    if [ $? -ne 0 ]; then
        clean_lock
        logme 2 "[ERROR] [none:${LINENO}] init path failed."
        exit 5
    fi
    
    #检查执行角色
    check_run_role
    if [ $? -ne 0 ]; then
        clean_lock
        logme 2 "[ERROR] [none:${LINENO}] check run role failed."
        exit 6
    fi
    
    #脚本输入所有参数的处理流程
    while getopts "skrl:w:h" options;
    do
        case ${options} in
            s)
                IS_START=1  #启动
                ;;
                
            k)
                IS_STOP=1  #停止
                ;;
                
            r)
                IS_RESTART=1 #重启
                ;;
                
            l)
                set_debuglog ${OPTARG}  #打印debug日志开关
                exit $?
                ;;
            w)
                set_wce ${OPTARG}
                exit $?
                ;; 
            h)
                show_help  #帮助
                exit $?
                ;;
                
            *)
                logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #输入参数非法
                show_help
                exit 1
                ;;
        esac
    done

    #判断输入参数有效性
    if [ $# -ne 1 ]; then
        logme 2 "[ERROR] [none:${LINENO}] invalid input!"
        show_help
        exit 1
    fi
    
    #是启动、停止、重启的哪种接口
    if [ ${IS_START} -eq 1 ]; then
        diskmgtd_start
    elif [ ${IS_STOP} -eq 1 ]; then
        diskmgtd_stop
    elif [ ${IS_RESTART} -eq 1 ]; then 
        diskmgtd_restart
    else
        logme 2 "[ERROR] [none:${LINENO}] invalid input!!!"
        show_help
        exit 1
    fi
    
    #成功或失败后统一打印出口
    RESULT=$?
    if [ ${RESULT} -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] execute `basename $0` failed."
    else
        logme 2 "[INFO] [none:${LINENO}] execute `basename $0` successfully."
    fi
    
    #清理锁并返回特定返回码
    clean_lock
    exit ${RESULT}
} 3<>"${LOCKFILE}"

