#!/bin/bash


###############################################################################
#author:   none
#date:     2014/02/16
#function: 硬盘管理服务对外的卸载接口
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#设置当前的umask
UMASK=0077
umask "$UMASK"

#服务目录和文件
SERVICEPATH="/usr/local/diskmgt"
SERVICERUN="/usr/local/diskmgt/inner/diskmgtd"

#luksmgt服务目录和卸载脚本
LUKSPATH="/usr/local/luksmgt/"
LUKSUNISTALL="/usr/local/luksmgt/script/uninstall.sh"

#日志目录和文件
LOGPATH="/var/log/diskmgt/"
LOGFILE="/var/log/diskmgt/uninsdmgt.log"

#文件系统配置文件
FSTABFILE="/etc/fstab"
CRYPTTABFILE="/etc/crypttab"

#文件系统挂载记录文件
PROCMOUNTFILE="/proc/mounts"

INITD_PATH="/etc/init.d/"
SERVICE_NAME="diskmgt"

SYSTEMD_SYSTEM_DIR="/usr/lib/systemd/system/"
SYSTEMD_UNIT_FILE="diskmgt.service"

#命令
CMD_CAT="/bin/cat"
CMD_UMOUNT="/bin/umount"
CMD_READLINK=readlink

IS_FORCE=0
IS_UMOUNT=0
RESULT=0
###############################################################################


S_RM="sudo /bin/rm"
S_SYSTEMCTL="sudo systemctl"
S_CHKCONFIG="sudo chkconfig"
if [ $(id -u) -eq 0 ]; then
    S_RM="rm"
    S_SYSTEMCTL="systemctl"
    S_CHKCONFIG="chkconfig"
fi

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] [EMER] diskmgtmodule mkdir -p ${LOGPATH} failed." >>/var/log/messages
            echo "[$(date)] ${log_info}" >>/var/log/messages
            return 1
        fi
    fi
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"
    else
        echo "logme invalid mode para."
        return 1
    fi

    if [ -f "${LOGFILE}" ]; then
        chmod 640 "${LOGFILE}"
    fi
    
    return 0
}

########################################################################
# 函数描述： 记录审计日志
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function syslog()
{
    local component=$1
    local filename=$2
    local status=$3
    local message=$4

    which logger >/dev/null 2>&1
    [ "$?" -ne "0" ] && return 0;

    login_user_ip="$(who -m | sed 's/.*(//g;s/)*$//g')"
    execute_user_name="$(whoami)"
    logger -p local0.notice -i "FusionInsight;${component};[${filename}];${status};${login_user_ip};${execute_user_name};${message}"
    return 0
}


########################################################################
# 函数描述： 记录脚本执行的上下文，主要用于定位用户的执行历史明细
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function log_begin()
{
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 "************************UNINSTALL-START************************"
    logme 1 "==$0 $*=="
    logme 1 "==who=="
    logme 1 "$(who)"
    get_call_stack
    logme 1 "************************UNINSTALL-END**************************"
    sync
    
    return 0
}


########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "                      Notes: Uninstall diskmgt."
    echo "-u                    Notes: Uninstall diskmgt, umount the partions monitored by diskmgt, and delete these items from /etc/fstab."
    echo "-f                    Notes: Uninstall diskmgt, umount the partions monitored by diskmgt(compulsively), and delete these items from /etc/fstab."
    echo "-h                    Notes: Show help."
    echo
    echo "Example:"
    echo "`basename $0` -f"
    echo
    return 0
}


########################################################################
# 函数描述： 停止服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function kill_diskmgtd()
{
    local run_pids=0
    local count=0
    local count_max=60
    
    #关闭diskmgt服务的开机启动
    which "systemctl" > /dev/null 2>&1
    if [ $? -eq 0 -a -d "${SYSTEMD_SYSTEM_DIR}" ]; then
        #redhat7.X版本及suse12.X版本，使用systemctl的方式
        ${S_SYSTEMCTL} stop ${SYSTEMD_UNIT_FILE}
        ${S_RM} -f ${SYSTEMD_SYSTEM_DIR}/${SYSTEMD_UNIT_FILE}
        ${S_SYSTEMCTL} daemon-reload
    else
        #低版本使用chkconfig的方式
        ${S_CHKCONFIG} --del ${SERVICE_NAME}
        ${S_RM} -f ${INITD_PATH}/${SERVICE_NAME}       
    fi
    
    #停止之前，切记先把cron的配置清除，免得进程停止后又被拉起
    if [ -f "/etc/cron.d/diskmgtd.cron" ]; then
        ${S_RM} -rf "/etc/cron.d/diskmgtd.cron"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${S_RM} -rf /etc/cron.d/diskmgtd.cron failed."
            return 1
        fi
    fi

    #循环停止服务进程，如果120秒后还停止不了，则停止服务失败
    #一般要注意都是权限不对导致停止不了，比如omm用户要停止root的进程当然失败
    while [ ${count} -le ${count_max} ]
    do
        run_pids=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
        if [ -z "${run_pids}" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] kill deamon ok."
            return 0
        fi
        
        logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill deamon pid(${run_pids})."
        echo ${run_pids} | xargs kill >>"${LOGFILE}" 2>&1
        
        ((count++))
        if [ ${count} -eq ${count_max} ]; then
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill force deamon pid(${run_pids})."
            echo ${run_pids} | xargs kill -9 >>"${LOGFILE}" 2>&1
            ${S_RM} -f "/usr/local/diskmgtd.lock"
        fi
        
        sleep 2
    done
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] kill deamon failed after ${count_max} times."
    return 1
}


########################################################################
# 函数描述： 获取脚本执行的调用栈
# 参数1说明：进程pid号
#
# 返回值  ： 0 成功
########################################################################
function get_call_stack()
{
    local pid_val=$1
    local line=""

    if [ -z "${pid_val}" ]; then
        pid_val=$$
    fi
    
    logme 1 "==stack-begin=="
    
    while [[ ${pid_val} != 1 && ${pid_val} != 0 ]]
    do
        line=$(ps -wweo pid,ppid,uname,tname,args | grep "^[[:space:]]*${pid_val}[[:space:]]")
        if [ -z "${line}" ]; then
           break
        fi
        pid_val=$(echo ${line} | awk '{print $2}')
        logme 1 "${line}"
    done
    
    logme 1 "==stack-end=="
    
    return 0
}

########################################################################
# 函数描述： 按照规则把目录名转换为加密后的映射名
# 参数1说明：挂载项的数据目录的目录名（如"/srv/BigData/dbdata_service"）
#
# 返回值  ： 转换后的映射名（如"encrypted-srv-BigData-dbdata_service"）
########################################################################
function get_dir_encrypt_mapper()
{
    local tmp_dir="$1"
    local dir=""
    local tmp_mapper=""
    local mapper=""
    
    dir="$(eval ${CMD_READLINK} -m ${tmp_dir})"
    #把目录中的'\'替换成'-'，并在开头加上"encrypted"字段
    tmp_mapper=$(echo "$dir" | sed 's/\//-/g' 2>/dev/null)
    mapper="encrypted$tmp_mapper"
    echo "$mapper"
    return 0
}

########################################################################
# 函数描述： 清除fstab中需要diskmgt监控的挂载目录
# 参数1说明：需要回退的挂载目录信息
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function clean_fstab()
{
    local dir="$1"
    local dev_mapper=""
    
    if [ ! -f $FSTABFILE ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] $FSTABFILE does not exist."
        return 1
    fi
    
    local dir_count=$(grep -v "^[[:blank:]]*#" $FSTABFILE | grep -c "[[:blank:]]$dir[[:blank:]]")
    if [ $dir_count -eq 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] item $dir is not found in $FSTABFILE. So skip to delete it."
        return 0
    fi
    
    sed -i 's# '${dir}' # EXCLUSIVE_IMPOSSIBLE #g;/ EXCLUSIVE_IMPOSSIBLE /d' "$FSTABFILE"  >>"${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] delete item $dir in $FSTABFILE failed."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 清除crypttab中需要diskmgt监控的挂载目录
# 参数1说明：需要回退的挂载目录信息
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function clean_crypttab()
{
    local dir="$1"
    local encrypt="$2"
    if [ ! -f $CRYPTTABFILE ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] $CRYPTTABFILE does not exist."
        return 0
    fi
    
    if [ ! -z "$encrypt" ]; then
        dev_mapper=$(get_dir_encrypt_mapper "$dir")
        sed -i 's#^[[:space:]]*'${dev_mapper}' # EXCLUSIVE_IMPOSSIBLE #g;/ EXCLUSIVE_IMPOSSIBLE /d' "$CRYPTTABFILE" >> "$LOGFILE" 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to delete item $dev_part in $CRYPTTABFILE."
            return 1
        fi
    fi
    
    return 0
}


########################################################################
# 函数描述： 卸载需要回退的挂载目录
# 参数1说明：需要回退的挂载目录信息
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function umount_dir()
{
    local line="$1"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} now."
    
    #第1次卸载
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed once?"
        return 0
    fi
    
    eval ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} ok."
        return 0
    else
        if [ ${IS_FORCE} -eq 0 ];then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount ${line} failed."
            return 1
        fi
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} failed once."
    
    
    #第2次卸载
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1 
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed twice?"
        return 0
    fi
    
    fuser -mk "${line}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] fuser -mk ${line} failed."
    fi
    
    eval ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} failed twice."
    
    
    
    #第3次卸载
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed third times?"
        return 0
    fi
    
    kill -9 $(lsof | grep -w " ${line}" | awk '{print $2}') >/dev/null 2>&1
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed after kill?"
        return 0
    fi
    
    fuser -mk "${line}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] fuser -mk ${line} failed."
    fi
    eval ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} failed third times, need to umount force."
    
    
    eval ${CMD_UMOUNT} -l "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount -l ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount -l ${line} failed."
    
    
    #最后再确认下是否卸载成功
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed fourth times?"
        return 0
    fi
    
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount ${line} failed finally."
    return 1
}

########################################################################
# 函数描述： 定期监控挂载配置文件的目录
# 参数1说明：指定的配置文件的每一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function clean_line()
{
    local line="$1"
    local line_column_num=4
    local line_column_nf=0
    
    local column_care=""
    local column_conf=""
    local item_nf=0
    local index=0
    
    
    #参数判断
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] clean line para is null."
        return 1
    fi
    line_column_nf=$(echo ${line} | awk '{print NF}')
    if [ ${line_column_nf} -ne ${line_column_num} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column num(${line_column_nf}) is not ${line_column_num}."
        return 1
    fi
    
    
    #是否需要关注，如果不需要就直接退出，主要是针对类似UDN的客户对象，他们可能不需要我们帮他们监控硬盘设备，我们的这个脚本就空转
    column_care="$(echo ${line} | awk '{print $3}')"
    if [ "${column_care}" != "y" ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] i do not care ${line}."
        return 0
    fi
    
    
    #获取每个配置项，并循环处理
    column_conf="$(echo ${line} | awk '{print $1}')"
    item_nf=$(echo "${column_conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        local item=""
        local item_file=""
        local item_i_dir=""
        local item_i_dir_tmp=""
        local item_i_mapper=""
        local ret=0
        
        #挂载项
        item=$(echo "${column_conf}" | awk -F '#' "{print \$${index}}")
        if [ -z "${item}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get item is null."
            return 1
        fi
        item_file="${SERVICEPATH}/ini-plugin/conf/${item}"
        if [ ! -f "${item_file}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${item_file} is not exist."
            return 1
        fi
        
        #挂载目录
        item_i_dir_tmp="$(cat ${item_file} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_dir_tmp}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get dir failed."
            return 1
        fi
        item_i_dir="$(eval ${CMD_READLINK} -m ${item_i_dir_tmp})"
        if [ -z "${item_i_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] readlink -m ${item_i_dir_tmp} failed."
            return 1
        fi
        
        item_i_encrypt="$(cat ${item_file} | grep "^[[:space:]]*encrypt=" | awk '{print $1}' | sed 's/encrypt=//g' | head -n 1 2>/dev/null)"
        
        #如果挂载信息中未找到该目录，在日志中记录错误信息后继续进行
        eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${item_i_dir} " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] mount item(${item}) dir(${item_i_dir}) doesn't exist. Maybe dir(${item_i_dir}) is already umounted"
            clean_fstab "${item_i_dir}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] delete item(${item}) dir(${item_i_dir}) in $FSTABFILE failed! You may need to do it manually."
                return 1
            fi
            
            clean_crypttab "${item_i_dir}" "${item_i_encrypt}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] delete item(${item}) dir(${item_i_dir}) in $CRYPTTABFILE failed! You may need to do it manually."
                return 1
            fi
        else
            umount_dir "${item_i_dir}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount item(${item}) dir(${item_i_dir}) failed! You may need to do it manually."
                return 1
            fi
            
            if [ ! -z "${item_i_encrypt}" ]; then
                item_i_mapper=$(get_dir_encrypt_mapper "$item_i_dir")
                cryptsetup luksClose "$item_i_mapper" >> "$LOGFILE" 2>&1
            fi
            
            clean_fstab "${item_i_dir}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] delete item(${item}) dir(${item_i_dir}) in $FSTABFILE failed! You may need to do it manually."
                return 1
            fi
            
            clean_crypttab "${item_i_dir}" "${item_i_encrypt}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] delete item(${item}) dir(${item_i_dir}) in $CRYPTTABFILE failed! You may need to do it manually."
                return 1
            fi
        fi
        
    done
    
    
    return 0
}


########################################################################
# 函数描述： 对diskmgt所监控的目录进行umount，并将记录从fstab中删除
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function umount_monitored_dir()
{
    local ini_num=0
    
    #只能有唯一的一个挂载配置文件，否则不知道要根据哪个配置文件卸载挂载点了
    ini_num=$(find ${SERVICEPATH}/ini -type f | grep "\.ini$" | wc -l)
    if [ ${ini_num} -ne 1 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile num(${ini_num}) in ${SERVICEPATH}/ini is not only."
        return 1
    fi

    local ini_file="$(find ${SERVICEPATH}/ini -type f | grep "\.ini$")"
    if [ -z "${ini_file}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get inifile in ${SERVICEPATH}/ini failed."
        return 1
    fi

    #对每一个挂载配置项进行监控
    while read line
    do
        if [ -z "${line}" ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] line para is null."
            continue
        fi

        clean_line "${line}"
        if [ $? -ne 0 ];then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] clean line ${line} failed."
            return 1
        fi
    done <"${ini_file}"
    
    if [ -d $"{LUKSPATH}" ]; then
        bash ${LUKSUNISTALL}
        if [ $? -ne 0 ];then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] uninstall luksmgt failed."
            return 1
        fi
    fi
}


########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{
    local crontabFile="/etc/crontab"
    #首先停止diskmgt服务并关闭开机启动
    kill_diskmgtd
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] kill diskmgtd failed."
        return 1
    fi

    #是否需要umount掉diskmgt所监控的目录
    if [ ${IS_FORCE} -eq 1 -o ${IS_UMOUNT} -eq 1 ]; then
        umount_monitored_dir
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount monitored dir failed."
            return 1
        fi
    fi

    #再删除安装目录
    ${S_RM} -rf "${SERVICEPATH}"
    sed -i  '/mount_encrypt_partition/d' "${crontabFile}"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] uninstall diskmgt successfully."
    return 0
}

syslog "diskmgt" "uninstall.sh" "begin" "diskmgt uninstall.sh $* begin."
log_begin "$@"

#脚本输入所有参数的处理流程
while getopts "ufh" options;
do
    case ${options} in
        u)
            IS_UMOUNT=1  #是否umount磁盘管理服务所监控的分区
            ;;

        f)
            IS_FORCE=1  #是否强制umount磁盘管理服务所监控的分区
            ;;
            
        h)
            show_help   #显示帮助
            exit $?
            ;;
            
        *)
            logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #非法参数输入
            show_help
            exit 1
            ;;
    esac
done

#调用主函数
main "$@"
ret=$?

syslog "diskmgt" "uninstall.sh" "exit=${ret}" "diskmgt uninstall.sh $* end."
exit $ret
