#!/bin/bash
###################################################
# 功能: 安装入口
###################################################

###################################################
# 全局变量
###################################################
gc_ipaddr=""       #本机的ip地址(用于日志输出)
gc_debug=0         #是否输出debug日志
gc_check=1         #是否在OS适配之前进行检查
gc_parted=2        #是否格式化磁盘,默认强制进行磁盘格式化
gc_parted_conf=""  #磁盘格式化的配置文件
gc_sched_file=""   #记录进度的文件
gc_support_os=""   #支持的OS版本
gc_swap_off=1      #是否关闭swap的标记
gc_services_ip=""  #本机的service ip地址
gc_ip_model=""     #ip地址格式
gc_hostname=""     #本机的hostname
gc_feed_random_setup=0   #本机的random data source
gc_mian_path="$(dirname "$0")"                  #install.sh脚本所在目录
gc_p_sched_file="${gc_mian_path}/parted_sched"  #格式化磁盘的进度

###################################################
# 输出error日志
###################################################
function error()
{
    #直接使用echo 输出error日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [ERROR]" "$@"
}

###################################################
# 输出info日志
###################################################
function info()
{
    #直接使用echo 输出info日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [INFO]" "$@"
}

###################################################
# 输出debug日志
###################################################
function debug()
{
    #直接使用echo 输出debug日志 服务端会重定向该日志
    if [ "${gc_debug}" -eq 1 ]; then
        echo "[$(date)][${gc_ipaddr}] [DEBUG]" "$@"
    fi
}

###################################################
#
# 获取OS的名称
#
###################################################
function get_osname_c()
{
    local cent_release="/etc/centos-release"
    local ole_release="/etc/oracle-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local euler_release="/etc/euleros-release"
    local neokylin_relase="/etc/neokylin-release"
    local bclinux_release="/etc/bclinux-release"
    local kylin_relase="/etc/kylin-release"
    local hce_release="/etc/hce-release"
    local linxos_release="/etc/linxos-release"
    local name=""
    
    if [ -f "${linxos_release}" ]; then
        name=$(awk '{print $1"-"$2}' ${linxos_release})
        echo "$name"
        return 0
    fi

    #获取Euler的版本, Euleros中存在redhat-release和CentOS-release，最先检查    
    if [ -f "${euler_release}" ]; then
        if grep -q "release" "${euler_release}"; then
            name="$(cat $euler_release | sed 's/x86_64//g' | awk -F " |SP|)" '{print "euleros-"$3"sp"$5}')"
        else
            name="euleros-$(cat "${euler_release}" | awk '{print $2}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
        fi
        echo "$name"
        return 0
    fi
    
    if [ -f "${bclinux_release}" ];then
        if grep -q "Euler" "${bclinux_release}";then
            name="bclinux-$(cat ${bclinux_release} | awk '{print $7}' | awk -F. '{print $1"."$2}')"
        else
            name="bclinux-$(cat "${bclinux_release}" | awk '{print $5}' | awk -F. '{print $1"."$2}')"
        fi
        echo "$name"
        return 0
    fi
    
    #备注: CentOS中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${cent_release}" ]; then
        if grep -q "Linux" "${cent_release}"; then
            name="centos-$(cat "${cent_release}" | awk '{print $4}' | awk -F. '{print $1"."$2}')"
        else
            name="centos-$(cat "${cent_release}" | awk '{print $3}')"
        fi
        echo "$name"
        return 0
    fi

    if [ -f "${neokylin_relase}" ];then
        if grep -q "release" "${neokylin_relase}"; then
            name="$(cat "${neokylin_relase}" | awk '{print $6}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            if [ -z "${name}" ];then
                name="$(cat "${neokylin_relase}" | awk '{print $4}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            fi
            name="neokylin-$(echo ${name/update/'.'})"
        fi
        echo "$name"
        return 0
    fi
    #获取KylinSecOS的版本
    if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}"|awk -F " " '{print $1}')" ]; then
        name="kylinSecOS-$(grep "^VERSION_ID=" "${sles15_release}" | awk -F "\"" '{print $2}')"
        echo "$name"
        return 0
    fi
    #备注: oracle linux中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${ole_release}" ]; then
        name="oracle-$(cat "${ole_release}" | awk '{print $5}')"
        echo "$name"
        return 0
    fi

    #获取RedHat的版本
    if [ -f "${rhel_release}" ]; then
        name="redhat-$(cat "${rhel_release}" | awk '{print $7}')"
        echo "$name"
        return 0
    fi

    #获取SuSE的版本
    if [ -f "${sles_release}" ]; then
        name="suse-$(cat "${sles_release}" | grep "^VERSION" | awk '{print $3}')"
        name="${name}.$(cat "${sles_release}" | grep "^PATCHLEVEL" | awk '{print $3}')"
        echo "$name"
        return 0
    fi
    if [ -f "${hce_release}" ];then
        if [ "hce" = "$(cat "${sles15_release}" | grep "^ID=" | awk -F "\"" '{print $2}')" ]; then
            name="hce-$(cat "${sles15_release}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            return 0
        fi
    fi
    #获取SuSE15的版本
    if [ -f "${sles15_release}" ]; then
        if [ "suse" = "$(cat "${sles15_release}" | grep "^ID_LIKE=" | awk -F "\"" '{print $2}')" ]; then
            name="suse-$(cat "${sles15_release}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            return 0
        fi
        raw=$(cat "${sles15_release}")
        eval $raw
        name="$ID-$VERSION_ID"
        echo "$name"
        return 0
    fi

    return 1
}

###################################################
# 解析参数
###################################################
function parse_argv()
{
    local options=""

    options=$(getopt -o a:i:p:c:d:k:o:w:s:n:r:f:l:v:h: -n "$0" -- "$@")
    if [ $? -ne 0 ] ; then
        error "[${FUNCNAME}:${LINENO}]" "getopt failed"
        return 1
    fi
    eval set -- "$options"
    while true;
    do
        case "$1" in
        -a)
            gc_platform="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_platform:${gc_platform}"
            shift 2
            ;;
        -i)
            gc_ipaddr="$2"
            export LOCAL_IPADDR="${gc_ipaddr}"
            info "[${FUNCNAME}:${LINENO}]" "gc_ipaddr:${gc_ipaddr}"
            shift 2
            ;;
        -p)
            gc_parted="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_parted:${gc_parted}"
            shift 2
            ;;
        -c)
            gc_parted_conf="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_parted_conf:${gc_parted_conf}"
            shift 2
            ;;
        -d)
            gc_debug="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_debug:${gc_debug}"
            shift 2
            ;;
        -k)
            gc_check="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_check:${gc_check}"
            shift 2
            ;;
        -o)
            gc_support_os="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_support_os:${gc_support_os}"
            shift 2
            ;;
        -w)
            gc_swap_off="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_swap_off:${gc_swap_off}"
            shift 2
            ;;
        -s)
            gc_services_ip="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_services_ip:${gc_services_ip}"
            shift 2
            ;;
        -n)
            gc_hostname="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_hostname:${gc_hostname}"
            shift 2
            ;;
        -r)
            gc_feed_random_setup="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_feed_random_setup:${gc_feed_random_setup}"
            shift 2
            ;;
        -f)
            gc_optimize="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_optimize:${gc_optimize}"
            shift 2
            ;;
        -l)
            gc_kms_conf="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_kms_conf:${gc_kms_conf}"
            shift 2
            ;;
        -v)
            gc_ip_model="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_ip_model:${gc_ip_model}"
            shift 2
            ;;
        -h)
            gc_data_home="$2"
            info "[${FUNCNAME}:${LINENO}]" "gc_data_home:${gc_data_home}"
            shift 2
            ;;
        --)
            shift
            break
            ;;
        *)
            error "[${FUNCNAME}:${LINENO}]" "Unknown options $1, skip."
            shift
            ;;
        esac
    done

    return 0
}

###################################################
# 检查参数
###################################################
function check_argv()
{
    debug "Enter [${FUNCNAME}:${LINENO}]"

    #判断分区配置文件
    if [ "$gc_parted" -ne 0 ]; then
        if [ -z "$gc_parted_conf" ]; then
            error "[${FUNCNAME}:${LINENO}]" "parted conf is null"
            return 1
        fi
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
# 引入配置文件 及 脚本
###################################################
function source_file()
{
    debug "Enter [${FUNCNAME}:${LINENO}]"

    #引入模块主函数
    while read line
    do
        source "${line}/install.sh"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "source ${line}/install.sh failed"
            return 1
        fi

        debug "source ${line}/install.sh"

    done < <(find "$gc_mian_path" -mindepth 1 -maxdepth 1 \
             -name "[0-9][0-9][0-9].*" -type d | sort)

    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Source file failed. Maybe the softlink file /dev/fd which point /proc/self/fd is missing."
        return 1
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
# 进行适配操作
###################################################
function install()
{
    local main_func=""
    local path=""
    local ret=0

    debug "Enter [${FUNCNAME}:${LINENO}]"

    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi
        
        #获取模块目录
        path="$(basename "${line}")"
        
        #组合模块主函数名
        main_func="$(echo "$path" | awk -F'.' '{print $2}')_main"

        debug "Run ${main_func} ${path} [${FUNCNAME}:${LINENO}]"

        #运行模块主函数
        $main_func "${path}"
        ret=$?
        if [ $ret -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "run ${main_func} ${path} in [${line}] failed"
            return $ret
        fi
    done < <(find "$gc_mian_path" -mindepth 1 -maxdepth 1 \
             -name "[0-9][0-9][0-9].*" -type d | sort)

    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Execute preinstall modules failed. Maybe the softlink file /dev/fd which point /proc/self/fd is missing."
        return 1
    fi
 
    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
# kill后台进程
###################################################
function kill_back_proc()
{
    local pids=""   #后台运行的进程PID    
    
    pids="$(jobs -p)"
    if [ -n "$pids" ]; then
        kill $pids
    fi

    return
}

###################################################
# 入口函数
###################################################
function main()
{
    local ret=0
    
    #设置当前的umask
    UMASK=0077
    umask "$UMASK"

    #引入脚本库
    source_file
    if [ $? -ne 0 ]; then
        return 1
    fi

    #解析参数
    parse_argv "$@"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "parse argv failed"
        return 1
    fi

    #检查参数
    check_argv
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "check argv failed"
        return 1
    fi

    #生成进度文件
    gc_sched_file="${gc_mian_path}/sched_c_${gc_ipaddr}"
    echo "0" > "$gc_sched_file"

    #进行适配操作
    install "$@"
    ret=$?
    if [ $ret -ne 0 ]; then
        kill_back_proc
        error "[${FUNCNAME}:${LINENO}]" "install failed"
        return $ret
    fi
    
    kill_back_proc
    return 0
}

main "$@"
