#!/bin/bash

OLD_KEY=""
SRC_FILE=""
IS_RETRY=""
KEY_NUM=""

G_CURRENT_PATH=""
G_LIB_SCRIPT="cluster_lib.sh"
G_REMOTE_SCRIPT="remote.sh"
G_CONF_FILE="cluster.ini"
G_CONF_FILE_TMP="cluster.ini.tmp.$$"
G_RESULT=0
G_HOSTS_ARRAY=()
G_PASSWDS_ARRAY=()

#日志文件
LOGPATH="/var/log/diskmgt"
LOGFILE="/var/log/diskmgt/replace_cluser_key.log"
#用于记录失败节点列表
FAILLIST="/var/log/diskmgt/replace_cluser_key.fail"
#diskmgt服务安装路径
DISKMGTPATH="/usr/local/diskmgt"
REPLACE_SCRIPT="${DISKMGTPATH}/script/replace_key.sh"

#后台执行替换秘钥的进程保存全局变量
BACKPIDS=""
TGT_FILE_NAME="new_key"

#脚本解释器 强制设置为 bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
    bash $0 "$@"
    exit $?
fi

###################################################
#
# 记录审计日志
#
###################################################
function syslog() {
    local component=$1
    local filename=$2
    local status=$3
    local message=$4

    which logger >/dev/null 2>&1
    [ "$?" -ne "0" ] && return 0

    login_user_ip="$(who -m | sed 's/.*(//g;s/)*$//g')"
    execute_user_name="$(whoami)"
    logger -p local0.notice -i "FusionInsight;${component};[${filename}];${status};${login_user_ip};${execute_user_name};${message}"
    return 0
}

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme() {
    local log_mode="$1"
    local log_info="$2"

    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi

    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi

    chmod 700 "${LOGPATH}"

    if [ "${log_mode}" -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ "${log_mode}" -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ "${log_mode}" -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}" #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi

    return 0
}

function log_echo() {
    local level="${1}"
    local message="${2}"

    if [ "${level}" == "ERROR" ]; then
        echo -e "\033[31m ${message} \033[0m"
        return 0
    fi

    if [ "${level}" == "WARN" ]; then
        echo -e "\033[33m ${message} \033[0m"
        return 0
    fi

    if [ "${level}" == "INFO" ]; then
        echo -e "\033[32m ${message} \033[0m"
        return 0
    fi

    if [ "${level}" == "TIP" ]; then
        echo -e "\033[34m ${message} \033[0m"
        return 0
    fi
    echo "${message}"
}

function validateNewKeyContent() {
    local inpwd="$1"
    if [ -z "${inpwd}" ]; then
        log_echo "ERROR" "the content of the new key is empty."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the content of the new-key is empty."
        return 1
    fi

    #字符串长度
    echo "${inpwd}" | grep -E "^.{8,32}$" 1>/dev/null
    if [ $? -ne 0 ]; then
        log_echo "ERROR" "the new-key does not meet the password policy,length not enough."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the new-key does not meet the password policy,length not enough."
        return 1
    fi

    #判断是否包含' \ 或 " 当前不支持
    echo "${inpwd}" | grep -P "[\\\'\"\$]" 1>/dev/null
    if [ $? -eq 0 ]; then
        log_echo "ERROR" "the new-key may contains unsupported char, such as ' \\ or \" or \$"
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the new-key may contains unsupported char, such as ' \\ or \" or \$"
        return 1
    fi

    #判断是否包含特殊字符
    echo "${inpwd}" | grep -P "[\x21-\x2F\x3A-\x40\x5B-\x60\x7B-\x7E _-]" 1>/dev/null
    local charvalue=$?
    if [ $charvalue -ne 0 ]; then
        log_echo "ERROR" "the new-key must contains special char"
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the new-key must contains special char"
        return 1
    fi

    #判断是否包数字
    local temp_pwd=($(echo "${inpwd}" | sed "s/[\x21-\x2F\x3A-\x40\x5B-\x60\x7B-\x7E _-|]//g"))
    echo "${temp_pwd}" | grep -E "[0-9]{1,}" 1>/dev/null
    local digitvalue=$?
    if [ $digitvalue -ne 0 ]; then
        log_echo "ERROR" "the new-key must contains digit char"
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the new-key must contains digit char"
        return 1
    fi

    #判断是否包小写字母
    echo "${inpwd}" | grep -E "[a-z]{1,}" 1>/dev/null
    local lowercasevalue=$?
    if [ $lowercasevalue -ne 0 ]; then
        log_echo "ERROR" "the new-key must contains lowercase char"
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the new-key must contains lowercase char"
        return 1
    fi

    #判断是否包大写字母
    echo "${inpwd}" | grep -E "[A-Z]{1,}" 1>/dev/null
    local uppercasevalue=$?

    #至少包含大写字母，小写字母，数字，特殊字符中三种，如果四个返回值的和大于等于2，则说明不满足至少包含三种，返回失败。
    local valid_value=$(expr $digitvalue + $uppercasevalue + $lowercasevalue + $charvalue)
    if [ $valid_value -ge 2 ]; then
        log_echo "ERROR" "the new-key does not meet the password policy,digitvalue=$digitvalue,uppercasevalue=$uppercasevalue,lowercasevalue=$lowercasevalue,charvalue=$charvalue."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] the new-key does not meet the password policy,digitvalue=$digitvalue,uppercasevalue=$uppercasevalue,lowercasevalue=$lowercasevalue,charvalue=$charvalue."
        return 1
    fi

    return 0
}

###################################################
#
# 捕获信号后删除中间文件
#
###################################################
function sig_handle() {
    #删除中间文件
    rm "${G_CONF_FILE_TMP}" -f
    stty sane
    echo ""
    syslog "replace_cluster_key" "replace_cluster_key.sh" "exit=1" "replace_cluster_key.sh, catch signal and exit."
    exit 1
}
trap "sig_handle" INT TERM QUIT

###################################################
#
# 显示帮助
#
###################################################
function show_help() {
    log_echo "TIP" "Help:"
    log_echo "TIP" "  Parameters:"
    log_echo "TIP" "      -k: Access key, Mandatory parameters, Enumerated Value: accessKey1 | accessKey2"
    log_echo "TIP" "          accessKey1: Master Access Key; accessKey2: Secondary Access Key"
    log_echo "TIP" "      -f: New key file, Optional parameters, If the new key file does not exist, use a random key."
    log_echo "TIP" "      -r: Retry or Not, Optional parameters, Enumerated Value: retry"
    log_echo "TIP" "  Usage:"
    log_echo "TIP" "        $0 -k [accessKey1 | accessKey2] -f [new_key_file] -r [retry]"
}

###################################################
#
# 获取绝对路径
#
###################################################
function get_path() {
    local path=""

    pushd $(dirname "$0") >/dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi

    G_CURRENT_PATH="${path}"
    G_LIB_SCRIPT="${path}/lib/${G_LIB_SCRIPT}"
    G_REMOTE_SCRIPT="${path}/lib/${G_REMOTE_SCRIPT}"
    G_CONF_FILE="${path}/${G_CONF_FILE}"
    G_CONF_FILE_TMP="${path}/lib/${G_CONF_FILE_TMP}"

    return 0
}

###################################################
#
# 引入配置文件 及 脚本
#
###################################################
function source_file() {
    #去除配置文件中的\r字符
    cat "${G_CONF_FILE}" | grep -v "^[[:space:]]*$" |
        grep -v "^#" | tr -d '\r' >"${G_CONF_FILE_TMP}"

    #引入配置文件
    source "${G_CONF_FILE_TMP}"
    if [ $? -ne 0 ]; then
        log_echo "ERROR" "source ${G_CONF_FILE} failed."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] source ${G_CONF_FILE} failed."
        return 1
    fi

    #引入库函数
    source "${G_LIB_SCRIPT}"
    if [ $? -ne 0 ]; then
        log_echo "ERROR" "source ${G_LIB_SCRIPT} failed."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] source ${G_LIB_SCRIPT} failed."
        return 1
    fi

    #设置超时时间，替换单个加密分区秘钥需要8s
    export g_timeout=360

    return 0
}

###################################################
#
# 参数检查
#
###################################################
function check_params() {
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] parameters[OLD_KEY:${OLD_KEY}, SRC_FILE:${SRC_FILE}, IS_RETRY:${IS_RETRY}]."
    if [ X"${OLD_KEY}" == "XaccessKey1" ]; then
        KEY_NUM=1
    elif [ X"${OLD_KEY}" == "XaccessKey2" ]; then
        KEY_NUM=2
    else
        log_echo "ERROR" "Invalid access key parameter[${OLD_KEY}]."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Invalid access key parameter[${OLD_KEY}]."
        show_help
        return 1
    fi

    if [ ! -z "${SRC_FILE}" ] && [ ! -f ${SRC_FILE} ]; then
        log_echo "ERROR" "Invalid new key file[${SRC_FILE}]."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Invalid new key file[${SRC_FILE}]."
        show_help
        return 1
    fi

    if [ -s "${SRC_FILE}" ]; then
        validateNewKeyContent "$(cat ${SRC_FILE})"
        if [ $? -ne 0 ]; then
            log_echo "ERROR" "Invalid new key content[${SRC_FILE}]."
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Invalid new key content[${SRC_FILE}]."
            return 1
        fi
    fi

    #参数3必须为空或者retry
    if [ ! -z "${IS_RETRY}" -a X"${IS_RETRY}" != "Xretry" ]; then
        log_echo "ERROR" "Invalid retry parameter[${IS_RETRY}]."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Invalid retry parameter[${IS_RETRY}]."
        show_help
        return 1
    fi

    #判断配置中用户名
    if ! [ X"${g_user_name}" == "Xroot" ]; then
        log_echo "ERROR" "g_user_name must be root. Now is ${g_user_name}."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] g_user_name must be root. Now is ${g_user_name}."
        return 1
    fi

    #检查 expect 工具
    which expect >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        log_echo "ERROR" "Please install expect command."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Please install expect command."
        return 1
    fi

    log_echo "INFO" "Replacing the cluster key is a high risk operation. Do you want to continue? [Y/N]:\c"
    read yes_or_no
    if [ -z "${yes_or_no}" ] || [ "${yes_or_no}" = "y" ] || [ "${yes_or_no}" = "Y" ]; then
        log_echo "INFO" "Begin replace..."
    else
        log_echo "INFO" "User aborted the operation."
        return 1
    fi

    return 0
}

########################################################################
# 函数描述： 等待后台的格式化进程执行完毕
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function wait_back_pid() {
    log_echo "INFO" "back pids are (${BACKPIDS})"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] back pids are (${BACKPIDS})"

    if [ -z "${BACKPIDS}" ]; then
        log_echo "INFO" "no back pid."
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] no back pid."
        return
    fi

    #等待所有的后台格式化进程直到完毕
    #注意等待完毕后把全局进程号保存变量清空，防止系统中pid号重复使用
    wait ${BACKPIDS}
    BACKPIDS=""
}

########################################################################
# 函数描述： 等待后台的格式化进程执行完毕
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function replace_one_host() {
    local host="$1"
    log_echo "INFO" "***** Begin to replace key for ${host} *****"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ***** Begin to replace key for ${host} *****"

    if [ -s "${SRC_FILE}" ]; then
        #拷贝新的秘钥文件到节点
        TGT_FILE="${DISKMGTPATH}/ini-plugin/keys/${TGT_FILE_NAME}"
        cp_file_to_r "${host}" "${passwd}" "${SRC_FILE}" "${TGT_FILE}" >>"$LOGFILE" 2>&1
        if [ $? -ne 0 ]; then
            echo -n "${host}," >>"$FAILLIST"
            log_echo "ERROR" "${host}: Failed to copy ${SRC_FILE} to ${TGT_FILE}."
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ${host}: Failed to copy ${SRC_FILE} to ${TGT_FILE}."
            return 1
        fi
    fi

    #在节点上执行秘钥替换脚本
    REPLACE_CMD="${REPLACE_SCRIPT} ${KEY_NUM} ${TGT_FILE}"
    exec_cmd_r "${host}" "${passwd}" "${REPLACE_CMD}" >>"$LOGFILE" 2>&1
    if [ $? -ne 0 ]; then
        echo -n "${host}," >>"$FAILLIST"
        log_echo "ERROR" "${host}: Failed to replace key file[${OLD_KEY}]."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ${host}: Failed to replace key file[${OLD_KEY}]."
        return 1
    fi
    log_echo "INFO" "***** End to replace key for ${host} *****"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ***** End to replace key for ${host} *****"
}

########################################################################
# 函数描述： 等待后台的格式化进程执行完毕
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function replace_all_hosts() {
    rm -f "$FAILLIST"
    if [ $? -ne 0 ]; then
        log_echo "ERROR" "Failed to rm -rf $FAILLIST."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to rm -rf $FAILLIST."
        return 1
    fi

    Nproc=200 # 可同时运行的最大作业数
    PID=()    # 记录PID到数组, 检查PID是否存在以确定是否运行完毕

    #在集群中执行命令
    for ((i = 0; i < ${#G_HOSTS_ARRAY[@]}; i++)); do
        ip=${G_HOSTS_ARRAY[${i}]}
        passwd="${G_PASSWDS_ARRAY[${i}]}"

        if [ -z "${ip}" ]; then
            log_echo "WARN" "the ip:${ip} of host ${i} is empty. Stop checking on this host."
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] the ip:${ip} of host ${i} is empty. Stop checking on this host."
            continue
        fi

        #控制并发,检查PID是否存在以确定是否运行完毕,找到一个就退出
        while true; do
            for ((Ijob = 0; Ijob < Nproc; Ijob++)); do
                if [[ ! "${PID[Ijob]}" ]] || ! kill -0 ${PID[Ijob]} 2>/dev/null; then
                    break
                fi
            done

            if [[ "$Ijob" -lt "$Nproc" ]]; then
                break
            fi

            sleep 2
        done

        #ping目标端 检测是否可以ping通
        ping -c 1 "${ip}" -i 0.2 -W 1 >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            ping -c 1 -6 "${ip}" -i 0.2 -W 1 >/dev/null 2>&1
            if [ $? -ne 0 ]; then
                log_echo "ERROR" "ping ${ip} failed."
                logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ping ${ip} failed."
                echo -n "${ip}," >>"$FAILLIST"
                continue
            fi
        fi

        replace_one_host "${ip}" &
        PID[Ijob]=$!
    done

    BACKPIDS="$(echo "${PID[@]}")"
    wait_back_pid

    if [ -s "$FAILLIST" ]; then
        sed -i s'/,$//' "$FAILLIST"
        log_echo "ERROR" "Failed to replace cluster key(fail list: $(cat $FAILLIST)). Please view $LOGFILE and $LOGPATH/replace_key.log for detail infomations. After the problem is solved, please run [ $0 $OLD_KEY $SRC_FILE retry ] to replace key for those failed hosts!"
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace cluster key(fail list: $(cat $FAILLIST)). Please view $LOGFILE and $LOGPATH/replace_key.log for detail infomations. After the problem is solved, please run [ $0 $OLD_KEY $SRC_FILE retry ] to replace key for those failed hosts!"
        return 1
    fi

    if [ -z "${SRC_FILE}" ]; then
        log_echo "INFO" "Success to replace cluster key[${OLD_KEY}]."
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace cluster key[${OLD_KEY}]."
    else
        log_echo "INFO" "Success to replace cluster key[${OLD_KEY}] with key-file[$SRC_FILE]."
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace cluster key[${OLD_KEY}] with key-file[$SRC_FILE]."
    fi

    return 0
}

function parse_params() {
    # when no input param print help msg
    if [ $# -eq 0 ]; then
        show_help
        exit 1
    fi

    while getopts "k:f:r:" options 2>/dev/null; do
        case $options in
        k)
            OLD_KEY=${OPTARG}
            ;;
        f)
            SRC_FILE=${OPTARG}
            ;;
        r)
            IS_RETRY=${OPTARG}
            ;;
        ?)
            show_help
            exit 1
            ;;
        esac
    done
}

###################################################
#
# 入口函数
#
###################################################
function main() {
    local ip=""
    local hosts=""
    local host=""
    local passwd=""

    get_path
    if [ $? -ne 0 ]; then
        return 1
    fi

    source_file
    if [ $? -ne 0 ]; then
        return 1
    fi

    parse_params "$@"
    check_params
    if [ $? -ne 0 ]; then
        return 1
    fi

    if [ X"${IS_RETRY}" == "Xretry" ]; then
        if [ -s "${FAILLIST}" ]; then
            hosts="$(cat "$FAILLIST")" >>"$LOGFILE" 2>&1
            ret=$?
        else
            log_echo "INFO" "No node failed to be replaced. You do not need to try again."
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] No node failed to be replaced. You do not need to try again."
            return 0
        fi
    else
        hosts="$(parse_host "${g_hosts}")" >>"$LOGFILE" 2>&1
        ret=$?
    fi

    if [ "$ret" -ne 0 ]; then
        log_echo "ERROR" "Failed to parse hosts."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to parse hosts."
        return 1
    fi

    #获取集群节点地址数组
    G_HOSTS_ARRAY=($(echo "$hosts" | tr ',' ' '))
    if [ ${#G_HOSTS_ARRAY[@]} -eq 0 ]; then
        log_echo "ERROR" "G_HOSTS_ARRAY is null."
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] G_HOSTS_ARRAY is null."
        return 1
    fi

    #设置密码
    if [ -z "${g_password}" ]; then
        set_passwd
        if [ $? -ne 0 ]; then
            return 1
        fi
    else
        if [ "${g_password:0:1}" != "/" ]; then
            g_password="${G_CURRENT_PATH}/${g_password}"
        fi
        parse_passwd "${g_password}"
        if [ $? -ne 0 ]; then
            log_echo "ERROR" "can not parse password."
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] can not parse password."
            return 1
        fi
    fi

    #为所有节点替换秘钥文件
    replace_all_hosts
    return $?
}

###################################################
# 脚本开始执行
###################################################
syslog "replace_cluster_key" "replace_cluster_key.sh" "begin" "replace_cluster_key.sh $* begin."
main "$@"
G_RESULT=$?
rm "${G_CONF_FILE_TMP}" -f

syslog "replace_cluster_key" "replace_cluster_key.sh" "exit=${G_RESULT}" "replace_cluster_key.sh $* end."
exit ${G_RESULT}
