#!/bin/bash

#脚本解释器 强制设置为 bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
   bash $0 "$@"
   exit $?
fi

g_pre_home="../../"                         #preinstall主目录
g_script_lib_dir="script/function/"         #脚本库路径
g_aes_rand_key="$(date +%s%N)"              #用于保存加密密码的随机key
g_passwd_aes_file="${HOME}/pres_in_pw.bin"     #用于保存密码密钥的文件
g_remote="remote.sh"                        #运行命令或拷贝文件
g_readme="readme.txt"
g_work_space="${HOME}/getrpms_home"            #各节点的工作目录
g_rpms_list_dir="tools/rpms/missing"
g_add_packages_dir="add_packages"
g_get_rpms_sh="add_packages/get_missing_rpms_lst.sh"
g_result=0

#############################################################
#
# 获取绝对路径
#
#############################################################
function get_absolute_path()
{
    local path=""
    
    pushd $(dirname "$0") > /dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi
    
    #获取 g_pre_home 的绝对路径
    g_pre_home="${path}/${g_pre_home}"   
    pushd "${g_pre_home}" > /dev/null
    g_pre_home="$(readlink -e "$(pwd)")"
    popd >/dev/null    
    
    g_script_lib_dir="${g_pre_home}/${g_script_lib_dir}"    
    g_add_packages_dir="${g_script_lib_dir}/${g_add_packages_dir}"
    g_remote="${g_script_lib_dir}/${g_remote}"
    g_get_rpms_sh="${g_work_space}/${g_get_rpms_sh}"
    g_rpms_list_dir="${g_pre_home}/${g_rpms_list_dir}"
    g_readme="${g_pre_home}/doc/${g_readme}"

    return 0
}

###################################################
#
# 引入配置文件 及 脚本
# 
###################################################
function source_file()
{
    export g_aes_rand_key="${g_aes_rand_key}"
    
    #引入配置文件
    source "${g_pre_home}/preinstall.ini"
    if [ $? -ne 0 ]; then
        echo "source ${g_pre_home} failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/001.log.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/001.log.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/002.parse_host.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}}/002.parse_host.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/006.set_passwd.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/006.set_passwd.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/008.run_in_remote.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/008.run_in_remote.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/009.ctrl_workspace.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/009.ctrl_workspace.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/014.check_login.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/014.check_login.sh failed"
        return 1
    fi
    
    return 0
} 

###################################################
#
# 在各节点上获取列表
#
###################################################
function get_rpms_from_remote()
{
    local ipaddr="$1"     #主机名或ip地址
    local parted_conf=""  #节点对应的格式化配置
    local ret=0
    
    debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]"
    
    echo "Get missing rpm packages from ${ipaddr}"

    #入参检查
    if [ -z "$ipaddr" ]; then
        error "Parameter error."
        error "Get failed from ${ipaddr}"
        return 1
    fi

    #在各个节点创建工作空间
    gen_workspace "$g_work_space" "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "Gen workspace failed on ${ipaddr}"
        return 1
    fi

    #把适配脚本拷贝至远端服务器
    cp_file_to_r "$ipaddr" "240" "${g_add_packages_dir}" "${g_work_space}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1
        error "Copy files to ${ipaddr} failed" 
        return 1
    fi

    #在各节点上执行命令
    exec_cmd_r "$ipaddr" "180" "bash ${g_get_rpms_sh} ${ipaddr}"
    if [ $? -ne 0 ]; then
        del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1
        error "[${ipaddr}] OS version not supported. Please read ${g_readme}"
        return 1
    fi
    
    #把列表从远端 拷贝至本机
    cp_file_from_r "$ipaddr" "240" "${g_work_space}/add_packages/*_${ipaddr}_missing-rpms.lst" "${g_rpms_list_dir}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1
        error "Copy [${g_work_space}/add_packages/*_${ipaddr}_missing-rpms.lst] from $ipaddr failed"
        return 1
    fi
    
    #删除节点上的工作空间
    del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1

    debug "Leave [${FUNCNAME}:${LINENO}]"
    
    echo "Get successfully from ${ipaddr}"

    return $ret
}

###################################################
#
# 安装 磁盘监控工具
#
###################################################
function get_rpms()
{
    local i=""       #循环变量
    local ipaddr=""    
    
    debug "Enter [${FUNCNAME}:${LINENO}]"    

    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"
        
        #在各节点上执行命令
        get_rpms_from_remote "${ipaddr}" &

        #休眠0.1秒
        sleep .1

    done
    
    wait $(jobs -p)

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 入口函数
#
###################################################
function main()
{
    local hosts=""
    
    which expect > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Please Install expect."
        return 1
    fi
    
    #获取绝对路径
    get_absolute_path
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #引入配置文件 及 脚本
    source_file
    if [ $? -ne 0 ]; then
        return 1
    fi   
    
    #解析HOST列表
    g_hosts="$(parse_host "$g_hosts")"
    if [ $? -ne 0 ]; then
        error "Please check g_hosts in ${g_pre_home}/preinstall.ini"
        return 1
    fi
    
    #输入集群的密码(各节点密码是一致的)
    set_passwd "$g_passwd_aes_file" 
    if [ $? -ne 0 ]; then
        return 1
    fi 
    
    #检查各节点是否可以联通
    only_check_login "$g_hosts"
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #创建目录
    if [ ! -d "${g_rpms_list_dir}" ]; then
        mkdir -p "${g_rpms_list_dir}"
        if [ $? -ne 0 ]; then
            error "mkdir -p ${g_rpms_list_dir} failed"
            return 1
        fi
    fi
    
    get_rpms
    
    return 0
}

main "$@"
g_result=$?

#删除临时文件
rm "${g_passwd_aes_file}" -f
rm "${g_known_hosts}" -f

exit $g_result
