#!/bin/bash
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    
## Description: 
## Version:     FusionInsight V100R002C60
## Created:     Friday, 11 18, 2015
################################################################################

declare g_logPath="/var/log/Bigdata/sudo"
declare g_logFile="postset.log"
declare g_upgrade_type="$1"  #commit,rollback
declare src_version_is812="$2"

################################################################################
# Function: get_cur_path
# Description: 获取脚本所在的目录及脚本名
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="${PWD}"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    cd - >/dev/null
}

################################################################################
# Function: init_log
# Description: 初始化preset日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_log()
{
    [ -d ${g_logPath} ] || mkdir -p ${g_logPath}
    [ -f ${g_logPath}/${g_logFile} ] || touch "${g_logPath}/${g_logFile}"
    chmod 600 "${g_logPath}/${g_logFile}"
}

################################################################################
# Function: main
# Description: Entrance of the script
# Parameter:
#   input:
#  
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
################################################################################
# Function: init_log
# Description: 812做过sudo目录切换,812之前版本回滚后ha进程加载环境变量还是高版本
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
reload_HA_env_configuration()
{
   local array=(Solr dbservice Flink GraphBase)
   if [ -f "/etc/euleros-release" -a "${g_upgrade_type}" == "rollback" -a "${src_version_is812}" == "false" ];then
      for element in ${array[@]}
	    do
	        if [ "$(ps -ef | grep ha_monitor | grep -i ${element} | grep -v grep | awk '{print $2}')" != "" ];then
              kill -9 $(ps -ef | grep ha_monitor | grep -i ${element} | grep -v grep | awk '{print $2}')
			        if [ $? -ne 0 ];then
                  showLog_error "${LINENO}"  "Failed to kill the ${element} service ha_monitor process."
 			        fi
          fi
          if [ "$(ps -ef | grep ha.bin | grep -i ${element} | grep -v grep | awk '{print $2}')" != "" ];then
                kill -9 $(ps -ef | grep ha.bin | grep -i ${element} | grep -v grep | awk '{print $2}')
			          if [ $? -ne 0 ];then
                    showLog_error "${LINENO}" "Failed to kill the ${element} service ha.bin process."
 			          fi
          fi

          log_info "${LINENO}" "kill the ${element} service ha process success."
 	    done
   fi
}
main()
{
    # 如果不是root用户，则退出安装
    if [ $(id -u) -ne 0 ] ; then
        showLog_info "${LINENO}" "Only super user can execute this script."
        return 1
    fi
    
    showLog_info "${LINENO}" "start to postSet" 
    
    #删除脚本update_sudoExecute.sh
    if [ -f "${BIGDATA_SUDO_HOME}/sudo/runtime/update_sudoExecute.sh" ] ; then
        log_info "${LINENO}" "start to delete the script update_sudoExecute.sh"
        rm -rf "${BIGDATA_SUDO_HOME}/sudo/runtime/update_sudoExecute.sh"
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}"  "Failed to delete the update_sudoExecute.sh."
            return 1
        fi
    fi
    
    grep "^${INSTALL_USER} ALL=(${RUN_USER}) NOPASSWD: ALL" ${SUDOERS}  > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
        tmp=$(echo "^${INSTALL_USER} ALL=(${RUN_USER}) NOPASSWD: ALL" | sed -e 's/\//\\\//g')
        sed -i "/$tmp/d" ${SUDOERS}
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}"  "Failed to clear ${SUDOERS}."
            return 1
        fi
    fi
    
    #删除/home/${INSTALL_USER}/env_file,清理/home/${INSTALL_USER}/.bashrc
    log_info "${LINENO}" "start to clear the envs"  
    if [ -f "${HOME_BASHRC}" ] ; then
        grep "^source ${HOME_ENV_FILE}" ${HOME_BASHRC}  > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
           tmp=$(echo "^source ${HOME_ENV_FILE}" | sed -e 's/\//\\\//g')
           sed -i "/$tmp/d" ${HOME_BASHRC}
           if [ $? -ne 0 ] ; then
               showLog_error "${LINENO}"  "Failed to clear ${HOME_BASHRC}."
               return 1
           fi
        fi
    fi
    if [ -f "${HOME_ENV_FILE}" ] ; then
        rm -rf "${HOME_ENV_FILE}" > /dev/null 2>&1
    fi
    
    log_info "${LINENO}" "g_upgrade_type=${g_upgrade_type}"
    if [ "${g_upgrade_type}" == "commit" ];then
        #删除/etc/suderos里配置的sudoExecute.sh
        local sudoExecute="${OLD_SUDO_HOME}/sudo/runtime/sudoExecute.sh"
        grep "^${INSTALL_USER} ALL=(root) NOPASSWD: ${sudoExecute}" ${SUDOERS}  > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
            tmp=$(echo "^${INSTALL_USER} ALL=(root) NOPASSWD: ${sudoExecute}" | sed -e 's/\//\\\//g')  
            sed -i "/$tmp/d" ${SUDOERS}
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}"  "Failed to clear ${SUDOERS}."
                return 1
            fi
        fi
        
        grep "^backup ALL=(root) NOPASSWD: ${sudoExecute}" ${SUDOERS}  > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
            tmp=$(echo "^backup ALL=(root) NOPASSWD: ${sudoExecute}" | sed -e 's/\//\\\//g')  
            sed -i "/$tmp/d" ${SUDOERS}
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}"  "Failed to clear ${SUDOERS}."
                return 1
            fi
        fi
        
        log_info "${LINENO}" "start to delete the path ${OLD_SUDO_HOME}/sudo/runtime"  
        if [ -d "${OLD_SUDO_HOME}/sudo/runtime" ] ; then
            rm -rf "${OLD_SUDO_HOME}/sudo/runtime"
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}"  "Failed to delete the path ${OLD_SUDO_HOME}/sudo/runtime."
                return 1
            fi
        fi
    elif [ "${g_upgrade_type}" == "rollback" ];then
        # 回滚时删除新路径/opt/huawei/Bigdata_func 
        log_info "${LINENO}" "start to delete the path ${BIGDATA_SUDO_HOME}"  
        if [ -d "${BIGDATA_SUDO_HOME}" -a "${src_version_is812}" == "false" ] ; then
            rm -rf "${BIGDATA_SUDO_HOME}"
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}"  "Failed to delete the path ${BIGDATA_SUDO_HOME}."
                return 1
            fi
        fi
         
        # 回滚时恢复备份的旧路径/var/lib/sudo/Bigdata
        if [ -d "${OLD_SUDO_HOME_BACKUP}" ] ; then
            # 回滚时删除旧路径/var/lib/sudo/Bigdata
            log_info "${LINENO}" "start to delete the path ${OLD_SUDO_HOME}"  
            if [ -d "${OLD_SUDO_HOME}" ] ; then
                rm -rf "${OLD_SUDO_HOME}"
                if [ $? -ne 0 ] ; then
                    showLog_error "${LINENO}"  "Failed to delete the path ${OLD_SUDO_HOME}."
                    return 1
                fi
            fi
        
            log_info "${LINENO}" "start to restore the path ${OLD_SUDO_HOME}" 
            mv ${OLD_SUDO_HOME_BACKUP} ${OLD_SUDO_HOME}
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}" "Failed to restore sudo path ${OLD_SUDO_HOME_BACKUP} to ${OLD_SUDO_HOME}"
                return 1
            fi
        fi
        
        local newSudoPath="${BIGDATA_SUDO_HOME}"
        if [ "${src_version_is812}" == "false" ];then
            grep "${newSudoPath}" ${SUDOERS}  > /dev/null 2>&1
            if [ $? -eq 0 ] ; then
                tmp=$(echo " ALL=(root) NOPASSWD: ${newSudoPath}" | sed -e 's/\//\\\//g') 
                sed -i "/$tmp/d" ${SUDOERS}
                if [ $? -ne 0 ] ; then
                    showLog_error "${LINENO}"  "Failed to clear ${SUDOERS}."
                    return 1
                fi
            fi
        fi
        
    fi
    
    log_info "${LINENO}" "start to delete the path ${OLD_SUDO_HOME_BACKUP}"
    if [ -d "${OLD_SUDO_HOME_BACKUP}" ] ; then
        rm -rf "${OLD_SUDO_HOME_BACKUP}"
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}"  "Failed to delete the path ${OLD_SUDO_HOME_BACKUP}."
            return 1
        fi
    fi
    
    # 正常节点安装或扩容场景
    # 删除脚本installSudoExecute.sh
    log_info "${LINENO}" "start to delete the script installSudoExecute.sh"  
    if [ -d "${INSTALL_SUDO_DIR}" ] ; then
        rm -rf "${INSTALL_SUDO_DIR}"
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}"  "Failed to delete the installSudoExecute.sh."
            return 1
        fi
    fi
    
    #删除/etc/suderos里配置的installSudoExecute.sh
    log_info "${LINENO}" "start to clear ${SUDOERS}."  
    grep "^${INSTALL_USER} ALL=(root) NOPASSWD: ${INSTALL_SUDO_PATH}" ${SUDOERS}  > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
        tmp=$(echo "^${INSTALL_USER} ALL=(root) NOPASSWD: ${INSTALL_SUDO_PATH}" | sed -e 's/\//\\\//g')  
        sed -i "/$tmp/d" ${SUDOERS}
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}"  "Failed to clear ${SUDOERS}."
            return 1
        fi
    fi
    #812做过sudo目录切换,812之前版本回滚后ha进展加载环境变量还是高版本
	  reload_HA_env_configuration
    if [ $? -ne 0 ];then
        showLog_error "${LINENO}"  "Failed to reload the HA environment configuration."
    fi
    showLog_info "${LINENO}" "postSet success" 
}
# ---------------------------------------------------------------------------- #
#                         获取当前路径，初始化日志文件                         #
# ---------------------------------------------------------------------------- #
get_cur_path
cd "${g_curPath}"
init_log

# ---------------------------------------------------------------------------- #
#                                   导入头文件                                 #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/func/log.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${g_curPath}/log.sh."; exit 1;}
. "${g_curPath}/func/constants.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/func/constants.sh."; exit 1; }

# ---------------------------------------------------------------------------- #
#                                 脚本开始运行                                 #
# ---------------------------------------------------------------------------- #

main "$@"
ret=$?

#在操作系统日志中记录审计日志
syslog "manager" "postset.sh" "$ret" "User root reclaims rights from user omm after user omm performs operations."
exit $ret