#!/bin/bash
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    preSetInstall.sh
## Description:
## Version:     FusionInsight V100R002C60
## Created:     Friday, 11 18, 2015
################################################################################

declare g_curPath=""                      # 当前脚本所在的目录
declare g_run_userID=2000                 # 默认运行用户id
declare g_logPath="/var/log/Bigdata/sudo" # 日志路径
declare g_logFile="preset.log"            # 日志文件
declare g_setup_tool_package_home=""      # Setup Tool package home directory
declare g_release_file="release"          # release file path
declare GROUP_WHEEL="wheel"
declare OMM_USER="omm"
declare g_upgrade_type="$1" #upgrade或空
declare src_version_is812="$2" #true false 空
################################################################################
# Function: init_path
# Description: 初始化脚本所在的目录及脚本名
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="${PWD}"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    g_setup_tool_package_home="$(dirname "${g_curPath}")"
    cd - >/dev/null
}

################################################################################
# Function: init_log
# Description: 初始化preset日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_log()
{
    [ -d ${g_logPath} ] || mkdir -p ${g_logPath}
    [ -f ${g_logPath}/${g_logFile} ] || touch "${g_logPath}/${g_logFile}"
    chmod 600 "${g_logPath}/${g_logFile}"
}

################################################################################
# Function: config_sudoers
# Description: 配置/etc/sudoers文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
config_sudoers()
{
    log_info "${LINENO}" "start to config ${SUDOERS}."

    set_sudoers

    # 若/etc/sudoers里没有配置该命令，则配置
    grep "^${INSTALL_USER} ALL=(root) NOPASSWD: ${INSTALL_SUDO_PATH}" ${SUDOERS}  > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "${INSTALL_USER} ALL=(root) NOPASSWD: ${INSTALL_SUDO_PATH}" >> ${SUDOERS}
    fi

    #${INSTALL_USER}    ALL=(${RUN_USER})       NOPASSWD: ALL
    grep "^${INSTALL_USER} ALL=(${RUN_USER}) NOPASSWD: ALL" ${SUDOERS}  > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
      echo "${INSTALL_USER} ALL=(${RUN_USER}) NOPASSWD: ALL" >> ${SUDOERS}
    fi
    
    # 8.1.2版本sudo路径变更，为保证升级期间sudo始终可用，preset时，把新老路径下的sudo脚本都注册到sudoers中，postset时删掉多余路径。
    if [ x"$g_upgrade_type" == x"upgrade" ]; then
        grep -q "${BIGDATA_SUDO_HOME}/sudo/runtime/" ${SUDOERS}
        if [ $? -ne 0 ] ; then
            grep "${OLD_SUDO_HOME}" ${SUDOERS} | grep -v "${INSTALL_SUDO_NAME}" | sed "s|${OLD_SUDO_HOME}|${BIGDATA_SUDO_HOME}|g"  >> ${SUDOERS}
            if [ $? -ne 0 ] ; then
                log_error "${LINENO}" "Falied to config ${SUDOERS} for upgrade."
                return 1
            fi
        fi
    fi

    chmod 0440 ${SUDOERS}

    return 0
}

################################################################################
# Function: set_sudoers
# Description: 配置/etc/sudoers
# Parameter:
#   input:
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
set_sudoers()
{
   if ! [ -f "/etc/SuSE-release" ] ; then
      grep '^[[:space:]]*Defaults[[:space:]]*requiretty[[:space:]]*$'  ${SUDOERS} >/dev/null 2>&1
      if [ $? -eq 0 ] ; then
         sed -i s/'^[[:space:]]*Defaults[[:space:]]*requiretty[[:space:]]*$'/'Defaults !requiretty'/g ${SUDOERS}
      fi

      grep '^[[:space:]]*Defaults[[:space:]]*!visiblepw[[:space:]]*$'  ${SUDOERS} >/dev/null 2>&1
      if [ $? -eq 0 ] ; then
         sed -i s/'^[[:space:]]*Defaults[[:space:]]*!visiblepw[[:space:]]*$'/'Defaults visiblepw'/g ${SUDOERS}
      fi

      grep '^[[:space:]]*Defaults[[:space:]]*visiblepw[[:space:]]*$'  ${SUDOERS} >/dev/null 2>&1
      if [ $? -ne 0 ] ; then
         echo 'Defaults visiblepw' >> ${SUDOERS}
      fi
    fi

    # 匹配Default !env_reset，如果存在则替换!为空
    sed -i '/^[[:space:]]*Defaults[[:space:]]*.env_reset/{s#!##g}' ${SUDOERS} >/dev/null 2>&1

    chmod 0440 ${SUDOERS}
}

################################################################################
# Function: save_input_param
# Description: 在用户目录/home/xxx/的.bashrc中添加两个环境变量
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
save_input_param()
{
    log_info "${LINENO}" "start to add env to ${HOME_BASHRC}."
    if [ -f "${HOME_ENV_FILE}" ] ; then
        rm -f ${HOME_ENV_FILE}
    fi
    touch ${HOME_ENV_FILE}
    chmod 700 ${HOME_ENV_FILE}
    chown ${INSTALL_USER}: ${HOME_ENV_FILE}
        
    if [ ! -f "${HOME_BASHRC}" ] ; then
        touch ${HOME_BASHRC}
        chmod 700 ${HOME_BASHRC}
        chown ${INSTALL_USER}: ${HOME_BASHRC}
    fi
    # 在${INSTALL_USER}用户目录/home/${INSTALL_USER}/的.bashrc中添加INSTALL_SUDO_PATH、BIGDATA_SUDO_HOME、INSTALL_USER环境变量：
    grep "^export INSTALL_SUDO_PATH=${INSTALL_SUDO_PATH}" ${HOME_ENV_FILE}  > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "export INSTALL_SUDO_PATH=${INSTALL_SUDO_PATH}" >> ${HOME_ENV_FILE}
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to set INSTALL_SUDO_PATH to ${HOME_ENV_FILE},check the disk size."
            return 1
        fi
    fi

    grep "^export BIGDATA_SUDO_HOME=${BIGDATA_SUDO_HOME}" ${HOME_ENV_FILE}  > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "export BIGDATA_SUDO_HOME=${BIGDATA_SUDO_HOME}" >> ${HOME_ENV_FILE}
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to set BIGDATA_SUDO_HOME to ${HOME_ENV_FILE},check the disk size."
            return 1
        fi
    fi

    grep "^export INSTALL_USER=${INSTALL_USER}" ${HOME_ENV_FILE}  > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "export INSTALL_USER=${INSTALL_USER}" >> ${HOME_ENV_FILE}
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to set INSTALL_USER to ${HOME_ENV_FILE},check the disk size."
            return 1
        fi
    fi

    if [ -f ${ETC_SUDO_PROFILE} ]; then
        sed -i 's/INSTALL_USER=root/INSTALL_USER=omm/g' ${ETC_SUDO_PROFILE}
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to set INSTALL_USER to ${ETC_SUDO_PROFILE},check the disk size."
            return 1
        fi
    fi

    grep "^source ${HOME_ENV_FILE}" ${HOME_BASHRC}  > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "source ${HOME_ENV_FILE}" >> ${HOME_BASHRC}
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "Failed to set para to ${HOME_BASHRC},check the disk size."
            return 1
        fi
    fi
    return 0
}

add_watches_conf_to_sysctl()
{
    echo fs.inotify.max_user_watches=100000 >> /etc/sysctl.conf
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to execute echo fs.inotify.max_user_watches=100000 >> /etc/sysctl.conf "
        return 1
    fi

    sysctl -p  >> "${g_logPath}/${g_logFile}"
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to execute sysctl -p"
        return 1
    fi
    return 0
}

################################################################################
# Function: modify_max_user_watches
# Description: tomcat监听文件较多时需要调整fs.inotify.max_user_watches
# Parameter:
#   input:
#   N:A
#   output:
#   N:A
# Return: 0 -- success; not 0 -- failure
# Others: N:A
################################################################################
modify_max_user_watches()
{
    g_controller_home=$(su - omm -c 'echo $CONTROLLER_HOME' 2>/dev/null)
    if [ -z "$g_controller_home" ] ; then
        return 0
    fi

    #非oms节点不需要修改
    if [ ! -d "$g_controller_home" ] ; then
        return 0
    fi

    is_exists=$(grep "^fs.inotify.max_user_watches" /etc/sysctl.conf)
    if [ ! -z "$is_exists" ] ; then
        conf_value=$(grep "^fs.inotify.max_user_watches" /etc/sysctl.conf | tail -1 | awk -F '=' '{print $2}' | sed "s/ //g")
        if [ $conf_value -lt 100000 ] ; then
            sed -i '/fs.inotify.max_user_watches/d' /etc/sysctl.conf
            if [ $? -ne 0 ]; then
                showLog_error "${LINENO}" "Failed to remove fs.inotify.max_user_watches from /etc/sysctl.conf."
                return 1
            fi

            add_watches_conf_to_sysctl
            if [ $? -ne 0 ]; then
                return 1
            fi
        fi
    else
        add_watches_conf_to_sysctl
        if [ $? -ne 0 ]; then
            return 1
        fi
    fi

    return 0
}

modify_kernel_pty()
{
    if [ -f "/proc/sys/kernel/pty/max" ]; then
        ptyValue=$(cat /proc/sys/kernel/pty/max)
        if [ ${ptyValue} -lt 8192 ]; then
            echo 8192 > /proc/sys/kernel/pty/max
        fi
    fi
}

################################################################################
# Function: addRunUser
# Description: 添加用户
# Parameter:
#   input:
#   N:A
#   output:
#   N:A
# Return: 0 -- success; not 0 -- failure
# Others: N:A
################################################################################
addRunUser()
{
  local username=${1}
  local password=${2}
  local userid=${3}
  local user_home=${4}
  local user_group=${5}

  log_info "${LINENO}" "start to add user ${username}."
  id -u "${username}" > /dev/null 2>&1
  if [ $? -eq 0 ]; then
    if [ "$(id -u "${username}")" != "${userid}" ] ; then
        echo "The existing user ${username} with id ${userid} exist." ;return 1
    fi
  else
    # 北京凝思6.0.99 x86上删除用户后，再创建同样id的用户时会失败，需要先清除uid
    grep -qi 'LinxOS' /etc/os-release
    if [ $? -eq 0 ]; then
      linx_uidcheck -r ${userid}
    fi
    
    add_user ${username} ${password} ${userid} ${user_home} ${user_group} || return 1
    chmod 750 ${user_home}  -R
    [ -f "/home/${username}/.bash_history" ] && chmod 600 /home/${username}/.bash_history
  fi

  return 0
}

################################################################################
# Function: add_user
# Description: 添加运行用户子操作
# Parameter:
#   input:
#   N:A
#   output:
#   N:A
# Return: 0 -- success; not 0 -- failure
# Others: N:A
################################################################################
add_user()
{
    local username=${1}
    local password=${2}
    local userid=${3}
    local user_home=${4}
    local user_group=${5}

    cat /etc/group | cut -d ":" -f 1 | grep -w "${user_group}" > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "The ${user_group} group does not exist." ;return 1
    fi

    useradd "${username}" -m -d "${user_home}" -g "${user_group}" -u "${userid}" -s /bin/bash
    if [ $? -ne 0 ]; then
        echo "Failed to add the $1 user." ;return 1
    fi
    passwd "${username}"<<EOF
${password}
${password}
EOF
    if [ $? -ne 0 ]; then
        echo "Failed to change the $1 user password." ;return 1
    fi
}


################################################################################
# Function: install_sudo
# Description: 安装sudo
# Parameter:
#   input:
#   N:A
#   output:
#   N:A
# Return: 0 -- success; not 0 -- failure
# Others: N:A
################################################################################
install_sudo()
{
    #拷贝installSudoExecute.sh脚本到 /var/lib/sudo/Bigdata/setuptool目录下
    local installSudoExecuteFile="${g_curPath}/installSudoExecute.sh"
    if [ ! -d "${INSTALL_SUDO_DIR}" ] ; then
        mkdir -p "${INSTALL_SUDO_DIR}" > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to mkdir ${INSTALL_SUDO_DIR}"
            return 1
        fi
        chmod 700 "${INSTALL_SUDO_DIR}"
    fi
    /bin/cp -af ${installSudoExecuteFile} ${INSTALL_SUDO_DIR} > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to copy the ${installSudoExecuteFile} file to the ${INSTALL_SUDO_DIR} directory."
        return 1
    fi
    
    /bin/cp -af "${g_curPath}/components" ${INSTALL_SUDO_DIR} > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to copy the ${g_curPath}/components file to the ${INSTALL_SUDO_DIR} directory."
        return 1
    fi
    
    # 升级场景下对sudo文件做兼容处理，保证无论环境变量指向哪个sudo路径，都能正常使用
    if [ x"$g_upgrade_type" == x"upgrade" ]; then
        # 1）升级先备份/var/lib/sudo/Bigdata到/var/lib/sudo/Bigdata_bakup目录下
        if [ ! -d ${OLD_SUDO_HOME_BACKUP} ];then
            cp -rpf ${OLD_SUDO_HOME} ${OLD_SUDO_HOME_BACKUP}
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}" "Failed to backup sudo path to ${OLD_SUDO_HOME_BACKUP}"
                return 1
            fi
        fi
        
        # 2）拷贝/var/lib/sudo/Bigdata/sudo到/opt/huawei/Bigdata_func目录下
        if [ ! -d ${BIGDATA_SUDO_HOME} ];then
            mkdir -p ${BIGDATA_SUDO_HOME}
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}" "Failed to mkdir ${BIGDATA_SUDO_HOME}"
                return 1
            fi
            chmod 750 ${BIGDATA_SUDO_HOME}
        fi
        
        if [ -d ${OLD_SUDO_HOME}/sudo ] && [ x"${src_version_is812}" != x"true" ];then
            cp -rpf ${OLD_SUDO_HOME}/sudo ${BIGDATA_SUDO_HOME}
            if [ $? -ne 0 ] ; then
                showLog_error "${LINENO}" "Failed to copy sudo files to ${BIGDATA_SUDO_HOME}"
                return 1
            fi
        fi
        
        # 3）复制update_sudoExecute.sh（升级相关的sudo函数）到${BIGDATA_SUDO_HOME}/sudo/runtime/目录下
        /bin/cp -af "${g_curPath}/update_sudoExecute.sh" ${BIGDATA_SUDO_HOME}/sudo/runtime/ > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to copy ${g_curPath}/update_sudoExecute.sh to the ${BIGDATA_SUDO_HOME}/sudo/runtime/."
            return 1
        fi
        
        # 4）修改update_sudoExecute.sh脚本的权限
        chown -R "root":"root" "${BIGDATA_SUDO_HOME}/sudo/runtime/update_sudoExecute.sh" > /dev/null 2>&1
        chmod -R 500 "${BIGDATA_SUDO_HOME}/sudo/runtime/update_sudoExecute.sh" > /dev/null 2>&1
    fi

    if [ -f "${g_setup_tool_package_home}/${g_release_file}" ] ; then
        /bin/cp -f ${g_setup_tool_package_home}/${g_release_file} ${INSTALL_SUDO_DIR} > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to copy the ${g_release_file} file to the ${INSTALL_SUDO_DIR} directory."
            return 1
        fi
    fi

    #修改installSudoExecute.sh脚本的权限
    chown -R "root":"root" ${INSTALL_SUDO_DIR} > /dev/null 2>&1
    chmod -R 500 "${INSTALL_SUDO_DIR}" > /dev/null 2>&1
    

    #在/etc/suderos里配置installSudoExecute.sh脚本。
    config_sudoers "$1"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi
}

function removeGaussDBConfig() {
    local configFile="/home/omm/.bashrc"
    if [ -f "${configFile}" ]; then
        sed -i '/export GAUSSHOME=/d' "${configFile}"
        sed -i '/export PATH=/d' "${configFile}"
    fi
}
function validatePwd()
{
    local inpwd="$1"
    if [ -z "${inpwd}" ] ; then
        showLog_error "${LINENO}" "the passwd is null."
        return 1
    fi

    #字符串长度
    echo "${inpwd}" | grep -E "^.{8,32}$" 1>/dev/null
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "the passwd does not meet the password policy,length not engouth."
        return 1
    fi

    #判断是否包含' \ 或 " 当前不支持
    echo "${inpwd}" | grep -P "[\\\'\"]" 1>/dev/null
    if [ $? -eq 0 ];then
        showLog_error "${LINENO}"  "the passwd may contains unsupported char, such as ' \\ or \""
        return 1
    fi

    #判断是否包含特殊字符
    echo "${inpwd}" | grep -P "[\x21-\x2F\x3A-\x40\x5B-\x60\x7B-\x7E _-]" 1>/dev/null
    local charvalue=$?
    if [ $charvalue -ne 0 ];then
        showLog_error "${LINENO}"  "the passwd must contains special char"
        return 1
    fi

    #判断是否包数字
    local temp_pwd=($(echo "${inpwd}" | sed "s/[\x21-\x2F\x3A-\x40\x5B-\x60\x7B-\x7E _-|]//g"))
    echo "${temp_pwd}" | grep -E "[0-9]{1,}" 1>/dev/null
    local digitvalue=$?
    if [ $digitvalue -ne 0 ];then
        showLog_error "${LINENO}"  "the passwd must contains digit char"
        return 1
    fi

    #判断是否包小写字母
    echo "${inpwd}" | grep -E "[a-z]{1,}" 1>/dev/null
    local lowercasevalue=$?
    if [ $lowercasevalue -ne 0 ];then
        showLog_error "${LINENO}" "the passwd must contains lowercase char"
        return 1
    fi

    #判断是否包大写字母
    echo "${temp_pwd}" | grep -E "[A-Z]{1,}" 1>/dev/null
    local uppercasevalue=$?
    if [ $uppercasevalue -ne 0 ];then
        showLog_error "${LINENO}" "the passwd must contains uppercase char"
        return 1
    fi
    #至少包含大写字母，小写字母，数字，特殊字符中三种，如果四个返回值的和大于等于2，则说明不满足至少包含三种，返回失败。
    local valid_value=`expr $digitvalue + $uppercasevalue + $lowercasevalue + $charvalue`
    if [ $valid_value -ge 2 ];then
        showLog_error "${LINENO}" "the new passwd does not meet the password policy,digitvalue=$digitvalue,uppercasevalue=$uppercasevalue,lowercasevalue=$lowercasevalue,charvalue=$charvalue."
        return 1
    fi

    return 0
}

function create_soft_link_for_libboost()
{
    boost_version="${1}"
    dst_version="${boost_version}";
    files=$(rpm -qa | grep -E "*boost[_-](system*|atomic-m*)" | xargs rpm -ql | grep lib64) >/dev/null 2>&1
    echo "$files" | tr ' ' '\n' | while read line
    do
        dst_file=`echo ${line} | sed "s/[0-9.]\+\$/.${dst_version}/g"`;  ln -s ${line} ${dst_file} >/dev/null 2>&1;
        dst_mt_file=`echo ${dst_file} | sed 's/-mt//g' | sed 's/.so/-mt.so/g'`; ln -s ${line} ${dst_mt_file} >/dev/null 2>&1;
        dst_no_mt_file=`echo ${dst_file} | sed 's/-mt//g'`; ln -s ${line} ${dst_no_mt_file} >/dev/null 2>&1;
    done
    return 0
}

########################################################################
# 函数描述： 启用pam_rootok.so模块，这样从root用户su到其他用户时无需输入密码
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function enable_pam_rootok()
{
    grep -qi 'LinxOS' /etc/os-release
    if [ $? -ne 0 ]; then
        return 0
    fi
    
    sed -i '/pam_rootok.so/s/#//' /etc/pam.d/su
    return $?
}

################################################################################
# Function: main
# Description: 主函数
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    # 如果不是root用户，则退出
    if [ $(id -u) -ne 0 ] ; then
        echo  "ERROR: Only super user can execute this script."
        return 1
    fi

    showLog_info "${LINENO}" "start to preSet"

    # 如果用户环境上不存在运行用户，则创建运行用户，如果已存在，则判断是否属于${RUN_USER_GROUP}组，如果不属于则报错退出
    id -u ${RUN_USER} > /dev/null 2>&1
    if [ $? -ne 0 ] ; then

        #创建${RUN_USER}用户
        omm_pwd_path="${g_setup_tool_package_home}/password.ini"
        if [ -f ${omm_pwd_path} ];then
            dos2unix ${omm_pwd_path} > /dev/null 2>&1
            omm_password=$(cat ${omm_pwd_path} | awk -F'=' '{print $2}')
            validatePwd ${omm_password}
            if [ $? -ne 0 ];then
               return 1
            fi
        else
            g_password=""
            c_password=""

            read -p "Please input omm password:" -s -t 60 g_password
            echo -e ""
            validatePwd ${g_password}
            if [ $? -ne 0 ];then
               return 1
            fi
            read -p "Please Confirm password: "  -s -t 60 c_password
            echo -e ""
            g_password=$(echo "${g_password}" | tr -d ['\r'])

            if [ -z "${g_password}" ];then
                echo  "The new password cannot be empty."
            return 1;
            fi

            if [ -z "${c_password}" ];then
                echo  "The confirm password cannot be empty."
                return 1;
            fi

            if [  "${g_password}" != "${c_password}" ];then
                echo  "Passwords do not match."
                return 1;
            fi
            omm_password=${g_password}
        fi
        addRunUser ${RUN_USER} "${omm_password}" ${g_run_userID} ${OMM_USER_HOME} ${RUN_USER_GROUP} > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to creat the ${RUN_USER} user."
            return 1
        fi
    else
        local group=$(groups ${RUN_USER} | awk -F ' ' '{print $3}')
        if [ "${group}" != "${RUN_USER_GROUP}" ] ; then
            showLog_error "${LINENO}" "${RUN_USER} user already exists, but its group is not ${RUN_USER_GROUP}, please delete the ${RUN_USER} user first."
            return 1
        fi
        local omm_home=$(cat /etc/passwd | grep "^${RUN_USER}:" | awk -F: '{print $6}')
        if [ "${OMM_USER_HOME}" != "${omm_home}" ]; then
            showLog_error "${LINENO}" "${RUN_USER} user already exists, but its home dir is not ${OMM_USER_HOME}."
            return 1
        fi
    fi


    #install installSudoExecutor.sh
    install_sudo "$1"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    #在用户目录/home/${INSTALL_USER}/的.bashrc中添加两个环境变量：
    save_input_param
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi
    
    # 启用root用户切换其他普通用户无需输入密码
    enable_pam_rootok
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    #sysctl.conf中增加fs.inotify.max_user_watches
    modify_max_user_watches
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi
    
    #修改/proc/sys/kernel/pty/max参数
    modify_kernel_pty

    removeGaussDBConfig
    
    create_soft_link_for_libboost 1.53.0 >/dev/null 2>&1
 
    create_soft_link_for_libboost 1.66.0 >/dev/null 2>&1

    showLog_info "${LINENO}" "success to preSet."
    return 0
}

# ---------------------------------------------------------------------------- #
#                        获取当前路径,初始化日志文件                           #
# ---------------------------------------------------------------------------- #
init_path
cd "${g_curPath}"
init_log

# ---------------------------------------------------------------------------- #
#                                   导入头文件                                 #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/func/log.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${g_curPath}/log.sh."; exit 1;}
. "${g_curPath}/func/constants.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/func/constants.sh."; exit 1; }

# ---------------------------------------------------------------------------- #
#                                 脚本开始运行                                 #
# ---------------------------------------------------------------------------- #
main "$@"
ret=$?

#在操作系统日志中记录审计日志
syslog "manager" "preset.sh" "$ret" "User root grants rights to user omm before user omm performs operations."
exit $ret