#!/bin/bash
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    loader_sudoExecute.sh
## Description: loader can execute sudo command via this script
## Version:     FusionInsight Hadoop V100R002C30
## Created:     Friday, May 7, 2021
################################################################################

source `dirname $0`/common_sudoExecute.sh

declare g_sudo_profile="/etc/profile.d/sudoProfile.sh" # sudo脚本中用到的环境变量文件
if [ -f ${g_sudo_profile} ] ; then
    source ${g_sudo_profile}
fi

declare -r g_fuctionArray=(loader_getInterface yarn_postinstall_chown_r yarn_postinstall_chown yarn_postinstall_chmod_jars yarn_postinstall_chmod_r yarn_postinstall_chmod yarn_prestart_nodemanager_chmod yarn_prestart_nodemanager_createUsercache yarn_prestart_nodemanager yarn_rollback_chown kafka_delete_dir kafka_postinstall_checkdir datanode_create_dn_socket_path krbc_upgrade_backup krbc_upgrade_restore ldapc_upgrade_backup krbc_hmon_copySrcConfFile ldaps_install_configSysLog ldapc_hmon_copySrcConfFile ntp_mon_copySrcConfFile chrony_mon_copySrcConfFile recursive_create_directory m_remove_upgrade_bak_file m_clear_upgrade_package m_adapter_specified_dir_profile  chowntimestamp m_init_logFile m_copy_scrpitToLocal remove_tmp ch_dir_owner ch_dir_mod replace_file delete_file add_file back_file preinstall_cp configIP configInterfaceDown configInterfaceUp m_chown_log LdapServer_getFloatIpInterface LdapServer_getFloatIpNetmask LdapServer_validateFloatIp chmodGaussdbLog replace_old_feed_watchdog_cron check_iftop mk_link_f add_sudospawner_setting remove_sudospawner_setting m_config_sudoers sssd_backupConfig_cpf sssd_restoreConfig_cpf agent_config_ntp backup_ntp_sys_conf agent_config_chrony kill_ntp_process backup_chrony_sys_conf krbc_hmon_repairConfigFile om_func_forceLogrotate om_func_rotateCompressLogs mk_link_f ldapc_install_configNscdServer setup_container_executor get_pg_search_passwd)


################################################################################
# Function: m_chown_log
# Description: change own
# Parameter:
#   input:
#   $1    own flag
#   $2    path name
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_chown_log()
{
    if [[ -z "$1" || -z "$2" ]]; then
        echo "chown arguments ($1) ($2) is null."
        return 1
    fi

    if [ ! -e "$2" ]; then
        echo "chown path does not exist."
        return 1
    fi

    #检查日志目录
    if [ "$2" = "/var/log/osperf" -o "$2" = "${LOG_HOME}/Bigdata/nodeagent/scriptlog" ] ; then
        recursive_chown "$1" "$2"
        return $?
    else
        echo "chown $2: Permission denied."
        return 1
    fi
}

####################################################################
#  FUNCTION     : configInterfaceUp
#  DESCRIPTION  : 配置interface生效
#  INPUT        : $1: interface; $2: ipadress
#  OUTPUT       : NA
#  RETURN       : 0: success,  非0：错误码
####################################################################
configInterfaceUp()
{
    local interface="$1"

    if [ -z "${interface}" ]; then
        echo "interface is empty, interface is $interface."
        return 1
    fi

    ${IFCONFIG} "${interface}" up
    return $?
}


####################################################################
#  FUNCTION     : configInterfaceDown
#  DESCRIPTION  : 配置interface失效
#  INPUT        : $1: interface; $2: ipadress
#  OUTPUT       : NA
#  RETURN       : 0: success,  非0：错误码
####################################################################
configInterfaceDown()
{
    local interface="$1"

    if [ -z "${interface}" ]; then
        echo "interface is empty, interface is $interface."
        return 1
    fi

    ${IFCONFIG} "${interface}" down
    return $?
}

####################################################################
#  FUNCTION     : configIP
#  DESCRIPTION  : 配置ip生效
#  INPUT        : $1: interface; $2: ipadress
#  OUTPUT       : NA
#  RETURN       : 0: success,  非0：错误码
####################################################################
configIP()
{
    local interface="$1"
    local ipAdress="$2"
    local ipnetmask="$3"

    if [ -z "${interface}" ] || [ -z "${ipAdress}" ] || [ -z "${ipnetmask}" ]; then
        echo "interface,netmask or ipAdress is empty. interface is $interface,netmask is $ipnetmask, ipAdress is ipAdress."
        return 1
    fi

    ${IFCONFIG} "${interface}" "${ipAdress}" netmask "${ipnetmask}"
    return $?
}


################################################################################
# Function: preinstall_cp
# Description: copy file
# Parameter:
#   input:
#   $1      source path
#   $2      target path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function preinstall_cp ()
{
    local paths_array=("/usr/lib/systemd/system//osperf.service" "/usr/lib/systemd/system/osperf.service" \
                       "/usr/sbin//statmon" "/usr/sbin/statmon" \
                       "/etc/init.d//statmon" "/etc/init.d/statmon" \
                       "/usr/lib/systemd/system//statmon.service" "/usr/lib/systemd/system/statmon.service" \
                       "/etc/sysconfig/rngd" "/usr/local/rngd")

    if [[ -z "$1" || -z "$2" ]]; then
        echo "path is null."
        return 1
    fi

    if [ ! -e "$1" ]; then
        echo "source path $1 does not exist."
        return 1
    fi

    check_linkfile "$2" || return 1

    echo "${paths_array[@]}" | grep -q -E "^$2[[:space:]]|[[:space:]]$2[[:space:]]|[[:space:]]$2$"
    if [ $? -eq 0 ]; then
        cp -arf "$1" "$2"
        return $?
    fi

    echo "cp -arf $1 $2 : Permission denied"
    return 1
}

function back_file()
{
    local file="$1"
    local dstDir="$2"
    verify_path "$dstDir" "$file" || return 1

    if ! [ -f "${file}" ]; then
        return 1
    fi

    check_linkfile "${file}"
    if [ $? -ne 0 ]; then
        return 1
    fi
    check_linkfile "${dstDir}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    local fuser=
    fuser=$(stat -c %U "$file")

    if [ "$fuser" != "ommdba" ] && [ "$fuser" != "omm" ]; then
        return 1
    fi

    if cp -pf "${file}" "${dstDir}"; then
        return 0
    fi
    return 1
}

##############################################################################
# Function: delete_file
# Description:
# Parameter:
#   input:
#   N/A
#   output:
#   NA
# Others: N/A
################################################################################
function delete_file()
{
    local file="$1"

    verify_path "$file" || return 1

    if ! [ -e "$file" ]; then
        return 0
    fi

    local fuser=
    fuser=$(stat -c %U "$file")
    if [ "$fuser" != "ommdba" ] && [ "$fuser" != "omm" ]; then
        return 1
    fi

    rm -rf "$file"

    return $?
}
##############################################################################
# Function: add_file
# Description:
# Parameter:
#   input:
#   N/A
#   output:
#   NA
# Others: N/A
################################################################################
function add_file()
{
    local file="$1"
    local dstDir="$2"

    verify_path "$dstDir" "$file" || return 1

    if ! [ -e "$file" ]; then
        return 1
    fi

    check_linkfile "${file}"
    if [ $? -ne 0 ]; then
        return 1
    fi
    check_linkfile "${dstDir}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    local fuser=
    fuser=$(stat -c %U "${dstDir}")
    if [ "$fuser" != "ommdba" ] && [ "$fuser" != "omm" ]; then
        return 1
    fi

    local fileName="$(basename "${file}")"
    if [ -e "${dstDir}/${fileName}" ]; then
        local srcfuser=
        srcfuser=$(stat -c %U "${dstDir}/${fileName}")
        if [ "$srcfuser" != "ommdba" ] && [ "$srcfuser" != "omm" ]; then
            return 1
        fi
        rm -rf "${dstDir}/${fileName}"
    fi

    if cp -af "$file" "$dstDir"; then
        return 0
    fi

    return 1
}

##############################################################################
# Function: replace_file
# Description:
# Parameter:
#   input:
#   N/A
#   output:
#   NA
# Others: N/A
################################################################################
function replace_file()
{
    local srcfile="$1"
    local dstfile="$2"

    verify_path "$dstfile" "$srcfile" || return 1

    if ! [ -e "$dstfile" ]; then
        return 1
    fi

    if [ -d "$dstfile" ]; then
        return 1
    fi

    check_linkfile "${srcfile}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    check_linkfile "${dstfile}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    local fuser=
    local fgroup=
    local facl=
    fuser=$(stat -c %U "$dstfile")
    fgroup=$(stat -c %G "$dstfile")
    facl=$(stat -c %a "$dstfile")

    if [ "$fuser" != "ommdba" ] && [ "$fuser" != "omm" ]; then
        return 1
    fi

    if [ $(stat -c %h "$dstfile") -gt 1 ]; then
        if ! cancel_hardLink "${dstfile}"; then
            return 1
        fi
    fi

    if ! cp -rf "$srcfile" "$dstfile"; then
        return 1
    fi
    chmod "$facl" "$dstfile"
    chown "$fuser:$fgroup" "$dstfile"

    return 0
}

##############################################################################
# Function: ch_dir_mod
# Description:
# Parameter:
#   input:
#   N/A
#   output:
#   NA
# Others: N/A
################################################################################
function ch_dir_mod()
{
    local facl="$1"
    local dstPath="$2"

    verify_path "$dstPath" || return 1

    if ! [ -d "$dstPath" ]; then
        return 1
    fi

    local fuser=
    fuser=$(stat -c %U "$dstPath")
    if [ "$fuser" != "ommdba" ] && [ "$fuser" != "omm" ]; then
        return 1
    fi

    single_chmod "$facl" "$dstPath"
    return $?

}

##############################################################################
# Function: ch_dir_owner
# Description:
# Parameter:
#   input:
#   N/A
#   output:
#   NA
# Others: N/A
################################################################################
function ch_dir_owner()
{
    local owner="$1"
    local path="$2"

    verify_path "$path" || return 1

    if ! [ -d "$path" ]; then
        return 1
    fi

    local fuser=
    fuser=$(stat -c %U "$path")
    if [ "$fuser" != "ommdba" ] && [ "$fuser" != "omm" ]; then
        return 1
    fi

    if [ "$owner" != "ommdba:wheel" ] && [ "$owner" != "omm:wheel" ] && [ "$owner" != "omm:ficommon" ]; then
        return 1
    fi

    single_chown "$owner" "$path"
    return $?

}

################################################################################
# Function: remove_tmp
# Description: 清除/tmp目录下的临时文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
remove_tmp()
{
    # 清除/tmp目录下的临时文件
    log "INFO" "${LINENO}" "start to remove the temporary files in the tmp directory."
    find /tmp -type f -user "${OMM_USER}" -group "${WHEEL_GROUP}" -print0 | xargs -0 rm -rf
    find /tmp -type d -user "${OMM_USER}" -group "${WHEEL_GROUP}" -print0 | xargs -0 rm -rf

}

################################################################################
# Function: m_copy_scrpitToLocal
# Description: copy scrpit to /usr/local/
# Parameter:
#   input:
#   $1    local file path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_copy_scrpitToLocal()
{
    local copyPath="$1"

    if [ -z "${copyPath}" ]; then
        echo "copy source path is null."
        return 1
    fi

    if [ ! -e "${copyPath}" ]; then
        echo "copy source path does not exist."
        return 1
    fi

    local ret1=1
    echo "${copyPath}" | grep -E "${bigdata_home}/om-server_?[A-Z0-9\.]*/om/inst/plugin/osperf" >>/dev/null 2>&1
    if [ $? -eq 0 ] ; then
        ret1=0
    fi

    local ret2=1
    echo "${copyPath}" | grep -E "${bigdata_home}/om-agent_?[A-Z0-9\.]*/nodeagent/setup/plugin/osperf" >>/dev/null 2>&1
    if [ $? -eq 0 ] ; then
        ret2=0
    fi

    if [ "${ret1}" -ne 0 -a "${ret2}" -ne 0 ] ; then
        return 1
    fi

    #拷贝脚本修改权限
    cp "${copyPath}/" /usr/local/ -arf
    recursive_chmod 750 "/usr/local/osperf/"

    #转换格式并执行
    hash dos2unix > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        find /usr/local/osperf/ -maxdepth 5 -mindepth 1 -type f -name "*" | while read line
        do
            dos2unix "$line" > /dev/null 2>&1
        done
    fi

    recursive_chown 0:0 "/usr/local/osperf"
    return $?
}


################################################################################
# Function: m_init_logFile
# Description: mkdir
# Parameter:
#   input:
#   $1        dir path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_init_logFile()
{
    if [ $# -eq 0 ]; then
        echo "chown arguments ($1) is null."
        return 1
    fi

    #检查日志目录
    if [ ! -d "$1" ]; then
        mkdir -p "$1"
    fi
}


####################################################################
#  FUNCTION     : chowntimestamp
#  DESCRIPTION  : 修改ntp配置文件/etc/sysconfig/ntpd(redhat)/etc/sysconfig/ntp(suse)
#  INPUT        : NA
#  OUTPUT       : NA
#  RETURN       : 0: success,  非0：错误码
####################################################################
function chowntimestamp()
{
    local ntptimestamp_txt=$bigdata_log_home/timestamp/ntptimestamp.txt
    local restart_stamp=$bigdata_log_home/timestamp/restart_stamp
    local inodeValue=0

    if [ -d $bigdata_log_home/timestamp/ ];then
        if [ -L $bigdata_log_home/timestamp/ ];then
            return 1
        fi
        chown omm:wheel $bigdata_log_home/timestamp/

        if [ -f ${ntptimestamp_txt} ];then
            if [ -L ${ntptimestamp_txt} ];then
                return 1
            fi
            inodeValue=$(ls -l ${ntptimestamp_txt} | awk '{print $2}')
            if [ ${inodeValue} -ge 2 ];then
                return 1
            fi
            chown omm:wheel $bigdata_log_home/timestamp/ntptimestamp.txt
        fi

        if [ -f ${restart_stamp} ];then
            if [ -L ${restart_stamp} ];then
                return 1
            fi

            inodeValue=$(ls -l ${restart_stamp} | awk '{print $2}')
            if [ ${inodeValue} -ge 2 ];then
                return 1
            fi
            chown omm:wheel $bigdata_log_home/timestamp/restart_stamp
        fi
    fi
}

################################################################################
# Function: m_adapter_specified_dir_profile
# Description: C50升级C60时，添加root用户环境变量
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
m_adapter_specified_dir_profile()
{
    mkdir -p /etc/profile.d
    local sudo_profile='/etc/profile.d/sudoProfile.sh'
    if [ -f ${sudo_profile} ]; then
        return 0
    fi
    touch ${sudo_profile}

    if [ -n "${BIGDATA_HOME}" ];then
        echo "BIGDATA_HOME=${BIGDATA_HOME}" >> ${sudo_profile}
    fi
    
    if [ -n "${BIGDATA_DATA_HOME}" ];then
        echo "BIGDATA_DATA_HOME=${BIGDATA_DATA_HOME}" >> ${sudo_profile}
    fi
    
    if [ -n "${BIGDATA_LOG_HOME}" ];then
        echo "BIGDATA_LOG_HOME=${BIGDATA_LOG_HOME}" >> ${sudo_profile}
    fi
}

################################################################################
# Function: m_clear_upgrade_package
# Description: 回滚或提交升级后，清理原安装包
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
m_clear_upgrade_package()
{
    local UPGRADE_STATUS_FILE="UPGRADE_STATUS"
    local ROLLBACK_STATUS_FILE="ROLLBACK_STATUS"

    local upgrade_path="$1/FusionInsight/software/upgrade"

    if [ ! -d "${upgrade_path}" ]; then
        log "INFO" "${LINENO}" "${upgrade_path} does not exist, treat as clear success."
        return 0
    fi

    log "INFO" "${LINENO}" "sudo: /bin/rm -rf ${upgrade_path}/sbin/${UPGRADE_STATUS_FILE}."
    [ ! -L "${upgrade_path}/sbin/${UPGRADE_STATUS_FILE}" ] && /bin/rm -rf "${upgrade_path}/sbin/${UPGRADE_STATUS_FILE}"

    log "INFO" "${LINENO}" "sudo: /bin/rm -rf ${upgrade_path}/sbin/${ROLLBACK_STATUS_FILE}."
    [ ! -L "${upgrade_path}/sbin/${ROLLBACK_STATUS_FILE}" ] && /bin/rm -rf "${upgrade_path}/sbin/${ROLLBACK_STATUS_FILE}"

    #排除软链接后删除"${upgrade_path}/share/"目录下的install-*.ini文件
    log "INFO" "${LINENO}" "sudo: /bin/rm -rf ${upgrade_path}/share/install-*.ini."
    find "${upgrade_path}/share/" -name "install-*.ini" -type f -links 1 | xargs -I{} /bin/rm -rf '{}'

    log "INFO" "${LINENO}" "sudo: echo '' > ${upgrade_path}/share/upgrading.profile."
    [ ! -L "${upgrade_path}/share/upgrading.profile" ] && echo '' > "${upgrade_path}/share/upgrading.profile"

    log "INFO" "${LINENO}" "sudo: /bin/rm -rf ${upgrade_path}/backup."
    [ ! -L "${upgrade_path}/backup" ] && /bin/rm -rf "${upgrade_path}/backup"
}


####################################################################
#  FUNCTION     : m_remove_upgrade_bak_file
#  DESCRIPTION  : 升级提交删除旧版本程序备份文件
#  INPUT        : $1 日志文件
#  OUTPUT       : NA
#  RETURN       : 0: success,  非0：错误码
####################################################################
function m_remove_upgrade_bak_file()
{
    if [ $# -lt 1 ]; then
        return 1
    fi

    local log_file="$1"
    log "INFO" "${LINENO}" "sudo: upgrade commit, remove programs." "${log_file}"

    if [ -d "${bigdata_home}/upgrade_backup" ]; then
        /bin/rm -rf "${bigdata_home}/upgrade_backup"
        log "INFO" "${LINENO}" "sudo: /bin/rm -rf ${bigdata_home}/upgrade_backup." "${log_file}"
    fi

    return 0
}

##########################################################
#    Description: recursive create directory
#    Parameter:   None
#    Return:      Return: 0 -- success; not 0 -- failure
#
##########################################################
function recursive_create_directory()
{
    local paths=()
    local path="$1"
    local i=0
    local j=0
    local USER="omm"
    local GROUP="wheel"

    #目录不可以为空
    if [ -z "${path}" ]; then
        echo "path is empty, error."
        return 1
    fi

    #指定的目录已经存在 不需要创建 也不修改权限
    if [ -d "${path}" ]; then
        return 0
    fi

    #记录不存在的路径
    paths[${i}]="${path}"

    # 查询 不存在的目录数
    while true
    do
        ((i++))
        path="$(dirname ${path})"
        if [ -d "${path}" ]; then
            break;
        fi

        #记录不存在的路径
        paths[${i}]="${path}"
    done

    #逐级创建目录
    for ((j=${i}-1; j>=0; j--))
    do
        if [ -z "${paths[${j}]}" ]; then
            continue
        fi

        #创建目录
        mkdir "${paths[${j}]}"
        if [ $? -ne 0 ]; then
            echo "${S_MKDIR} ${paths[${j}]} failed"
            return 1
        fi

        #修改权限
        chmod 755 "${paths[${j}]}"
        if [ $? -ne 0 ]; then
            echo "${S_CHMOD} 755 ${paths[${j}]} failed"
            return 1
        fi

        #修改属主
        chown "${USER}":"${GROUP}" "${paths[${j}]}"
        if [ $? -ne 0 ]; then
            echo "${S_CHOWN} ${USER}:${GROUP} ${paths[${j}]} failed"
            return 1
        fi

    done

    return 0
}


################################################################################
# Function: ntp_mon_copySrcConfFile
# Description: copy ConfFile ntp.conf
# Parameter:
#   input:
#         ${1}: src  Conf File path
#         ${2}: dst  Conf File path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
ntp_mon_copySrcConfFile()
{
    local src_location1="/etc/ntp.conf"
    local tmp_location="^${bigdata_home}/tmp/ntp/ntp.*"

    local src_location2=""
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then
        src_location2="/etc/ntp.keys"
    else
        src_location2="/etc/ntp/ntpkeys"
    fi

    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    if [ "${1}" = "${src_location1}" ] ; then
        echo ${2} | grep ${tmp_location}   1>/dev/null
        if [  $? = 0    ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi

    if [ "${1}" = "${src_location2}" ] ; then
        echo ${2} | grep ${tmp_location}   1>/dev/null
        if [  $? = 0    ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi

    return 1
}

################################################################################
# Function: chrony_mon_copySrcConfFile
# Description: copy ConfFile chrony.conf
# Parameter:
#   input:
#         ${1}: src  Conf File path
#         ${2}: dst  Conf File path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
chrony_mon_copySrcConfFile()
{
    local src_location1="/etc/chrony.conf"
    local tmp_location="^${bigdata_home}/tmp/ntp/chrony.*"

    local src_location2=""
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then
        src_location2="/etc/chrony.keys"
    else
        src_location2="/etc/chrony.keys"
    fi

    local src_location3=""
    if [ -n "$IS_SUSE_OS" ]; then
        src_location3="/etc/ntp.keys"
    else
        src_location3="/etc/ntp/ntpkeys"
    fi

    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    if [ "${1}" = "${src_location1}" ] ; then
        echo ${2} | grep ${tmp_location}   1>/dev/null
        if [  $? = 0    ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi

    if [ "${1}" = "${src_location2}" -o "${1}" = "${src_location3}" ] ; then
        echo ${2} | grep ${tmp_location}   1>/dev/null
        if [  $? = 0    ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi

    return 1
}

##########################################################
#    Description: kafka postinstall checkdir
#    Parameter:   kafka data dir
#    Return:      Return: 0 -- success; not 0 -- failure
##########################################################
kafka_postinstall_checkdir(){
    DATA_DIR=$1

    if [ ! -d "$DATA_DIR" ]; then
        recursive_create_directory $DATA_DIR
        if [ $? -ne 0 ]; then
            echo "Error: sudo make data dir $DATA_DIR error."
            return 1
        fi
        echo "INFO: sudo make data dir $DATA_DIR success."
    fi

    single_chown omm:wheel "$DATA_DIR" && single_chmod 700 "$DATA_DIR"
    if [ $? -ne 0 ]; then
        echo "Error: sudo set data dir $DATA_DIR  owner or authority error."
        return 1
    fi
    echo "INFO: sudo set data dir $DATA_DIR  owner or authority success."
    return 0
}
####################################################################
#  Description  : delete dir
#  Parameter    : del_dir
#  Return       : 0 -- success; not 0 -- failure
####################################################################
kafka_delete_dir()
{
    del_dir=$1
    if [ -d "$del_dir" ]; then
        if [ "-bak" = ${del_dir:0-4} ]; then
            /bin/rm -rf $del_dir
            return $?
        else
            echo "Error: sudo to delet data dir $del_dir ,but it is not bak_dir."
            return 1
        fi

    else
        echo "Error: sudo to delet data dir $del_dir ,but it is not exist."
        return 1
    fi
}

##########################################################
#    Description:  get interface by ip
#    Parameter:   $1 ip
#    Return:
##########################################################
function loader_getInterface()
{
    interface=$( /sbin/ip addr | grep -B0 -w "$1"  | awk '{print $NF}' )
    echo "${interface}"
}

yarn_postinstall_chown_r(){
    local targetPath="$1"
    recursive_chown omm:ficommon "$targetPath"
}

yarn_postinstall_chown(){
    local targetPath="$1"
    single_chown omm:ficommon "$targetPath"
}

yarn_postinstall_chmod_jars(){
    local targetPath="$1"
    find "$targetPath" -name "*.jar" -type f -links 1 | xargs chmod 0550
    #find "$targetPath" -name "*.jar" | chown omm:ficommon
    find "$targetPath" -name "*.so" -type f -links 1 | xargs chmod 0550
    #find "$targetPath" -name "*.so" | chown omm:ficommon
}

yarn_postinstall_chmod_r(){
    local targetPath="$1"
    recursive_chmod 750 $targetPath
}

yarn_postinstall_chmod(){
    local permission="$1"
    local targetPath="$2"
    single_chmod $permission $targetPath
}


yarn_prestart_nodemanager_chmod()
{
    local nm_dirs="$1"
    local tmp_dir=""
    local srv_dir=$(cd "$2"; pwd)
    local file_group=""
    local exist_parent_dir=""

    for nm_dir in ${nm_dirs//,/ }; do

        tmp_dir="${nm_dir}"

        while true; do
            if [ -d "$tmp_dir" ]; then
                exist_parent_dir=$(cd "$tmp_dir"; pwd)
                break
            fi

            tmp_dir=$(dirname "${tmp_dir}")
        done

        if [ ! -d "${nm_dir}" ]; then
            mkdir -p "${nm_dir}"
        fi

        single_chmod 755 "${nm_dir}"
        single_chown omm:ficommon "${nm_dir}"

        tmp_dir="${nm_dir}"

        while true; do
            tmp_dir=$(dirname "${tmp_dir}")

            if [ "${tmp_dir}" == "/" ]; then
                break
            fi

            file_group=$(stat -c "%G" "$tmp_dir")

            if [ ${#tmp_dir} -gt ${#exist_parent_dir} ]; then
                single_chmod 750 "${tmp_dir}"
                single_chown omm:ficommon "${tmp_dir}"
            else
                if [ "$file_group" == "ficommon" ]; then
                    single_chmod g+x "$tmp_dir"
                else
                    single_chmod o+x "$tmp_dir"
                fi
            fi
        done

    done
}


yarn_prestart_nodemanager_createUsercache(){
    local nm_dirs="$1"
    local nm_usercache_dir=""
    local nm_filecache_dir=""

    for nm_dir in ${nm_dirs//,/ }; do

        nm_usercache_dir="${nm_dir}/usercache"
        nm_filecache_dir="${nm_dir}/filecache"

        if [ ! -d "${nm_usercache_dir}" ]; then
            mkdir "${nm_usercache_dir}"
            single_chmod 755 "${nm_usercache_dir}"
            single_chown omm:wheel "${nm_usercache_dir}"
        fi

        if [ ! -d "${nm_filecache_dir}" ]; then
            mkdir "${nm_filecache_dir}"
            single_chmod 755 "${nm_filecache_dir}"
            single_chown omm:wheel "${nm_filecache_dir}"
        fi

    done
}

yarn_prestart_nodemanager() {
    local nm_local_dirs="$2"
    local nm_log_dirs="$3"
    local srv_dir="$4"

    if [ -f "/etc/hadoop/conf/container-executor.cfg" ]; then
        sed -i "/yarn.nodemanager.local-dirs/d" /etc/hadoop/conf/container-executor.cfg
        sed -i "/yarn.nodemanager.log-dirs/d" /etc/hadoop/conf/container-executor.cfg
        /bin/echo "yarn.nodemanager.local-dirs=$2" >> /etc/hadoop/conf/container-executor.cfg
        /bin/echo "yarn.nodemanager.log-dirs=$3" >> /etc/hadoop/conf/container-executor.cfg

        high_os_version_check
        local os_version=$?
        local CGROUP_ROOT_PATH
        if [ -z "$CGROUP_ROOT_PATH" ]; then
            if [ "$os_version" == 0 ]; then
                CGROUP_ROOT_PATH="/dev/cgroup"
            else
                CGROUP_ROOT_PATH="/sys/fs/cgroup"
            fi
        fi

        /bin/echo "
[cgroups]
# This should be same as yarn.nodemanager.linux-container-executor.cgroups.mount-path inside yarn-site.xml
root=$CGROUP_ROOT_PATH
# This should be same as yarn.nodemanager.linux-container-executor.cgroups.hierarchy inside yarn-site.xml
yarn-hierarchy=/hadoop-yarn

[gpu]" >> /etc/hadoop/conf/container-executor.cfg

        type nvidia-smi >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            /bin/echo "module.enabled=true

[docker]
module.enabled=true
docker.allowed.volume-drivers=nvidia-docker
docker.allowed.rw-mounts=${bigdata_data_home}/hadoop/
docker.allowed.capabilities=SYS_CHROOT,MKNOD,SETFCAP,SETPCAP,FSETID,CHOWN,AUDIT_WRITE,SETGID,NET_RAW,FOWNER,SETUID,DAC_OVERRIDE,KILL,NET_BIND_SERVICE
docker.allowed.networks=bridge,host,none
docker.binary=/usr/bin/docker
docker.privileged-containers.enabled=false
docker.trusted.registries=127.0.0.1:5000" >> /etc/hadoop/conf/container-executor.cfg

            nvidia_devices=$(ls /dev/nvidia*)
            nvidia_devices=$(echo $nvidia_devices)
            nvidia_devices=$(echo ${nvidia_devices// /,})
            /bin/echo "docker.allowed.devices=$nvidia_devices" >> /etc/hadoop/conf/container-executor.cfg

            nvidia_version=$(nvidia-smi | grep "Driver Version" | awk -F" " '{ print $6 }' )
            /bin/echo "docker.allowed.ro-mounts=nvidia_driver_$nvidia_version,${bigdata_data_home}/hadoop/" >> /etc/hadoop/conf/container-executor.cfg
        else
            /bin/echo "module.enabled=false

[docker]
module.enabled=true
docker.allowed.ro-mounts=${bigdata_data_home}/hadoop/
docker.allowed.rw-mounts=${bigdata_data_home}/hadoop/
docker.allowed.capabilities=SYS_CHROOT,MKNOD,SETFCAP,SETPCAP,FSETID,CHOWN,AUDIT_WRITE,SETGID,NET_RAW,FOWNER,SETUID,DAC_OVERRIDE,KILL,NET_BIND_SERVICE
docker.allowed.networks=bridge,host,none
docker.binary=/usr/bin/docker
docker.privileged-containers.enabled=false
docker.trusted.registries=127.0.0.1:5000" >> /etc/hadoop/conf/container-executor.cfg
        fi
    fi

    yarn_prestart_nodemanager_chmod "${nm_local_dirs}" "${srv_dir}"
    yarn_prestart_nodemanager_chmod "${nm_log_dirs}" "${srv_dir}"
    yarn_prestart_nodemanager_createUsercache "${nm_local_dirs}"
}

yarn_rollback_chown()
{
    local nm_dirs="$1"
    local tmp_dir=""
    local srv_dir=$BIGDATA_DATA_HOME

    for nm_dir in ${nm_dirs//,/ }; do
        if [ ! -d "${nm_dir}" ]; then
            mkdir -p "${nm_dir}"
        fi

        single_chmod 755 "${nm_dir}"
        single_chown omm:wheel "${nm_dir}"

        tmp_dir="${nm_dir}"

        while true; do
            tmp_dir="${tmp_dir}/.."
            tmp_dir=$(cd "${tmp_dir}"; pwd)

            if [ "${tmp_dir}" == "${srv_dir}" ] || [ "${tmp_dir}" == "/" ]; then
                break
            fi

            single_chmod 755 "${tmp_dir}"
            single_chown omm:wheel "${tmp_dir}"
        done

    done
}

datanode_create_dn_socket_path()
{
    local dn_socket_path="$1"
    
    if [ -z "$dn_socket_path" ]; then
        return 1
    fi
    
    local dn_socket_dir=$(dirname "$dn_socket_path")
    
    if [ "$dn_socket_dir" == "/" ]; then
        return 2
    fi
    
    if [ -f "$dn_socket_path" ]; then
        rm -f "$dn_socket_path"
    fi
    
    mkdir -p "$dn_socket_dir"
    single_chmod 750 "$dn_socket_dir"
    single_chown omm:ficommon "$dn_socket_dir"
    
    return 0
}

################################################################################
# Function: krbc_upgrade_backup
# Description: 备份/etc/krb5.conf目录
# Parameter:
#   input:
#    $1:${BIGDATA_DATA_HOME}/upgrade/krbclient C70 changed to ${BIGDATA_HOME}/package
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function krbc_upgrade_backup()
{
    local krbclient_data="${1}"
    
    check_linkfile "${1}"
    if [ $? -ne 0 ]; then
        return 1
    fi 
    
    verify_path "${krbclient_data}" 
    if [ $? -ne 0 ] ; then
        log "ERROR" "${LINENO}" "Incorrect backup path: ${krbclient_data}."
        return 1
    fi
    
    /bin/cp -af /etc/krb5.conf  "${krbclient_data}"/
    return $?
}

################################################################################
# Function: krbc_upgrade_restore
# Description: 还原/etc/krb5.conf目录
# Parameter:
#   input:
#    $1:${BIGDATA_DATA_HOME}/upgrade/krbclient
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function krbc_upgrade_restore()
{
    local krbclient_data="${1}"
    
    verify_path "${krbclient_data}"
    if [ $? -ne 0 ] ; then
        log "ERROR" "${LINENO}" "Incorrect backup path: ${krbclient_data}."
        return 1
    fi
    
        /bin/cp -af "${krbclient_data}"/krb5.conf /etc/krb5.conf
         return $?
}

################################################################################
# Function: ldapc_upgrade_backup
# Description: /bin/cp -arf "${oldver_install_dir}"/* "${oldver_backup_dir}"
# Parameter:
#   input:
#           ${1}:  #{install_home}/ldapclient
#           ${2}:  #{upgrade_home}/ldapclient
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
ldapc_upgrade_backup()
{
    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    verify_path "${2}"
    if [ $? -ne 0 ] ; then
        log "ERROR" "${LINENO}" "Incorrect backup path: ${2}."
        return 1
    fi

    local src_file="^${bigdata_home}/.*/ldapclient/"
    echo "${1}"   | grep  "${src_file}"   1>/dev/null
    if [ $? = 0  ] ; then
        /bin/cp -arf  "${1}" "${2}"
        return $?
    fi
    return 1
}

################################################################################
# Function: krbc_hmon_copySrcConfFile
# Description: copy krb5.conf
# Parameter:
#   input:
#         ${1}:"/etc/krb5.conf"
#         ${2}:"^${bigdata_home}/FusionInsight/etc/"
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
krbc_hmon_copySrcConfFile()
{
    local src_file="/etc/krb5.conf"
    local correct_file="^${bigdata_home}/.*/etc/"

    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    #the head of ${1} contains  ${data_home}
    echo "${2}" | grep "${correct_file}"  1>/dev/null
    if [ $? = 0 ]  ;  then
        if [ "${1}" = "${src_file}"   ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi

    return 1
}

################################################################################
# Function: ldaps_install_configSysLog
# Description: cp -af   mv -f  rsyslog.conf
# Parameter:
#   input:
#           ${1}:  src conf file rsyslog.conf
#           ${2}:  dst conf file rsyslog.conf
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
ldaps_install_configSysLog()
{
    local redhat_syslogConf="/etc/rsyslog.conf"
    local redhat_temp_syslogConf="^${bigdata_home}/.*/ldapserver/package/rsyslog.conf"

    if [ "${1}" = "${redhat_syslogConf}" ] ; then
        check_linkfile "${2}"
        if [ $? -ne 0 ]; then
            return 1
        fi

        echo "${2}"   | grep  "${redhat_temp_syslogConf}"   1>/dev/null
        if [ $? = 0  ] ; then
            /bin/cp -af  "${1}" "${2}"
            return $?
        fi
    fi

    if [ "${2}" = "${redhat_syslogConf}" ] ; then
        echo "${1}"   | grep  "${redhat_temp_syslogConf}"   1>/dev/null
        if [  $? = 0   ] ; then
            /bin/mv -f  "${1}" "${2}"
            return $?
        fi
    fi
    return 1
}


function LdapServer_getFloatIpNetmask()
{
    service_ip="$1"
    if [ -z "${service_ip}" ];then
        echo ""
        exit 0
    fi

    mask=$( ${IFCONFIG} -a |grep -w -F "${service_ip}" | awk '{print $4}' | tr -d 'Mask:' )
    echo ${mask}

    exit 0
}

function LdapServer_getFloatIpInterface()
{
    service_ip="$1"
    if [ -z "${service_ip}" ];then
        echo ""
        exit 0
    fi
    ether=$( ${IFCONFIG} -a | grep -B1 -w "${service_ip}"  | grep HWaddr  | awk '{print $1}')
    if [ -z "${ether}" ];then
        echo ""
        exit 0
    fi
    echo ${ether}:LDAP
    exit 0
}

function LdapServer_validateFloatIp()
{
   local localip="$1"
   local floatip="$2"
   local interface="$(LdapServer_getFloatIpInterface ${localip})"

   ifconfig | grep -A 2 -w "${interface}" | grep -w -F "${floatip}"
   return $?
}

###########################################################
#    Description:  Chmod GaussdbLog 750
#    Parameter:   no
#    Return:
##########################################################
function chmodGaussdbLog()
{
    recursive_chmod 0755 "/var/log/gaussdb"
}


#替换旧版本喂狗任务，只适用于C70或更新版本向C70之前版本回滚
replace_old_feed_watchdog_cron()
{
    if [ -z "${bigdata_home}" ]; then
        LOG "ERROR" "replace_old_feed_watchdog_cron failed due to bigdata_home is null."
        return 1
    fi
    OLD_NODE_AGENT_SUDO_SHELL_PATH="${bigdata_home}/nodeagent/bin/sudo/sudoExecute.sh"
    sed_task_content="* * * * * sh $OLD_NODE_AGENT_SUDO_SHELL_PATH wdt_running > /dev/null 2>&1 &"
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    local cron_profile="/var/spool/cron/tabs/root"

    check_linkfile "${cron_profile}" || return 1

    local cron_service="cron"
    #redhat与suse的cron配置文件及服务名有所区别
    if [ -z "$IS_SUSE_OS" ]; then
        cron_profile="/var/spool/cron/root"
        cron_service="crond"
    fi
    local wdt_running="$(cat ${cron_profile} | grep wdt_running | grep -v grep)"
    #如果存在喂狗任务，则清理任务并使用旧版本的sudo路径加入到cron任务中
    if [ -n "${wdt_running}" ]; then
        sed -i '/wdt_running/d' "${cron_profile}"
        ret_temp=$?
        if [ $ret_temp -ne 0 ]; then
            LOG "ERROR" "replace_old_feed_watchdog_cron sed -i '/wdt_running/d' \"${cron_profile}\" failed, result=$ret_temp."
            return 1
        fi
        if [ ! -f "${OLD_NODE_AGENT_SUDO_SHELL_PATH}" ]; then
            LOG "ERROR" "replace_old_feed_watchdog_cron ${OLD_NODE_AGENT_SUDO_SHELL_PATH} does not exist."
            return 1
        fi
        echo "${sed_task_content}" >> "${cron_profile}"
        if [ $? -ne 0 ];then
            LOG 'ERROR' "replace_old_feed_watchdog_cron echo ${sed_task_content} >> \"${cron_profile}\" failed!"
            return 1
        fi
        service $cron_service restart
        if [ $? -ne 0 ];then
            LOG 'ERROR' "replace_old_feed_watchdog_cron service $cron_service restart failed!"
            return 1
        fi
    else
        LOG "INFO" "No wdt_runninng in ${cron_profile}."
    fi

    return 0
}

################################################################################
# Function: check_iftop
# Description: iftop获取网络流量
# Parameter:
#   input:
#   $1 : Path
#   $2 : lan
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function check_iftop()
{
    Path="$1"
    lan="$2"
    if [ -z "${Path}" ]||[ -z "${lan}" ];then
        return 1
    fi
    "${Path}"/Lib/iftop/iftop -n -N -B -t -s 15 -i "${lan}"
}

################################################################################
# Description: create hard link
# Parameter:
#   input:
#   $1      source path
#   $2      target path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function mk_link_f()
{
    verify_path "$1" || return 1
    verify_path "$2" || return 1

    ln -f "$1" "$2"
    return $?
}

################################################################################
# Function: add_sudospawner_setting
# Description: add sudospawner setting into /etc/sudoers
# Return: 0
# Others: N/A
################################################################################
function add_sudospawner_setting()
{
    local sudopath=`ls ${bigdata_home}/FusionInsight_Miner_*/install/*/notebook/anaconda3/bin/sudospawner`
    if [ ! -z ${sudopath} ]; then
        log "INFO" "${LINENO}" "sudo: Config the sudospawner command to /etc/sudoers file."

        # 若/etc/sudoers里没有配置该命令，则配置
        grep "^${OMM_USER} ALL=(${OMM_USER}) NOPASSWD: ${sudopath}$" /etc/sudoers
        if [ $? -ne 0 ] ; then
            echo "${OMM_USER} ALL=(${OMM_USER}) NOPASSWD: ${sudopath}" >> /etc/sudoers
        fi
        return 0
    else
        log "INFO" "${LINENO}" "sudo: Config sudospawner to /etc/sudoers fail."
        return 1
    fi
}

################################################################################
# Function: remove_sudospawner_setting
# Description: remove sudospawner setting from /etc/sudoers
# Return: 0
# Others: N/A
################################################################################
function remove_sudospawner_setting()
{
    local sudopath=`ls ${bigdata_home}/FusionInsight_Miner_*/install/*/notebook/anaconda3/bin/sudospawner`

    if [ ! -z ${sudopath} ]; then
        log "INFO" "${LINENO}" "sudo: clear sudospawner command in /etc/sudoers file."
        # 若/etc/sudoers里有配置该命令，则清除
        grep "^${OMM_USER} ALL=(${OMM_USER}) NOPASSWD: ${sudopath}$" /etc/sudoers
        if [ $? -eq 0 ] ; then
            tmp=$(echo $sudopath | sed -e 's/\//\\\//g')
            sed -i "/${OMM_USER} ALL=(${OMM_USER}) NOPASSWD: $tmp/d" /etc/sudoers
        fi
        return 0
    else
         log "INFO" "${LINENO}" "sudo: clear sudospawner from /etc/sudoers fail."
         return 1
    fi
}

################################################################################
# Function: m_config_sudoers
# Description: config sudoers
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
m_config_sudoers()
{
    local componentType="$1"
    log "INFO" "${LINENO}" "sudo: Begin to regist sudo file, componentType=$componentType"

    local agentSudoPath="${BIGDATA_SUDO_HOME}/sudo/runtime/sudoExecute.sh"

    cmdArray=("${agentSudoPath}")
    for cmd in "${cmdArray[@]}" ; do

        if [ -z "${cmd}" ] ; then
            return 1
        fi

        log "INFO" "${LINENO}" "sudo: Config the ${cmd} command to /etc/sudoers file."

        # 若/etc/sudoers里没有配置该命令，则配置
        grep "^${OMM_USER} ALL=(root) NOPASSWD: ${cmd}[[:space:]]*$" /etc/sudoers
        if [ $? -ne 0 ] ; then
            echo "${OMM_USER} ALL=(root) NOPASSWD: ${cmd}" >> /etc/sudoers
        fi

        # 若/etc/sudoers里没有配置该命令，则配置
        grep "^${BACKUP_USER} ALL=(root) NOPASSWD: ${cmd}[[:space:]]*$" /etc/sudoers >> "${logFile}" 2>&1
        if [ $? -ne 0 ] ; then
            echo "${BACKUP_USER} ALL=(root) NOPASSWD: ${cmd}" >> /etc/sudoers
        fi

        if [ $? -ne 0 ] ; then
            log "ERROR" "${LINENO}" "Failed to config the ${cmd} command to /etc/sudoers."
            return 1
        fi
    done

    return 0
}

##备份/etc/sssd/sssd.conf，升级时候使用
#######################################################
sssd_backupConfig_cpf()
{
    local sssd_conf="/etc/sssd/sssd.conf"
    local sssd_tmp_conf="$1"


    verify_path "${sssd_tmp_conf}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    cp -afp "${sssd_conf}" "${sssd_tmp_conf}"
    ret=$?
    if [ $ret -ne 0 ]; then
        echo "cp -afp ${sssd_conf} ${sssd_tmp_conf} failed"
        return 1
    fi

    return 0
}

################################################################################
# Function: 恢复/etc/sssd/sssd.conf
################################################################################
sssd_restoreConfig_cpf()
{
    local backup_dir="^${bigdata_home}/om-agent/configbackup"
    local redhat_sssd_conf="/etc/sssd/sssd.conf"
    local redhat_sssd_conf_bak="${backup_dir}/sssd.conf"

    #redhat
    echo "${1}"  | grep "${redhat_sssd_conf_bak}"  1>/dev/null
    if [ $? -eq 0 ] ; then
        if [ "${2}" = "${redhat_sssd_conf}" ]  ; then
            check_linkfile "${2}"
            if [ $? -ne 0 ]; then
                return 1
            fi

            /bin/cp -f  "${1}" "${2}"
            return $?
        fi
    fi

    return 0
}

check_linkfile()
{
    local file="${1}"

    if [ -L "${file}" ]; then
        return 1
    fi

    if [ -d "${file}" ] ; then
        return 0
    fi

    if [ -f "${file}" ]; then
        local inodeValue=$(ls -l "${file}" | awk '{print $2}')
        if [ ${inodeValue} -ge 2 ];then
            return 1
        fi
    fi

    return 0
}

################################################################################
# Function: agent_config_ntp
# Description: agent生成ntp配置文件
# Parameter:
#   input:
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function agent_config_ntp()
{
    local ntpFile="${1}"
    local g_controllerIP="${2}"
    verify_path "${ntpFile}" || return $?

    check_linkfile "${ntpFile}"
    if [ $? -ne 0 ]; then
        return 1
    fi
    if [ -z "$g_controllerIP" ]; then
        log "ERROR" "${LINENO}" "controllerIP is empty."
        return 1
    fi
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then
    cat <<EOF > "${ntpFile}"
server ${g_controllerIP} key 1 prefer burst iburst minpoll 4 maxpoll 4
restrict -4 default kod notrap nomodify nopeer noquery
restrict -6 default kod notrap nomodify nopeer noquery
restrict 127.0.0.1
restrict ::1
tinker step 0
tinker panic 0
disable monitor
disable kernel
enable auth
keys /etc/ntp.keys
trustedkey 1
requestkey 1
controlkey 1
EOF
    else
        cat <<EOF > "${ntpFile}"
server ${g_controllerIP} key 1 prefer burst iburst minpoll 4 maxpoll 4
restrict -4 default kod notrap nomodify nopeer noquery
restrict -6 default kod notrap nomodify nopeer noquery
restrict 127.0.0.1
restrict ::1
tinker step 0
tinker panic 0
disable monitor
disable kernel
enable auth
keys /etc/ntp/ntpkeys
trustedkey 1
requestkey 1
controlkey 1
EOF
    fi
    if [ $? -ne 0 ] ; then
        log "ERROR"  "${LINENO}" "Failed to modify ${ntpFile} file."
        return ${INS_CREATE_FILE_FAILURE}
    fi

    #check_linkfile check link type already
    chown "${OMM_USER}":"${WHEEL_GROUP}" "${ntpFile}"
    chmod 600 "${ntpFile}"
}

####################################################################
#  FUNCTION     : backup_ntp_sys_conf
#  DESCRIPTION  : 备份系统配置文件
#  INPUT        : $1: 源文件
#  OUTPUT       : NA
#  RETURN       : 0: success
####################################################################
backup_ntp_sys_conf()
{
    local ntp_conf_location="/etc/ntp.conf"
    local ntp_keys_location=""
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then
        ntp_keys_location="/etc/ntp.keys"
    else
        ntp_keys_location="/etc/ntp/ntpkeys"
    fi
    local backup_dir="${bigdata_home}/om-agent_?[A-Z0-9\.]*/configbackup/"
    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi

    echo "${2}"   | grep -E "^${backup_dir}" >/dev/null 2>&1
    if [ $? -eq 0 ] ;then
        if [ "${1}" = "${ntp_conf_location}" ] ; then
            /bin/cp -f "${1}" "${2}"
        return $?
        elif [ "${1}" = "${ntp_keys_location}" ] ; then
            /bin/cp -f "${1}" "${2}"
        return $?
        fi
    fi

    return 1
}

################################################################################
# Function: agent_config_chrony
# Description: agent生成chrony配置文件
# Parameter:
#   input:
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function agent_config_chrony()
{
    local ntpFile="${1}"
    local g_controllerIP="${2}"
    verify_path "${ntpFile}" || return $?

    check_linkfile "${ntpFile}"
    if [ $? -ne 0 ]; then
        return 1
    fi    
    if [ -z "$g_controllerIP" ]; then
        log "ERROR" "${LINENO}" "controllerIP is empty."
        return 1
    fi
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then
    cat <<EOF > "${ntpFile}"
server ${g_controllerIP}
driftfile /var/lib/chrony/drift
logdir /var/log/chrony
allow all
log measurements statistics tracking rtc refclocks tempcomp
keyfile /etc/chrony.keys
commandkey 1
EOF
    else
        cat <<EOF > "${ntpFile}"
server ${g_controllerIP}
driftfile /var/lib/chrony/drift
logdir /var/log/chrony
allow all
log measurements statistics tracking rtc refclocks tempcomp
keyfile /etc/chrony.keys
commandkey 1
EOF
    fi
    if [ $? -ne 0 ] ; then
        log "ERROR"  "${LINENO}" "Failed to modify ${ntpFile} file."
        return ${INS_CREATE_FILE_FAILURE}
    fi
    
    #check_linkfile check link type already
    chown "${OMM_USER}":"${WHEEL_GROUP}" "${ntpFile}"
    chmod 600 "${ntpFile}"
}

################################################################################
# Function: kill_ntp_process
# Description: 杀掉ntp监控进程
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
kill_ntp_process()
{
    log "INFO" "${LINENO}" "start to kill ntp process."
    kill -9 $(ps -ef | grep 'ntpMonitor' | grep -v grep | awk '{print $2}')
}

####################################################################
#  FUNCTION     : backup_chrony_sys_conf
#  DESCRIPTION  : 备份系统配置文件
#  INPUT        : $1: 源文件
#  OUTPUT       : NA
#  RETURN       : 0: success
####################################################################
backup_chrony_sys_conf()
{
    local ntp_conf_location="/etc/chrony.conf"
    local ntp_keys_location=""
    IS_SUSE_OS="$(cat /proc/version | grep "SUSE")"
    if [ -n "$IS_SUSE_OS" ]; then
        ntp_keys_location="/etc/chrony.keys"
    else
        ntp_keys_location="/etc/chrony.keys"
    fi

    if [ -n "$IS_SUSE_OS" ]; then
        ntp_keys_location2="/etc/ntp.keys"
    else
        ntp_keys_location2="/etc/ntp/ntpkeys"
    fi
    local backup_dir="${bigdata_home}/om-agent_?[A-Z0-9\.]*/configbackup/"
    local backup_dir2="${bigdata_home}/om-agent/configbackup/"
    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi 
   
    echo "${2}"   | grep -E "^${backup_dir}" >/dev/null 2>&1
    if [ $? -eq 0 ] ;then
        if [ "${1}" = "${ntp_conf_location}" ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        elif [ "${1}" = "${ntp_keys_location}" ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi
    echo "${2}"   | grep -E "^${backup_dir2}" >/dev/null 2>&1
    if [ $? -eq 0 ] ;then
        if [ "${1}" = "${ntp_conf_location}" ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        elif [ "${1}" = "${ntp_keys_location}" ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi
    
    local work_backup_dir="${BIGDATA_HOME}/om-server_?[A-Z0-9\.]*/OMS/workspace/conf"
    echo "${2}"   | grep -E "^${work_backup_dir}" >/dev/null 2>&1
    if [ $? -eq 0 ] ;then
        if [ "${1}" = "${ntp_keys_location2}" ] ; then
            /bin/cp -f "${1}" "${2}"
            chmod 600 "${2}"
            chown omm:wheel "${2}"
            return $?
        fi
    fi
    local work_backup_dir2="${BIGDATA_HOME}/om-server/OMS/workspace/conf"
    echo "${2}"   | grep -E "^${work_backup_dir2}" >/dev/null 2>&1
    if [ $? -eq 0 ] ;then
        if [ "${1}" = "${ntp_keys_location2}" ] ; then
            /bin/cp -f "${1}" "${2}"
            chmod 600 "${2}"
            chown omm:wheel "${2}"
            return $?
        fi
    fi

    return 1
}

################################################################################
# Function: ldap_copySrcConfFile
# Description: copy ConfFile ldap.conf  sssd.conf
# Parameter:
#   input:
#         ${1}: src  Conf File path
#         ${2}: dst  Conf File path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
ldapc_hmon_copySrcConfFile()
{
    local src1_location="/etc/ldap.conf"
    local src2_location="/etc/openldap/ldap.conf"
    local tmp_location="^${bigdata_home}/tmp/ldapclient/ldap.conf"
    
    check_linkfile "${2}"
    if [ $? -ne 0 ]; then
        return 1
    fi 

    if [ "${1}" = "${src1_location}" -o "${1}" = "${src2_location}" ] ; then
        echo "${2}" | grep ${tmp_location}   1>/dev/null
        if [  $? = 0    ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi

    local redhat_src1_locat_sssd="/etc/sssd/sssd.conf"
    local redhat_tmp_locat_sssd="^${bigdata_home}/tmp/ldapclient/sssd.conf"

    if [ "${1}" = "${redhat_src1_locat_sssd}"  ] ; then
      echo   "${2}" | grep ${redhat_tmp_locat_sssd}   1>/dev/null
        if [  $? = 0    ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi
    return 1
}

################################################################################
# Function: krbc_hmon_repairConfigFile
# Description: repair  ConfigFile
# Parameter:
#   input:
#         ${1}:Config  Parameter
#         ${2}:"/etc/krb5.conf"
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
krbc_hmon_repairConfigFile()
{
    local src_file="/etc/krb5.conf"
    local file_mode_num="644"
    local file_user="root"
    local file_group="root"
    local correct_file="^${bigdata_home}/.*/etc/"
 
    #the head of ${1} contains  ${correct_file}
    echo "${1}" | grep "${correct_file}"  1>/dev/null
    if [ $? = 0 ]  ;  then
        if [ "${2}" = "${src_file}"   ] ; then
            /bin/cp -f "${1}" "${2}"
            return $?
        fi
    fi
 
    if [ "${2}" = "${src_file}"  ] ; then
        if  [ "${1}" = "${file_mode_num}" ] ; then
            single_chmod   "${1}" "${2}"
            return $?
        elif [ "${1}" = "${file_user}:${file_group}" ] ; then
            single_chown   "${1}" "${2}"
            return $?
        fi
    fi
 
    return 1
}

##########################################################
#    Description: force to excute logrotate
#    Parameter:   logrotate config file
#    Return:      logrotate executing result
##########################################################
function om_func_forceLogrotate()
{
    declare -i rotateSize=10

    if [ ! -f "${1}" ];then
        return 1
    fi
    
    if [ "${1}" != "/etc/logrotate.d/okrbserver_kadmin" -a "${1}" != "/etc/logrotate.d/okrbserver_krb5kdc" -a "${1}" != "/etc/logrotate.d/krbserver_kadmind" -a "${1}" != "/etc/logrotate.d/krbserver_krb5kdc" ] ; then
        return 1
    fi

    #to get config rotate size
    om_func_getConfRotateLogSize "${1}"
    retSize=$?
    if [ ${retSize} -gt 0 ]; then
        rotateSize=${retSize}
    fi

    #if size of kerberos log file larger than logrotate conf compress size, compress it.
    if [ ${2} -lt ${rotateSize} ];then
        return 0
    fi

    /usr/sbin/logrotate -f "${1}" 1>/dev/null 2>&1

    return $?
}

##########################################################
#    Description: To get Log Begin Num
#    Parameter:   logrotate config file
#    Return:      0: represent no need to rotate logs
#                 1~20: represent the beging num
##########################################################
function om_func_getLogBeginNum()
{
    declare -i beginNum=0
    declare -i flag=0

    for k in $( seq 1 "$1" )
    do
        postfix=$(echo "$2" | awk -v i=$k '{print $i}' | awk -F '.' '{print $3}')
        num=$(echo "$2" | awk -v i=$k '{print $i}' | awk -F '.' '{print $3}')
        if [ "${postfix}" = "bz2" -o "${postfix}" = "gz" ];then
            flag=1
        else
            if [ ${num} -ge ${beginNum} ]; then
                beginNum=${num}
            fi
        fi
    done

    if [ ${flag} -eq 0 ]; then
        return  ${flag}
    else
        beginNum=${beginNum}+1
    fi

    return ${beginNum}
}

##########################################################
#    Description: force to excute logrotate
#    Parameter:   logrotate config file
#    Return:      logrotate executing result
##########################################################
function om_func_rotateCompressLogs()
{
    declare -i logZipNum=0
    declare -i oldVerAmount=0
    declare -i beginNum=1
    declare -i rotateNum=20

    local compressExt=".gz"

    local log_path_pattern="^${bigdata_log_home}/.*"

    echo "${1}"  | grep "${log_path_pattern}"   1>/dev/null
    if [ $? -ne 0 ] ; then
        return 1
    fi

    # 增加白名单校验
    if [ "${2}"!="krb5kdc.log" ] && [ "${2}"!="oms-krb5kdc.log" ] && [ "${2}"!="oms-kadmind.log" ] ; then
        return 1
    fi

    local logrotate_conf_pattern="^/etc/logrotate.d/.*"

    echo "${3}"  | grep "${logrotate_conf_pattern}"   1>/dev/null
    if [ $? -ne 0 ] ; then
        return 1
    fi

    return 0
}

#########################################################
#    Description: get config logrotate log compress ext
#    Parameter:   logrotate config file
#    Return:      logrotate log compress ext
##########################################################
function om_func_getConfCompressExt()
{
    local ext=".gz"

    if [ ! -f "$1" ];then
        echo ${ext}
    fi

    confExt=$(grep -i "compressext" "$1" | awk '{print $2}')
    if [ -n "${confExt}" ];then
        ext=${confExt}
    fi

    echo ${ext}
}

##########################################################
#    Description: get config logrotate log compress size
#    Parameter:   logrotate config file
#    Return:      logrotate log compress size
##########################################################
function om_func_getConfRotateLogSize()
{
    declare -i size=0

    if [ ! -f "$1" ];then
        return ${size}
    fi

    conSize=$(grep -i "size" "$1" | awk -F '=' '{print $2}'|sed 's/M//')
    if [ -n "${conSize}" ];then
        size=${conSize}
    fi

    return ${size}
}

################################################################################
# Description: create hard link
# Parameter:
#   input:
#   $1      source path
#   $2      target path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function mk_link_f()
{
    verify_path "$1" || return 1
    verify_path "$2" || return 1
 
    ln -f "$1" "$2"
    return $?
}

################################################################################
# Function: ldapc_install_configNscdServer
# Description:
# Parameter:
#   input:
#          ${1}:  src conf file  nscd.conf
#          ${2}:  dst conf file  nscd.conf
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
ldapc_install_configNscdServer()
{
    local nscd_conf="/etc/nscd.conf"
    local nscd_conf_tmp="/etc/nscd.conf.tmp"
    local nscd_conf_template="${BIGDATA_HOME}/tmp/nscd.conf.template"
    
    local enable_cache_key="enable-cache"
    
    local passwd_key="passwd"
    local passwd_line=""
    local replace_passwd_string="enable-cache        passwd      yes"
    
    local group_key="group"
    local group_line=""
    local replace_group_string="enable-cache        group      yes"
    
  
    local positive_live_time_key="positive-time-to-live"
    local group_live_time_line=""
    local replace_group_live_time_string="positive-time-to-live      group      180"
    
    local passwd_live_time_line=""
    local replace_passwd_live_time_string="positive-time-to-live      passwd      180"
 
    rm -f "${nscd_conf_tmp}"
    
    /bin/cp -f "${nscd_conf_template}" "${nscd_conf_tmp}"
    if [ $? -ne 0 ];then        
        return $RET_ERR
    fi        
    
    # find replace line
    passwd_line=$(grep -wn "${enable_cache_key}" "${nscd_conf_tmp}" | grep -w "${passwd_key}" | grep -v "#" | awk -F ":" '{print $1}' )
    if [ -z "${passwd_line}" ];then
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "can not find passwd_key ,["${passwd_key}"]"
        return $RET_ERR
    fi 
    
    group_line=$(grep -wn "${enable_cache_key}" "${nscd_conf_tmp}" | grep -w "${group_key}" | grep -v "#" | awk -F ":" '{print $1}' )
    if [ -z "${group_line}" ];then
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "can not find group_key ,["${group_key}"]"
        return $RET_ERR
    fi      
    
   
    group_live_time_line=$(grep -wn "${positive_live_time_key}" "${nscd_conf_tmp}" | grep -w "${group_key}" | grep -v "#" | awk -F ":" '{print $1}' )
    if [ -z "${group_live_time_line}" ];then
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "["${group_key}"] is not exist"
        return $RET_ERR
    fi
    
    passwd_live_time_line=$(grep -wn "${positive_live_time_key}" "${nscd_conf_tmp}" | grep -w "${passwd_key}" | grep -v "#" | awk -F ":" '{print $1}' )
    if [ -z "${passwd_live_time_line}" ];then
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "["${passwd_key}"] is not exist"
        return $RET_ERR
    fi
    
    # replace string 
    sed -i "${passwd_line}c ${replace_passwd_string}" "${nscd_conf_tmp}"
    if [ $? -ne 0 ];then   
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "sed -i s#${passwd_line}#${replace_passwd_string}#g ${nscd_conf_tmp} failed" 
        return $RET_ERR
    fi    
    
    sed -i "${group_line}c ${replace_group_string}" "${nscd_conf_tmp}"  
    if [ $? -ne 0 ];then    
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "sed -i s#${group_line}#${replace_group_string}#g ${nscd_conf_tmp} failed" 
        return $RET_ERR
    fi   
       
    sed -i "${group_live_time_line}c ${replace_group_live_time_string}" "${nscd_conf_tmp}"      
    if [ $? -ne 0 ];then 
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "sed -i s#${group_live_time_line}#${replace_group_live_time_string}#g ${nscd_conf_tmp} failed" 
        return $RET_ERR
    fi
     
    sed -i "${passwd_live_time_line}c ${replace_passwd_live_time_string}" "${nscd_conf_tmp}"  
    if [ $? -ne 0 ];then 
        log  "${LINENO}"   "$DETAIL_LOG_FILE_PATH" "${LAUNCHROLE}" "${ERR}" "sed -i s#${passwd_live_time_line}#${replace_passwd_live_time_string}#g ${nscd_conf_tmp} failed" 
        return $RET_ERR
    fi
    
    /bin/mv -f  "${nscd_conf_tmp}" "${nscd_conf}" 
    return $?
}

function setup_container_executor(){
    local containerExecutor="${1}"
    local nmLocalDirs="${2}"
    local nmLogDirs="${3}"
    local -i version="${4}"
    local -i gpuMiniVersion=651
    local containerCheckPath="^${bigdata_home}/.*/bin/container-executor"
    
    isValidMRSDirConfs "${nmLocalDirs}"
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    isValidMRSDirConfs "${nmLogDirs}"
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    echo ${containerExecutor} | grep ${containerCheckPath}  1>/dev/null
    if [ $? -ne 0 ]  ;  then
        log "ERROR" "${LINENO}" "${containerExecutor} is illegal."
        return 1
    fi

    if [ ! -f "${containerExecutor}" ]; then
        log "ERROR" "${LINENO}" "${containerExecutor} does not exist."
        return 1
    fi

    # change permission for container-executor
    single_chown root:wheel "${containerExecutor}"
    single_chmod 6150 "${containerExecutor}"

    # create container-executor.cfg
    local containerExecutorConf="/etc/hadoop/conf/container-executor.cfg"
    /bin/mkdir -p /etc/hadoop/conf
    /bin/rm -f "$containerExecutorConf"
    /bin/touch "$containerExecutorConf"

    high_os_version_check
    local os_version=$?
    local CGROUP_ROOT_PATH
    if [ -z "$CGROUP_ROOT_PATH" ]; then
        if [ "$os_version" == 0 ]; then
            CGROUP_ROOT_PATH="/dev/cgroup"
        else
            CGROUP_ROOT_PATH="/sys/fs/cgroup"
        fi
    fi
    
    /bin/echo "yarn.nodemanager.linux-container-executor.group=wheel
banned.users=banneduser
min.user.id=1000
allowed.system.users=omm
yarn.nodemanager.local-dirs=${nmLocalDirs}
yarn.nodemanager.log-dirs=${nmLogDirs}" >> "$containerExecutorConf"
    if [ $version -ge $gpuMiniVersion ]; then
        /bin/echo "
[cgroups]
# This should be same as yarn.nodemanager.linux-container-executor.cgroups.mount-path inside yarn-site.xml
root=$CGROUP_ROOT_PATH
# This should be same as yarn.nodemanager.linux-container-executor.cgroups.hierarchy inside yarn-site.xml
yarn-hierarchy=/hadoop-yarn

[gpu]" >> "$containerExecutorConf"

        type nvidia-smi >/dev/null 2>&1
        if [ $? -eq 0 ];then
            /bin/echo "module.enabled=true

[docker]
module.enabled=true
docker.allowed.volume-drivers=nvidia-docker
docker.allowed.rw-mounts=${bigdata_data_home}/hadoop/
docker.allowed.capabilities=SYS_CHROOT,MKNOD,SETFCAP,SETPCAP,FSETID,CHOWN,AUDIT_WRITE,SETGID,NET_RAW,FOWNER,SETUID,DAC_OVERRIDE,KILL,NET_BIND_SERVICE
docker.allowed.networks=bridge,host,none
docker.binary=/usr/bin/docker
docker.privileged-containers.enabled=false
docker.trusted.registries=127.0.0.1:5000" >> "$containerExecutorConf"

            nvidia_devices=$(ls /dev/nvidia*)
            nvidia_devices=$(echo $nvidia_devices)
            nvidia_devices=$(echo ${nvidia_devices// /,})
            /bin/echo "docker.allowed.devices=$nvidia_devices" >> "$containerExecutorConf"

            nvidia_version=$(nvidia-smi | grep "Driver Version" | awk -F" " '{ print $6 }' )
            /bin/echo "docker.allowed.ro-mounts=nvidia_driver_$nvidia_version,${bigdata_data_home}/hadoop/" >> "$containerExecutorConf"
        else
            /bin/echo "module.enabled=false

[docker]
module.enabled=true
docker.allowed.ro-mounts=${bigdata_data_home}/hadoop/
docker.allowed.rw-mounts=${bigdata_data_home}/hadoop/
docker.allowed.capabilities=SYS_CHROOT,MKNOD,SETFCAP,SETPCAP,FSETID,CHOWN,AUDIT_WRITE,SETGID,NET_RAW,FOWNER,SETUID,DAC_OVERRIDE,KILL,NET_BIND_SERVICE
docker.allowed.networks=bridge,host,none
docker.binary=/usr/bin/docker
docker.privileged-containers.enabled=false
docker.trusted.registries=127.0.0.1:5000" >> "$containerExecutorConf"
        fi
    fi
    single_chown root:wheel "$containerExecutorConf"
    single_chmod 400 "$containerExecutorConf"
}

get_pg_search_passwd()
{
    local pg_passwd=""
    pg_passwd=$(cat /etc/sssd/sssd.conf |grep ldap_default_authtok)
    pg_passwd=$(echo ${pg_passwd#*=} | sed 's/^[ \t]*//g')
    echo $pg_passwd
}

main()
{
    local function_name="$1"
    shift
    if echo "${g_fuctionArray[@]}" | grep -w "${function_name}" &>/dev/null; then
       "$function_name" "$@"
    else
       echo "Fuction $function_name is not exist in $(basename $0)."
       return 1
    fi
}

main "$@"
exit $?