#!/bin/bash
# --------------------------  variables  ------------------------------------- #


#----------------global variable, control outprint style-----------------------#
CURRENT_PATH=""
LOG_FILE=""
UPD_PREINSTALL_CFG=""
MANAGE_IP_LIST=""
AGENT_PRESET_PATH="/opt/preset"
ROOT_PASSWD=""
G_REMOTE_SCRIPT=""
GROUP_WHEEL="wheel"

# -------------------------------  retcode  ---------------------------------- #
declare -ri RET_OK=0                            # operate successfully
declare -ri RET_FAIL=1                          # operate failed
declare -ri INPUT_PARAM_ERROR=2                 # received invalid param
declare g_upgrade_type="commit"  #commit,rollback
declare src_version_is812="false"

# ----------------------------  functions  ----------------------------------- #

# **************************************************************************** #
# Function Name: get_cur_path
# Description: get current script path
# Parameter: none
# Return:
# **************************************************************************** #
function get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    local curPath="${PWD}"
    local g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    echo "$curPath"
    cd - > /dev/null
}

# **************************************************************************** #
# Function Name: PrintHelp
# Description: print help ussage
# Parameter: none
# Return:
# **************************************************************************** #
function PrintHelp()
{
    echo "Ussage:"
    echo -e "\t-p -u -n: perform preinstall before upgrade."
    echo -e "\t-p -e -n: perform preinstall after commit upgrade task."
    echo -e "\t-p -r -n: perform preinstall after rollback."
    echo -e "\t-s [preset|postset]: perform preset or postset before upgrade."
}

# **************************************************************************** #
# Function Name: init_log
# Description: initialize log module
# Parameter: none
# Return:
# **************************************************************************** #
function init_log()
{
    G_LOG_FILE="${LOG_FILE}"
    mkdir -p "$(dirname ${G_LOG_FILE})"
    touch "${G_LOG_FILE}"
    chown  -R "$OMM_USER":"$GROUP_WHEEL" "$(dirname ${G_LOG_FILE})"
    chmod 600 "${G_LOG_FILE}"
    chmod 700 "$(dirname ${G_LOG_FILE})"
}

##############################################################
##@Usage get_node_list
## @Return
## @Description 获取数据库中的节点IP列表
##############################################################
function init_node_list()
{
    #数据库用户名密码
    local cipher_username="$(cat ${CONTROLLER_CONF_HOME}/controller.properties | egrep -w '^database.username' | cut -d '=' -f2-)"
    [ -z "${cipher_username}" ] && error_exit "Get cipher username failed."
    DATABASE_USERNAME=$(decrypt_ciphertext ${cipher_username} AES256_CBC_IGNORE_KEY) || return $?

    local cipher_password="$(cat ${CONTROLLER_CONF_HOME}/controller.properties | egrep -w '^database.password' | cut -d '=' -f2-)"
    [ -z "${cipher_password}" ] && error_exit "Get cipher password failed."
    DATABASE_PASSWORD=$(decrypt_ciphertext ${cipher_password} AES256_CBC_IGNORE_KEY) || return $?
    
    if [ $options == "p" ] ; then
        local cpuType=$(cat ${CURRENT_PATH}/.local_cpu_type)
        #判断是否存在cpu字段
        has_cpu=$(gsql -U "${DATABASE_USERNAME}" -p "${PG_PORT}"  -d "${OMM_DB_NAME}" -W "${DATABASE_PASSWORD}" -t -c "\d om_nodes" | grep -i CPUARCHITECTURE)
        if [ -z "${has_cpu}" ] ; then
            get_nodes_sql="select ip from om_nodes;"
        else
            get_nodes_sql="select ip from om_nodes where CPUARCHITECTURE='${cpuType}';"
        fi
    fi
    
    if [ $options == "s" ] ; then
        get_nodes_sql="select ip from om_nodes;"
    fi
    
    #数据库中获取节点列表
    MANAGE_IP_LIST=( $(gsql -U "${DATABASE_USERNAME}" -p "${PG_PORT}"  -d "${OMM_DB_NAME}" -W "${DATABASE_PASSWORD}" -h 127.0.0.1 -c "$get_nodes_sql" | grep -vwE "rows|row" | sed 's/^[[:space:]]*//g;s/[[:space:]]*$//g;1,2d;/^$/d') )
    
    if [ -z "$MANAGE_IP_LIST" ] ; then
        showLog_error ${LINENO} "Failed to get nodes list,node list is empty,please check if the FusionInsight_SetupTool package is correct."
        showLog_error ${LINENO} "This tool is applicable to $cpuType,maybe there's no nodes of $cpuType type in the cluster,check and Re-download the correct package please."
        return 1
    fi
}


# **************************************************************************** #
# Function Name: get_root_password
# Description: get root password from user
# Parameter: none
# Return:
# **************************************************************************** #
get_root_password()
{
    #如果密码文件存在,则从此文件中获取每个节点的密码，此文件格式为
    # ip1 password1
    # ip2 password2
    if [ -f "$curPath/password.ini" ]; then
        return 0
    fi
    
    # 输入root用户密码
    stty -echo
    read -p "Please input root password for agent nodes:" -r -s -t 60 ROOT_PASSWD
    echo -e ""
    if [ ${#ROOT_PASSWD} -eq 0 ]; then
       echo "Root password cannot be empty."
       stty echo
       return 1
    fi
    stty echo
    export ROOT_PASSWD
}

# **************************************************************************** #
# Function Name: init_env_variable
# Description: initialize some env variable
# Parameter: none
# Return:
# **************************************************************************** #
init_env_variable()
{
    source /home/${OMM_USER}/.profile || return 1

    UPD_PREINSTALL_CFG="${CURRENT_PATH}/etc/preinstall.ini"
    LOG_FILE="$(echo ${BIGDATA_LOG_HOME})/upgrade/setup_tool.log_$(date +%F)"
    G_REMOTE_SCRIPT="${CURRENT_PATH}/../preinstall/tools/cluster/lib/remote.sh"

    init_log
}


##############################################################
##@Usage check_user_config
## @Return
## @Description 判断用户是否已经填写 etc/preinstall.ini 中的挂载配置项
##############################################################
check_user_config()
{
    if [ ! -f "${UPD_PREINSTALL_CFG}"    ] ;then
        showLog_error ${LINENO} "${UPD_PREINSTALL_CFG} does not exist."
        return 1
    fi

    #校验配置的挂载光盘项目是否合法
    source  "${UPD_PREINSTALL_CFG}"
    g_pkgs_dir="$(echo "$g_pkgs_dir" | tr -d '[:space:]' | tr -s ';' | tr -s ':' | sed 's/^;//g' | sed 's/;$//g')"
    if [ "${g_pkgs_dir}" == "" ]; then
        showLog_error ${LINENO} "The file ${UPD_PREINSTALL_CFG} configuration items g_pkgs_dir can not be empty."
        return 1
    fi

    return 0
}

##############################################################
##@Usage construct_preinstll_ini
## @Return
## @Description 构造preinstall可用的配置文件
##############################################################
function construct_preinstll_ini()
{
    if [ "${G_NODE_LIST}" == ""  ] ;then
        log_error ${LINENO} " node list is null."
        return 1
    fi

    sed -i '/g_host/d'  "${UPD_PREINSTALL_CFG}"> /dev/null
    sed -i '/g_parted/d' "${UPD_PREINSTALL_CFG}" > /dev/null
    sed -i '/g_add_pkg=/d' "${UPD_PREINSTALL_CFG}" > /dev/null
    sed -i '/g_swap_off=/d' "${UPD_PREINSTALL_CFG}" > /dev/null
    sed -i '/g_hostname_conf=/d' "${UPD_PREINSTALL_CFG}" > /dev/null


    grep 'g_parted' "${UPD_PREINSTALL_CFG}" > /dev/null
    if [ $? -eq 0 ]; then
        log_error ${LINENO} "file parameter error, please delete g_parted=2 in ${UPD_PREINSTALL_CFG} and retry."
        return 1
    fi

    echo "g_hosts=${G_NODE_LIST}" >> "${UPD_PREINSTALL_CFG}"
    echo "g_parted=0" >> "${UPD_PREINSTALL_CFG}"
    echo "g_add_pkg=1" >> "${UPD_PREINSTALL_CFG}"
    echo "g_swap_off=0" >> "${UPD_PREINSTALL_CFG}"
    echo "g_hostname_conf=\"\"" >> "${UPD_PREINSTALL_CFG}"
    echo "g_platform=\"upgrade\"" >> "${UPD_PREINSTALL_CFG}"

    return 0
}

# **************************************************************************** #
# Function Name: do_preinstall
# Description: execute preinstall
# Parameter: none
# Return:
# **************************************************************************** #
function do_preinstall()
{
    init_node_list || return 1
    
    local G_DST_PREINSTALL_INI_FILE="${CURRENT_PATH}/../preinstall"
    check_user_config || return 1

    #生成节点列表
    for agent_ip in "${MANAGE_IP_LIST[@]}" ;
    do
        if [ "${G_NODE_LIST}" == "" ];then
            G_NODE_LIST="${agent_ip}"
        else
            G_NODE_LIST="${G_NODE_LIST},${agent_ip}"
        fi
    done

    construct_preinstll_ini || return 1

    /bin/cp  -rp "${UPD_PREINSTALL_CFG}" "${G_DST_PREINSTALL_INI_FILE}"
    export LD_LIBRARY_PATH=/usr/lib64:$LD_LIBRARY_PATH
    ldconfig
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "ldconfig command execute error, May cause openssl to be unavailable."
    fi
    bash  "${G_DST_PREINSTALL_INI_FILE}/preinstall.sh" "$@"

    log_info "$LINENO"  "preinstall adapter end."
    return 0
}

##############################################################
## @Usage scp_preset
## @Return
## @Description 拷贝Preset到其他节点
##############################################################
scp_preset()
{
    local hostIP="$1"
    
    tgtPath="$(dirname ${AGENT_PRESET_PATH})"
    exec_remote_cmd 180 "${hostIP}" "rm -rf ${tgtPath}/${dir_name}" >> ${G_LOG_FILE} 2>&1
    upload_file_to_remote_check "$temp_tgz_pkg" "$tmpDir" "${hostIP}"  "$checkFile"|| return 1
    exec_remote_cmd 180 "${hostIP}" "tar -xzf $temp_tgz_pkg -C ${tgtPath}/" >> ${G_LOG_FILE} 2>&1 || return 1
    
    #清理临时文件，失败不退出，只是打印日志
    exec_remote_cmd 30 "${hostIP}" "rm -rf $temp_tgz_pkg" >> ${G_LOG_FILE} 2>&1
}

##############################################################
## @Usage do_preset
## @Return
## @Description 在某个节点上执行Preset
##############################################################
do_preset()
{
    #MRS发放集群场景下补齐确实的少量rpm包
    g_controller_home=$(su - omm -c 'echo $CONTROLLER_HOME')

    KNOX_HOME="/opt/knox"
    FI_PROP_FILE="$g_controller_home/etc/om/fi.properties"
    local os_version=""
    if [ -f /etc/euleros-release ]; then
        os_version=$(cat /etc/euleros-release | tr -d '()' | awk '{print $4}' | tr [A-Z] [a-z])
    fi

    if [ -f /sbin/ifconfig ]; then
        IFCONFIG="/sbin/ifconfig"
    else
        IFCONFIG="/usr/bin/ifconfig"
    fi
    ${IFCONFIG} | grep -w -F "$1" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        /bin/bash ${CURRENT_PATH}/../preset/preset.sh "upgrade" "${src_version_is812}" || return 1
        if [ -d ${KNOX_HOME} -a ! -f ${FI_PROP_FILE} ] && [ "${os_version}" == "sp5" -o "${os_version}" == "sp8" ]; then
            /bin/bash ${CURRENT_PATH}/../preset/add_pkgs_for_mrs.sh || return 1
        fi
        return 0
    fi

    # 拷贝preset到其他路径
    scp_preset "$1"
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "Failed to scp preset on $1."
        return 1
    fi

    exec_remote_cmd 20 "$1" "[ -x ${AGENT_PRESET_PATH}/preset.sh ]" >> ${G_LOG_FILE} 2>&1
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "${AGENT_PRESET_PATH}/preset.sh is not exist on $1."
        return 1
    fi
    exec_remote_cmd 20 "$1" "/bin/bash ${AGENT_PRESET_PATH}/preset.sh upgrade ${src_version_is812}" >> ${G_LOG_FILE} 2>&1
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "Connect $1 to perform preset failed."
        return 1
    fi

    #MRS集群没有预置环境变量，通过版本号和6.5.1.1补丁新增的文件联合判断
    if [ -d ${KNOX_HOME} -a ! -f ${FI_PROP_FILE} ]; then
        exec_remote_cmd 60 "$1" "/bin/bash ${AGENT_PRESET_PATH}/add_pkgs_for_mrs.sh" >> ${G_LOG_FILE} 2>&1
        if [ $? -ne 0 ]; then
            log_error "$LINENO" "Connect $1 to add rpms failed."
            return 1
        fi
    fi
}

##############################################################
## @Usage compress_preset
## @Return
## @Description 压缩preset相关脚本
##############################################################
compress_preset()
{
    tmpDir="/tmp"
    srcPath="${CURRENT_PATH}/../preset"

    if [ ! -d "$srcPath" ]; then
        log_error "${LINENO}" "$srcPath is not a directory."
        return 1
    fi
    
    dir_name="$(basename ${CURRENT_PATH}/../preset)"
    temp_tgz_pkg="$tmpDir/${dir_name}_temp.tar.gz"
    tar -C "$(dirname $srcPath)" -czf "$temp_tgz_pkg" "$dir_name"  >> ${G_LOG_FILE} 2>&1
    if [ $? -ne 0 ]; then
        log_error "${LINENO}" "compress $srcPath failed."
        rm -rf "$temp_tgz_pkg"
        cd - >/dev/null 2>&1
        return 1
    fi
    ##create the check sha256 file
    checkFile="$temp_tgz_pkg.sha256.temp"
    /bin/rm -rf $checkFile
    sha256sum  $temp_tgz_pkg > $checkFile
    if [ $? -ne 0 ] ; then
        log_error "${LINENO}" "Failed to create the check file[$checkFile]."
        return 1
    fi
}

##############################################################
## @Usage do_preset_on_all_nodes
## @Return
## @Description 执行preset在所有节点
##############################################################
do_preset_on_all_nodes()
{
    ##compress preset
    compress_preset
    if [ $? -ne 0 ]; then
        log_error "${LINENO}" "Failed to compress preset."
        return 1
    fi

    if [ -n "${MANAGE_IP_LIST}" ] ; then
        echo -e "The task is in progress, It may take several minutes. Please wait..."
        concurrent_exec "${MANAGE_IP_LIST[*]}" do_preset "null" ${G_MAX_PARALLEL_TASK}
        if [ $? -ne 0 ] ; then
            showLog_error "$LINENO" "Do preset on all nodes failed. Failed ip list: $G_FAILED_TASK_LIST."
            return 1
        fi
        showLog_info "$LINENO" "Succeed to do preset operation on all agent nodes."
    else
        log_info "$LINENO" "No host information database."
    fi
}

##############################################################
## @Usage do_postset
## @Return
## @Description 在某个节点上执行postSet
##############################################################
do_postset()
{
    if [ -f /sbin/ifconfig ]; then
        IFCONFIG="/sbin/ifconfig"
    else
        IFCONFIG="/usr/bin/ifconfig"
    fi
    ${IFCONFIG} | grep -w -F "$1" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        /bin/bash ${CURRENT_PATH}/../preset/postset.sh "${g_upgrade_type}" "${src_version_is812}"
        return $?
    fi
    
    # 拷贝preset到其他路径
    scp_preset "$1"
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "Failed to scp preset on $1."
        return 1
    fi
    
    exec_remote_cmd 20 "$1" "[ -x ${AGENT_PRESET_PATH}/postset.sh ]" >> ${G_LOG_FILE} 2>&1
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "${AGENT_PRESET_PATH}/postset.sh is not exist on $1."
        return 1
    fi

    exec_remote_cmd 20 "$1" "/bin/bash ${AGENT_PRESET_PATH}/postset.sh ${g_upgrade_type} ${src_version_is812}" >> ${G_LOG_FILE} 2>&1
    if [ $? -ne 0 ]; then
        log_error "$LINENO" "Connect $1 to perform postset failed."
        return 1
    fi

    exec_remote_cmd 20 "$1" "rm -rf ${AGENT_PRESET_PATH}" >> ${G_LOG_FILE} 2>&1
}

##############################################################
## @Usage do_postset_on_all_nodes
## @Return
## @Description 到各个节点上执行postset
##############################################################
do_postset_on_all_nodes()
{
    ##compress preset
    compress_preset
    if [ $? -ne 0 ]; then
        log_error "${LINENO}" "Failed to compress preset."
        return 1
    fi
    
    if [ -n "${MANAGE_IP_LIST}" ] ; then
        echo -e "The task is in progress, It may take several  minutes. Please wait..."
        concurrent_exec "${MANAGE_IP_LIST[*]}" do_postset "null" ${G_MAX_PARALLEL_TASK}
        if [ $? -ne 0 ] ; then
            showLog_error "$LINENO" "Do postset on all nodes failed. Failed ip list: $G_FAILED_TASK_LIST.Please view log ${G_LOG_FILE}. "
            return 1
        fi
        showLog_info "$LINENO" "Succeed to do postset operation on all agent nodes."
    else
        log_info "$LINENO" "No host information database."
    fi
}

init_postset_upgrade()
{
    local version=$(cat "${BIGDATA_HOME}/om-server/om/etc/om/VERSION" | tr -d [:space:])
    local updsversion=""
    if [ -f "${BIGDATA_HOME}/update-service/conf/upd.cfg" ];then
        updsversion=$(grep "SRC_VERSION=" ${BIGDATA_HOME}/update-service/conf/upd.cfg |awk -F"=" '{print $2}' | tr -d [:space:])
        #8.1.2版本开始目录结构发生变化
        if [ "${updsversion}" \< "8.1.2" ] ;then
            src_version_is812="false"
        else
            src_version_is812="true"
        fi
    fi
    if [ X"${version}" = X"${updsversion}" ];then
        g_upgrade_type="rollback"
    elif [ -z "${updsversion}" ];then
        g_upgrade_type=""
        src_version_is812=""
    fi
}

init_preset_upgrade()
{
    if [ ! -f "${BIGDATA_HOME}/om-server/om/etc/om/VERSION" ];then
        #omm用户扩容，非升级环境
        src_version_is812=""
        return 0
    fi
    
    local version=$(cat "${BIGDATA_HOME}/om-server/om/etc/om/VERSION" | tr -d [:space:])
    #8.1.2版本开始目录结构发生变化
    if [ "${version}" \< "8.1.2" ] ;then
        src_version_is812="false"
    else
        src_version_is812="true"
    fi
}

# **************************************************************************** #
# Function Name: perform_preset
# Description: execute preset or postset when upgrade
# Parameter: none
# Return:
# **************************************************************************** #
function perform_preset()
{
    init_node_list || return 1

    local opType="$1"

    if [ "$opType" == "postset" ]; then
        get_root_password || return 1
        init_postset_upgrade
        do_postset_on_all_nodes || return 1
    elif [ "$opType" == "preset" ]; then
        get_root_password || return 1
        init_preset_upgrade
        do_preset_on_all_nodes || return 1
    else
        showLog_error "$LINENO" "Received unrecognized option $opType."
        PrintHelp
    fi
}

function main()
{
    #when no input param print help msg
    if [ $# -eq 0 ]; then
        PrintHelp
        return $?
    fi

    #limit only omm user can execute
    if [ "$(whoami)" != "${ROOT_USER}" ] ; then
        echo  "Error: Only the root user can execute this command."
        exit $RET_FAIL
    fi

    init_env_variable

    while getopts "p:s:" options 2>/dev/null
    do
        case $options in
            p)
                shift
                do_preinstall "$@"
                exit $?
                ;;
            s)
                shift
                perform_preset "$@"
                exit $?
                ;;
            ?)
                showLog_error "$LINENO" "Error: Received unrecognized option $1."
                PrintHelp
                exit $INPUT_PARAM_ERROR
                ;;
        esac
    done
}

# set local current path
CURRENT_PATH=$(get_cur_path)

. "${CURRENT_PATH}/func/constants.sh"  || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${CURRENT_PATH}/func/constants.sh."; exit 1; }
. "${CURRENT_PATH}/func/functions.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${CURRENT_PATH}/func/functions.sh."; exit 1; }
. "${CURRENT_PATH}/func/log.sh"  || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${CURRENT_PATH}/func/log.sh."; exit 1; }

main "$@"
exit $?
