/*******************************************************************************
* Copyright @ Huawei Technologies Co., Ltd. 1998-2014. All rights reserved.
* File name: WSEC_ErrorCode.h
* Decription: 错误码
*********************************************************************************/
#ifndef SC_ERRCODE_H
#define SC_ERRCODE_H

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
    SEC_SUCCESS                                           =0,    /* 成功 */
    SEC_FAILURE                                           =1,   /* 通用错误 */

/* 文件操作错误 */
    SEC_ERR_OPEN_FILE_FAIL                                =11,   /* 打开文件失败 */
    SEC_ERR_READ_FILE_FAIL                                =12,   /* 读文件失败 */
    SEC_ERR_WRI_FILE_FAIL                                 =13,   /* 写文件失败 */
    SEC_ERR_GET_FILE_LEN_FAIL                             =14,   /* 获取文件长度失败 */
    SEC_ERR_FILE_FORMAT                                   =15,   /* 文件格式错误 */
    SEC_ERR_FILE_COPY_FAIL                                =16,   /* 文件拷贝失败 */


/* 内存操作错误 */
    SEC_ERR_MALLOC_FAIL                                   =51,  /* 内存分配失败 */
    SEC_ERR_MEMCPY_FAIL                                   =52,  /* 内存拷贝失败 */
    SEC_ERR_MEMCLONE_FAIL                                 =53,  /* 内存克隆失败 */
    SEC_ERR_STRCPY_FAIL                                   =54,  /* 字符串拷贝失败 */
    SEC_ERR_OPER_ARRAY_FAIL                               =55,  /* 数组操作失败 */


/* 组件公共内部错误 */
    SEC_ERR_NO_INIT                                       =1001,   /* 未初始化 */
    SEC_ERR_REPEAT_INIT                                   =1002,   /* 重复初始化 */
    SEC_ERR_INIT_SYS_LOG_FAIL                             =1003,   /* 初始化系统日志失败 */
    SEC_ERR_INIT_LOG_FAIL                                 =1004,   /* 初始化日志失败 */
    SEC_ERR_INIT_CFG_FAIL                                 =1005,   /* 初始化配置文件失败 */
    SEC_ERR_INVALID_INPUT                                 =1006,   /* 无效输入 */
    SEC_ERR_REPEAT_RELEASE_LOG                            =1007,   /* 重复调用释放日志 */
    SEC_ERR_BASE64_ENCODE                                 =1008,   /* BASE64编码失败 */
    SEC_ERR_BASE64_DECODE                                 =1009,   /* 无效的BASE64数据，解码失败 */

/* SecCrypt组件内部错误 */
    SEC_ERR_INIT_KMC_FAIL                                 =2001,   /* 初始化KMC失败 */
    SEC_ERR_INIT_DOMAIN_FAIL                              =2002,   /* 初始化扩展的domain失败 */
    SEC_ERR_INIT_KMC_HARD_FAIL                            =2003,   /* 初始化硬件根实例失败 */
    SEC_ERR_RELEASE_KMC_HARD_FAIL                         =2004,   /* 释放硬件根实例失败 */
    SEC_ERR_COMMON_KMC_HARD_FAIL                          =2005,   /* kmc硬件根实例操作失败 */
    SEC_DOMAIN_KEY_TOO_LONG_FAIL                          =2006,   /* domain key 超过1M */

/* Pwdpolicy组件内部错误 */
    SEC_ERR_INIT_WEAKPWD_FAIL                             =3001,   /* 初始化弱口令字典失败 */
    SEC_ERR_INIT_TIMER_FAIL                               =3002,   /* 初始化更新Cache定时器失败 */
    SEC_ERR_INIT_MONITOR_FAIL                             =3003,   /* 初始化文件变更跟踪事件失败 */
    SEC_ERR_INIT_DATA_FAIL                                =3004,   /* 初始化数据失败 */
    SEC_ERR_INVALID_POLICY                                =3005,   /* 无效的策略 */
    SEC_ERR_INVALID_PWDLEN                                =3006,   /* 无效的密码长度 */
    SEC_ERR_INVALID_ARRAY_SIZE                            =3007,   /* 无效的数组长度 */

/* CertSign组件内部错误 */
    CSC_ERR_INVALID_KEY_ALG_ID                            =4001,   /* 无效的Key算法ID */
    CSC_ERR_INVALID_SUBJECT                               =4002,   /* 无效的subject */
    CSC_ERR_INVALID_CERT_FORMAT                           =4003,   /* 无效的证书格式*/
    CSC_ERR_INVALID_CA_KEY                                =4004,   /* 无效的CA key对象*/
    CSC_ERR_INVALID_PUBLIC_KEY                            =4005,   /* 无效的公钥*/
    CSC_ERR_INVALID_PUBLIC_KEY_PKCS1                      =4006,   /* 无效的PKCS1格式公钥，格式BEGIN RSA PUBLIC KEY*/
    CSC_ERR_INVALID_PUBLIC_KEY_PKCS8                      =4007,   /* 无效的PKCS8格式公钥，格式BEGIN PUBLIC KEY*/
    CSC_ERR_INVALID_PRIVATE_KEY                           =4008,   /* 无效的私钥*/
    CSC_ERR_INVALID_PRIVATE_KEY_PKCS1                     =4009,   /* 无效的PKCS1格式私钥，格式BEGIN RSA PRIVATE KEY*/
    CSC_ERR_INVALID_PRIVATE_KEY_PKCS8                     =4010,   /* 无效的PKCS8格式私钥，格式BEGIN PRIVATE KEY*/
    CSC_ERR_INVALID_CERT                                  =4011,   /* 无效的证书*/
    CSC_ERR_INVALID_CSR                                   =4012,   /* 无效的证书请求*/
    CSC_ERR_INVALID_EXTENTION                             =4013,   /* 无效的证书Extention */
    CSC_ERR_INVALID_CERT_SIGN_INFO                        =4014,   /* 无效的证书签发信息参数 */
    CSC_ERR_INVALID_CRL_SIGN_INFO                         =4015,   /* 无效的证书CRL签发信息参数 */
    CSC_ERR_INVALID_CRL_REVOKE_CERTS                      =4016,   /* 无效的证书CRL吊销证书列表 */
    CSC_ERR_INVALID_CERT_KEY_INFO                         =4017,   /* 无效的证书公私钥信息 */
    CSC_ERR_INVALID_ISSUE_INFO                            =4018,   /* 无效的签发者信息,外部签发对象和私钥都为空 */
    CSC_ERR_INVALID_EC_CURVE_NAME                         =4019,   /* 无效的EC curve name */

    CSC_ERR_CREATE_CERT                                   =4050,   /* 创建证书失败 */
    CSC_ERR_CREATE_CSR                                    =4051,   /* 创建证书CSR失败 */
    CSC_ERR_CREATE_CRL                                    =4052,   /* 创建证书CRL失败 */
    CSC_ERR_CREATE_RSA_KEY                                =4053,   /* 创建RSA公私钥对对象失败 */
    CSC_ERR_CREATE_EC_GROUP                               =4054,   /* 创建EC公私钥GROUP失败 */
    CSC_ERR_CREATE_EC_KEY                                 =4055,   /* 创建EC公私钥对对象失败 */
    CSC_ERR_GENERATE_KEY                                  =4056,   /* 生成公私钥对失败 */

    CSC_ERR_SET_VERSION                                   =4070,   /* 设置版本信息失败 */
    CSC_ERR_SET_SERIAL_NO                                 =4071,   /* 设置序列号失败 */
    CSC_ERR_SET_VALID_DATE                                =4072,   /* 设置证书有效期失败 */
    CSC_ERR_SET_ISSUE_NAME                                =4073,   /* 设置签发者信息失败 */
    CSC_ERR_SET_SUBJECT_NAME                              =4074,   /* 设置证书Subject失败 */
    CSC_ERR_SET_PUBLIC_KEY                                =4075,   /* 设置公钥失败 */
    CSC_ERR_SET_EXTENTION                                 =4076,   /* 设置证书扩展信息失败 */
    CSC_ERR_SET_EXTS_BASICCONSTRAINTS                     =4077,   /* 设置证书basicConstraints扩展信息失败 */
    CSC_ERR_SET_EXTS_CRLDISTRIBUTIONPOINTS                =4078,   /* 设置证书crlDistributionPoints扩展信息失败 */
    CSC_ERR_SET_EXTS_SUBJECTALTNAME                       =4079,   /* 设置证书subjectAltName扩展信息失败 */
    CSC_ERR_SET_EXTS_KEYUSAGE                             =4080,   /* 设置证书keyUsage扩展信息失败 */
    CSC_ERR_SET_EXTS_AUTHORITYINFOACCESS                  =4081,   /* 设置证书authorityInfoAccess扩展信息失败 */
    CSC_ERR_SET_EXTS_SUBJECTKEYIDENTIFIER                 =4082,   /* 设置证书subjectKeyIdentifier扩展信息失败 */
    CSC_ERR_SET_EXTS_AUTHORITYKEYIDENTIFIER               =4084,   /* 设置证书authorityKeyIdentifier扩展信息失败 */
    CSC_ERR_SET_EXTS_EXTENDEDKEYUSAGE                     =4085,   /* 设置证书extendedKeyUsage扩展信息失败 */
    CSC_ERR_SET_EC_GROUP                                  =4086,   /* 设置EC的GROUP失败 */
    CSC_ERR_SET_CRL_TIME                                  =4087,   /* 设置CRL创建和下次生成时间失败 */
    CSC_ERR_SET_CRL_REVOKE_CERTS                          =4088,   /* 设置CRL吊销证书列表失败 */

    CSC_ERR_SIGN_DATA                                     =4100,   /* 签名数据失败 */
    CSC_ERR_SIGN_CERT                                     =4101,   /* 签发证书失败 */
    CSC_ERR_SIGN_CSR                                      =4102,   /* 签发证书CSR失败 */
    CSC_ERR_SIGN_CRL                                      =4103,   /* 签发证书CRL失败 */
    CSC_ERR_VERIFY_SIGN                                   =4104,   /* 校验证书/CSR/CRL签名失败 */

    CSC_ERR_OUTPUT_KEY                                    =4110,   /* 转换或输出key字符串失败 */
    CSC_ERR_OUTPUT_PUBLIC_KEY                             =4111,   /* 转换或输出公钥字符串失败 */
    CSC_ERR_OUTPUT_PRIVATE_KEY                            =4112,   /* 转换或输出私钥字符串失败 */
    CSC_ERR_OUTPUT_CERT_KEY                               =4113,   /* 转换或输出证书字符串失败 */
    CSC_ERR_OUTPUT_CSR_KEY                                =4114,   /* 转换或输出证书请求字符串失败 */
    CSC_ERR_OUTPUT_CRL_KEY                                =4115,   /* 转换或输出证书CRL字符串失败 */

    CSC_ERR_WRITE_PRIVATE_KEY_PKCS1                       =4130,   /* 写PKCS1格式私钥失败，格式BEGIN RSA PRIVATE KEY*/
    CSC_ERR_WRITE_PRIVATE_KEY_PKCS8                       =4131,   /* 写PKCS8格式私钥失败，格式BEGIN PRIVATE KEY*/

    CSC_ERR_GET_PUBLIC_KEY                                =4132,   /* 获取公钥失败 */
    CSC_ERR_PARSE_SUBJECT                                 =4133,   /* 解析subject 失败*/
    CSC_ERR_COPY_EXTENTION                                =4134   /* 拷贝证书扩展信息失败 */

} SC_ERROR_CODE;

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif/* SC_ERRCODE_H */
