#!/bin/bash


###############################################################################
#author:   none
#date:     2014/01/19
#function: 此脚本为自动化分区、格式化、挂载的入口脚本
#以下是逻辑设备在各种呈现形态的相互映射关系图，维护此脚本的人务必先清楚此关系图
#
#                  UUID(findfs)
#                    |   |
#                    |     |
#                    |       |
#                    |         |
#                    |           |
#                    |             |
#                    |               mapper(readlink)
#                    |               |
#                    |               |
#                    |               |
#                    |               |
#                    |               |
#  by-id(readlink)   |              dm-0(slaves)
#        |           |            |
#          |         |          |
#            |       |        |
#              |     |      |
#                |   |    |
#                  | | |
#                   sda
###############################################################################
#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#主机名或IP地址
IPADDR="$(echo ${SSH_CONNECTION} | awk '{print $3}')"

#版本号，不同的版本可能特性、BUG不同
APVERSION=V100R001C00B001

#设置当前路径
CURRPATH=""

#设置格式化路径
NOHUPTXT=""

#设置信息收集脚本名，要求收集硬盘格式化失败的历史记录
#所以此脚本后台执行，定期收集D状态的系统进程信息
COLLECT_SCRIPT=""

#临时目录，主要用于存放临时文件，比如系统分区表、系统swap表、拷贝临时文件等
TMPDIR=""




#***********************************
#以下为收集系统中设备信息的临时文件
#***********************************

#系统中所有的设备名的保存文件
DEVLIST_ALL=""

#系统中挂载的设备名的保存文件
DEVLIST_MOUNT=""

#系统中的swap设备信息的保存文件
DEVLIST_SWAP=""

#正在格式化的设备名的保存文件
DEVLIST_MKFS=""

#系统的分区信息的保存文件
DEVLIST_PARTITION=""

#系统中做过软raid或者类似软raid设备名的保存文件
DEVLIST_RAID=""

#系统中空白盘(没有分区和文件系统的设备)的保存文件
DEVLIST_BLANK=""

#准备用于开始挂载大数据目录的设备名的保存文件
DEVLIST_READY=""

#准备挂载的scsi设备名的保存文件
#此文件和READY文件目的不同，在于此文件保存的可能是插拔之后的设备分区名
#也就是此文件是最后要挂载的设备分区列表
DEVLIST_SCSI=""

#回滚的挂载设备信息（挂载目录、设备、文件系统、挂载选项等）的保存文件
#用于整个挂载流程成功后，整个文件一次性追加到/etc/fstab文件系统配置文件中
DEVLIST_ROLLBACK=""

#回滚的设备名的保存文件，用于脚本执行失败后，需要知道哪些设备需要回退，保存执行脚本前后样貌的一致性
DEVLIST_PARTED=""

#回滚的加密设备挂载设备信息（映射名、设备UUID、秘钥文件等）的保存文件
#用于整个挂载流程成功后，整个文件一次性追加到/etc/crypttab文件系统配置文件中
CRYPTTAB_ROLLBACK=""

#用于记录创建的加密映射，如果升级失败，需要删除
ENCRYPT_MAPPER_ROLLBACK=""
ENCRYPT_MAPPER_LIST=""
#记录 dev_mapper(加密设备名) uuid(fstab) 挂载点 文件系统类型
ENCRYPT_PARTITION_INFO_ROLLBACK=""
#记录 dev_mapper(加密设备名) uuid(fstab) 挂载点 文件系统类型
ENCRYPT_PARTITION_INFO=""

#加密分区的访问秘钥
KEY_FILE_1=""
KEY_FILE_2=""

#diskmgt服务的安装路径
DISKMGTPATH="/usr/local/diskmgt"

#记录基于kms服务加密的设备信息（设备UUID、映射名、挂载目录、文件系统、挂载选项等）
#用于OS重启后对基于kms服务加密的分区进行自动加密及挂载
KMS_ENCRYPT_DEVS=""



#***********************************
#以下为脚本的各个输入参数保存变量
#***********************************

#输入的挂载配置文件
INIFILE=""

#输入的审核挂载配置文件（不挂载分区等，只是检查配置文件的有效性和合法性）
INICHECK=""

#根据输入的挂载配置文件INIFILE，经过处理（比如去除空行、#号开头行等）的挂载配置文件
ININEWFILE=""

#根据处理过的ININEWFILE挂载配置文件，再进行一次处理（比如去除多余的可忽略挂载项等）的挂载配置文件
INIFINALFILE=""

#显示进度的记录文件，注意调用者自身必须保存文件的存在和可写
RATEFILE=""

#是否强制进行分区（不考虑本来已经有分区和文件在设备中）
#0表示不破坏本来有分区的设备，1表示会把有本来有分区的设备强制用来挂载大数据目录 2表示不进行格式化、挂载，仅仅安装diskmgt
PARTITION_MODE=0

#是否强制安装，如果强制安装则会覆盖之前的整个安装目录
#对于第一次安装，此配置变量不影响，主要是多次安装使用
#0表示不强制安装，1表示强制安装
IS_FORCE=0

#是否安装硬盘管理进程，这个和IS_FORCE变量配套使用
#0表示不安装，1表示安装
IS_INSTALL=0

#是否集群初次安装阶段，如果初次安装则磁盘异常概率低，获取磁盘前无需等待
#0表示非初次安装 1表示初次安装
IS_BOOT_INSTALL=0

#是否使用kms服务进行分区加密
#0表示不使用，1表示使用
IS_KMS_ENCRYPT=0

#该节点的raid级别，必须是nonraid、raid0、raid1
#用于检查该节点的所有的raid配置是否一致
#不能存在既要求raid0，又要求raid1等等矛盾的场景
RAID_LEVEL=""





#***********************************
#以下为系统的配置变量等
#***********************************
#Linux操作系统的文件系统挂载配置文件
FSTABFILE="/etc/fstab"

#Linux操作系统的文件系统加密分区挂载配置文件
CRYPTTABFILE="/etc/crypttab"

PROCMOUNTSFILE="/proc/mounts"

#Linux操作系统的文件系统挂载配置文件的备份
FSTABBAKFILE="${FSTABFILE}_$(date +%s)_bak"

#Linux操作系统的文件系统加密分区挂载配置文件的备份文件
CRYPTTABBAKFILE="${CRYPTTABFILE}_$(date +%s)_bak"

#根分区设备
ROOTDEV=""

#根分区文件系统
ROOTFS=""

#根分区格式化工具
ROOTMKFS=""

#最大格式化并行硬盘数量
#主要是考虑系统中一般最大有12个盘，可以分2次并行格式化
#如果太多盘一起格式化，吃内存太多会整体失败
MAXPARALLEL=12

#当前的最大并行格式化数量
CURRPARALLEL=0

#当前的并行格式化数量
PARALLELCNT=0

#系统中存在此脚本执行的总进程数
#主要用于此脚本如果被强杀那么就遗留锁文件，等待下一次进来此脚本被锁定判断其实是之前被强杀，就返回特定错误码告知此事
#并删除遗留锁
CMDCNT=0

#保存此脚本执行的总进程数的临时文件
TMPFILE=""

#锁文件
LOCKFILE="${HOME}/autopart.lock"

#日志目录
LOGPATH="${HOME}/diskmgt/"

#日志文件
LOGFILE="${LOGPATH}/autopart.log"

#数据目录根目录
BIGDATA_DATA_HOME="/srv/BigData"

#全局返回结果变量
RESULT=0

#后台执行格式化（因为时间比较长）的进程保存全局变量
BACKPIDS=""

SCC_INSTALL_PATH="${DISKMGTPATH}/scc"
SCC_INIT_INSTALL_PATH="${HOME}/scc"
SCC_CONF_DIR="${SCC_INIT_INSTALL_PATH}/config"
#############################################################################

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_WHICH="sudo ${INSTALL_SUDO_PATH} preinstall_which"
    CMD_MKFS_EXT3="sudo ${INSTALL_SUDO_PATH} preinstall_mkfs_ext3"
    CMD_MKFS_EXT4="sudo ${INSTALL_SUDO_PATH} preinstall_mkfs_ext4"
    CMD_MKFS_XFS="sudo ${INSTALL_SUDO_PATH} preinstall_mkfs_xfs"
    CMD_READLINK_E="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_e"
    CMD_READLINK_M="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_m"
    CMD_CAT_FSTAB="sudo ${INSTALL_SUDO_PATH} cat_fstab"
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_FILE_S="sudo ${INSTALL_SUDO_PATH} preinstall_file_s"
    CMD_FDISK_L="sudo ${INSTALL_SUDO_PATH} preinstall_fdisk_l"
    CMD_PARTED="sudo ${INSTALL_SUDO_PATH} preinstall_parted"
    CMD_BLKID="sudo ${INSTALL_SUDO_PATH} preinstall_blkid"
    CMD_SFDISK_G="sudo ${INSTALL_SUDO_PATH} preinstall_sfdisk_g"
    CMD_WAIT="sudo ${INSTALL_SUDO_PATH} preinstall_wait"
    CMD_PARTPROBE="sudo ${INSTALL_SUDO_PATH} preinstall_partprobe"
    CMD_PARTED_S_MK_GPT="sudo ${INSTALL_SUDO_PATH} preinstall_parted_s_mk_gpt"
    CMD_PARTED_UNIT_PRINT="sudo ${INSTALL_SUDO_PATH} preinstall_parted_unit_print"
    CMD_PARTED_UNIT_GB_PRINT="sudo ${INSTALL_SUDO_PATH} preinstall_parted_unit_gb_print"
    CMD_PARTED_S_MK_LOGICAL="sudo ${INSTALL_SUDO_PATH} preinstall_parted_s_mk_logical"
    CMD_PARTED_S_MK_OPT_LOGICAL="sudo ${INSTALL_SUDO_PATH} preinstall_parted_s_mk_opt_logical"
    CMD_CHOWN="sudo ${INSTALL_SUDO_PATH} preinstall_chown"
    CMD_CHOWN_R="sudo ${INSTALL_SUDO_PATH} preinstall_chown_r"
    CMD_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
    CMD_MOUNT="sudo ${INSTALL_SUDO_PATH} preinstall_mount"
    CMD_CP="sudo ${INSTALL_SUDO_PATH} preinstall_cp"
    CMD_UMOUNT="sudo ${INSTALL_SUDO_PATH} preinstall_umount"
    CMD_FUSER_MK="sudo ${INSTALL_SUDO_PATH} preinstall_fuser_mk"
    CMD_DD_CLEAN_ZERO="sudo ${INSTALL_SUDO_PATH} preinstall_dd_clean_zero"
    CMD_MKDIR_P="sudo ${INSTALL_SUDO_PATH} preinstall_mkdir_p"
    CMD_INIT_DATA_DIR="sudo ${INSTALL_SUDO_PATH} preinstall_init_data_dir"
    CMD_IS_DIR="sudo ${INSTALL_SUDO_PATH} preinstall_is_dir"
    CMD_IS_FILE="sudo ${INSTALL_SUDO_PATH} preinstall_is_file"
    CMD_RM_F="sudo ${INSTALL_SUDO_PATH} preinstall_rm_f"
    CMD_ECHO_PROC_SCSI="sudo ${INSTALL_SUDO_PATH} preinstall_echo_proc_scsi"
    CMD_CAT="sudo ${INSTALL_SUDO_PATH} preinstall_cat"
    CMD_SWAPON_S="sudo ${INSTALL_SUDO_PATH} preinstall_swapon_s"
    CMD_ADD_TO_FSTAB="sudo ${INSTALL_SUDO_PATH} preinstall_add_to_fstab"
    CMD_ADD_TO_CRYPTTAB="sudo ${INSTALL_SUDO_PATH} preinstall_add_to_crypttab"
    CMD_PVSCAN_S="sudo ${INSTALL_SUDO_PATH} preinstall_pvscan_s"
    CMD_CRYPTSETUP="sudo ${INSTALL_SUDO_PATH} preinstall_cryptsetup"
    CMD_TOUCH="sudo ${INSTALL_SUDO_PATH} preinstall_touch"
    CMD_CHECK_DFP="sudo ${INSTALL_SUDO_PATH} check_dfP"
    CMD_CHECK_MOUNTA="sudo ${INSTALL_SUDO_PATH} check_mounta"
    CMD_UPDATE_RPM_PACKAGE="sudo ${INSTALL_SUDO_PATH} update_rpm_package"
else
    CMD_WHICH="which"
    CMD_MKFS_EXT3="preinstall_mkfs_ext3"
    CMD_MKFS_EXT4="preinstall_mkfs_ext4"
    CMD_MKFS_XFS="preinstall_mkfs_xfs"
    CMD_READLINK_E="readlink -e"
    CMD_READLINK_M="readlink -m"
    CMD_CAT_FSTAB=" cat /etc/fstab"
    CMD_FINDFS="findfs"
    CMD_FILE_S="file -s"
    CMD_FDISK_L="fdisk -l"
    CMD_PARTED="parted"
    CMD_BLKID="blkid"
    CMD_SFDISK_G="sfdisk -g"
    CMD_WAIT="preinstall_wait"
    CMD_PARTPROBE="partprobe"
    CMD_PARTED_S_MK_GPT="preinstall_parted_s_mk_gpt"
    CMD_PARTED_UNIT_PRINT="preinstall_parted_unit_print"
    CMD_PARTED_UNIT_GB_PRINT="preinstall_parted_unit_gb_print"
    CMD_PARTED_S_MK_LOGICAL="preinstall_parted_s_mk_logical"
    CMD_PARTED_S_MK_OPT_LOGICAL="preinstall_parted_s_mk_opt_logical"
    CMD_CHOWN="chown"
    CMD_CHOWN_R="chown -R"
    CMD_CHMOD="chmod"
    CMD_MOUNT="preinstall_mount"
    CMD_CP="cp -arf"
    CMD_UMOUNT="umount"
    CMD_FUSER_MK="fuser -mk"
    CMD_DD_CLEAN_ZERO="preinstall_dd_clean_zero"
    CMD_MKDIR_P="mkdir -p"
    CMD_INIT_DATA_DIR="preinstall_init_data_dir"
    CMD_IS_DIR="preinstall_is_dir"
    CMD_IS_FILE="preinstall_is_file"
    CMD_RM_F="rm -rf"
    CMD_ECHO_PROC_SCSI="preinstall_echo_proc_scsi"
    CMD_CAT="cat"
    CMD_SWAPON_S="swapon -s"
    CMD_ADD_TO_FSTAB="preinstall_add_to_fstab"
    CMD_ADD_TO_CRYPTTAB="preinstall_add_to_crypttab"
    CMD_PVSCAN_S="pvscan -s"
    CMD_CRYPTSETUP="cryptsetup"
    CMD_TOUCH="touch"
    CMD_CHECK_DFP="df -P"
    CMD_CHECK_MOUNTA="mount -a"
    CMD_UPDATE_RPM_PACKAGE="update_rpm_package"
fi

########################################################################
#                           sudo封装开始
########################################################################
########################################################################
# 函数描述： 判断路径是否是一个目录
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_dir ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -d "$1" ]; then
        return 0
    fi

    return 1
}

########################################################################
# 函数描述： 判断路径是否是一个文件
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_file ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -f "$1" ]; then
        return 0
    fi

    return 1
}

########################################################################
# 函数描述： 将文件中内容加入到fstab中
# 参数1说明：文件路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_add_to_fstab()
{
    if [ -z "$1" ]; then
        echo "path is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "cat path does not exist."
        return 1
    fi
    
    cat "$1" >> /etc/fstab
    return $?
}

########################################################################
# 函数描述： 将文件中内容加入到crypttab中
# 参数1说明：文件路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_add_to_crypttab()
{
    if [ -z "$1" ]; then
        echo "path is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "cat path does not exist."
        return 1
    fi
    
    local line=""
    local mapper=""
    
    if [ ! -f /etc/crypttab ]; then
        touch /etc/crypttab
        chmod 644 /etc/crypttab
    fi
    
    ${CMD_CP} /etc/crypttab /etc/crypttab_bak
    if [ $? -ne 0 ]; then
        echo "cp -pf /etc/crypttab /etc/crypttab_bak failed."
        return 1
    fi
    
    while read line
    do
        mapper=$(echo "$line" | awk '{print $1}')
        sed -i "/$mapper /d" /etc/crypttab_bak
    done <"$1"
    
    ${CMD_CAT} "$1" >> /etc/crypttab_bak
    if [ $? -ne 0 ]; then
        echo "cat $1 >> /etc/crypttab_bak failed."
        return 1
    fi
    
    mv -f /etc/crypttab_bak /etc/crypttab
    return $?
}

########################################################################
# 函数描述： 格式化磁盘成ext3文件系统
# 参数1说明：设备名
# 参数2说明：日志路径名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_mkfs_ext3()
{
    if [[ -z "$1" || -z "$2" ]]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    nohup mkfs.ext3 -m 0 $1 >> $2 2>&1 &
    echo $!
    return 0
}

########################################################################
# 函数描述： 格式化磁盘成ext4文件系统
# 参数1说明：设备名
# 参数2说明：日志路径名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_mkfs_ext4()
{
    if [[ -z "$1" || -z "$2" ]]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    nohup mkfs.ext4 -F -m 0 $1 >> $2  2>&1 &
    echo $!
    return 0
}

########################################################################
# 函数描述： 格式化磁盘成xfs文件系统
# 参数1说明：设备名
# 参数2说明：日志路径名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_mkfs_xfs()
{
    if [[ -z "$1" || -z "$2" ]]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    nohup mkfs.xfs -f $1 >> $2  2>&1 &
    echo $!
    return 0
}

########################################################################
# 函数描述： 将磁盘格式化成GPT分区
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_parted_s_mk_gpt()
{
        if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    parted -s "$1"  mklabel gpt
    return $?
}

########################################################################
# 函数描述： 显示磁盘信息
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_parted_unit_print()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    parted "$1" unit s print
    return $?
}

########################################################################
# 函数描述： 显示磁盘信息
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_parted_unit_gb_print()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    parted "$1" unit GB print
    return $?
}

########################################################################
# 函数描述： 将磁盘分区
# 参数1说明：设备名
# 参数2说明：分区的开始扇区
# 参数3说明：分区的结束扇区
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_parted_s_mk_logical()
{
        if [ -z "$1" -o -z "$2" -o -z "$3" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    parted -s "$1" mkpart logical ${2}s ${3}s
    return $?
}


########################################################################
# 函数描述： 将磁盘分区(采用默认的最优选项)
# 参数1说明：设备名
# 参数2说明：分区的开始GB
# 参数3说明：分区的结束GB
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_parted_s_mk_opt_logical()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    parted -a optimal -s "$1" mkpart logical $2 $3
    return $?
}

########################################################################
# 函数描述： mount指定的磁盘
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_mount()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    mount "$1" "$2" -o noatime,nodiratime
    return $?
}

########################################################################
# 函数描述： 清空磁盘的分区信息
# 参数1说明：设备名
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_dd_clean_zero()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    dd if=/dev/zero of="$1" bs=512 count=1
    return $?
}

#############################################################################################################################
# 函数描述： 初始化数据目录，递归向上查找挂载点目录的父目录，直到数据目录根目录的下一层，然后递归修改这个目录的权限和属主
#            如：挂载点为/srv/BigData/test1/test2/test3，配置文件中设置的属主为2000:wheel，权限为700
#            则会执行chown 2000:wheel /srv/BigData/test1 -R 以及 chmod 700 /srv/BigData/test1 -R
#            这样做主要是为了解决挂载点和数据根目录之间的层级目录权限未修改的问题
# 参数1说明：数据挂载点目录
#
# 返回值  ：0 成功
#           1 失败
##############################################################################################################################
function preinstall_init_data_dir()
{
    local dir="${1}"
    local user_group="${2}"
    local permission="${3}"
    local data_home=$(readlink -m "${4}")
    
    echo "$1" | grep -q ^"${data_home}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Dir ${1} is not under ${data_home}."
        return 1
    fi
    
    # 递归向上查找父父目录，直到父目录为数据目录根目录
    local currentDir="${dir}"
    local parentDir=$(dirname "${currentDir}")
    while [[ "${parentDir}" != "${data_home}" ]] && [[ "${parentDir}" != "/" ]]
    do
        currentDir="${parentDir}"
        parentDir=$(dirname "${currentDir}")
    done
    
    if [ "${parentDir}" = "/" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Invalid dir ${dir}. It is not a sub dir of ${data_home}."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to init dir ${currentDir} ."
    chown ${user_group} "${currentDir}" -R
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown ${user_group} ${currentDir} -R failed."
        return 1
    fi
    
    if [ "${permission}" != "default" ]; then
        chmod "${permission}" "${currentDir}" -R
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod ${permission} ${currentDir} -R failed."
            return 1
        fi 
    fi
}


########################################################################
# 函数描述： echo 信息到/proc/scsi/scsi目录
# 参数1说明：信息
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_echo_proc_scsi()
{
    if [ $# -eq 0 ]; then
        echo "arguments is null"
        return 1
    fi
    
    echo "$*" > /proc/scsi/scsi
        return $?
}

########################################################################
# 函数描述： 等待进程结束
# 参数1说明：进程PID
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_wait()
{
    if [ $# -eq 0 ]; then
        echo "pid is empty"
        return 0
    fi
    
        local completed=0
        while true
        do
                completed=0
                for pid in "$@"
                do
                        kill -0 $pid
                        if [ $? -ne 0 ]; then
                                ((completed++))
                        fi
                done

                if [ $completed -eq $# ]; then
                        break;
                fi

                sleep 5
        done

        return $?
}

########################################################################
#                           sudo封装结束
########################################################################


########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    #参数个数判断
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] [EMER] diskmgtmodule ap: mkdir -p ${LOGPATH} failed." >>/var/log/messages
            echo "[$(date)] ${log_info}" >>/var/log/messages
            return 1
        fi
    fi
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 记录脚本执行的上下文，主要用于定位用户的执行历史明细
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function log_begin()
{
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 "************************AUTOPART-START************************"
    logme 1 "==$0 $*=="
    logme 1 "==who=="
    logme 1 "$(who)"
    get_call_stack
    logme 1 "************************AUTOPART-END**************************"
    
    return 0
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-l                                 Notes: show all ini-files."
    echo "-n XXX.ini [-p] -r file -i [-f]    Notes: auto part by ini file(force), then install service diskmgt(force)."
    echo "-c XXX.ini/os [-p]                 Notes: check environment first."
    echo "-h                                 Notes: show help."
    echo "-v                                 Notes: show version."
    echo
    echo "Example:"
    echo "`basename $0` -l"
    echo "`basename $0` -n master.ini"
    echo "`basename $0` -n master.ini -p"
    echo "`basename $0` -n master.ini -p -r rate.txt"
    echo "`basename $0` -n master.ini -p -r rate.txt -i"
    echo "`basename $0` -n master.ini -p -r rate.txt -i -f"
    echo "`basename $0` -c os"
    echo "`basename $0` -c os -p"
    echo "`basename $0` -c master.ini"
    echo "`basename $0` -c master.ini -p"
    echo "`basename $0` -h"
    echo "`basename $0` -v"
    echo
    return 0
}


########################################################################
# 函数描述： 显示版本信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_ap_version()
{
    echo "autopart ${APVERSION}"
    echo 
    return 0
}


########################################################################
# 函数描述： 初始化路径变量
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_path()
{
    #初始化当前路径，为其他变量赋值做准备
    pushd $(dirname "$0") >/dev/null 2>&1
    CURRPATH="$(readlink -e "`pwd`")"
    find ./ini -type f | xargs dos2unix >/dev/null 2>&1
    find ./ini-plugin -type f | xargs dos2unix >/dev/null 2>&1
    find ./inner -type f | xargs dos2unix >/dev/null 2>&1
    find ./script -type f | xargs dos2unix >/dev/null 2>&1
    popd >/dev/null 2>&1
    if [ -z "${CURRPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current path failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPATH is:${CURRPATH}"
    
    #初始化分区加密文件路径
    KEY_FILE_1="$CURRPATH/ini-plugin/keys/fi_disk_partition_key_1"
    KEY_FILE_2="$CURRPATH/ini-plugin/keys/fi_disk_partition_key_2"
    ENCRYPT_MAPPER_LIST="$CURRPATH/ini-plugin/keys/encrypt_mapper.lst"
    ENCRYPT_PARTITION_INFO="$CURRPATH/ini-plugin/keys/encrypt_partition_info.lst"

    NOHUPTXT="${CURRPATH}/nohup.txt"
    ${CMD_RM_F} "${NOHUPTXT}"
    
    #设置收集日志路径（比如D状态收集）
    set_collect_file
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] set collect file failed."
        return 1
    fi
    
    #设置全局保存设备变量
    set_temp_file
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] set temp file failed."
        return 1
    fi
    
    #压缩日志，具体压缩规格见scriptlog.sh脚本中描述
    compress_log
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] compress log failed."
        return 1
    fi

    return 0
}


########################################################################
# 函数描述： 设置后台收集硬盘分区信息（如D状态等）脚本的绝对路径、设置权限等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_collect_file()
{
    COLLECT_SCRIPT="${CURRPATH}/inner/atpt_collect.sh"
    if [ ! -f "${COLLECT_SCRIPT}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] collect-script ${COLLECT_SCRIPT} is not exist."
        return 1
    fi
    
    chmod 750 "${COLLECT_SCRIPT}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 750 ${COLLECT_SCRIPT} failed."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 运行后台硬盘分区信息（如D状态等）脚本
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function run_collect()
{
    local count=0
    local count_max=10
    local pid_ap_clt=0
    
    #首先要保证后台没有收集脚本在运行
    #如果有的话，先杀死进程
    while [ ${count} -le ${count_max} ]
    do
        pid_ap_clt=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "atpt_collect.sh" | awk '{print $1}')
        if [ -z "${pid_ap_clt}" ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] atpt_collect.sh is not in back."
            break
        fi
        
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] atpt_collect.sh is running in back, ready to kill it."
        echo ${pid_ap_clt} | xargs kill >>"${LOGFILE}" 2>&1
        
        ((count++))
        if [ ${count} -eq ${count_max} ]; then
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill force collect pid(${pid_ap_clt})."
            echo ${pid_ap_clt} | xargs kill -9 >>"${LOGFILE}" 2>&1
        fi
        
        sleep 2
    done

    pid_ap_clt=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "atpt_collect.sh" | awk '{print $1}')
    if [ ! -z "${pid_ap_clt}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] can not stop atpt_collect.sh in back."
        return 1
    fi
    
    #启动收集进程（比如D状态）
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] ready to run ${COLLECT_SCRIPT} in back."
    nohup ${COLLECT_SCRIPT} >/dev/null 2>&1 &
    
    return 0
}


########################################################################
# 函数描述： 设置系统中的设备的保存文件绝对路径
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_temp_file()
{
    TMPDIR="${CURRPATH}/tmpdir"
    rm -rf "${TMPDIR}"
    mkdir -p "${TMPDIR}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkdir -p ${TMPDIR} failed."
        return 1
    fi
    
    #全局保存文件名绝对路径变量
    DEVLIST_ALL="${TMPDIR}"/devlist_all
    DEVLIST_MOUNT="${TMPDIR}"/devlist_mount
    DEVLIST_SWAP="${TMPDIR}"/devlist_swap
    DEVLIST_MKFS="${TMPDIR}"/devlist_mkfs
    DEVLIST_PARTITION="${TMPDIR}"/devlist_partition
    DEVLIST_RAID="${TMPDIR}"/devlist_raid
    DEVLIST_BLANK="${TMPDIR}"/devlist_blank
    DEVLIST_READY="${TMPDIR}"/devlist_ready
    DEVLIST_SCSI="${TMPDIR}"/devlist_scsi
    DEVLIST_ROLLBACK="${TMPDIR}"/devlist_rollback
    DEVLIST_PARTED="${TMPDIR}"/devlist_parted
    CRYPTTAB_ROLLBACK="${TMPDIR}"/crypttab_rollback
    ENCRYPT_MAPPER_ROLLBACK="${TMPDIR}"/encrypt_mapper_rollback
    ENCRYPT_PARTITION_INFO_ROLLBACK="${TMPDIR}"/ENCRYPT_PARTITION_INFO_ROLLBACK
    
    #先清空，后面会再生成
    rm -rf "${DEVLIST_ALL}"
    rm -rf "${DEVLIST_MOUNT}"
    rm -rf "${DEVLIST_SWAP}"
    rm -rf "${DEVLIST_MKFS}"
    rm -rf "${DEVLIST_PARTITION}"
    rm -rf "${DEVLIST_RAID}"
    rm -rf "${DEVLIST_BLANK}"
    rm -rf "${DEVLIST_READY}"
    rm -rf "${DEVLIST_SCSI}"
    rm -rf "${DEVLIST_ROLLBACK}"
    rm -rf "${DEVLIST_PARTED}"
    rm -rf "${CRYPTTAB_ROLLBACK}"
    rm -rf "${ENCRYPT_MAPPER_ROLLBACK}"
    rm -rf "${ENCRYPT_PARTITION_INFO_ROLLBACK}"

    return 0
}


########################################################################
# 函数描述： 压缩日志
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function compress_log()
{
    script_log="${CURRPATH}/inner/scriptlog.sh"
    
    if [ ! -f "${script_log}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] compress script(${script_log}) is not exist."
        return 1
    fi
    
    if [ ! -x "${script_log}" ]; then
        chmod 750 "${script_log}"
    fi
    
    #压缩日志
    if [ -f "${LOGFILE}" ]; then
        "${script_log}" "${LOGFILE}"
    fi
    
    return 0
}


########################################################################
# 函数描述： 获取脚本执行的调用栈
# 参数1说明：进程pid号
#
# 返回值  ： 0 成功
########################################################################
function get_call_stack()
{
    local pid_val=$1
    local line=""

    if [ -z "${pid_val}" ]; then
        pid_val=$$
    fi
    
    logme 1 "==stack-begin=="
    
    #获取调用栈
    while [[ ${pid_val} != 1 && ${pid_val} != 0 ]]
    do
        line=$(ps -wweo pid,ppid,uname,tname,args | grep "^[[:space:]]*${pid_val}[[:space:]]")
        if [ -z "${line}" ]; then
           break
        fi
        pid_val=$(echo ${line} | awk '{print $2}')
        logme 1 "${line}"
    done
    
    logme 1 "==stack-end=="
    
    return 0
}


########################################################################
# 函数描述： 显示目前包含的所有挂载配置规格项
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function show_ini()
{
    if [ ! -d "${CURRPATH}/ini" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ini dir(${CURRPATH}/ini) is not exist."
        return 1
    fi
    
    #从ini目录下读取所有文件
    echo
    echo "All ini-files: "
    echo "==============="
    find "${CURRPATH}/ini/" -type f | grep "\.ini$" | awk -F / '{print $NF}'
    echo "==============="
    echo

    return 0
}


########################################################################
# 函数描述： 检查系统的有效信息，包括用户、操作系统、命令等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_system()
{   
    #检查操作系统
    check_osversion
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check os version failed."
        return 1
    fi
    
    #检查需要用到的命令
    check_command
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check command failed."
        return 1
    fi
    
    #检查根分区信息
    check_rootinfo
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check root info failed."
        return 1
    fi

    #检查挂载项和fstab是否一致，如果不一致可能本来重启就会挂住，我们不会再冒险做任何操作
    check_mount
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check mount failed."
        return 1
    fi
    
    #检查swap挂载项和fstab是否一致
    check_swap
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check swap failed."
        return 1
    fi
    
    
    return 0
}


########################################################################
# 函数描述： 检查节点是否安装diskmgt
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_diskmgt()
{
    local install_path="$DISKMGTPATH"
    
    #执行自动化格式化分区脚本，必须保证之前可能安装过的硬盘管理服务不能在后台运行
    ps -wweo pid,ppid,uname,tname,args | grep -v grep | grep "diskmgtd" | grep "/bin/" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        logme 2 "[WARN] [none:${LINENO}] diskmgtd is running background."
        return 1
    fi
    
    #如果diskmgt安装目录已经存在，说明Preinstall已经成功，因此直接退出
    ${CMD_IS_DIR} "${install_path}"
    if [ $? -eq 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] diskmgt install path ${install_path} exists."
        return 1
    fi
    
    return 0
}



########################################################################
# 函数描述： 检查执行操作系统版本，目前支持redhat/suse/centos
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_osversion()
{
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local centos_release="/etc/centos-release"
    local euler_release="/etc/euleros-release"
    local neokylin_relase="/etc/neokylin-release"
    local bclinux_release="/etc/bclinux-release"
    
    #支持红帽/suse/centos
    #为了最大程度上的支持，都不会异常退出，但是会打印告警，表示可能不太支持，毕竟没有在其他操作系统上做过测试
    if [ -f "${centos_release}" ]; then
        cat "${centos_release}" | grep -i "CentOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is centos."
    #获取Euler的版本
    elif [ -f "${euler_release}" ]; then
        cat "${euler_release}" | grep -i "EulerOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not euleros, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is euleros."
    elif [ -f "${neokylin_relase}" ];then
        cat "${neokylin_relase}" | grep -i "NeoKylin" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not NeoKylin, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is NeoKylin."   
    elif [ -f ${bclinux_release} ]; then
        cat "${bclinux_release}" | grep -i "BigCloud" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not bclinux, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is bclinux." 
    elif [ -f "${rhel_release}" ]; then
        cat "${rhel_release}" | grep -i "Red Hat" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not redhat, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is redhat."
    elif [ -f "${sles_release}" ]; then
        cat "${sles_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse."
    elif [ -f "${sles15_release}" ]; then
        cat "${sles15_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            cat "${sles15_release}" | grep -i "openEuler " >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is openEuler."
                return 0
            fi
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse-15, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse-15."
    else
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos/redhat/suse, maybe it will cause unknown result!"
        return 0
    fi
    
    
    return 0
}


########################################################################
# 函数描述： 检查系统包含的有效命令，如lsscsi/fdisk等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_command()
{
    #要求系统中存在lsscsi命令，因为是我们自己适配的os，这些命令都可以保证有，下同
    which lsscsi >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] lsscsi command not found."
        return 1
    fi

    ${CMD_WHICH} fdisk >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fdisk command not found."
        return 1
    fi

    ${CMD_WHICH} sfdisk >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] sfdisk command not found."
        return 1
    fi
    
    ${CMD_WHICH} parted >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] parted command not found."
        return 1
    fi

    which free >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] free command not found."
        return 1
    fi
    
    ${CMD_WHICH} mkfs.ext3 >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs.ext3 command not found."
        return 1
    fi
    
    ${CMD_WHICH} mkfs.ext4 >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs.ext4 command not found."
        return 1
    fi
    
    ${CMD_WHICH} swapon >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] swapon command not found."
        return 1
    fi
    
    which getent >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] getent command not found."
        return 1
    fi

    ${CMD_WHICH} blkid >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] blkid command not found."
        return 1
    fi

    ${CMD_WHICH} findfs >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] findfs command not found."
        return 1
    fi

    which readlink >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] readlink command not found."
        return 1
    fi

    which file >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] file command not found."
        return 1
    fi
    
    which ln >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ln command not found."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查操作系统根分区信息，包括文件系统、分区等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_rootinfo()
{
    if [ ! -f "${FSTABFILE}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab(${FSTABFILE}) is not exist."
        return 1
    fi
    
    #获取根分区
    ROOTDEV=$(mount 2>/dev/null | awk '{if( $3=="/" ) print $1}' | sed 's/\/dev\///' | sed 's/[0-9]//g' | sed 's/[[:space:]]//g')
    if [ -z "${ROOTDEV}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get root device failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ROOTDEV is:${ROOTDEV}"
    
    #获取根分区的文件系统
    ROOTFS=$(mount 2>/dev/null | awk '{if( $3=="/" ) print $5}')
    if [ -z "${ROOTFS}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get root filesystem failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ROOTFS is:${ROOTFS}"
    
    #获取根分区对应的格式化工具
    if [[ "${ROOTFS}" == "ext3" ]]; then
        ROOTMKFS="${CMD_MKFS_EXT3}"
    elif [[ "${ROOTFS}" == "ext4" ]]; then
        ROOTMKFS="${CMD_MKFS_EXT4}"
    elif [ "${ROOTFS}" == "btrfs" -o "${ROOTFS}" == "xfs" ]; then
        ROOTFS=xfs
        ROOTMKFS="${CMD_MKFS_XFS}"
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] root filesystem(${ROOTFS}) is not ext3/ext4/btrfs/xfs."
        return 1    
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ROOTMKFS is:${ROOTMKFS}"
    
    
    return 0
}


########################################################################
# 函数描述： 检查操作系统目前的挂载信息，是否和/etc/fstab一致
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_mount()
{
    local line=""
    local mnt_dev=""
    local mnt_link_dev=""
    local mnt_dir=""
    local mnt_link_dir=""
    local kylin_relase="/etc/kylin-release"
    #临时保存文件
    local devlist_mnt_sys="${TMPDIR}"/devlist_mnt_sys
    local devlist_mnt_sys_tmp="${TMPDIR}"/devlist_mnt_sys.tmp
    local devlist_mnt_fstab="${TMPDIR}"/devlist_mnt_fstab
    local devlist_mnt_fstab_tmp="${TMPDIR}"/devlist_mnt_fstab.tmp

    #先清空，后面会再生成
    rm -rf "${devlist_mnt_sys}"
    rm -rf "${devlist_mnt_sys_tmp}"
    rm -rf "${devlist_mnt_fstab}"
    rm -rf "${devlist_mnt_fstab_tmp}"
    
    
    #获取当前系统中的挂载设备分区信息
    mount | grep "^/dev/" | awk '{print $1,$3}' >"${devlist_mnt_sys}"
    if [ ! -s "${devlist_mnt_sys}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get system mnt failed."
        return 1
    fi
    
    while read line
    do
        mnt_dev="$(echo ${line} | awk '{print $1}')"
        if [ -z "${mnt_dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] system mnt(${line}) get first column dev failed."
            return 1
        fi
        mnt_link_dev="$(${CMD_READLINK_E} ${mnt_dev})"
        if [ -z "${mnt_link_dev}" ]; then     
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] system mnt(${line}) readlink -e ${mnt_dev} failed."
            return 1
        fi
        
        mnt_dir="$(echo ${line} | awk '{print $2}')"
        if [ -z "${mnt_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] system mnt(${line}) get second column dir failed."
            return 1
        fi
        
        echo "${mnt_link_dev} ${mnt_dir}" >>"${devlist_mnt_sys_tmp}"
    done <"${devlist_mnt_sys}"
    
    
    
    #获取fstab里配置的分区信息
    ${CMD_CAT_FSTAB} | grep -v "^[[:space:]]*#" | grep -v "[[:space:]]swap[[:space:]]" | grep -v -E "cdrom|sr[0-9]|iso9660" \
        | grep -E "^[[:space:]]*/dev/|^[[:space:]]*UUID=" | awk '{print $1,$2}' >"${devlist_mnt_fstab}"
    if [ ! -s "${devlist_mnt_fstab}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get fstab mnt failed."
        return 1
    fi
    
    #对每一行挂载信息进行判断，只有都合法才能继续执行，否则就失败：原因是本来系统里就存在不一致，为了安全起见我们不会进行任何操作，否则后面更不一致，原因就更难定位了
    #要求用户必须先排除这些异常因素
    while read line
    do
        if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}" |awk -F " " '{print $1}')" ]; then
            if grep -q "/data" "/etc/fstab";then
               continue
            fi
        fi
        mnt_dev="$(echo ${line} | awk '{print $1}')"
        if [ -z "${mnt_dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) get first column dev failed."
            return 1
        fi
        
        if [[ ${mnt_dev} =~ ^UUID=.* ]]; then
            mnt_link_dev="${mnt_dev}"
            mnt_dev=$(echo ${mnt_dev} | xargs ${CMD_FINDFS} 2>/dev/null)
            if [ -z "${mnt_dev}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) findfs ${mnt_link_dev} failed."
                return 1
            fi
        fi
        
        mnt_link_dev="$(${CMD_READLINK_E} ${mnt_dev})"
        if [ -z "${mnt_link_dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) readlink -e ${mnt_dev} failed."
            return 1
        fi

        mnt_dir="$(echo ${line} | awk '{print $2}')"
        if [ -z "${mnt_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) get second column dir failed."
            return 1
        fi
        
        #包括的格式可能有：/// 或者/opt/ 或者/opt/// 或者//opt/abc//ad// 等，所以要通过readlink来获取绝对路径，这样统一
        mnt_link_dir="$(${CMD_READLINK_M} ${mnt_dir})"
        if [ -z "${mnt_link_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) readlink -m ${mnt_dir} failed."
            return 1
        fi
        
        echo "${mnt_link_dev} ${mnt_link_dir}" >>"${devlist_mnt_fstab_tmp}"
        
        cat "${devlist_mnt_sys_tmp}" | grep "^${mnt_link_dev} ${mnt_link_dir}$" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            #光驱挂载项要忽略，比如：/dev/cdrom #/dev/sr0 #/dev/loop0
            #或者如下信息：/dev/sr0: ISO 9660 CD-ROM filesystem data 'HUAWEIOS ' (bootable) 
            #或者如下信息：/dev/loop0: ISO 9660 CD-ROM filesystem data 'SLE-11-SP1-SDK-DVD-x86_64.02.00'
            ${CMD_FILE_S} "${mnt_link_dev}" | grep -i " ISO " >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) dev(${mnt_link_dev}) is iso."
                continue
            fi
            
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) dev(${mnt_link_dev}) dir(${mnt_link_dir}) can not find in system mnt."
            return 1
        fi
    done <"${devlist_mnt_fstab}"
    
    
    return 0
}


########################################################################
# 函数描述： 记录操作系统目前的swap使用信息，用于定位失败原因
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function record_swap_info()
{
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info start=="
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info in sys=="
    cat "${1}" >>"${LOGFILE}"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info in fstab=="
    cat "${2}" >>"${LOGFILE}"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info end=="
    
    return 0
}


########################################################################
# 函数描述： 检查操作系统目前的swap使用信息，是否和/etc/fstab一致
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_swap()
{
    local line=""
    local line_link=""

    #临时保存文件
    local swap_dev_sys="${TMPDIR}"/devlist_swp_sys
    local swap_dev_sys_tmp="${TMPDIR}"/devlist_swp_sys.tmp
    local swap_dev_sys_tmp_sort="${TMPDIR}"/devlist_swp_sys.tmp.sort
    local swap_dev_fstab="${TMPDIR}"/devlist_swp_fstab
    local swap_dev_fstab_tmp="${TMPDIR}"/devlist_swp_fstab.tmp
    local swap_dev_fstab_tmp_sort="${TMPDIR}"/devlist_swp_fstab.tmp.sort
    
    
    #清空临时配置文件，后面会重新生成，否则会对后面可能有影响
    rm -rf "${swap_dev_sys}"
    rm -rf "${swap_dev_sys_tmp}"
    rm -rf "${swap_dev_sys_tmp_sort}"
    rm -rf "${swap_dev_fstab}"
    rm -rf "${swap_dev_fstab_tmp}"
    rm -rf "${swap_dev_fstab_tmp_sort}"
    
    
    #获取系统中目前挂载的swap信息
    ${CMD_SWAPON_S} | grep "^/dev/" | awk '{print $1}' >${swap_dev_sys}
    while read line
    do
        line_link="$(${CMD_READLINK_E} ${line})"
        if [ -z "${line_link}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] sys swap(${line}) readlink -e ${line} failed."
            return 1
        fi
        echo ${line_link} >>"${swap_dev_sys_tmp}"
    done <"${swap_dev_sys}"
    cat "${swap_dev_sys_tmp}" 2>/dev/null | sort >"${swap_dev_sys_tmp_sort}"
    
    
    #获取fstab里配置的swap信息
    ${CMD_CAT_FSTAB} | grep -v "^[[:space:]]*#" | grep "[[:space:]]swap[[:space:]]" | grep -E "^[[:space:]]*/dev/|^[[:space:]]*UUID=" | awk '{print $1}' >${swap_dev_fstab}
    while read line
    do
        if [[ ${line} =~ ^UUID=.* ]]; then
            local line_save="${line}"
            line="$(${CMD_FINDFS} ${line} 2>/dev/null)"
            if [ -z "${line}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab swap(${line_save}) findfs ${line_save} failed."
                return 1
            fi
        fi
        
        line_link="$(${CMD_READLINK_E} ${line})"
        if [ -z "${line_link}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab swap(${line}) readlink -e ${line} failed."
            return 1
        fi
        echo "${line_link}" >>"${swap_dev_fstab_tmp}"
    done <"${swap_dev_fstab}"
    cat "${swap_dev_fstab_tmp}" 2>/dev/null | sort >"${swap_dev_fstab_tmp_sort}"

    #对比
    diff "${swap_dev_sys_tmp_sort}" "${swap_dev_fstab_tmp_sort}" >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] sys swap and fstab swap are different."
        record_swap_info "${swap_dev_sys_tmp_sort}" "${swap_dev_fstab_tmp_sort}"
        return 1
    fi
    
    
    #创建swap分区列表
    get_slaves "${swap_dev_sys_tmp_sort}" "${DEVLIST_SWAP}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get slave ${swap_dev_sys_tmp_sort} ${DEVLIST_SWAP} failed."
        return 1
    fi
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] check swap ok."
    
    #记录方便后续定位
    record_swap_info "${swap_dev_sys}" "${swap_dev_fstab}" 
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的item列有效性
# 参数1说明：配置项的item列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_item()
{
    local item=$1
    
    local item_i_dir=""
    local item_i_size=0
    local item_i_fs=""
    local item_i_user=""
    local item_i_group=""
    local item_i_permission=0
    local item_i_notify=""
    local item_i_ntytimeout=0
    local item_i_raid=""
    
    #最大通知脚本的执行时间
    local max_ntftimeout=1800
    local dir_link=""
    
    
    #合法性判断
    if [ -z "${item}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item para is null."
        return 1
    fi
    
    if [ ! -f "${CURRPATH}/ini-plugin/conf/${item}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${CURRPATH}/ini-plugin/conf/${item}) is not exist."
        return 1
    fi
    
    
    #挂载目录
    item_i_dir="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_dir}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get dir failed."
        return 1
    fi
    if [ -L "${item_i_dir}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${item_i_dir}) is a symbolic link."
        return 1
    fi
    
    dir_link="$(readlink -m ${item_i_dir})"
    echo "${item_i_dir}" | grep "^${dir_link}$\|^${dir_link}/$" > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${item_i_dir}) is a symbolic path."
        return 1
    fi
    
    #PARTITION_MODE!=2,仅仅是安装diskmgt，不需要检查mount
    if [ $PARTITION_MODE -ne 2 ]; then
        #如果系统已经挂载了，那么整个流程失败
        #说明客户对挂载的目的不明确
        mount | grep " ${dir_link} " >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${dir_link}) already mount."
            return 1
        fi
    fi
    if [ -f "${dir_link}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${dir_link}) is regular file."
        return 1
    fi
    
    
    #设备大小要求
    item_i_size="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*size=" | awk '{print $1}' | sed 's/size=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_size}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get size failed."
        return 1
    fi
    if [ "${item_i_size}" != "max" ]; then
        if [[ ${item_i_size} =~ ^[0-9]+$ ]]; then
            if [ ${item_i_size} -le 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) size(${item_i_size}) is invalid."
                return 1
            fi
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) size(${item_i_size}) is not max/digital."
            return 1
        fi
    fi
    
    
    #文件系统
    item_i_fs="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*filesystem=" | awk '{print $1}' | sed 's/filesystem=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_fs}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get filesystem failed."
        return 1
    fi
    if [ "${item_i_fs}" != "ext3" -a "${item_i_fs}" != "ext4" -a "${item_i_fs}" != "xfs" -a "${item_i_fs}" != "default" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) filesystem(${item_i_fs}) is not ext3/ext4/xfs/default."
        return 1
    fi
    
    
    #用户
    item_i_user="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*user=" | awk '{print $1}' | sed 's/user=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_user}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get user failed."
        return 1
    fi
    

    #群主
    item_i_group="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*group=" | awk '{print $1}' | sed 's/group=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_group}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get group failed."
        return 1
    fi
    getent group | grep "^${item_i_group}:" >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) group(${item_i_group}) is not exist."
        return 1
    fi
    
    
    #权限
    item_i_permission="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*permission=" | awk '{print $1}' | sed 's/permission=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_permission}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get permission failed."
        return 1
    fi
    if [ "${item_i_permission}" != "default" ]; then
        if [[ ${item_i_permission} =~ ^[0-9]+$ ]]; then
            local tmpfile="${HOME}/autopart.permission.tmp_`date +%s`"
            touch "${tmpfile}"
            while [ ! -f "${tmpfile}" ]
            do
                logme 2 "[WARN] [${FUNCNAME}:${LINENO}] ${tmpfile} is not exist still."
                touch "${tmpfile}"
                sleep 1
            done
            chmod ${item_i_permission} "${tmpfile}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) permission(${item_i_permission}) is invalid."
                rm -f "${tmpfile}"
                return 1
            fi
            rm -f "${tmpfile}"
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) permission(${item_i_permission}) is not default/digital."
            return 1
        fi
    fi


    #通知脚本的判断
    item_i_notify="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*notify=" | awk '{print $1}' | sed 's/notify=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_notify}" ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] item(${item}) notify is not set."
    else
        if [ ! -f "${CURRPATH}/ini-plugin/notify/${item_i_notify}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notify(${CURRPATH}/ini-plugin/notify/${item_i_notify}) is not exist."
            return 1
        fi
    fi
    

    #通知脚本的超时时间判断
    item_i_ntytimeout="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*notifytimeout=" | awk '{print $1}' | sed 's/notifytimeout=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_ntytimeout}" ]; then
        if [ ! -z "${item_i_notify}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notify(${item_i_notify}) get timeout failed."
            return 1
        fi
    else
        if [[ ${item_i_ntytimeout} =~ ^[0-9]+$ ]]; then
            if [ ${item_i_ntytimeout} -le 0 -o ${item_i_ntytimeout} -gt ${max_ntftimeout} ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notifytimeout(${item_i_ntytimeout}) is invalid, it should between[1,1800]."
                return 1
            fi
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notifytimeout(${item_i_ntytimeout}) is not digital."
            return 1
        fi
    fi
    
    #RAID 级别检查
    item_i_raid="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*raid=" | awk '{print $1}' | sed 's/raid=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_raid}" ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] item(${item}) raid is not set, default is non raid."
        item_i_raid=nonraid
    else
        if [ "${item_i_raid}" != "nonraid" -a "${item_i_raid}" != "raid0" -a "${item_i_raid}" != "raid1" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] raid(${item_i_raid}) is not nonraid/raid0/raid1."
            return 1
        fi
    fi
    
    #保证该节点的所有的RAID的级别都是一致的
    if [ -z "${RAID_LEVEL}" ]; then
        RAID_LEVEL="${item_i_raid}"
    fi
    
    if [ "X${item_i_raid}" != "X${RAID_LEVEL}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] raid(${item_i_raid}) is not ${RAID_LEVEL}."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的conf列有效性
# 参数1说明：配置项的conf列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_conf()
{
    local conf=$1
    local item=""
    local item_nf=0
    local index=1
    
    
    if [ -z "${conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column conf para is null."
        return 1
    fi
    
    #对conf列进行检查，可能一个设备分为多个分区，要对多个conf进行分别检查
    item_nf=$(echo "${conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${conf}" | awk -F '#' "{print \$${index}}")
        check_column_item "${item}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check item(${item}) invalid."
            return 1
        fi
    done
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的required列有效性
# 参数1说明：配置项的required列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_required()
{
    local required=$1
    
    if [ -z "${required}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column required para is null."
        return 1
    fi
    
    #只能配置为n或者y
    #为n表示此挂载项不是必须挂载的
    #为y表示此挂载项必须挂载的，否则整个挂载流程失败
    if [ "${required}" != "y" -a "${required}" != "n" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] required(${required}) is not y/n."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的care列有效性
# 参数1说明：配置项的care列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_care()
{
    local care=$1
    
    if [ -z "${care}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column care para is null."
        return 1
    fi
    
    #只能配置为n或者y
    #为n表示后面安装后，监控流程中不监控他
    #为y表示后面安装后，监控流程中监控他，包括盘拔出等会进行处理和通知
    if [ "${care}" != "y" -a "${care}" != "n" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] care(${care}) is not y/n."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的condition列有效性
# 参数1说明：配置项的condition列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_condition()
{
    local condition=$1
    local condition_file=""
    
    if [ -z "${condition}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column condition para is null."
        return 1
    fi
    
    #如果为n表示对设备没有任何要求，包括大小、盘符等
    #否则就必须指定脚本，而且这个脚本必须存在
    if [ "${condition}" != "n" ]; then
        condition_file="${CURRPATH}/ini-plugin/condition/${condition}"
        if [ ! -f "${condition_file}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] condition(${condition_file}) is not exist."
            return 1
        fi
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项文件的每一行有效性
# 参数1说明：配置项的每一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_line_inifile()
{
    local line=$1
    local line_column_num=4
    local line_column_nf=0
    
    local column_conf=""
    local column_required=""
    local column_care=""
    local column_condition=""
    
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] iniline para is null."
        return 1
    fi
    
    line_column_nf=$(echo ${line} | awk '{print NF}')
    if [ ${line_column_nf} -ne ${line_column_num} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column num(${line_column_nf}) is not ${line_column_num}."
        return 1
    fi
    
    
    #判断每个列的合法性
    column_conf="$(echo ${line} | awk '{print $1}')"
    column_required="$(echo ${line} | awk '{print $2}')"
    column_care="$(echo ${line} | awk '{print $3}')"
    column_condition="$(echo ${line} | awk '{print $4}')"
    
    #判断conf列
    check_column_conf "${column_conf}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column conf(${column_conf}) invalid."
        return 1
    fi
    
    #判断required列
    check_column_required "${column_required}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column required(${column_required}) invalid."
        return 1
    fi
    
    #判断care列
    check_column_care "${column_care}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column care(${column_care}) invalid."
        return 1
    fi
    
    #判断condition列
    check_column_condition "${column_condition}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column condition(${column_condition}) invalid."
        return 1
    fi
    
    
    return 0   
}


########################################################################
# 函数描述： 检查配置项文件的所有行有效性
# 参数1说明：配置项文件名
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_inifile()
{
    local line=""
    local ini_file="$1"
    local ini_base=""
    local ini_tmp_file=""
    
    
    #合法性判断
    if [ -z "${ini_file}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile para is null."
        return 1
    fi
    echo ${ini_file} | grep '\.ini$' >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile(${ini_file}) end without .ini"
        show_help
        return 1
    fi
    if [ ! -f "${ini_file}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile(${ini_file}) is not exist."
        return 1
    fi
    
    ini_base="$(basename ${ini_file})"
    if [ -z "${ini_base}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] basename ${ini_file} failed."
        show_help
        return 1
    fi
    
    #判断配置文件每一行的有效性
    ini_tmp_file="${TMPDIR}/${ini_base}.aptmp"
    cat "${ini_file}" | grep -v "^[[:space:]]*#" | grep -v "^[[:space:]]*$" >"${ini_tmp_file}"
    while read line
    do
        check_line_inifile "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check inifile line(${line}) invalid."
            return 1
        fi
    done <"${ini_tmp_file}"
    
    
    #如果挂载配置文件为空，也是合法的
    #把有条件要求的放在前面首先满足
    ININEWFILE="${TMPDIR}/${ini_base}.ap"
    awk '$2 == "y" && $4 != "n" {print $0}' "${ini_tmp_file}" >"${ININEWFILE}"
    awk '$2 == "y" && $4 == "n" {print $0}' "${ini_tmp_file}" >>"${ININEWFILE}"
    awk '$2 == "n" && $4 != "n" {print $0}' "${ini_tmp_file}" >>"${ININEWFILE}"
    awk '$2 == "n" && $4 == "n" {print $0}' "${ini_tmp_file}" >>"${ININEWFILE}"
    
    #最终的挂载配置文件内容
    INIFINALFILE="${CURRPATH}/ini/${ini_base}_ap.ini"
    cat /dev/null >"${INIFINALFILE}"
    
    return 0
}


########################################################################
# 函数描述： 初始化RAID中的物理设备，如果RAID中还存在未配置实用的RAID
#            那么就创建新的RAID，防止磁盘个数不足导致失败
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_raid_dev()
{
    local level=$1
    
    local raid_adapter="${CURRPATH}/raidadapter/raid_adapter_ctl.sh"
    
    #如果初始化失败也不用在意，记录日志就可以了，也许操作系统内的磁盘就足够了
    bash "${raid_adapter}" -i "${level}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] init raid adapter failed."
    fi
    
    return 0
}

########################################################################
# 函数描述： 获取系统中所有的设备列表，并保存到DEVLIST_ALL中
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_all_disk()
{
    #根据RAID级别初始化RAID设备
    init_raid_dev "${RAID_LEVEL}"
    if [ $? -ne 0 ];then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] init raid dev failed."
        return 1
    fi
    
    #重新扫描
    ${CMD_ECHO_PROC_SCSI} "scsi add-single-device - - - -"
    if [ $IS_BOOT_INSTALL -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] sleep 30s before getting empty disks."
        sleep 30
    fi
    
    #用fdisk命令获取
    ${CMD_FDISK_L} | grep "Disk /dev/" | grep -v "/dev/mapper/" | awk '{ print $2 }' | awk -F'/' '{ print $NF }' | sed s/:$//g >${DEVLIST_ALL}
    if [ ! -s "${DEVLIST_ALL}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fdisk -l get disk failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 获取系统中所有的挂载设备列表，并保存到DEVLIST_MOUNT中
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_mount_disk()
{
    local mount_txt="${TMPDIR}"/devlist_mount.txt
    
    mount | grep "^/dev/" | awk '{print $1}' >"${mount_txt}"
    if [ ! -s "${mount_txt}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get mount empty."
        return 1
    fi
    
    #注意如果是lvm格式化的，需要通过附属信息来获取他包含的所有设备分区
    get_slaves "${mount_txt}" "${DEVLIST_MOUNT}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get slave ${mount_txt} ${DEVLIST_MOUNT} failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 获取系统中所有的正在后台格式化的设备列表，并保存到DEVLIST_MKFS中
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_mkfs_disk()
{
    local mkfs_txt="${TMPDIR}/devlist_mkfs.txt"
    
    #保存所有后台正在格式化的设备，这些设备可能是客户本身触发的，我们不能使用，否则会有冲突
    ps aux | grep " mkfs\." | grep -o "/dev/[^ ]*" >"${mkfs_txt}"
    get_slaves "${mkfs_txt}" "${DEVLIST_MKFS}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get slave ${mkfs_txt} ${DEVLIST_MKFS} failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 获取系统中所有的存在分区的设备列表，并保存到DEVLIST_PARTITION中
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_partition_disk()
{
    local line=""
    local partitions_txt="${TMPDIR}"/devlist_partition.txt
    
    #获取所有的有分区的设备
    cat /proc/partitions | awk '{print $NF}' | grep -v name | grep -v '^[[:space:]]*$' >"${partitions_txt}"
    if [ ! -s "${partitions_txt}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get /proc/partitions empty."
        return 1
    fi
    cat "${partitions_txt}" >"${DEVLIST_PARTITION}"
    
    #再增加一次，重复没关系，因为后续判断在有分区列表里即可
    while read line
    do
        ls /sys/block/${line}/slaves/ 2>/dev/null >>"${DEVLIST_PARTITION}"
    done <"${partitions_txt}"
    
    
    return 0
}


########################################################################
# 函数描述： 获取系统中所有的软raid的设备列表，并保存到DEVLIST_RAID中
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_raid_disk()
{
    local line=""
    local raid_device_txt="${TMPDIR}"/raid_devlist.txt

    #获取所有的软raid设备
    cat /proc/mdstat | grep "^[[:space:]]*md[[:digit:]]*[[:space:]]*:" | awk '{print $1}' >"${raid_device_txt}"
    cat "${raid_device_txt}" >"${DEVLIST_RAID}"

    #获取软raid设备中的相关的设备
    while read line
    do
        cat /proc/mdstat | grep ${line} | awk '{for(i=2;i<NF+1;i++){print $i}}' | grep '\[' | awk -F '[' '{print $1}' 2>/dev/null >>"${DEVLIST_RAID}"
    done <"${raid_device_txt}"

    return 0
}


########################################################################
# 函数描述： 获取指定设备包含的附属设备名，并保存到指定文件中
# 参数1说明：指定的设备列表
# 参数2说明：指定的保存文件
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_slaves()
{
    local list="$1"
    local record="$2"
    
    local line=""
    local line_link=""
    local dev=""
    
    local slaves_dir=""
    local slaves_file=""
    
    
    #参数合法性检查
    if [ -z "${list}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get slave list para is null."
        return 1
    fi
    if [ ! -f "${list}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get slave list(${list}) is not exist."
        return 1
    fi

    if [ -z "${record}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get slave record para is null."
        return 1
    fi
    if [ -f "${record}" ]; then
        rm -f "${record}"
    fi
    
    #对输入的所有设备，依次进行记录
    while read line
    do
        if [ -z "${line}" ]; then
            continue
        fi
        
        #如果是uuid形式的要通过findfs转换
        if [[ ${line} =~ ^UUID=.* ]]; then
            local line_save="${line}"
            line="$(${CMD_FINDFS} ${line} 2>/dev/null)"
            if [ -z "${line}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] findfs ${line_save} failed."
                return 1
            fi
        fi
        
        #绝对路径
        line_link="$(${CMD_READLINK_E} ${line})"
        if [ -z "${line_link}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] readlink -e ${line} failed."
            return 1
        fi
        
        dev="$(basename ${line_link})"
        if [ -z "${dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] basename ${line_link} failed."
            return 1
        fi
        
        #如果是lvm格式，可以通过如下搜索出他包含的所有设备
        slaves_dir="/sys/block/${dev}/slaves/"
        if [ ! -d "${slaves_dir}" ]; then
            echo "${dev}" >>"${record}"
            continue
        fi
        
        slaves_file="${HOME}/${dev}.sys.block.slaves.txt"
        ls "${slaves_dir}" >"${slaves_file}"
        if [ -s "${slaves_file}" ]; then
            cat "${slaves_file}" >>"${record}"
        else
            echo "${dev}" >>"${record}"
        fi
    done <"${list}"
    
    return 0
}

########################################################################
# 函数描述： 根据磁盘名称获取磁盘分区后的分区名前缀
# 参数1说明：磁盘设备名称，如：sda、vda、nvme0n1
#
# 返回值  ： 分区前缀，如：sda、vda、nvme0n1p
########################################################################
function get_partition_prefix()
{
    local disk_name="$1"
    local partition_prefix="${disk_name}"
    # 获取分区前缀：如果是nvme盘，分区名称需要加p（如磁盘为nvme0n1，则分区名为nvme0n1p1）
    echo "${disk_name}" | grep -q "nvme"
    if [ $? -eq 0 ]; then
        partition_prefix="${disk_name}p"
    fi
    
    echo "${partition_prefix}"
}

########################################################################
# 函数描述： 获取系统中所有的空白盘，即无分区和文件系统信息的设备
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_blank_disk()
{
    local line=""

    #在系统中存在的所有设备中，过滤出空白盘
    while read line
    do
        #在挂载列表中寻找
        cat "${DEVLIST_MOUNT}" | grep -E "^${line}$|^${line}[1-9]" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) exists in mount."
            continue
        fi

        #在swap分区中寻找
        cat "${DEVLIST_SWAP}" 2>/dev/null | grep -E "^${line}$|^${line}[1-9]" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) exists in swap."
            continue
        fi

        #在后台格式化设备中寻找
        cat "${DEVLIST_MKFS}" 2>/dev/null | grep -E "^${line}$|^${line}[1-9]" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) exists in mkfs background."
            continue
        fi

        local partition_prefix=$(get_partition_prefix ${line})
        #过滤lvm设备，若已经做过pv，则跳过
        ${CMD_PVSCAN_S} 2>/dev/null | grep "/dev/${partition_prefix}[1-9]*$" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) exists in pvscan -s."
            continue
        fi

        #过滤loop设备（多数情况下是挂载的镜像文件，而非物理磁盘）
        echo /dev/"${line}" | grep -q "loop" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) is a loop device. Skip it."
            continue
        fi

        #在有分区的设备列表中寻找，主要如果是加了-p参数，那么就不检查过滤此列表了
        if [ ${PARTITION_MODE} -eq 0 ]; then
            ${CMD_BLKID} | grep -E "^/dev/${line}:|^/dev/${line}[1-9]*:" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) has partitions in blkid."
                continue
            fi

            cat "${DEVLIST_PARTITION}" | grep "^${line}[1-9]" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) has partitions in proc."
                continue
            fi

            #设备信息是否存在文件系统格式信息，如果想清空可以通过fdisk然后d选项，或者dd命令实现目的
            ${CMD_FILE_S} /dev/"${line}" | grep -i -E "ext4|ext3|GRUB|swap|partition|xfs|btrfs|iso9660" >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) has partition entire."
                continue
            fi
        fi

        #在raid挂载列表中寻找
        cat "${DEVLIST_RAID}" | grep -E "^${line}$|^${line}[1-9]" >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${line}) exists in /proc/mdstat."
            continue
        fi

        #确认是空白盘
        echo "${line}" >>"${DEVLIST_BLANK}"
    done <"${DEVLIST_ALL}"

    #系统中不存在空白盘
    if [ ! -f "${DEVLIST_BLANK}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] there is no blank disk in system."
    fi

    return 0
}


########################################################################
# 函数描述： 获取系统目前内存所能支持的最大并行格式化个数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_max_parallel()
{
    local free_mem=0
    local cached_mem=0
    local buffers_mem=0
    
    free_mem=$(grep -E "^MemFree:" /proc/meminfo | awk '{print $2}')
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get free mem failed."
        return 1
    fi
    
    buffers_mem=$(grep -E "^Buffers:" /proc/meminfo | awk '{print $2}')
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get buffers mem failed."
        return 1
    fi
    
    cached_mem=$(grep -E "^Cached:" /proc/meminfo | awk '{print $2}')
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get cached mem failed."
        return 1
    fi
    
    #我们认为，平均每1G内存支持1个硬盘格式化比较保险,提升并发度
    CURRPARALLEL=$(((free_mem + buffers_mem + cached_mem)/(1024 * 1024) + 1))
    [ -z "${CURRPARALLEL}" ] && CURRPARALLEL=1
    if [ ${CURRPARALLEL} -gt ${MAXPARALLEL} ]; then
        CURRPARALLEL=${MAXPARALLEL}
    fi
    
    #打印当前最大支持并行格式化个数
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPARALLEL is:${CURRPARALLEL}"
    
    return 0
}


########################################################################
# 函数描述： 判断某个设备是否符合最小硬盘空间的要求
# 参数1说明：指定的设备名
# 参数2说明：指定的最小分区大小
#
# 返回值  ： 0 判断符合要求
#            1 判断不符合要求
########################################################################
function match_dev_size()
{
    local dev="$1"
    local min_size=$2
    local dev_size=0

    #参数合法性判断
    if [ -z "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] match dev para is null."
        return 1
    fi

    if [ ! -b /dev/"${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] match dev(${dev}) is not block."
        return 1
    fi

    if [ -z "${min_size}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] match dev size para is null."
        return 1
    elif [[ ${min_size} =~ ^[0-9]+$ ]]; then
        if [ ${min_size} -le 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] match dev size(${min_size}) is invalid."
            return 1
        fi
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] match dev size(${min_size}) is not digital."
        return 1
    fi

    #获取设备size大小，单位GB
    dev_size=$(${CMD_PARTED} /dev/${dev} unit GB print 2>/dev/null | grep "^Disk /dev/${dev}:" | awk '{print $3}' | awk -F"." '{print $1}' | tr -d '[:alpha:]')  

    #比较是否设备大小符合用户指定要求的最小值
    if [ ${dev_size} -lt ${min_size} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dev(${dev}) size(${dev_size}G) less than min required size(${min_size}G)."
        return 1
    fi

    #打印日志方便定位
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] dev(${dev}) size(${dev_size}G) can match min required size(${min_size}G)."
    return 0
}


########################################################################
# 函数描述： 为需要挂载的每一行寻找匹配合适的设备
# 参数1说明：指定的挂载行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_line_ready()
{
    local ready_line="$1"
    local line_column_num=4
    local line_column_nf=0
    
    local column_conf=""
    local column_required=""
    local column_condition=""
    
    local line=""
    local index=0
    local is_find=0
    
    local item=""
    local item_nf=0
    local item_size=0
    local total_min_size=0
    
    local ret=0
    
    
    if [ -z "${ready_line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line para is null."
        return 1
    fi

    line_column_nf=$(echo ${ready_line} | awk '{print NF}')
    if [ ${line_column_nf} -ne ${line_column_num} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line column num(${line_column_nf}) is not ${line_column_num}."
        return 1
    fi
    
    #获取准备匹配的盘的配置信息
    column_conf="$(echo ${ready_line} | awk '{print $1}')"
    if [ -z "${column_conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get conf column failed."
        return 1
    fi

    column_required="$(echo ${ready_line} | awk '{print $2}')"
    if [ -z "${column_required}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get required column failed."
        return 1
    fi
    
    column_condition="$(echo ${ready_line} | awk '{print $4}')"
    if [ -z "${column_condition}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get condition column failed."
        return 1
    fi
    if [ "${column_condition}" != "n" ]; then
        if [ ! -f "${CURRPATH}/ini-plugin/condition/${column_condition}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line condition(${column_condition}) is not exist."
            return 1
        fi
    fi
    
    #一个设备可能会被分成多个分区
    item_nf=$(echo "${column_conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${column_conf}" | awk -F '#' "{print \$${index}}")
        item_size="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*size=" | awk '{print $1}' | sed 's/size=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_size}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get item(${item}) size failed."
            return 1
        fi
        
        #配置要求只能最后一个分区为max的size，而前面的必须指定具体的size大小
        if [ "${item_size}" == "max" ]; then
            if [ ${index} -ne ${item_nf} ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line item(${item}) size is max, but pos(${index}) is not last(${item_nf})."
                return 1
            fi
            ((total_min_size+=1))
        elif [[ ${item_size} =~ ^[0-9]+$ ]]; then
            if [ ${item_size} -le 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line item(${item}) size(${item_size}) is invalid."
                return 1
            fi
            ((total_min_size+=item_size))
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line item(${item}) size(${item_size}) is not max/digital."
            return 1
        fi
    done
    
    #非法设备大小
    if [ ${total_min_size} -le 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line total min size is invalid."
        return 1
    fi
    
    #在所有的空白盘中寻找合适的设备
    while read line
    do
        cat "${DEVLIST_READY}" 2>&1 | grep "^${line} " >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            continue
        fi
        
        #匹配大小
        match_dev_size "${line}" ${total_min_size}
        if [ $? -ne 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] ready line conf(${column_conf}) unmatch dev(${line}) size."
            continue
        fi
        
        #如果没有条件要求，找到即可
        if [ "${column_condition}" == "n" ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] ready line conf(${column_conf}) match dev(${line}) without condition."
            is_find=1
            echo "${ready_line}" >>"${INIFINALFILE}"
            echo "${line} ${column_conf}" >>"${DEVLIST_READY}"
            break
        fi
        
        #如果有条件要求，把设备名（比如sda等）作为参数调用脚本
        #如果返回0表示满足条件要求，否则不满足，继续找下个空白设备进行匹配
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ready line execute ${column_condition} ${line} now:"
        pushd "${CURRPATH}/ini-plugin/condition/" >/dev/null 2>&1
        if [ ! -x ${column_condition} ]; then
            chmod +x ${column_condition}
        fi
        ./${column_condition} "${line}" >>"${LOGFILE}" 2>&1
        ret=$?
        popd >/dev/null 2>&1
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ready line execute ${column_condition} ${line} return ${ret}."
        
        if [ ${ret} -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] ready line conf(${column_conf}) match dev(${line}) with condition(${column_condition})."
            is_find=1
            echo "${ready_line}" >>"${INIFINALFILE}"
            echo "${line} ${column_conf}" >>"${DEVLIST_READY}"
            break
        fi
    done <"${DEVLIST_BLANK}"
    
    
    #如果没有找到合适的盘
    if [ ${is_find} -eq 0 ]; then
        #但是用户又配置为：他不关心这个目录是否挂载，那么继续往下执行，否则就失败
        #这个用在数据的datanode目录，有几个盘就挂载几个这样的目录
        if [ "${column_required}" == "n" ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] ready line conf(${column_conf}) can not match dev, but is not required so ignore!"
            :
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line conf(${column_conf}) can not match dev."
            return 1
        fi
    fi
    
    return 0
}


########################################################################
# 函数描述： 为需要挂载的所有行寻找匹配合适的设备
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_ready()
{
    local line=""

    if [ -z "${ININEWFILE}" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] never check inifile before."
        return 0
    fi

    #如果系统没有空白盘，但是用户又需要至少挂载一个盘，那么整个流程失败
    if [ ! -f "${DEVLIST_BLANK}" ]; then
        if [ ! -z "$(awk '$2 == "y"' ${ININEWFILE})" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk ready ${ININEWFILE} need blank disk."
            return 1
        fi
        
        return 0
    fi
    
    #依次对每个盘进行寻找合适的设备
    while read line
    do
        disk_line_ready "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk ready line(${line}) invalid."
            return 1
        fi
    done <"${ININEWFILE}"
    
    return 0
}


########################################################################
# 函数描述： 等待后台的格式化进程执行完毕
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function wait_back_pid()
{
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] back pids are (${BACKPIDS})"
    
    if [ -z "${BACKPIDS}" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] no back pid."
        return
    fi
    
    #等待所有的后台格式化进程直到完毕
    #注意等待完毕后把全局进程号保存变量清空，防止系统中pid号重复使用
    ${CMD_WAIT} ${BACKPIDS}
    BACKPIDS=""
}


########################################################################
# 函数描述： 分区后等待具体的设备分区出现（操作系统需要一定时间加载到内核中）
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function wait_dev_appear()
{
    local dev=$1
    local count=0
    local timeout=15
    
    
    if [ -z "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] wait dev appear para is null."
        return 1
    fi
    
    #循环等待，直到设备分区号出现
    #否则等待30秒后，返回失败
    while :;
    do
        #将判断放在前面，提升性能
        if [ -e "${dev}" ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] wait ${dev} appear finally."
            return 0
        fi
        
        #指定设备，防止全局扫描，提升速度
        ${CMD_PARTPROBE} "${dev}" >>"${LOGFILE}" 2>&1
        sleep 2

        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] wait ${dev} appear..."
        ((count++))
        if [ ${count} -gt ${timeout} ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] wait ${dev} appear failed after 30s."
            return 1
        fi
    done
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] BUG."
    return 1
}


########################################################################
# 函数描述： 格式化后等待具体的设备分区对应的uuid生效（操作系统需要一定时间加载到内核中）
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function wait_uuid_appear()
{
    local devpart="$1"
    local devpart_uuid=""
    local count=0
    local timeout=15
    
    #合法性判断
    if [ -z "${devpart}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart para is null."
        return 1
    fi
    
    if [ ! -e "${devpart}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart(${devpart}) is not exist."
        return 1 
    fi
    
    if [ ! -b "${devpart}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart(${devpart}) is not block."
        return 1 
    fi
    
    #获取设备分区对应的uuid
    devpart_uuid=$(${CMD_BLKID} "${devpart}" | awk '{print $2}' | sed 's/\"//g')
    if [ -z "${devpart_uuid}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart(${devpart}) get uuid failed:"
        blkid "${devpart}" >>"${LOGFILE}" 2>&1
        return 1 
    fi
   
    #循环等待，知道查询成功，否则循环查询30秒后返回失败
    #主要针对suse操作系统格式化后，uuid信息不会自动更新到内核中，需要执行partprobe探测后才行
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] findfs ${devpart_uuid} for ${devpart} now..."
    while :;
    do
        #将判断放在前面，提升性能
        ${CMD_FINDFS} ${devpart_uuid} >>"${LOGFILE}" 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] findfs ok."
            return 0
        fi

        if [ $((${count} % 5)) == 0  ]; then
          logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Use partprobe to detect all devices."
          ${CMD_PARTPROBE} >>"${LOGFILE}" 2>&1
        else
          #指定设备，防止全局扫描，提升速度
          ${CMD_PARTPROBE} "${devpart}" >>"${LOGFILE}" 2>&1
        fi
        sleep 2
        
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] wait uuid ${devpart_uuid} appear..."
        ((count++))
        if [ ${count} -gt ${timeout} ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] wait uuid ${devpart_uuid} appear failed after 30s."
            return 1
        fi
    done
    
    logme 2 "[BUG] [${FUNCNAME}:${LINENO}] wait uuid ${devpart_uuid} appear failed in the end."
    return 1
}

########################################################################
# 函数描述： 根据磁盘设备名以及分区编号，获取分区设备名(适配nvme盘)
# 参数1说明：磁盘设备名
# 参数2说明：分区编号
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_partition()
{
    local disk="$1"
    local index="$2"
    local partation=""
    
    #如果是nvme协议的ssd盘，分区编号为p1, p2, p3...
    echo "${disk}" | grep -q 'nvme'
    if [ $? -eq 0 ]; then
        partation="/dev/${disk}p${index}"
    else
        partation="/dev/${disk}${index}"
    fi
    
    echo "${partation}"
}

########################################################################
# 函数描述： 对挂载项的每一行进行分区
# 参数1说明：挂载项的具体某一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_line_mklabel()
{
    local line="$1"
    local index=0
    
    local conf=""
    local item=""
    local item_nf=0
    local item_i_size=0
    local item_i_size_sectors=0
    
    local dev=""
    local dev_part=""
    local dev_cylinders=0
    local dev_heads=0
    local dev_sectors=0
    local dev_align=0
    local dev_size_sector=0
    local dev_max_size=0

    local dev_begin_sct=0
    local dev_end_sct=0
    
    local parted_major_version=0
    local senior_parted=0

    local dev_begin_size="0"
    local dev_end_size="0"
    
    
    #合法性判断
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel line para is null."
        return 1
    fi
    
    dev="$(echo ${line} | awk '{print $1}')"
    if [ -z "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel get dev column failed."
        return 1
    fi
    if [ ! -b /dev/"${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel get dev(${dev}) is not block."
        return 1
    fi
    
    conf="$(echo ${line} | awk '{print $2}')"
    if [ -z "${conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel get conf column failed."
        return 1
    fi

    #只有2.0以上的版本的parted命令才包含自动对齐的功能
    parted_major_version=$(${CMD_PARTED} -v | awk '{if(NR==1)print $4}' | awk -F"." '{print $1}')
    if [ -n ${parted_major_version} -a ${parted_major_version} -ge 2 ]; then
        senior_parted=1
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] senior parted version ${parted_major_version}."
    fi

    #获取设备属性
    dev_cylinders=$(${CMD_SFDISK_G} /dev/"${dev}" 2>/dev/null | awk '{print $2}')
    if [ -z "${dev_cylinders}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) get cylinders failed."
        return 1
    fi
    
    dev_heads=$(${CMD_SFDISK_G} /dev/"${dev}" 2>/dev/null | awk '{print $4}')
    if [ -z "${dev_heads}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) get heads failed."
        return 1
    fi
    
    dev_sectors=$(${CMD_SFDISK_G} /dev/"${dev}" 2>/dev/null | awk '{print $6}')
    if [ -z "${dev_sectors}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) get sectors failed."
        return 1
    fi
    
    dev_align=$((dev_heads * dev_sectors * 8))
    if [ ${dev_align} -le 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) get align(${dev_align}) failed."
        return 1
    fi
    
    dev_size_sector=$((dev_heads * dev_sectors * dev_cylinders))
    if [ ${dev_size_sector} -le 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) get size(${dev_size_sector}s) failed."
        return 1
    fi
    
    #进行分区处理，如果失败，那么进行拔插处理后，再次进行分区
    #这在如果设备里之前已经有旧的文件系统的时候，很有效。
    echo "/dev/${dev}" >>"${DEVLIST_PARTED}"
    ${CMD_PARTED_S_MK_GPT} "/dev/${dev}"
    if [ $? -ne 0 ]; then
        local dev_new=""
        local serial_colon=""
        local serial_space=""
        
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] parted -s /dev/${dev} mklabel gpt failed, try to remove-add-scsi."
        
        if [ ! -b "/dev/${dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) is not block device."
            return 1
        fi
        
        mount | grep "^/dev/${dev}[1-9]* " >/dev/null
        if [ $? -eq 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] /dev/${dev} exists in current mount."
            return 1
        fi

        serial_colon="$(lsscsi | grep "[[:space:]]/dev/${dev}[[:space:]]*$" | awk '{print $1}' | sed 's/\[//g' | sed 's/\]//g')"
        if [ -z "${serial_colon}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get /dev/${dev} scsi serial colon failed."
            return 1
        fi
        serial_space="$(echo ${serial_colon} | sed 's/:/ /g')"
        if [ -z "${serial_space}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get /dev/${dev} scsi serial space failed."
            return 1
        fi
        
        #remove拔盘
        ${CMD_ECHO_PROC_SCSI} "scsi remove-single-device ${serial_space}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] echo scsi remove-single-device ${serial_space} > /proc/scsi/scsi for /dev/${dev} failed."
            return 1
        fi
        sleep 1
        
        #add插盘
         ${CMD_ECHO_PROC_SCSI} "scsi add-single-device ${serial_space}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] echo scsi add-single-device ${serial_space} > /proc/scsi/scsi for /dev/${dev} failed."
            return 1
        fi
        sleep 1
        dev_new="$(lsscsi | grep disk | grep "\/dev" | grep -v DVD | grep -v -i Floppy | grep "^\[${serial_colon}\]" | awk '{print $(NF)}' | awk -F '/' '{print $(NF)}')"
        if [ -z "${dev_new}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get new dev for /dev/${dev} failed."
            return 1
        fi
        wait_dev_appear "/dev/${dev_new}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] wait /dev/${dev_new} appear failed after remove-add-scsi."
            return 1
        fi
        
        dev="${dev_new}"
        ${CMD_PARTED_S_MK_GPT} "/dev/${dev}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] parted -s /dev/${dev} mklabel gpt failed after remove-add-scsi."
            return 1
        fi
        
    fi

    dev_max_size=$(${CMD_PARTED} /dev/${dev} unit GB print 2>/dev/null | grep "^Disk /dev/${dev}:" | awk '{print $3}' | awk -F"." '{print $1}' | tr -d '[:alpha:]')  
    if [ -z "${dev_max_size}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] parted dev(${dev}) get max size(${dev_max_size}GB) failed."
        return 1
    fi
    
    echo "${dev} ${conf}" >>"${DEVLIST_SCSI}"
    
    #一个设备可能分为多个分区
    item_nf=$(echo "${conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${conf}" | awk -F '#' "{print \$${index}}")
        
        if [ ${senior_parted} -eq 0 ]; then
            dev_begin_sct=$(($(${CMD_PARTED_UNIT_PRINT} /dev/${dev} | grep -v "^[[:space:]]*$" | sed -n '$p' | awk '{print $3}' | sed 's/s//g') + 1))
            if [ -z "${dev_begin_sct}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel get begin sector failed."
                return 1
            fi
            if [ ${dev_begin_sct} -lt ${dev_align} ]; then
                logme 1 "[INFO] [${FUNCNAME}:${LINENO}] mklabel dev(${dev}) first partition from align(${dev_align})."
                dev_begin_sct=${dev_align}
            fi
        else
            #使用parted命令进行自动对齐方案
            if [ $index -eq 1 ]; then
                dev_begin_size="0"
            else
                dev_begin_size=$(${CMD_PARTED_UNIT_GB_PRINT} /dev/${dev} | grep -v "^[[:space:]]*$" | sed -n '$p' | awk '{print $3}'  | tr -d '[:alpha:]')
                if [ -z "${dev_begin_size}" ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel get begin size failed."
                    return 1
                fi
            fi
        fi
        
        item_i_size="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*size=" | awk '{print $1}' | sed 's/size=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_size}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel item(${item}) get size failed."
            return 1
        fi
        if [ "${item_i_size}" == "max" ]; then
            if [ ${index} -ne ${item_nf} ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel item(${item}) size is max, but pos(${index}) is not last(${item_nf})."
                return 1
            fi
            if [ ${senior_parted} -eq 0 ]; then
                dev_end_sct=${dev_size_sector}
            else
                dev_end_size="${dev_max_size}"
            fi
        elif [[ ${item_i_size} =~ ^[0-9]+$ ]]; then
            if [ ${item_i_size} -le 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel item(${item}) size(${item_i_size}) is invalid."
                return 1
            fi
            if [ ${senior_parted} -eq 0 ]; then
                item_i_size_sectors=$((item_i_size * 1024 * 1024 * 2 / dev_align * dev_align))
                dev_end_sct=$((dev_begin_sct + item_i_size_sectors - 1))
            else
                dev_end_size=$(echo "${dev_begin_size}+${item_i_size}" | bc -l)
            fi
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel item(${item}) size(${item_i_size}) is not max/digital."
            return 1
        fi
        
        sleep 1
        if [ ${senior_parted} -eq 0 ]; then
            ${CMD_PARTED_S_MK_LOGICAL} "/dev/${dev}" ${dev_begin_sct} ${dev_end_sct}
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] parted -s /dev/${dev} mkpart logical ${dev_begin_sct}s ${dev_end_sct}s failed."
                return 1
            fi
            
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] parted -s /dev/${dev} mkpart logical ${dev_begin_sct}s ${dev_end_sct}s."
        else
            ${CMD_PARTED_S_MK_OPT_LOGICAL} "/dev/${dev}" "${dev_begin_size}GB" "${dev_end_size}GB"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] parted -a optimal -s /dev/${dev} mkpart logical ${dev_begin_size}GB ${dev_end_size}GB failed."
                return 1
            fi
            
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] parted -a optimal -s /dev/${dev} mkpart logical ${dev_begin_size}GB ${dev_end_size}GB."
        fi
        
        #适配nvme协议的ssd盘，分区编号为p1, p2, p3...
        dev_part=$(get_partition ${dev} ${index})
        
        wait_dev_appear "${dev_part}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mklabel wait ${dev_part} appear failed."
            return 1
        fi
    done
      
    return 0
}

########################################################################
# 函数描述： 按照规则把目录名转换为加密后的映射名
# 参数1说明：挂载项的数据目录的目录名（如"/srv/BigData/dbdata_service"）
#
# 返回值  ： 转换后的映射名（如"encrypted-srv-BigData-dbdata_service"）
########################################################################
function get_dir_encrypt_mapper()
{
    local tmp_dir="$1"
    local dir=""
    local tmp_mapper=""
    local mapper=""
    
    dir="$(${CMD_READLINK_M} ${tmp_dir})"
    #把目录中的'\'替换成'-'，并在开头加上"encrypted"字段
    tmp_mapper=$(echo "$dir" | sed 's/\//-/g' 2>/dev/null)
    mapper="encrypted$tmp_mapper"
    echo "$mapper"
    return 0
}



########################################################################
# 函数描述： 使用kms对挂载项的分区加密后格式化
# 参数1说明：挂载项的具体某一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function encrypt_partation_kms()
{
    local partation="$1"
    local dev_mapper="$2"
    local encrypt_option="$3"
    local kms_conf_file="${CURRPATH}/ini-plugin/kms/kms_info"
    local kms_server=""
    local kms_key_version_name=""
    local encrypt_cipher=""
    local key_size=""
    local kms_user_info=""
    local hash_algorithm="sha256"
    local iter_time="2000"
    local kmc_tool_dir="${CURRPATH}/kmsLuksMgt/kmc/"
    local sm4_sector_size=""

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to encrypt $partation."
    
    #获取kms配置
    kms_server=$(cat ${kms_conf_file} | grep "^[[:space:]]*kms_server=" | awk '{print $1}' | sed 's/kms_server=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_server" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms server from $kms_conf_file."
        return 1
    fi
    
    kms_key_version_name=$(cat ${kms_conf_file} | grep "^[[:space:]]*kms_key_version_name=" | awk '{print $1}' | sed 's/kms_key_version_name=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_key_version_name" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms key version name from $kms_conf_file."
        return 1
    fi
    

    encrypt_cipher=$(echo "$encrypt_option" | awk -F ':' '{print $1}')
    if [ -z "$encrypt_cipher" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get encrypt cipher from $encrypt_option."
        return 1
    fi
    
    key_size=$(echo "$encrypt_option" | awk -F ':' '{print $2}')
    if [ -z "$key_size" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key size from $encrypt_option."
        return 1
    fi
    
    if [ "$key_size" -le 0 ] 2>/dev/null ;then 
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Key size in $encrypt_option is not a valid number."
        return 1
    fi
    
    #根据密钥版本获取秘钥材料
    cd $kmc_tool_dir > /dev/null 2>&1
    material=$(./kmc_tool -g -i "GET https://${kms_server}/kms/v1/keyversion/${kms_key_version_name}")
    cd - > /dev/null 2>&1
    if [ -z "${material}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get Material from ${kms_server}."
        return 1
    fi

    #为避免加密分区被占用，先close，失败后也close
    ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null

    #判断是不是sm4算法
    echo ${encrypt_option} | grep "sm4" > /dev/null
    if [ $? -eq 0 ]; then
        sm4_sector_size="--sector-size=4096"
    fi

    #创建加密盘
expect <<EOF
set timeout 30;
spawn -noecho ${CMD_CRYPTSETUP} --batch-mode --cipher $encrypt_cipher --key-size $key_size --hash $hash_algorithm --iter-time $iter_time $sm4_sector_size luksFormat "$partation"
expect "passphrase"
send "${material}\r"
expect eof
EOF
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to encrypt $partation with command: [ ${CMD_CRYPTSETUP} --batch-mode --cipher $encrypt_cipher --key-size $key_size --hash $hash_algorithm --iter-time $iter_time $sm4_sector_size luksFormat $partation $key_file_1 ]."
        return 1
    fi
    
    
    #记录创建的加密映射，如果后边格式化失败，则需要删除
    echo "${dev_mapper}" >> "$ENCRYPT_MAPPER_ROLLBACK"
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start mapper encrypted $partation to $dev_mapper."
    
expect <<EOF
set timeout 30;
spawn -noecho ${CMD_CRYPTSETUP} luksOpen "$partation" "$dev_mapper" >> "$LOGFILE" 2>&1
expect   "passphrase"
send "${material}\r"
expect eof
EOF
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to mapper $partation to $dev_mapper."
        ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null
        return 1
    fi
    if [ ! -b "/dev/mapper/$dev_mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart /dev/mapper/$dev_mapper is not block."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] convert $partation to encrypted $dev_mapper. End encrypt $partation."
    return 0
}



########################################################################
# 函数描述： 对挂载项的分区加密后格式化
# 参数1说明：挂载项的具体某一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function encrypt_partation()
{
    local partation="$1"
    local dev_mapper="$2"
    local encrypt_option="$3"
    local key_file_1="$4"
    local key_file_2="$5"
    local encrypt_cipher=""
    local key_size=""
    local hash_algorithm="sha256"
    local iter_time="2000"
    local sm4_sector_size=""

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to encrypt $partation."
    
    if [ ! -f "${key_file_1}" -o ! -f "${key_file_2}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Key file $(basename ${key_file_1}) or $(basename ${key_file_2}) doesn't exist in {PREINSTALL_DIR}/partition/ini-plugin/keys/."
        return 1
    fi
    
    encrypt_cipher=$(echo "$encrypt_option" | awk -F ':' '{print $1}')
    if [ -z "$encrypt_cipher" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get encrypt cipher from $encrypt_option."
        return 1
    fi
    
    key_size=$(echo "$encrypt_option" | awk -F ':' '{print $2}')
    if [ -z "$key_size" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key size from $encrypt_option."
        return 1
    fi
    
    if [ "$key_size" -le 0 ] 2>/dev/null ;then 
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Key size in $encrypt_option is not a valid number."
        return 1
    fi

    #为避免加密分区被占用，先close，失败后也close
    ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null

    #判断是不是sm4算法
    echo ${encrypt_option} | grep "sm4" > /dev/null
    if [ $? -eq 0 ]; then
        sm4_sector_size="--sector-size=4096"
    fi

    #创建加密盘
    ${CMD_CRYPTSETUP} --batch-mode --cipher "$encrypt_cipher" --key-size "$key_size" --hash "$hash_algorithm" --iter-time "$iter_time" $sm4_sector_size luksFormat "$partation" "$key_file_1">> "$LOGFILE" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to encrypt $partation with command: [ ${CMD_CRYPTSETUP} --batch-mode --cipher $encrypt_cipher --key-size $key_size --hash $hash_algorithm --iter-time $iter_time $sm4_sector_size luksFormat $partation $key_file_1 ]."
        return 1
    fi
    
    #记录创建的加密映射，如果后边格式化失败，则需要删除
    echo "${dev_mapper}" >> "$ENCRYPT_MAPPER_ROLLBACK"
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start mapper encrypted $partation to $dev_mapper."
    ${CMD_CRYPTSETUP} --key-file "$key_file_1" luksOpen "$partation" "$dev_mapper" >> "$LOGFILE" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to mapper $partation to $dev_mapper."
        ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null
        return 1
    fi
    
    #添加备用秘钥
    ${CMD_CRYPTSETUP} --key-file "$key_file_1" luksAddKey "$dev_part" "$key_file_2" >> "$LOGFILE" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to add key $key_file_2 for $dev_mapper."
        return 1
    fi
    
    if [ ! -b "/dev/mapper/$dev_mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] devpart /dev/mapper/$dev_mapper is not block."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] convert $partation to encrypted $dev_mapper. End encrypt $partation."
    return 0
}

########################################################################
# 函数描述： 对挂载项的每一行进行格式化
# 参数1说明：挂载项的具体某一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_line_mkfs()
{
    local line="$1"
    local index=0
    
    local dev=""
    local dev_part=""
    local dev_mapper=""
    
    local conf=""
    local item=""
    local item_nf=0
    local item_i_fs=""
    local mkfs_tool=""
    local mkfs_pid=""
    local item_i_encrypt=""
    local item_i_dir=""
    local item_i_key_management=""
    
    
    #合法性判断
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs line para is null."
        return 1
    fi

    dev="$(echo ${line} | awk '{print $1}')"
    if [ -z "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs get dev column failed."
        return 1
    fi
    if [ ! -b /dev/"${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs get dev(${dev}) is not block."
        return 1
    fi
    
    conf="$(echo ${line} | awk '{print $2}')"
    if [ -z "${conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs get conf column failed."
        return 1
    fi
    
    #一个设备可能分为多个分区，分别进行格式化
    item_nf=$(echo "${conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${conf}" | awk -F '#' "{print \$${index}}")
        
        item_i_fs="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*filesystem=" | awk '{print $1}' | sed 's/filesystem=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_fs}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs item(${item}) get filesystem failed."
            return 1
        fi
        if [ "${item_i_fs}" == "default" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] mkfs item(${item}) chooses default filesystem(${ROOTFS})."
            item_i_fs="${ROOTFS}"
            mkfs_tool="${ROOTMKFS}"
        elif [ "${item_i_fs}" == "ext3" ]; then
            mkfs_tool="${CMD_MKFS_EXT3}"
        elif [ "${item_i_fs}" == "ext4" ]; then
            mkfs_tool="${CMD_MKFS_EXT4}"
        elif [ "${item_i_fs}" == "xfs" ]; then
            mkfs_tool="${CMD_MKFS_XFS}"
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs item(${item}) get filesystem(${item_i_fs}) is not ext3/ext4/xfs/default."
            return 1
        fi
        
        #适配nvme协议的ssd盘，分区编号为p1, p2, p3...
        dev_part=$(get_partition ${dev} ${index})

        item_i_dir="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get dir failed."
            return 1
        fi
        
        #分区加密算法（如：encrypt=aes-xts-plain64，C80新增；需要向前兼容历史版本，找不到该配置项或者该配置项的值为空时，不加密；为default时，加密算法为aes-xts-plain64）
        item_i_encrypt="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*encrypt=" | awk '{print $1}' | sed 's/encrypt=//g' | head -n 1 2>/dev/null)"

        #加密算法为sm4时，cryptsetup的版本不能低于2.1.0
        echo ${item_i_encrypt} | grep "sm4" > /dev/null
        if [ $? -eq 0 ]; then
            check_cryptsetup
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check cryptsetup version failed."
                return 1
            fi
        fi

        if [ ! -z "$item_i_encrypt" ]; then
            item_i_key_management="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*key_management=" | awk '{print $1}' | sed 's/key_management=//g' | head -n 1 2>/dev/null)"
            if [ "$item_i_key_management" == "kms_luks" ]; then
                #获取挂载点对应的映射
                dev_mapper=$(get_dir_encrypt_mapper "$item_i_dir")
                #创建加密盘
                encrypt_partation_kms "$dev_part" "$dev_mapper" "$item_i_encrypt" || return 1
                #把设备更新为加密映射，格式化操作要对加密映射进行操作
                dev_part="/dev/mapper/$dev_mapper"
            else
                #获取挂载点对应的映射
                dev_mapper=$(get_dir_encrypt_mapper "$item_i_dir")
                #创建加密盘
                encrypt_partation "$dev_part" "$dev_mapper" "$item_i_encrypt" "$KEY_FILE_1" "$KEY_FILE_2" || return 1
                #把设备更新为加密映射，格式化操作要对加密映射进行操作
                dev_part="/dev/mapper/$dev_mapper"
            fi
        fi
        
        mkfs_pid=$(${mkfs_tool} ${dev_part} "${NOHUPTXT}")
        BACKPIDS="${BACKPIDS} $mkfs_pid"
        ((PARALLELCNT++))
        if [ $((PARALLELCNT % CURRPARALLEL)) -eq 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] mkfs parallel limit, please wait patiently..."
            wait_back_pid
        fi
    done
        
    return 0
}

########################################################################
# 函数描述： 检查cryptsetup命令版本
# 参数说明： 无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_cryptsetup()
{
    #当前cryptsetup版本
    current_cryptsetup_version=$(cryptsetup --version 2>&1 | tr -cd "[0-9 .]" | awk '{print $1}' | sed -e "s/ //g")
    #cryptsetup最低版本
    lowest_cryptsetup_version="2.1.0"
    #比较当前版本是否高于最低版本
    if [ "${current_cryptsetup_version}" != "" ] && [ "${lowest_cryptsetup_version}" != "" ]; then
        current_array=( $(echo "${current_cryptsetup_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        lowest_array=( $(echo "${lowest_cryptsetup_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        current_array_length=${#current_array[@]}
        lowest_array_length=${#lowest_array[@]}

        length=$((${lowest_array_length}>${current_array_length}?${current_array_length}:${lowest_array_length}))
        local index=0
        while [ $index -lt "${length}" ]
        do
            v1=${current_array[$index]}
            v2=${lowest_array[$index]}
            if [ "${v1}" -lt "${v2}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] the current cryptsetup version(${current_cryptsetup_version}) is earlier than ${lowest_cryptsetup_version},please upgrade cryptsetup."
                return 1
            elif [ "${v1}" -gt "${v2}" ] ; then
                break
            elif [ "${v1}" -eq "${v2}" ] ; then
                let index+=1
                continue
            fi
        done
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current cryptsetup version failed."
        return 1
    fi

    return 0
}

########################################################################
# 函数描述： 如果为加密分区，则更新crypttab临时文件及变量dev_part的值
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function update_crypttab()
{
    local dev_part="$1"
    local retry_count=""
    local retry_max=5
    local dev_uuid_part=""
    local dev_mapper="$2"
    local final_key_1=""
    
    #等待原物理分区的UUID
    retry_count=0
    while [ ${retry_count} -lt ${retry_max} ]
    do
        dev_uuid_part=$(${CMD_BLKID} "${dev_part}" | awk '{print $2}' | sed 's/\"//g')
        if [ -z "${dev_uuid_part}" ]; then
            ((retry_count++))
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] mount item(${item}) get dev(${dev_part}) uuid failed $retry_count times."
            sleep 5
        else
            break
        fi
    done
        
    if [ -z "${dev_uuid_part}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get dev(${dev_part}) uuid failed."
        return 1
    fi
    
    #工作空间在执行成功后会被删除，秘钥文件最终会被拷贝到diskmgt的安装目录下，因此要把这个路径写入文件
    final_key_1="$DISKMGTPATH/ini-plugin/keys/$(basename $KEY_FILE_1)"
    #在临时文件中增加“映射名  UUID  密钥文件”，挂载成功后该文件内容会被追加到/etc/crypttab，用于实现开机自动加载
    echo "$dev_mapper  $dev_uuid_part  $final_key_1" >> "$CRYPTTAB_ROLLBACK"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] echo [ $dev_mapper  $dev_uuid_part  $final_key_1 ] to $CRYPTTAB_ROLLBACK."
    
    return 0
}

########################################################################
# 函数描述： 对挂载项的每一行进行挂载
# 参数1说明：挂载项的具体某一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_line_mount()
{
    local line="$1"
    local index=0
    
    local dev=""
    local dev_part=""
    local dev_mapper=""
    local dev_uuid_part=""
    
    local conf=""
    local item=""
    local item_nf=0
    local item_i_dir=""
    local item_i_dir_tmp=""
    local item_i_fs=""
    local item_i_user=""
    local item_i_group=""
    local item_i_permission=0
    local item_i_encrypt=""
    local item_i_key_management=""
    
    local retry_count=0
    local retry_max=5
    
    #合法性检查
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount line para is null."
        return 1
    fi

    dev="$(echo ${line} | awk '{print $1}')"
    if [ -z "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount get dev column failed."
        return 1
    fi
    if [ ! -b /dev/"${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount get dev(${dev}) is not block."
        return 1
    fi
    
    conf="$(echo ${line} | awk '{print $2}')"
    if [ -z "${conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount get conf column failed."
        return 1
    fi
    
        
    item_nf=$(echo "${conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${conf}" | awk -F '#' "{print \$${index}}")
        
        #适配nvme协议的ssd盘，分区编号为p1, p2, p3...
        dev_part=$(get_partition ${dev} ${index})

        #挂载目录
        item_i_dir_tmp="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_dir_tmp}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get dir failed."
            return 1
        fi
        if [ -L "${item_i_dir_tmp}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${item_i_dir_tmp}) is a symbolic link."
            return 1
        fi
        item_i_dir="$(${CMD_READLINK_M} ${item_i_dir_tmp})"
        if [ -z "${item_i_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] readlink -m ${item_i_dir_tmp} failed."
            return 1
        fi
        echo "${item_i_dir_tmp}" | grep "^${item_i_dir}$\|^${item_i_dir}/$" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${item_i_dir_tmp}) is a symbolic path."
            return 1
        fi
        mount | grep " ${item_i_dir} " >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) dir(${item_i_dir}) already mount."
            return 1
        fi
        
        ${CMD_IS_FILE} "${item_i_dir}"
        if [ $? -eq 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) dir(${item_i_dir}) is regular file."
            return 1
        fi
        
        ${CMD_IS_DIR} "${item_i_dir}"
        if [ $? -ne 0 ]; then
            ${CMD_MKDIR_P} "${item_i_dir}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) mkdir -p ${item_i_dir} failed."
                return 1
            fi
        fi
        
        #如果该分区配置了加密，则需要把加密后的映射挂载到相应目录（创建加密分区的过程查看encrypt_partation函数）
        item_i_encrypt="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*encrypt=" | awk '{print $1}' | sed 's/encrypt=//g' | head -n 1 2>/dev/null)"
        if [ ! -z "$item_i_encrypt" ]; then
            dev_mapper=$(get_dir_encrypt_mapper "$item_i_dir")
            item_i_key_management="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*key_management=" | awk '{print $1}' | sed 's/key_management=//g' | head -n 1 2>/dev/null)"
            if [ "$item_i_key_management" != "kms_luks" ]; then
                update_crypttab "${dev_part}" "${dev_mapper}"
                if [ $? -ne 0 ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to update crypttab for item(${item})."
                    return 1
                fi
            else
                dev_part_kms=${dev_part}
            fi
            #把分区设备名更新为对应的加密映射设备，mount到挂载点时，需要用映射后的设备
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] mapper $dev_part to /dev/mapper/$dev_mapper"
            dev_part="/dev/mapper/$dev_mapper"
        fi
        
        #等待加密映射设备的UUID
        retry_count=0
        while [ ${retry_count} -lt ${retry_max} ]
        do
            if [ "$item_i_key_management" != "kms_luks" ]; then
                dev_uuid_part=$(${CMD_BLKID} "${dev_part}" | awk '{print $2}' | sed 's/\"//g')
            else
                dev_uuid_part=$(${CMD_BLKID} "${dev_part_kms}" | awk '{print $2}' | sed 's/\"//g')
                dev_uuid_mapper=$(${CMD_BLKID} "${dev_part}" | awk '{print $2}' | sed 's/\"//g')
            fi
            
            if [ -z "${dev_uuid_part}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get dev(${dev_part}) uuid failed."
                ((retry_count++))
                sleep 5
            else
                break
            fi
        done
        
        if [ -z "${dev_uuid_part}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get dev(${dev_part}) uuid failed."
            return 1
        fi
        
        
        #防止非root用户（omm用户)写入
        ${CMD_CHOWN} 0:0 "${item_i_dir}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown root:root ${item_i_dir} failed."
            return 1
        fi
        ${CMD_CHMOD} 000 "${item_i_dir}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 000 ${item_i_dir} failed."
            return 1
        fi
        
        
        #文件系统
        item_i_fs="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*filesystem=" | awk '{print $1}' | sed 's/filesystem=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_fs}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get filesystem failed."
            return 1
        fi
        if [ "${item_i_fs}" == "default" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] mount item(${item}) chooses default filesystem(${ROOTFS})."
            item_i_fs="${ROOTFS}"
        elif [ "${item_i_fs}" != "ext3" -a "${item_i_fs}" != "ext4" -a "${item_i_fs}" != "xfs" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) filesystem(${item_i_fs}) is not ext3/ext4/xfs/default."
            return 1
        fi

        #加密分区的映射为软链接，需要先readlink后再查看文件系统
        ${CMD_FILE_S} $(${CMD_READLINK_M} ${dev_part}) | grep -i " ${item_i_fs} " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) mkfs dev(${dev_part}) not filesystem(${item_i_fs}) format."
            ${CMD_FILE_S} "$(${CMD_READLINK_M} ${dev_part})" >>"${LOGFILE}" 2>&1
            return 1
        fi
        
        #用户（注：目前我们配置的是root，而不能配置为大数据需要的omm用户，是因为这个时候omm用户还没有添加进来）
        item_i_user="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*user=" | awk '{print $1}' | sed 's/user=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_user}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get user failed."
            return 1
        fi 
        
        
        #群主
        item_i_group="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*group=" | awk '{print $1}' | sed 's/group=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_group}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get group failed."
            return 1
        fi
        getent group | grep "^${item_i_group}:" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) group(${item_i_group}) is not exist."
            return 1
        fi        
        
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] ready to wait uuid for ${dev_part}."
        wait_uuid_appear "${dev_part}"
        if [ $? -ne 0 ]; then
             logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] wait uuid for ${dev_part} failed."
             return 1
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ready to mount ${dev_part} ${item_i_dir} -o noatime,nodiratime."
        #避免红帽systemd-219-19.el7版本的bug(在某些场景下会将已经mount的分区自动umount掉)
        udevadm settle  >/dev/null 2>&1
        #规避另外一个分区挂载上之后马上又被自动umount掉的OS bug
        systemctl daemon-reload >/dev/null 2>&1
        ${CMD_MOUNT} "${dev_part}" "${item_i_dir}" >>"${LOGFILE}" 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) mount ${dev_part} ${item_i_dir} -o noatime,nodiratime failed."
            return 1
        fi
        
        #kms服务加密分区，记录挂载、映射信息，用来在os重启后能够自动挂载
        KMS_ENCRYPT_DEVS="${CURRPATH}/kmsLuksMgt/ini/kms_encrypt_devs"
        if [ "$item_i_key_management" == "kms_luks" ]; then
            mapper=$(echo ${dev_part} | sed 's/\/dev\/mapper\///g')
            echo "${dev_uuid_part} ${mapper} ${item_i_dir} ${item_i_fs} ${dev_uuid_mapper} defaults,noatime,nodiratime,nofail 1 0" >>"${KMS_ENCRYPT_DEVS}"
        else
            echo "${dev_uuid_part} ${item_i_dir} ${item_i_fs} defaults,noatime,nodiratime,nofail 1 0" >>"${DEVLIST_ROLLBACK}"
        fi

        if [ ! -z "$item_i_encrypt" ]; then
            # 记录 dev_mapper(加密设备名) uuid(fstab) 挂载点 文件系统类型
            echo "${dev_mapper} ${dev_uuid_part} ${item_i_dir} ${item_i_fs}" >> ${ENCRYPT_PARTITION_INFO_ROLLBACK}
        fi

        ${CMD_CHOWN_R} ${item_i_user}:${item_i_group} "${item_i_dir}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) chown ${item_i_user}:${item_i_group} ${item_i_dir} failed."
            return 1
        fi
        
        #权限，如果是默认（default），则使用系统的umask值，我们不做任何修改
        item_i_permission="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*permission=" | awk '{print $1}' | sed 's/permission=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_permission}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get permission failed."
            return 1
        fi
        
        ${CMD_INIT_DATA_DIR} "${item_i_dir}" "${item_i_user}:${item_i_group}" "${item_i_permission}" "${BIGDATA_DATA_HOME}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) init ${item_i_dir} failed."
            return 1
        fi
    done
        
    return 0
}

########################################################################
# 函数描述： 脚本执行的加密分区的挂载结果写入文件系统配置文件中
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function modify_crypttab()
{
    if [ ! -f "${CRYPTTAB_ROLLBACK}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] modify nothing to crypttab."
        return 0
    fi
    
    #修改前先备份，防止信号、掉电等异常；如果文件不存在，则创建
    if [ -f "${CRYPTTABFILE}" ]; then
        ${CMD_CP} "${CRYPTTABFILE}" "${CRYPTTABBAKFILE}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CRYPTTABFILE} ${CRYPTTABBAKFILE} -arf failed."
            return 1
        fi
    else
        ${CMD_TOUCH} "${CRYPTTABFILE}"
        ${CMD_CHMOD} 644 "${CRYPTTABFILE}"
    fi
    
    #追加挂载信息
    ${CMD_ADD_TO_CRYPTTAB} "${CRYPTTAB_ROLLBACK}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cat ${CRYPTTAB_ROLLBACK} >>${CRYPTTABFILE} failed."
        return 1
    fi
    
    sync
}

########################################################################
# 函数描述： 回滚文件系统加密分区配置文件为初始状态（未执行此脚本之前的内容）
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function rollback_crypttab()
{
    #如果crypttab的备份存在，就覆盖回退
    if [ -f "${CRYPTTABBAKFILE}" ]; then
        ${CMD_CP} "${CRYPTTABBAKFILE}" "${CRYPTTABFILE}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CRYPTTABBAKFILE} ${CRYPTTABFILE} -arf failed."
            return 1
        fi
    fi
    sync
}

########################################################################
# 函数描述： 回滚为加密分区创建的映射（避免下次创建时失败）
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function rollback_encrypt_mappers()
{
    local line=""
    local line_link=""
    
    while read line
    do
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to delete mapper ${line}."
        line_link="$(${CMD_READLINK_E} "/dev/mapper/${line}")"
        ${CMD_CRYPTSETUP} luksClose "${line}"
        ${CMD_RM_F} "/dev/mapper/${line}"
        if [ -z "${line_link}" ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] mapper(${line}) readlink -e ${line} failed."
            continue
        fi
        ${CMD_RM_F} "${line_link}"  
    done <"${ENCRYPT_MAPPER_ROLLBACK}"
}

########################################################################
# 函数描述： 脚本执行的挂载结果写入文件系统配置文件中
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function modify_fstab()
{
    if [ ! -f "${DEVLIST_ROLLBACK}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] modify nothing to fstab."
        return 0
    fi
    
    #修改前先备份，防止信号、掉电等异常
    ${CMD_CP} "${FSTABFILE}" "${FSTABBAKFILE}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${FSTABFILE} ${FSTABBAKFILE} -arf failed."
        return 1
    fi
    
    #追加挂载信息
    ${CMD_ADD_TO_FSTAB} "${DEVLIST_ROLLBACK}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cat ${DEVLIST_ROLLBACK} >>${FSTABFILE} failed."
        return 1
    fi
    
    sync
}


########################################################################
# 函数描述： 回滚文件系统配置文件为初始状态（未执行此脚本之前的内容）
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function rollback_fstab()
{
    #如果fstab的备份存在，就覆盖回退
    if [ -f "${FSTABBAKFILE}" ]; then
        ${CMD_CP} "${FSTABBAKFILE}" "${FSTABFILE}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${FSTABBAKFILE} ${FSTABFILE} -arf failed."
        fi
    fi
    sync
}


########################################################################
# 函数描述： 卸载需要回退的挂载信息的每一行
# 参数1说明：需要回退的挂载信息的具体行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback_line_mount()
{
    local line="$1"
    if [ -z "${line}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback line para is null."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback ${line} now."
    
    #卸载1次
    mount | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback ${line} needed once?"
        return 0
    fi
    
    ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback umount ${line} failed once."
    
    
    #卸载2次
    mount | grep " ${line} " >/dev/null 2>&1 
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback ${line} needed twice?"
        return 0
    fi
    
    #强杀进程
    ${CMD_FUSER_MK} "${line}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback fuser -mk ${line} failed."
    fi
    
    ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback umount ${line} failed twice."
    
    
    
    #卸载3次
    mount | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback ${line} needed third times?"
        return 0
    fi
    
    #强杀进程
    kill -9 $(lsof | grep " ${line}" | awk '{print $2}') >/dev/null 2>&1
    mount | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback ${line} needed after kill?"
        return 0
    fi
    
    ${CMD_FUSER_MK} "${line}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback fuser -mk ${line} failed."
    fi
    ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback umount ${line} failed third times, need to umount force."
    
    
    #最后强制卸载
    ${CMD_UMOUNT} -l "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback umount -l ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback umount -l ${line} failed."
    
    
    #再确认下
    mount | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback ${line} needed fourth times?"
        return 0
    fi
    
    
    #这样都卸载不掉，那真没办法了，返回失败吧
    #但是经过测试，不可能代码走到这步的
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rollback ${line} failed finally."
    return 1
}


########################################################################
# 函数描述： 对所有需要回退的挂载信息进行卸载
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback_mount()
{
    local line=""
    
    ${CMD_RM_F} "${DISKMGTPATH}"
    if [ ! -f "${DEVLIST_ROLLBACK}" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback no mount any yet."
        return
    fi
    
    #依次读每一行进行处理
    while read line
    do
        line="$(echo ${line} | awk '{print $2}')"
        rollback_line_mount "${line}"
    done <"${DEVLIST_ROLLBACK}"
    #如果基于kms服务进行分区加密，回滚信息还包括KMS_ENCRYPT_DEVS中的数据
    if [ -s ${KMS_ENCRYPT_DEVS} ]; then
        while read line
        do
            line="$(echo ${line} | awk '{print $3}')"
            rollback_line_mount "${line}"
        done <"${KMS_ENCRYPT_DEVS}"
    fi
    rm -rf "${ENCRYPT_PARTITION_INFO_ROLLBACK}"
    return
}


########################################################################
# 函数描述： 对需要挂载的设备进行分区
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_mklabel()
{
    local line=""

    if [ ! -f "${DEVLIST_READY}" ]; then
        return 0
    fi
    
    #依次读每一行进行处理
    while read line
    do
        disk_line_mklabel "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mklabel line(${line}) failed."
            return 1
        fi
    done <"${DEVLIST_READY}"
    
    return 0
}


########################################################################
# 函数描述： 对需要挂载的设备进行格式化
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_mkfs()
{
    local line=""
    
    local scsi_lines=0
    local each_rate=0
    local now_rate=10
    
    if [ ! -f "${DEVLIST_READY}" ]; then
        return 0
    fi
    
    #粗略统计进度
    scsi_lines=$(cat "${DEVLIST_SCSI}" | wc -l)
    if [ -z "${scsi_lines}" -o ${scsi_lines} -le 0 ]; then
        scsi_lines=1
    fi
    ((each_rate=85/scsi_lines))
    
    #依次读每一行进行处理
    while read line
    do
        disk_line_mkfs "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mkfs line(${line}) failed."
            wait_back_pid
            return 1
        fi
        
        #进度增加
        ((now_rate=now_rate+each_rate))
        rate_mark ${now_rate}
    done <"${DEVLIST_SCSI}"
    
    
    #等待后台格式化完毕
    wait_back_pid
    
    #记录加密分区列表，用于更换秘钥
    if [ -s "${ENCRYPT_MAPPER_ROLLBACK}" ]; then
        cat "${ENCRYPT_MAPPER_ROLLBACK}" | uniq | sort > "${ENCRYPT_MAPPER_LIST}"
    fi

    return 0
}

########################################################################
# 函数描述： 对需要挂载的设备进行挂载
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_mount()
{
    local line=""

    if [ ! -f "${DEVLIST_READY}" ]; then
        return 0
    fi
    
    #依次读每一行进行处理
    while read line
    do
        disk_line_mount "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mount line(${line}) failed."
            return 1
        fi
    done <"${DEVLIST_SCSI}"
    
    return 0
}

########################################################################
# 函数描述： 安装硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            其他返回值 失败
########################################################################
function ins_dmgt()
{
    local install_script="${CURRPATH}/installdmgt.sh"
    
    #判断调用者是否指定要安装
    if [ ${IS_INSTALL} -eq 0 ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] skip install!"
        return 0
    fi
    
    #合法性判断
    if [ ! -f "${install_script}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install ${install_script} is not exist."
        return 1
    fi
    
    #开始安装，调用同目录下的installdmgt.sh脚本
    #所以installdmgt.sh脚本也可以单独调用，指定相应的参数即可，具体执行-h查询
    if [ ! -x "${install_script}" ]; then
        chmod 755 "${install_script}"
    fi

    # 加密分区加密秘钥，安装硬盘管理服务时，直接拷贝加密后的密文
    local sccConf="${SCC_CONF_DIR}/scc.conf"
    local diskPartitionKey1Cipher=$(encrypt_password $(cat ${KEY_FILE_1}) ${sccConf})
    local diskPartitionKey2Cipher=$(encrypt_password $(cat ${KEY_FILE_2}) ${sccConf})
    echo "${diskPartitionKey1Cipher}" > ${KEY_FILE_1}
    echo "${diskPartitionKey2Cipher}" > ${KEY_FILE_2}

    if [ ${IS_FORCE} -eq 0 ]; then
        "${install_script}" -n "${INIFILE}_ap.ini"
    else
        "${install_script}" -n "${INIFILE}_ap.ini" -f
    fi
    
    return $?
}


########################################################################
# 函数描述： 记录进度
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function rate_mark()
{
    local rate=$1
    
    #合法性判断
    if [  -z "${rate}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] rate para is null."
        return
    fi
    
    #进度有效范围
    if [ ${rate} -lt 0 -o ${rate} -gt 100 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] rate(${rate}) is invalid."
        return
    fi
    
    if [ -z "${RATEFILE}" ]; then
        return
    fi
    
    #调用者要保证文件存在
    if [ ! -f "${RATEFILE}" ]; then
        logme 1 "[WARN] [${FUNCNAME}:${LINENO}] ratefile(${RATEFILE}) is not exist."
        return
    fi
    
    #调用者要保证文件可写
    echo ${rate} >"${RATEFILE}"
    return
}


########################################################################
# 函数描述： 回滚格式化过的每一行设备信息
# 参数1说明：准备挂载的分区表项信息
#
# 返回值  ： 0 成功
#            其他返回值 失败
########################################################################
function rollback_line_parted()
{
    local line="$1"

    #合法性判断
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rollback line parted para is null."
        return 1
    fi
    
    if [ ! -b "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rollback line parted dev(${line}) is not block."
        return 1
    fi
    
    #安全起见 判断是否在挂载中
    mount | grep "^${line}[1-9]* " >>"${LOGFILE}" 2>&1
    if [ $? -eq 0 ]; then
        logme 2 "[BUG] [${FUNCNAME}:${LINENO}] rollback line parted dev(${line}) is mounting, can not dd it."
        return 1
    fi
    
    #使用dd命令清除分区表信息
    ${CMD_DD_CLEAN_ZERO} "${line}" >>"${LOGFILE}" 2>&1
    return $?
}


########################################################################
# 函数描述： 回滚格式化过的所有设备信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function rollback_parted()
{
    local line=""
    
    if [ ! -f "${DEVLIST_PARTED}" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] do not need to rollback line parted."
        return 0
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] ready to rollback line parted."
    
    #依次读每一行进行处理
    while read line
    do
        rollback_line_parted "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] rollback line parted ${line} failed."
        fi
    done <"${DEVLIST_PARTED}"
    
    return 0
}

########################################################################
# 函数描述： 检查挂载
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_line_mount_check() {
    local ready_line="$1"
    local line_column_num=4
    local line_column_nf=0
    
    local column_conf=""
    local column_required=""
    local column_condition=""
    
    local line=""
    local index=0
    local is_find=0
    
    local item=""
    local item_nf=0
    local item_size=0
    local total_min_size=0
    
    local ret=0
    
    
    if [ -z "${ready_line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line para is null."
        return 1
    fi

    line_column_nf=$(echo ${ready_line} | awk '{print NF}')
    if [ ${line_column_nf} -ne ${line_column_num} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line column num(${line_column_nf}) is not ${line_column_num}."
        return 1
    fi
    
    #获取准备匹配的盘的配置信息
    column_conf="$(echo ${ready_line} | awk '{print $1}')"
    if [ -z "${column_conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get conf column failed."
        return 1
    fi

    column_required="$(echo ${ready_line} | awk '{print $2}')"
    if [ -z "${column_required}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get required column failed."
        return 1
    fi
    
    column_condition="$(echo ${ready_line} | awk '{print $4}')"
    if [ -z "${column_condition}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get condition column failed."
        return 1
    fi
    if [ "${column_condition}" != "n" ]; then
        if [ ! -f "${CURRPATH}/ini-plugin/condition/${column_condition}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line condition(${column_condition}) is not exist."
            return 1
        fi
    fi
    
    #一个设备可能会被分成多个分区
    item_nf=$(echo "${column_conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${column_conf}" | awk -F '#' "{print \$${index}}")
        item_size="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*size=" | awk '{print $1}' | sed 's/size=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_size}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line get item(${item}) size failed."
            return 1
        fi
        
        #配置要求只能最后一个分区为max的size，而前面的必须指定具体的size大小
        if [ "${item_size}" == "max" ]; then
            if [ ${index} -ne ${item_nf} ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line item(${item}) size is max, but pos(${index}) is not last(${item_nf})."
                return 1
            fi
            ((total_min_size+=1))
        elif [[ ${item_size} =~ ^[0-9]+$ ]]; then
            if [ ${item_size} -le 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line item(${item}) size(${item_size}) is invalid."
                return 1
            fi
            ((total_min_size+=item_size))
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ready line item(${item}) size(${item_size}) is not max/digital."
            return 1
        fi
        
        item_dir="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
        if [ -L "${item_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${item_dir}) is a symbolic link."
            return 1
        fi
        grep " ${item_dir} " "${PROCMOUNTSFILE}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] line item(${item}) mount dir(${item_dir}) not exist in ${PROCMOUNTSFILE}."
            return 1
        fi
        
        expand -t 4 "${FSTABFILE}" | grep -v "^[[:space:]]*#" | grep " ${item_dir} " > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] line item(${item}) mount dir(${item_dir}) not exist in ${FSTABFILE}."
            return 1
        fi
        
        #文件系统
        item_i_fs="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*filesystem=" | awk '{print $1}' | sed 's/filesystem=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_fs}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get filesystem failed."
            return 1
        fi
        if [ "${item_i_fs}" == "default" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] mount item(${item}) chooses default filesystem(${ROOTFS})."
            item_i_fs="${ROOTFS}"
        elif [ "${item_i_fs}" != "ext3" -a "${item_i_fs}" != "ext4" -a "${item_i_fs}" != "xfs" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) filesystem(${item_i_fs}) is not ext3/ext4/xfs/default."
            return 1
        fi

        expand -t 4 "${FSTABFILE}" | grep -v "^[[:space:]]*#" | grep " ${item_dir} " | grep " ${item_i_fs} " > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] line item(${item}) mount dir(${item_dir}) filesystem is not ${item_i_fs}."
            return 1
        fi
        
        #用户（注：目前我们配置的是root，而不能配置为大数据需要的omm用户，是因为这个时候omm用户还没有添加进来）
        item_i_user="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*user=" | awk '{print $1}' | sed 's/user=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_user}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get user failed."
            return 1
        fi
        
        #群主
        item_i_group="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*group=" | awk '{print $1}' | sed 's/group=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_group}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get group failed."
            return 1
        fi
        getent group | grep "^${item_i_group}:" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) group(${item_i_group}) is not exist."
            return 1
        fi
        
        ${CMD_CHOWN_R} ${item_i_user}:${item_i_group} "${item_dir}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) chown ${item_i_user}:${item_i_group} ${item_dir} failed."
            return 1
        fi
        
        #权限，如果是默认（default），则使用系统的umask值，我们不做任何修改
        item_i_permission="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*permission=" | awk '{print $1}' | sed 's/permission=//g' | head -n 1 2>/dev/null)"
        if [ -z "${item_i_permission}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) get permission failed."
            return 1
        fi
        if [ "${item_i_permission}" != "default" ]; then
            ${CMD_CHMOD} ${item_i_permission} "${item_dir}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mount item(${item}) chmod ${item_i_permission} ${item_dir} failed."
                return 1
            fi 
        fi
    done
    
    echo "${ready_line}" >>"${INIFINALFILE}"
    return 0
}

########################################################################
# 函数描述： 对每一个配置项进行检查挂载情况
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function disk_mount_check() {
    local line=""

    if [ -z "${ININEWFILE}" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] never check inifile before."
        return 0
    fi
    
    #依次对每个配置项进行检查
    while read line
    do
        disk_line_mount_check "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mount check line(${line}) invalid."
            return 1
        fi
    done <"${ININEWFILE}"
    
    return 0
}

########################################################################
# 函数描述： 安装基于kms服务加密分区的管理工具
# 参数1说明：无
#
# 返回值  ： 0 成功
#            其他返回值 失败
########################################################################
function ins_kms_manager()
{
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] start install kmsLuksMgt!"
    local install_script="${CURRPATH}/installkms.sh"
    
    #判断是否安装
    if [ ${IS_KMS_ENCRYPT} -eq 0 ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] skip install kms manager!"
        return 0
    fi
    
    #合法性判断
    if [ ! -f "${install_script}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install ${install_script} is not exist."
        return 1
    fi
    
    #开始安装，调用同目录下的installkms.sh脚本
    if [ ! -x "${install_script}" ]; then
        chmod 755 "${install_script}"
    fi
    
    bash "${install_script}"
    
    return $?
    
}


########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{    
    #检查系统有效性
    check_system
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check system failed."
        return 1
    fi
    
    #记录进度为1
    rate_mark 1   
    
    #判断是否只是检查配置文件和系统有效性
    if [ -z "${INICHECK}" ]; then
        check_inifile "${CURRPATH}/ini/${INIFILE}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check inifile failed."
            return 1
        fi
    #不检查配置文件，只是检查操作系统有效性
    elif [ "${INICHECK}" == "os" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] check os only."
    
    #默认是直接分区格式化指定的挂载配置文件
    else
        check_inifile "${CURRPATH}/ini/${INICHECK}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check inicheck failed."
            return 1
        fi
    fi
    
    #记录进度为2
    rate_mark 2
    
    #获取所有的设备
    get_all_disk
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get all disk failed."
        return 1
    fi
    
    #记录进度为3
    rate_mark 3
    
    #获取系统中挂载的设备
    get_mount_disk
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get mount disk failed."
        return 1
    fi
    
    #记录进度为4
    rate_mark 4
    
    #获取站在后台格式化的设备
    get_mkfs_disk
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get mkfs disk failed."
        return 1
    fi
    
    #记录进度为5
    rate_mark 5
    
    #获取有分区的设备
    get_partition_disk
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get partition disk failed."
        return 1
    fi

    #获取有软raid的设备
    get_raid_disk
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get /proc/mdstat disk failed."
        return 1
    fi

    #记录进度为6
    rate_mark 6
    
    #获取系统中存在的所有空白设备
    get_blank_disk
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get blank disk failed."
        return 1
    fi
    
    #记录进度为7
    rate_mark 7
    
    if [ $PARTITION_MODE -ne 2 ]; then
        #为指定的每个挂载项匹配合适的设备
        disk_ready
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk ready failed."
            return 1
        fi
    
        #记录进度为8
        rate_mark 8
    
        #获取最大并行格式化个数
        get_max_parallel
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get max parallel failed."
            return 1
        fi
    
        #如果只是检查配置文件或操作系统有效性，到这里就结束了，没必要往下面继续分区格式化了
        if [ ! -z "${INICHECK}" ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] check os ok."
            return 0
        fi
    
        #执行后台收集脚本（如D状态等）
        run_collect
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] run collect failed."
            return 1
        fi
    
        #进行设备分区
        disk_mklabel
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mklabel failed."
            return 1
        fi
    
        #记录进度为10
        rate_mark 10
    
        #进行设备格式化
        disk_mkfs
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mkfs failed."
            return 1
        fi
    
        #探测分区结构，促使信息写入内核，后续可以通过命令查询某些信息了;partprobe是一个异步命令，执行过程中会短暂导致分区设备找不到
        #${CMD_PARTPROBE}
    
        #记录进度为95
        rate_mark 95
    
        #进行设备挂载
        disk_mount
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] disk mount failed."
            return 1
	    else
            logme 2 "[INFO] current mount status $(${CMD_CHECK_DFP})"
        fi
    else
        disk_mount_check
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check mount failed."
            return 1
        fi
    fi
    
    #记录进度为96
    rate_mark 96
    
    #安装硬盘管理服务
    ins_dmgt
    if [ $? -ne 0 ]; then
        return 1
    else
	    if [ ${IS_INSTALL} -ne 0 ]; then
            ${CMD_CP} "${CURRPATH}/ini/${INIFILE}" /usr/local/diskmgt/inner/ >>"${LOGFILE}" 2>&1
            ${CMD_CHMOD} 644 "/usr/local/diskmgt/inner/${INIFILE}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 644 /usr/local/diskmgt/inner/${INIFILE} failed."
                return 1
            fi
        
            ${CMD_CHOWN} 0:0 "/usr/local/diskmgt/inner/${INIFILE}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown root:root /usr/local/diskmgt/inner/${INIFILE} failed."
                return 1
            fi
		else
		    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] skip post install for diskmgt!"
        fi
    fi
    
    #安装基于kms服务加密分区的管理工具
    ins_kms_manager
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install kmsLuksMgt failed."
        return 1
    fi
    
    #记录进度为99
    rate_mark 99
    
    #修改文件系统配置文件
    modify_crypttab || return 1
    modify_fstab || return 1
    
    #自动检查并挂载fstab文件中的目录
    ${CMD_CHECK_MOUNTA}
    
    #返回成功
    return 0
}


########################################################################
# 函数描述： 清除锁函数
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function clean_lock()
{
    rm -rf "${LOCKFILE}"
}


########################################################################
# 函数描述： 信号处理函数
# 参数1说明：无
#
# 返回值  ： 2 返回信号处理特殊错误码
########################################################################
function sig_catch()
{
    #记得打印，方便后面定位
    logme 2 "[EMER] [${FUNCNAME}:${LINENO}] catch signal!!"
    
    #回退文件系统加密分区配置文件
    rollback_crypttab
    
    #回退创建的加密映射
    rollback_encrypt_mappers

    #回退文件系统配置文件
    rollback_fstab
    
    #回退挂载项
    rollback_mount
    
    #回退分区信息
    rollback_parted
    
    #等待后台格式化完毕
    wait_back_pid
    
    #清理锁
    clean_lock
    
    #返回特定错误码
    exit 2
}

########################################################################
# 函数描述： 备份fstab文件
#   备份/etc/fstab文件至/usr/local/diskmgt/ini/目录下
########################################################################
function backup_fstab_file() {
    local backupDir="/usr/local/diskmgt/ini"
    local result=0
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to backup ${FSTABFILE}"
    if [ ! -d ${backupDir} ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] The directory[${backupDir}] is not exist, so create it."
        ${CMD_MKDIR_P} ${backupDir}
        result=$?
        if [ ${result} -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Create the directory[${backupDir}] of fstab-backup failed[result=${result}]"
            return 1
        fi
    fi

    local backupFile="${backupDir}/fstab.$(date -d today +'%Y-%m-%d_%H-%M-%S')"
    ${CMD_CP} "${FSTABFILE}" "${backupFile}"
    result=$?
    if [ ${result} -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${FSTABFILE} to ${backupFile} -arf failed[result=${result}]."
        return 1
    fi
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Backup ${FSTABFILE} to ${backupFile} successfully"
}

function install_scc_tool() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to init install scc tool[$SCC_INIT_INSTALL_PATH]"
    ${CMD_RM_F} "$SCC_INIT_INSTALL_PATH"
    mkdir -p "$SCC_INIT_INSTALL_PATH"
    cp -r ${CURRPATH}/scc/* "$SCC_INIT_INSTALL_PATH"

    sed -i "s|#SCC_ROOT_DIR#|$SCC_INIT_INSTALL_PATH|g" "${SCC_CONF_DIR}/logger.conf"
    sed -i "s|#SCC_ROOT_DIR#|$SCC_INIT_INSTALL_PATH|g" "${SCC_CONF_DIR}/scc.conf"

    local cpuType=$(arch)
    local retVal=0
    ${CMD_UPDATE_RPM_PACKAGE} "$SECCOMPONENT_PACKAGE_NAME" "$SCC_INIT_INSTALL_PATH/package/${SECCOMPONENT_PACKAGE_NAME}.${cpuType}.rpm"
    retVal=$?
    if [ $retVal -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to update seccomponent rpm package."
        return 1
    fi

    encrypt_password "$RANDOM" "${SCC_CONF_DIR}/scc.conf" > /dev/null 2>&1
    retVal=$?
    if [ $retVal -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to init scc tool, return-code[$retVal]"
        return 1
    fi

    find ${SCC_INIT_INSTALL_PATH} -type f | xargs -I{} chmod 600 "{}" >/dev/null 2>&1
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to init install scc tool"
}

function init_partition_encryption_key() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to init disk partition encryption key"
    local diskPartitionKey1=$(generate_random_pwd)
    local diskPartitionKey2=$(generate_random_pwd)
    echo "${diskPartitionKey1}" > ${KEY_FILE_1}
    echo "${diskPartitionKey2}" > ${KEY_FILE_2}
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to init disk partition encryption key"
}

function adjust_diskmgt_service_file_permission() {
    #调整diskmgt服务安装目录下文件权限
    ${CMD_CHOWN_R} omm: "${DISKMGTPATH}"
    find "${DISKMGTPATH}" -type f | xargs -I{} "${CMD_CHMOD}" 640 "{}" >/dev/null 2>&1
    find "${DISKMGTPATH}" -name "*.sh" | xargs -I{} "${CMD_CHMOD}" 500 "{}" >/dev/null 2>&1
    find "${DISKMGTPATH}" -name "diskmgt*" -type f | xargs -I{} "${CMD_CHMOD}" 500 "{}" >/dev/null 2>&1
    find "${DISKMGTPATH}" -name "kmsLuksMgt*" -type f | xargs -I{} "${CMD_CHMOD}" 500 "{}" >/dev/null 2>&1
    ${CMD_CHMOD} 500 "${DISKMGTPATH}/ini-plugin/notify/clickhouse_get_replica.py"
    find "${DISKMGTPATH}/ini-plugin/keys" -name "fi_disk*" | xargs -I{} "${CMD_CHMOD}" 600 "{}" >/dev/null 2>&1
    find "${DISKMGTPATH}/scc/ks" -type f | xargs -I{} "${CMD_CHMOD}" 600 "{}" >/dev/null 2>&1
    ${CMD_CHOWN_R} root: "${DISKMGTPATH}"
}

########################################################################
# 磁盘分区后置动作
########################################################################
function post_auto_part() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to perform the automatic partitioning post-action."

    #磁盘分区、挂载和磁盘服务安装为并发异步操作，分区结束后拷贝分区信息至安装服务下
    cat "${ENCRYPT_PARTITION_INFO_ROLLBACK}" | uniq | sort > "${ENCRYPT_PARTITION_INFO}"
    ${CMD_CHMOD} 600 "${ENCRYPT_PARTITION_INFO}"
    ${CMD_CP} "${ENCRYPT_PARTITION_INFO}" "${DISKMGTPATH}/ini-plugin/keys/"
    ${CMD_CHOWN} root:root "${DISKMGTPATH}/ini-plugin/keys/"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] copy file[${ENCRYPT_PARTITION_INFO}] to disk-mgt install path."

    adjust_diskmgt_service_file_permission

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] End to perform the automatic partitioning post-action."
}

#设置当前的umask
UMASK=0027
umask "$UMASK"

#捕获信号
trap "sig_catch" INT TERM QUIT


########################################################################
# 函数描述： 以下为最外层函数，在锁保护的范围内
# 参数1说明：脚本的所有输入参数
#
# 返回值  ： 0 成功
#            1 普通失败错误码
#            2 返回信号处理特殊错误码
#            3 锁保护获取失败错误码
#            4 上次被强杀进程导致的锁遗留错误
#            5 后台正在运行硬盘管理服务错误码
########################################################################
log_begin "$@"

{
    flock -n 3
    #如果获取锁失败，要判断之前是否是被强杀进程导致锁遗留
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] `basename $0` locked by another user."
        
        #这里判断到之前是否是被强杀进程导致锁遗留，就把遗留锁清理干净
        TMPFILE="${HOME}/aptmpfile_`date +%s`.txt"
        ps -wweo pid,ppid,uname,tname,args | grep -v grep | grep "autopart.sh" | grep "/bin/" >"${TMPFILE}"
        CMDCNT=$(cat "${TMPFILE}" | wc -l)
        if [ ${CMDCNT} -eq 1 ]; then
            logme 2 "[ERROR] [none:${LINENO}] `basename $0` kill force last time, clean lock now."
            clean_lock
            exit 4
        fi
        
        exit 3
    fi

    #检查是否已经安装了diskmgt
    check_diskmgt
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [none:${LINENO}] diskmgt service has been installed."
        clean_lock
        backup_fstab_file
        exit 102
    fi
    
    #初始化路径全局变量
    init_path
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] init path failed."
        exit 1
    fi

    source ${CURRPATH}/script/scc_tool.sh
    install_scc_tool
    init_partition_encryption_key

    #脚本输入所有参数的处理流程
    while getopts "ln:p:r:e:ifc:hvd:b" options;
    do
        case ${options} in
            n)
                INIFILE="${OPTARG}"  #输入挂载配置文件
                ;;
                
            p)
                PARTITION_MODE="${OPTARG}"  #是否强制把有分区或文件系统信息的设备，拿来给大数据的目录挂载
                ;;
           
            r)
                RATEFILE="${OPTARG}"  #进度保存文件
                rate_mark 0
                ;;
            e)
                IS_KMS_ENCRYPT="${OPTARG}"  #是否使用kms服务进行分区加密
                ;;
           
            i)
                IS_INSTALL=1  #是否安装硬盘管理服务
                ;;
                
            f)
                IS_FORCE=1  #是否强制安装硬盘管理服务
                ;;

            b)
                IS_BOOT_INSTALL=1  #是否集群初次安装阶段
                ;;
                
            c)
                INICHECK="${OPTARG}"  #检查的配置文件或操作系统
                ;;
                
            l)
                show_ini  #显示所有支持的配置项列表
                exit $?
                ;;
                
            h)
                show_help   #显示帮助
                exit $?
                ;;
            d)
                BIGDATA_DATA_HOME="${OPTARG}"  #数据目录
                ;;
                
            v)
                show_ap_version  #脚本版本
                exit $?
                ;;
            *)
                logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #非法参数输入
                show_help
                exit 1
                ;;
        esac       
    done

    #执行主函数
    main
    #保存返回值，下面进行统一处理，保证出口唯一不会遗漏
    RESULT=$?
    if [ ${RESULT} -ne 0 ]; then
        #只要失败就需要进行回退
        rollback_mount
        logme 2 "[ERROR] [none:${LINENO}] execute `basename $0` failed."
    else
        logme 2 "[INFO] [none:${LINENO}] execute `basename $0` successfully."
        # 磁盘分区格式化挂载成功后，备份/etc/fstab文件
        backup_fstab_file
        post_auto_part
        rate_mark 100
    fi
    
    #保存格式化信息到日志中，方便后面定位
    ${CMD_IS_FILE} "${NOHUPTXT}"
    if [ $? -eq 0 ]; then
        ${CMD_CAT} "${NOHUPTXT}" >>"${LOGFILE}"
    fi

    rm -rf ${SCC_INIT_INSTALL_PATH}
    #清理锁并返回特定返回码
    clean_lock
    exit "${RESULT}"
} 3<>"${LOCKFILE}"

